!###############################################################
!#
!# Note: Module of DirectDet
!#
!# Author: Yue-Lin Sming Tsai
!# Email: smingtsai@gmail.com
!# Date: 2013-12-04
!# Date: 2017-01-31
!#
!# Author: Zuowei Liu, Yushan Su, Yue-Lin Sming Tsai and Binrong Yu
!# Email: zuoweiliu@nju.edu.cn, 1034026870@qq.com, smingtsai@gmail.com, 944174541@qq.com
!# Date: 2017-02-03
!###############################################################
  
module DirectDet
  use DirectDetLike
  implicit none


  real*8 LUX13to16__mx(100),LUX13to16__sigsip(100) ! in [GeV,pb]
  real*8 LUX14to16__mx(100),LUX14to16__sigsip(100) ! in [GeV,pb]

  real*8 LUX13__Q(100),LUX13__eff(100)! Q in [GeV]
  real*8 LUX16__Q(100),LUX16__eff(100)! Q in [GeV]

  real*8 PandaX16__mx(100),PandaX16__sigsip(100)! in [GeV,pb]
  real*8 PandaX16__Q(100),PandaX16__eff(100)! Q in [GeV]
  
  real*8 PandaXRun9Run10__mx(100),PandaXRun9Run10__sigsip(100)! in [GeV,pb]
  real*8 PandaXRun9__Q(100),PandaXRun9__eff(100)! Q in [GeV]
  real*8 PandaXRun10__Q(100),PandaXRun10__eff(100)! Q in [GeV]

  real*8 XENON1T__mx(100),XENON1T__sigsip(100)! in [GeV,pb]
  real*8 XENON1T__Q(100),XENON1T__eff(100)! Q in [GeV]

  ! Astrophysics parameters
  real*8 :: DD__vrot,DD__v0,DD__vesc,DD__vobs
  real*8 :: DD__vrms,DD__vmean
  real*8 :: DD__vloc(3),DD__vpec(3)
  real*8 :: DD__vbulk(3),DD__vsun(3)
  real*8 :: DD__rho0
  real*8 :: DD__f_Distribution(2,200) 

  logical :: DD__exist_Ftable ! The internal table using for velocity distribution.
  logical :: DD__use_formula

  real*8 :: DD__v0_list(50),DD__vesc_list(50)
  real*8 :: DD__vMaps(50,50,2,200)

  logical :: DD__loaded
  logical :: DD__seebug 


contains


!=======================================================================






  !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  !*** Loading all the constants.
  !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  subroutine load_Velocities(Table_or_Formula,v0,vesc)
    implicit none
    real*8,parameter :: pi=dacos(-1.0d0)
    real*8,optional,intent(in) :: v0,vesc 
    CHARACTER(LEN=*), intent(in) :: Table_or_Formula
    logical problem 
    integer i

    if (trim(Table_or_Formula) .eq. 'Help' .or.  &
        trim(Table_or_Formula) .eq. 'help') call LikeDM_DD_doc('load_Velocities')



    if (present(v0)) then 
      DD__v0= v0
      DD__vrot = v0  ! The disk rotation speed [km/s]
    else
      DD__v0=220d0
      DD__vrot =220d0
    endif


    DD__vesc  = 544d0   ! escape velocity [km/s]
    if (present(vesc)) DD__vesc=vesc


    DD__vloc  = (/ 0d0, DD__vrot, 0d0 /)
    DD__vpec  = (/ 11d0, 12d0, 7d0 /)  !Sun's peculiar velocity vector, relative to a rest frame. [km/s]
    DD__vsun(:)  =  DD__vloc(:) + DD__vpec(:)

    DD__vbulk(:) = (/ 0d0, 0d0, 0d0 /)

    DD__vobs  = sum( (DD__vsun(:)-DD__vbulk(:))**2 )**0.5
    DD__v0    = dabs(DD__vrot)


    DD__vrms = dsqrt(3d0/2d0)*DD__v0    ! rms velocity
    DD__vmean = dsqrt(4d0/pi)*DD__v0    ! mean velocity

    DD__rho0   = 0.32d0 ! local density GeV/cm3



    DD__vloc = (/ 0d0, DD__v0, 0d0 /)
    DD__vsun(:)  =  DD__vloc(:) + DD__vpec(:)
    DD__vobs  = sum( (DD__vsun(:)-DD__vbulk(:))**2 )**0.5
    
    DD__exist_Ftable=.false. 

    if (trim(Table_or_Formula) .eq. 'Table') then 
        call Load_PreGen_Table(problem)
        ! *** If the large table is not exist, then we generate. 
        if (problem) then 
          call PreTableVelocity()
          ! *** The global varibles, DD__vesc and DD__v0 have been changed 
          !     so we have to restore it now. 
          DD__vesc=vesc
          DD__v0=v0
        endif
        call reset_vel_Distr(v0,DD__vesc+DD__vobs)

    elseif (trim(Table_or_Formula) .eq. 'None') then

        call integral_f_over_angle(1d0,DD__vesc+DD__vobs) 

    else
        DD__use_formula=.true.
        DD__exist_Ftable=.false.
    endif


    if (DD__seebug .and. DD__exist_Ftable) then 
      write(*,*)'   '
      write(*,*)' Velocity distribution '
      write(*,*)'   '
      do i=1,200,1
        write(*,*)DD__f_Distribution(1:2,i)
      enddo
    endif
 
    return 
  end subroutine load_Velocities








  !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  ! Load all Direct Detection data
  !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

  subroutine loadDirectDet(Table_or_Formula,v0,vesc)
    implicit none 
    CHARACTER(LEN=*), intent(in) :: Table_or_Formula
    real*8, intent(in) :: v0,vesc
    integer i
    CHARACTER(LEN=10) buf
    logical problem 
    real*8 tmp(100),tmp1(100)
    CHARACTER(LEN=200) file_name
    
  
    ! loading the velocity values 
    call load_Velocities(Table_or_Formula,v0,vesc)
  
    !--- LUX-WS2014-16 cross section limits ---
    file_name='DirectDetData/LUX/2016/cs-mx_LUX-WS2014-16.txt'
    call ResizeData(trim(file_name),100,LUX14to16__mx,LUX14to16__sigsip)
    LUX14to16__sigsip(:)=LUX14to16__sigsip(:)*1d-9 ! zb -> pb
  
    !--- LUX-WS2013-16 cross section limits ---
    file_name='DirectDetData/LUX/2016/cs-mx_LUX-WS2013+WS2014-16.txt'
    call ResizeData(trim(file_name),100,LUX13to16__mx,LUX13to16__sigsip)
    LUX13to16__sigsip(:)=LUX13to16__sigsip(:)*1d-9 ! zb -> pb
  
    !--- PandaX-Run8+Run9 cross section limits ---
    file_name='DirectDetData/PandaX/2016/cs-mx_PandaX-Run8+Run9.txt'
    call ResizeData(trim(file_name),100,PandaX16__mx,PandaX16__sigsip)
    PandaX16__sigsip(:)=PandaX16__sigsip(:)*1d-9 ! zb -> pb
    
    !--- PandaX-Run9+Run10 cross section limits ---
    file_name='DirectDetData/PandaX/Run9+Run10/cs-mx_PandaX_Run9+Run10.txt'
    call ResizeData(trim(file_name),100,PandaXRun9Run10__mx,PandaXRun9Run10__sigsip)
    PandaXRun9Run10__sigsip(:)=PandaXRun9Run10__sigsip(:)*1d-9 ! zb -> pb
    
    !--- XENON1T-2017 cross section limits ---
    file_name='DirectDetData/XENON1T/2017/cs-mx_XENON1T_2017.txt'
    call ResizeData(trim(file_name),100,XENON1T__mx,XENON1T__sigsip)
    XENON1T__sigsip(:)=XENON1T__sigsip(:)*1d-9 ! zb -> pb  
  
    
    !--- LUX 2014-2016 eff. ---
    file_name='DirectDetData/LUX/2016/efficiency_2014-16_blue_LUX.txt'
    call ResizeData(trim(file_name),100,LUX16__Q,tmp)
    file_name='DirectDetData/LUX/2016/efficiency_2014-16_black_LUX.txt'
    call ResizeData(trim(file_name),100,LUX16__Q,LUX16__eff,.false.)
    do i=1,100,1
      LUX16__eff(i)=LUX16__eff(i)*tmp(i)
      LUX16__Q(i)=LUX16__Q(i)*1d-6 ! keV -> GeV
    enddo 
  
  
    !--- LUX 2013 eff. ---
    file_name='DirectDetData/LUX/2016/efficiency_2013_blue_LUX.txt'
    call ResizeData(trim(file_name),100,LUX13__Q,tmp)
    file_name='DirectDetData/LUX/2016/efficiency_2013_black_LUX.txt'
    call ResizeData(trim(file_name),100,LUX13__Q,LUX13__eff,.false.)
    do i=1,100,1
      LUX13__eff(i)=LUX13__eff(i)*tmp(i)
      LUX13__Q(i)=LUX13__Q(i)*1d-6 ! keV -> GeV
    enddo 
  
  
    !--- PandaX16 eff. ---
    file_name='DirectDetData/PandaX/2016/efficiency_PandaX16.txt'
    call ResizeData(trim(file_name),100,PandaX16__Q,PandaX16__eff)
    PandaX16__Q(:)=PandaX16__Q(:)*1d-6 ! keV -> GeV
  

    !--- PandaXRun9 eff. ---
    file_name='DirectDetData/PandaX/Run9+Run10/efficiency_PandaX_Run9.txt'
    call ResizeData(trim(file_name),100,PandaXRun9__Q,PandaXRun9__eff)
    PandaXRun9__Q(:)=PandaXRun9__Q(:)*1d-6 ! keV -> GeV
    
    
    !--- PandaXRun10 eff. ---
    file_name='DirectDetData/PandaX/Run9+Run10/efficiency_PandaX_Run10.txt'
    call ResizeData(trim(file_name),100,PandaXRun10__Q,PandaXRun10__eff)
    PandaXRun10__Q(:)=PandaXRun10__Q(:)*1d-6 ! keV -> GeV
    
    
    !--- XENON1T eff. ---
    file_name='DirectDetData/XENON1T/2017/efficiency_XENON1T.txt'
    call ResizeData(trim(file_name),100,XENON1T__Q,XENON1T__eff)
    XENON1T__Q(:)=XENON1T__Q(:)*1d-6 ! keV -> GeV
    
    
    write(*,*) '... Loaded DirectDet experimental data <<< '
    DD__loaded=.true.
    return
  end subroutine loadDirectDet






  !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  !*** pre-generating all the velocity.
  !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

  subroutine PreTableVelocity()
    implicit none
    integer i,j,k
    real*8 v0(50),vesc(50)

    ! DD__v0    = 235d0
    ! DD__vesc  = 550d0   ! escape velocity [km/s]

    call Gen_linspace(210d0,270d0,50,.false.,v0)
    call Gen_linspace(500d0,600d0,50,.false.,vesc)

    open(unit=23,file='./VelocTables/smooth_truncated.txt',status='unknown')
    write(*,*)'>>> Computing and tabulating velocity distribution.'
    write(*,*)'    Storing in folder, ./VelocTables/*.txt .'
    ! compute DM velocity distribution 
    do i=1,50,1
      do j=1,50,1

        DD__v0=v0(i)
        DD__vesc=vesc(j)

        call integral_f_over_angle(1d0,DD__vesc)
        do k=1,200,1
          write(23,*)v0(i),vesc(j),DD__f_Distribution(1:2,k)
        enddo

      enddo 
    enddo 

    write(*,*) 'Done the Table reading <<<'

    close(23)

    return 
  end subroutine PreTableVelocity



!=======================================================================
! Load_PreGen_Table
!=======================================================================

  subroutine Load_PreGen_Table(problem)
    implicit none
    logical, intent(out) :: problem
    integer i,j,k


    open(unit=23,file='./VelocTables/smooth_truncated.txt',status='old', err=531)

    do i=1,50,1
      do j=1,50,1
        do k=1,200,1
          read(23,*,end=532) DD__v0_list(i),DD__vesc_list(j),DD__vMaps(i,j,1:2,k)
        enddo
      enddo 
    enddo 
    close(23)

    problem=.false.
    write(*,*) '<<<  Successfully read the existing Table ...'
    return

532 close(23)
531 problem=.true.
    return
  end subroutine




  !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  ! Translate DD__vMaps(50,50,2,200) to DD__f_Distribution(2,200) by giving
  ! v and vesc
  !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  subroutine reset_vel_Distr(v0,vesc)
    implicit none
    real*8, intent(in) :: v0,vesc
    integer i

    do i=1,200,1
      call interpXYtoZ(DD__v0_list,DD__vesc_list,DD__vMaps(:,:,1,i), &
                       50,50,.true.,v0,vesc,DD__f_Distribution(1,i))

      call interpXYtoZ(DD__v0_list,DD__vesc_list,DD__vMaps(:,:,2,i), &
                       50,50,.true.,v0,vesc,DD__f_Distribution(2,i))
    enddo 

    DD__v0    = v0
    DD__vesc  = vesc 
    DD__exist_Ftable=.true.


    return
  end subroutine





   !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   !*** The halo velocity profile in the Maxwell-Boltzmann approximation.
   !*** input: u !velocity relative to earth [ km s^-1 ]
   !*** output: f(u)  [ (km/ s)^(-3) ]
    subroutine Maxwell_Boltzmann_f(u_,ans)
   !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
      implicit none
      real*8, intent(in) :: u_
      real*8, intent(out) :: ans
      real*8 v0,vesc,u,vdm(3)
      real*8 norm,z
      real*8,parameter :: pi=dacos(-1.0d0)
      real*8 theta(100),phi(100)  
      real*8 vea(3),ftheta(100),fphi(100)
      integer i,j


      ! using the analytic formula which can speed the computation.  
      if (DD__use_formula) then
        call Formulized_MB_f(u_,ans)
        return 
      endif


      if (DD__exist_Ftable) then 
        call linInt2p(DD__f_Distribution(1,:),    &
                      DD__f_Distribution(2,:),    &
                      200,.false.,u_,ans,.false.,.true.) 
        return
      endif


      vea(1:3)=DD__vsun(1:3)
      v0=DD__v0
      vesc=DD__vesc
      !u=u_+vea

      !--- run over all the parameter space of phi and theta

      call Gen_linspace(0d0,pi,100,.false.,theta)
      call Gen_linspace(0d0,2d0*pi,100,.false.,phi)

      do i=1,100,1
        do j=1,100,1
          vdm(1)=dabs(u_)*dsin(theta(i))*dcos(phi(j)) + vea(1)
          vdm(2)=dabs(u_)*dsin(theta(i))*dsin(phi(j)) + vea(2)
          vdm(3)=dabs(u_)*dcos(theta(i))+ vea(3)

          u=sum( vdm(:)**2 )**0.5

          if (u<=vesc) then
            ! truncated 
            z=vesc/v0


            ! sharply cut
            !    norm=derf(z)-2d0*z*dexp(-z*z)/dsqrt(pi)       
            !    fphi(j)=dexp(-1d0* u**2/ v0**2)/(norm* v0**3 * pi**1.5)

            ! smoothly truncated
            norm=(-2d0*vesc*(3d0*v0**2 + 2d0*vesc**2))/(3d0*dexp(z*z)*dsqrt(pi)*v0**3) + derf(z)
            fphi(j)=(dexp(-1d0* u**2/ v0**2)-dexp(-1d0* vesc**2/ v0**2))/(norm* v0**3 * pi**1.5)

          else
            fphi(j)=0.0d0
          endif 
          
        enddo
        call TableArea(phi,fphi,100,ftheta(i))
        ftheta(i)=ftheta(i)*dsin(theta(i)) !Jacobi
 
      enddo
      call TableArea(theta,ftheta,100,ans)

      return
    end subroutine




   !=======================================================================
   ! To formulize Maxwell_Boltzmann_f
   !=======================================================================
   subroutine Formulized_MB_f(u_,ans)
     implicit none
     real*8, intent(in) :: u_
     real*8, intent(out) :: ans
     real*8,parameter :: pi=dacos(-1.0d0)
     real*8 vea,v0,vesc

     vea=DD__vobs
     v0=DD__v0
     vesc=DD__vesc

     ! smoothly truncated
     if (u_+vea<=vesc) then
         ans=(3d0*dEXP(vesc**2/v0**2)*(-1d0 + dEXP((4d0*u_*vea)/v0**2))*v0**2 - 12d0*dEXP((u_ + vea)**2/v0**2)*u_*vea)&
             /(dEXP((u_ + vea)**2/v0**2)*u_*vea*(-6d0*v0**2*vesc - 4d0*vesc**3 + &
              3d0*dEXP(vesc**2/v0**2)*dSqrt(pi)*v0**3*dErf(vesc/v0)))
     elseif(u_+vea>vesc .and. dabs(u_-vea)<vesc) then
         ans=(3d0*((-1d0 + dEXP((-(u_ - vea)**2 + vesc**2)/v0**2))*v0**2 + (u_ - vea)**2 - vesc**2))&
             /(u_*vea*(-6d0*v0**2*vesc - 4d0*vesc**3 + 3d0*dEXP(vesc**2/v0**2)*dSqrt(pi)*v0**3*dErf(vesc/v0)))
     elseif(dabs(u_-vea)>=vesc) then
         ans=0d0
     endif

     return
   end subroutine





   !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   ! integral f over angle
   !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    subroutine integral_f_over_angle(vmin,vmax)
      implicit none
      real*8, intent(in) :: vmin,vmax
      real*8 v(200),f(200)
      integer i

      DD__exist_Ftable=.false.

      call Gen_linspace(vmin,vmax,200,.false.,v)


      do i=1,200,1
        call Maxwell_Boltzmann_f(v(i),f(i))
      enddo
      DD__f_Distribution(1,:)=v(:)
      DD__f_Distribution(2,:)=f(:)

      DD__exist_Ftable=.true.
 
      return
    end subroutine










   !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
   !@ 
   !@    Calc_dsigmadQ
   !@ 
   !@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

   subroutine Calc_dsigmadQ(mx,sigsip,Q,Z,A,mT,v,ans)
    implicit none
    real*8, intent(in) :: mx,sigsip
    real*8, intent(in) :: Q,Z,A
    real*8, intent(in) :: mT,v  ! in the unit of [GeV,GeV,c]
    real*8, intent(out) :: ans ! in the unit of [GeV-3]
    real*8 ff
    real*8,parameter :: pi=dacos(-1.0d0)
    real*8,parameter :: Mp=0.9383d0 ! [GeV]
    real*8 bn
    real*8 mr,mr_p ! reduced mass of WIMP-nucleus and WIMP-proton
    real*8, parameter :: cmGeV = 1d0/(197.327053d-16)  ! 1 cm-1=197.327053d-16 GeV

    mr=mT*mx/(mT+mx) 
    mr_p=mp*mx/(mp+mx)

    ans=A*A*sigsip*1d-36*cmGeV**2  ! [pb->GeV^-2]
    ans=ans*(mr/mr_p)**2 ! rescale to WIMP-nucleus
    call Helm_FormFactor(Q,int(A),int(Z),ff)
    ans=ans*mT*ff/2d0/(mr*v)**2

    return
  end subroutine 







!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!   Compute the dRdQ per nuclei mass per time. unit:[GeV-1]   
!   Detector_type=PandaX is now availible.
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@



  subroutine Calc_dRdQ(Detector_type,mx,sigsip,Q_,ans)
    implicit none 
    real*8, intent(in) :: mx,sigsip,Q_
    real*8 Q
    real*8, intent(out) :: ans
    CHARACTER(LEN=*), intent(in) :: Detector_type

    real*8, parameter :: c = 299792458d-3  ! Speed of light in km/s
    real*8,parameter :: Mp=0.9383d0 ! [GeV]
    real*8,parameter :: Mn=0.9396d0 ! [GeV]
    real*8,parameter :: NA=6.022d23
    real*8, parameter :: Gev2kg=1.782662e-27
    real*8, parameter :: atomicmassunit=0.931494028d0 ! [GeV]
    real*8, parameter :: cmGeV = 1d0/(197.327053d-16)  ! 1 cm-1=197.327053d-16 GeV
    real*8,parameter :: pi=dacos(-1.0d0)
    integer,parameter :: ngrid=200

    real*8 f(ngrid),v(ngrid),ProbDen(ngrid),avgP(ngrid)

    real*8 A(7),eta(7),Z,Abar,ans_iso(7),xi(7)
    real*8 mr  ! reduced mass
    real*8 mT  ! target mass
    real*8 dsigmadE ! d\sigma/dE_R
    real*8 vmin,vmax
    integer i,j,niso

    if (.not.DD__loaded) call loadDirectDet('formula',220d0,544d0)

    if (trim(Detector_type).eq. 'Xenon') then 
      if (Q_<=0d0) then 
        Q=2d-7 !set a nuclear- recoil energy threshold
      else
        Q=Q_
      endif

      Z=54.0d0
      ! eta is the natural abundance of the ith isotope
      eta(1:7)=(/1.9d-2,26.0d-2,4.1d-2,21.0d-2,27.0d-2,10.0d-2,8.9d-2/)  
      A(1:7)=(/128.0d0,129.0d0,130.0d0,131.0d0,132.0d0,134.0d0,136.0d0/)
      niso=7
      Abar=sum(A(:)*eta(:))
    endif


    do i=1,niso,1  ! run over all the targets 
      mT=A(i)*atomicmassunit
      mr=mT*mx/(mT+mx)

      vmin=dsqrt(Q*mT/2d0/mr/mr)
      !vmax=dmin1(DD__vesc/c,dsqrt(DD__detQmax*mT/2d0/mr/mr))
      vmax=(DD__vesc+DD__vobs)/c
      if (vmax>vmin) then 
        call Gen_linspace(vmin,vmax,ngrid,.false.,v)



        do j=1,ngrid,1

          call Maxwell_Boltzmann_f(v(j)*c,f(j)) ! all the theta and phi were integrated out. 

          f(j)=f(j)*(c**3) ! to nature unit

          call Calc_dsigmadQ(mx,sigsip,Q,Z,A(i),mT,v(j),dsigmadE)

          !avgP(j)=4d0*pi*v(j)**3*f(j)*dsigmadE  ! v*f*dsigmadQ *dv^3  
          avgP(j)=v(j)**3*f(j)*dsigmadE  ! v*f(v)*dsigmadQ *v^2*dv where the sin(theta) 
                                         !   is inside the subroutine: Maxwell_Boltzmann_f   
    
        enddo 
        call TableArea(v,avgP,ngrid,ans_iso(i))
      else
        ans_iso(i)=0d0
      endif

      !xi(i)=1d3*NA*mT*Gev2kg*eta(i)*Abar ! The expression of 1307.5955, not sure what is the definition.
      ans_iso(i)=eta(i)*ans_iso(i)
      ans_iso(i)=ans_iso(i)/mT ! per target mass


    enddo
    ans=sum(ans_iso(:))*DD__rho0/(cmGeV)**3/mx
    return
  end subroutine 







!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!    The total evints  per day per kg 
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  subroutine Predict_events(What_experiment,mx,sigsip,ans)
    implicit none
    real*8, intent(in) :: mx,sigsip
    real*8, intent(out) :: ans
    CHARACTER(LEN=*), intent(in) :: What_experiment
    integer :: i
    integer, parameter :: nQ=100
    real*8 dRdQ(nQ),Q(nQ),eff
    !real*8 :: exposure  ! kg-days
    real*8, parameter :: Gev2kg=1.782662d-27
    real*8, parameter :: cmGeV = 1d0/(197.327053d-16)  ! 1 cm-1=197.327053d-16 GeV
    real*8, parameter :: secGeV = 1d25/(6.582119d0)  ! 1 sec-1=6.582119d-25 GeV



    if (trim(What_experiment).eq. 'PandaX-II') then 

      !exposure=33000d0
      do i=1,nQ,1
        Q(i)=PandaX16__Q(i)
        eff=PandaX16__eff(i)
        call Calc_dRdQ('Xenon',mx,sigsip,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff       
      enddo 
 
      
    elseif (trim(What_experiment).eq. 'PandaXRun9') then 

      do i=1,nQ,1
        Q(i)=PandaXRun9__Q(i)
        eff=PandaXRun9__eff(i)
        call Calc_dRdQ('Xenon',mx,sigsip,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff       
      enddo

      
    elseif (trim(What_experiment).eq. 'PandaXRun10') then 

      do i=1,nQ,1
        Q(i)=PandaXRun10__Q(i)
        eff=PandaXRun10__eff(i)
        call Calc_dRdQ('Xenon',mx,sigsip,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff       
      enddo     
      
      
    elseif (trim(What_experiment).eq. 'LUX-2013') then 

      do i=1,nQ,1
        Q(i)=LUX13__Q(i)
        eff=LUX13__eff(i)
        call Calc_dRdQ('Xenon',mx,sigsip,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff       
      enddo 


    elseif (trim(What_experiment).eq. 'LUX-2014-2016') then 

      do i=1,nQ,1
        Q(i)=LUX16__Q(i)
        eff=LUX16__eff(i)
        call Calc_dRdQ('Xenon',mx,sigsip,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff       
      enddo 


    elseif (trim(What_experiment).eq. 'XENON1T') then 

      do i=1,nQ,1
        Q(i)=XENON1T__Q(i)
        eff=XENON1T__eff(i)
        call Calc_dRdQ('Xenon',mx,sigsip,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff       
      enddo 

    else
      write(*,*)'Not prepared yet!'
      STOP
    endif


    call TableArea(Q,dRdQ,nQ,ans)
    ! from GeV to 1/kg/day
    ans=ans/Gev2kg*secGeV 
    ans=ans*60d0*60d0*24d0

    return
  end subroutine












!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
! 
!  Auxiliary subroutines
! 
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    subroutine sigsip_scalar_exchange(WhatInputs,mx,parm_p,parm_n,ans_p,ans_n)
      !c=======================================================================
      !c  DM nucleon cross section.
      !c  input:
      !c    IF 
      !c      WhatInputs='fp_and_fn', parm_p=fp and parm_n=fn 
      !c    else
      !c      parm_p=sigsip, parm_n=sigsin : proton and neutron spin-independent cross sections
      !c      [units: pb]
      !c
      !c  output:
      !c
      !c    IF 
      !c      WhatInputs='fp_and_fn', ans_p=sigsip and parm_n=sigsin 
      !c    [units: pb]
      !c 
      !c    else
      !c      ans_p=fp and parm_n=fn
      !c   
      !c
      !c  02-Dec-2013 modified by Y.L.S. Tsai for LikeDM 
      !c=======================================================================

      implicit none
      CHARACTER(LEN=*), intent(in) :: WhatInputs
      real*8, intent(in) :: mx
      real*8, intent(in) :: parm_p,parm_n
      real*8, intent(out) :: ans_p,ans_n
      real*8 :: fp,fn
      real*8 :: sigsip,sigsin
      real*8,parameter :: pi=dacos(-1.0d0)
      real*8 mp,mn,fkinp,fkinn
      real*8, parameter :: gev2cm2 = (197.327053d-16)**2
      real*8, parameter :: atomicmassunit=0.931494028d0
      real*8, parameter :: mpu=1.00727646688d0
      real*8, parameter :: mnu=1.0086649156d0

      mp = mpu*atomicmassunit
      mn = mnu*atomicmassunit
      fkinp = 4./pi*(mp*mx/(mp+mx))**2  ! see Eq. 7.36 Physics Reports 267(1996)
      fkinn = 4./pi*(mn*mx/(mn+mx))**2

      if (trim(WhatInputs).eq. 'fp_and_fn') then 
         fp=parm_p
         fn=parm_n
         sigsip = fkinp*(fp/2.d0)**2   ! in gev^-2
         sigsin = fkinn*(fn/2.d0)**2   ! in gev^-2
         ans_p=sigsip
         ans_n=sigsin

      else
         sigsip = parm_p*1.0d-36/gev2cm2
         sigsin = parm_n*1.0d-36/gev2cm2
         fp=2.0d0* (sigsip/fkinp)**0.5d0
         fn=2.0d0* (sigsin/fkinn)**0.5d0
         ans_p=fp
         ans_n=fn

      endif 

      return
    end subroutine






!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!# Taken from micromegas 
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  subroutine calcScalarQuarkFF(muDmd,msDmd,sigmaPiN,sigma0, sigmaS, &
                               ScalarFFPd,ScalarFFPu,ScalarFFPs, &
                               ScalarFFNd,ScalarFFNu,ScalarFFNs)
    real*8 muDmd, msDmd,sigma0,sigmaPiN,sigmaS
    real*8 Mp,Mn
    parameter (Mp=0.9383d0)
    parameter (Mn=0.9396d0)
    real*8 y,z,alpha
    real*8 ScalarFFPd,ScalarFFPu,ScalarFFPs
    real*8 ScalarFFNd,ScalarFFNu,ScalarFFNs

    ! sigma0=sigmaPiN*(1.0d0-sigmaS/sigmaPiN*(1.0d0+muDmd)/msDmd)
    sigmaS=(1.0d0-(sigma0/sigmaPiN))*sigmaPiN*msDmd/(1.0d0+muDmd)

    y=1.0d0-sigma0/sigmaPiN 
    z=1.49d0
    alpha=(2.0d0*z-(z-1.0d0)*y)/(2.0d0+(z-1.0d0)*y)
    ScalarFFPd=2.0d0*sigmaPiN/((1.0d0+ muDmd)*Mp*(1.0d0+alpha))/1.0d3
    ScalarFFPu=muDmd*alpha*ScalarFFPd
    ScalarFFPs=sigmaPiN*y*msDmd/(1.0d0+ muDmd)/Mp/1.0d3

    ScalarFFNd=2.0d0*sigmaPiN*alpha/((1.0d0+ muDmd)*Mn*(1.0d0+alpha))/1.0d3
    ScalarFFNu=muDmd*ScalarFFNd/alpha;
    ScalarFFNs=sigmaPiN*y*msDmd/(1.0d0+ muDmd)/Mn/1.0d3   
    return
  end subroutine
  






!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  subroutine Helm_FormFactor(q,a,z,ff)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!c     Calculates F^2(q) 
!c     
!c     F(q) is Helm type form factor computed using
!c     R1 from Lewin and Smith with s = 0.9 fm and with a and c 
!c     using the least squares fit from Table IIIa in Lewin and Smith.
!c     
!c     Parameters and Equations found in:
!c     Lewin J D and Smith P F 1996 Astropart. Phys. 6 87-112.
!c     Fricke G et al. 1995 Atomic Data and Nuclear Data Tables 60 177-285.
!c     Duda, Kemper, and Gondolo JCAP 0704:012,2007. 
!c     
!c     
!c     Authors: Gintaras Duda gkduda@creighton.edu (2007-06-27)
!c              (w/ students George Reifenberger and Katherine Garrett)
!c
!c     Input: a: Mass number of Element
!c            z: Atomic Number of Element
!c            q: Momentum transfer in GeV
!c
!c     Output: ff: Form Factor squared (normalized to F^2(0) = 1)
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!c-------------------------------------------------------

      implicit none
      real*8, intent(in) :: q
      integer, intent(in) ::  a,z      
      real*8, intent(out) :: ff
      real*8,parameter :: pi=dacos(0d0)*2
      real*8,parameter :: fermiGeV=1.d0/0.1973269602d0 ! conversion [GeV fm]

      real*8 f,s,ha,cpa,hc
      real*8 r1,x,x2,y,rsq,r1tmp

      if (a.eq.1) then
         ff = 1.d0
      else
         s = 0.9d0              ! fm
         ha = 0.52d0            ! fm
         hc = 1.23d0*dexp(dlog(dble(a))/3.d0)-0.6d0
         cpa = (7.d0/3.d0)*(pi*ha)**2
         rsq = hc*hc+cpa
         r1tmp = rsq-5.d0*s*s
         if (r1tmp.gt.0.d0) then
            r1 = dsqrt(r1tmp)
         else
            r1 = dsqrt(rsq)
         endif 
         x = dabs(q*r1*fermiGeV)
         y = q*s*fermiGeV

         if (x.gt.5.d-8) then
            f = 3.d0*(dsin(x)-x*dcos(x))/x**3

         else
            x2 = x*x
            f = 1.d0 + x2*(-0.1d0+x2*(1.d0/280.d0+x2*(-1.d0/15120.d0+ x2/1330560.d0)))
         endif
         ff = f**2*dexp(-y**2)
      endif
      
      return
  end subroutine






!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    subroutine dsddismff_(q,a,z,l2jjpp,l2jjnn,l2jjpn)
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!  Spin-dependent structure functions for direct detection.
!  Full interacting shell model (ISM) calculations
!  input:
!    q : real*8  : momentum transfer in GeV ( q=sqrt(2ME) )
!    a : integer : mass number
!    z : integer : atomic number
!  output:
!    l2jjnn, l2jjpp, l2jjpn : the factors \lambda^2 J (J+1) made functions
!       of the momentum transfer q, for pp, nn, and pn; they are
!       related to Spp, Snn, and Spn through
!           S = \lambda^2 J (J+1) (2J+1)/\pi
!       In turn, Spp, Snn, and Spn are related to the spin-dependent 
!       structure functions S_{00}(q), S_{01}(q), S_{11}(q) defined in
!       Engel (PLB264,114,1991) via
!           Spp = S00+S11+S01
!           Snn = S00+S11-S01
!           Spn = 2*(S00-S11)
!  author: paolo gondolo (paolo@physics.utah.edu) 2008
!=======================================================================
      implicit none
      real*8 q
      integer a,z
      real*8 l2jjpp,l2jjnn,l2jjpn
      real*8 s00,s01,s11,spp,snn,spn
      real*8 y,j
      character*10 ddffsd
      real*8 fermiGeV,pi

      fermiGeV=1.d0/0.1973269602d0 ! conversion [GeV fm]
      pi=4.0d0*datan(1.d0)         ! pi

!     j = total nuclear spin
      ddffsd = ' best'
      if (a.eq.29.and.z.eq.14) then ! Si-29
         ! ressell et al, PRD 48 (1993) 5519
         y = (1.75d0*q*fermiGeV/2.d0)**2
         j = 0.5d0
         s00 = 0.00818d0*exp(-4.428d0*y)
         s11 = 0.00818d0*1.06d0*exp(-6.264d0*y)
         s01 = 0.00818d0*(-2.06d0)*exp(-5.413d0*y)
      else if (a.eq.73.and.z.eq.32) then ! Ge-73
         j = 4.5d0
         y = (2.04d0*q*fermiGeV/2.d0)**2
         if (trim(ddffsd).eq.'ISMR') then
            ! ressell et al, PRD 48 (1993) 5519
            s00 = 0.20313d0*1.102d0*exp(-7.468d0*y)
            s11 = 0.20313d0*exp(-8.856d0*y)
            s01 = 0.20313d0*(-2.099d0)*exp(-8.191d0*y)
         else
            ! dimitrov, engel, pittel, hep-ph/9408246
            s00 = (0.1606d0+y*(-1.1052d0+y*(3.2320d0+y*   &
                (-4.9245d0+y*(4.1229d0+y*(-1.8016d0+y*0.3211d0))))))
            s11 = (0.1164d0+y*(-0.9228d0+y*(2.9753d0+y*   &
                (-4.8709d0+y*(4.3099d0+y*(-1.9661d0+y*0.3624d0))))))
            s01 = (-0.2736d0+y*(2.0374d0+y*(-6.2803d0+y*  &
                (9.9426d0+y*(-8.5710+y*(3.8310d0-y*0.6948d0))))))
         endif
      else if (a.eq.27.and.z.eq.13) then ! Al-27
         ! engel, ressell, towner, ormand, hep-ph/9504322
         y = (1.73d0*q*fermiGeV/2.d0)**2
         j = 2.5d0
         s00 = (0.0929516d0+y*(-0.472059d0+y*(1.05996d0+y*  &
             (-1.01148d0))))
         s11 = (0.0657232d0+y*(-0.449840d0+y*(1.35041d0+y*  &
             (-1.68508d0))))
         s01 = (0.1563300d0+y*(-0.935958d0+y*(2.45779d0+y*  &
             (-2.72621))))
      else if (a.eq.39.and.z.eq.19) then ! K-39
         ! engel, ressell, towner, ormand, hep-ph/9504322
         y = (1.73d0*q*fermiGeV/2.d0)**2
         j = 1.5d0
         s00 = (0.0094999d0+y*(-0.0619718d0+y*(0.162844d0+y* &
             (-0.194282d0+y*0.0891054d0))))
         s11 = (0.0298127d0+y*(-0.2176360d0+y*(0.623646d0+y* &
             (-0.814418d0+y*0.4050270d0))))
         s01 = (0.332044d0+y*(-0.2319430d0+y*(0.638528d0+y*  &
             (-0.798523d0+y*0.3809750d0))))
      else if (a.eq.23.and.z.eq.11) then ! Na-23
         ! ressell, dean, hep-ph/9702290
         y = (1.6864d0*q*fermiGeV/2.d0)**2
         j = 1.5d0
         s00 = (0.0379935d0+y*(-0.174341d0+y*(0.378299d0+y* &
             (-0.342962d0))))
         s01 = (0.0646525d0+y*(-0.350289d0+y*(0.910031d0+y* &
             (-0.985833d0))))
         s11 = (0.0275013d0+y*(-0.169641d0+y*(0.507868d0+y* &
             (-0.617985d0))))
      else if (a.eq.127.and.z.eq.53) then ! I-127
         ! ressell, dean, hep-ph/9702290 Bonn A
         y = (2.282d0*q*fermiGeV/2.d0)**2
         j = 2.5d0
         s00 = exp(-2.d0*y)*(0.0983393d0+y*(-0.489096d0+y*  &
             (1.1402d0+y*(-1.47168d0+y*(1.1717d0+y*         &
             (-0.564574d0+y*(0.158287d0+y*(-0.0238874d0+y*  &
             (0.00154252d0)))))))))
         s11 = exp(-2.d0*y)*(0.0365709d0+y*(-0.194994d0+y*  &
             (0.504876d0+y*(-0.747451d0+y*(0.704334d0+y*    &
             (-0.393018d0+y*(0.121881d0+y*(-0.0191881d0+y*  &
             (0.00121021d0)))))))))
         s01 = exp(-2.d0*y)*(0.11994d0+y*(-0.618424d0+y*    &
             (1.50893d0+y*(-2.07367d0+y*(1.77307d0+y*       &
             (-0.903597d0+y*(0.26002d0+y*(-0.0387025d0+y*   &
             (0.00235675d0)))))))))
      else if (a.eq.129.and.z.eq.54) then ! Xe-129
         ! ressell, dean, hep-ph/9702290 Bonn A
         y = (2.282d0*q*fermiGeV/2.d0)**2
         j = 0.5d0
         s00 = exp(-2.d0*y)*(0.0713238d0+y*(-0.344779d0+y*  &
             (0.755895d0+y*(-0.933448d0+y*(0.690061d0+y*    &
             (-0.302476d0+y*(0.0765282d0+y*(-0.0103169d0+y* &
             (0.000573919d0)))))))))
         s11 = exp(-2.d0*y)*(-2.05825d0+y*(1.80756d0+y*           &
             (-1.27746d0+y*(0.654589d0+y*(-0.221971d0+y*          &
             (0.0454635d0+y*(-0.00425694d0+y*(-0.000136779d0+y*   &
             (0.00004396d0))))))))+2.11016d0/(1.d0+y))
         s01 = exp(-2.d0*y)*(-0.12166d0+y*(0.644351d0+y*          &
             (-1.52732d0+y*(2.02061d0+y*(-1.57689d0+y*            &
             (0.723976d0+y*(-0.190399d0+y*(0.0263823d0+y*         &
             (-0.00148593d0)))))))))
      else if (a.eq.131.and.z.eq.54) then ! Xe-131
         ! ressell, dean, hep-ph/9702290 Bonn A
         y = (2.282d0*q*fermiGeV/2.d0)**2
         j = 1.5d0
         s00 = exp(-2.d0*y)*(0.0296421d0+y*(-0.133427d0+y*  &
             (0.377987d0+y*(-0.579614d0+y*(0.578896d0+y*    &
             (-0.345562d0+y*(0.115952d0+y*(-0.0201178d0+y*  &
             (0.00141793d0)))))))))
         s11 = exp(-2.d0*y)*(0.0250994d0+y*(-0.137716d0+y*  &
             (0.366609d0+y*(-0.53851d0+y*(0.492545d0+y*     &
             (-0.269903d0+y*(0.0836943d0+y*(-0.0133959d0+y* &
             (0.000868668d0)))))))))
         s01 = exp(-2.d0*y)*(-0.0545474d0+y*(0.271757d0+y*  &
             (-0.723023d0+y*(1.0545d0+y*(-0.971333d0+y*     &
             (0.538422d0+y*(-0.168988d0+y*(0.027416d0+y*    &
             (-0.00180527d0)))))))))
      else
        write (*,*)'dsddismff: ISM form factor unavailable for A=',a,' Z=',z
         l2jjpp = -1.d0
         l2jjnn = -1.d0
         l2jjpn = -1.d0
         STOP
      endif

      spp = s00+s11+s01
      snn = s00+s11-s01
      spn = 2.d0*(s00-s11)
      l2jjpp = spp*pi/(2.d0*j+1.d0)
      l2jjnn = snn*pi/(2.d0*j+1.d0)
      l2jjpn = spn*pi/(2.d0*j+1.d0)

      return
    end subroutine 









!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!  Auxiliary functional subroutine  
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  subroutine ResizeData(file_name,n,xout,yout,newX_)
    implicit none 
    CHARACTER(LEN=*), intent(in) :: file_name
    integer, intent(in) :: n
    real*8, intent(inout) :: xout(n)
    real*8, intent(out) :: yout(n)
    logical,optional,intent(in) :: newX_
    logical newX 
    real*8 MAP(2,1000)
    integer i,nrows

    if (present(newX_)) then
      newX=newX_
    else
      newX=.true.   
    endif


    call count_nrows(file_name,nrows)
    write(*,*)trim(file_name),nrows
    open(unit=58,file=trim(file_name),status='old')
    do i=1,nrows,1
      read(58,*) MAP(1:2,i)
    enddo 
    close(58)

    if (newX) call Gen_linspace(MAP(1,1),MAP(1,nrows),100,.true.,xout)

    do i=1,n,1
      call linInt2p(MAP(1,1:nrows),MAP(2,1:nrows),nrows,.false.,&
                    xout(i),yout(i),.false.,.true.)
    enddo

    return 
  end subroutine








!=======================================================================
end module DirectDet
  

