﻿!###############################################################
!#
!# Note: Module of DMFormFactor
!#
!# Author: Zuowei Liu, Yushan Su, Yue-Lin Sming Tsai and Binrong Yu
!# Email: zuoweiliu@nju.edu.cn, 1034026870@qq.com, smingtsai@gmail.com, 944174541@qq.com
!# Date: 2017-01-09
!# Date: 2017-01-31
!###############################################################

module DMFormFactor
  use DirectDet
  implicit none


  ! Particle model parameters
  real*8 DD__coep(15)     ! coep(k) is dimensionless value of coefficient of operator k for protons
  real*8 DD__coen(15)     ! coen(k) is dimensionless value of coefficient of operator k for neutrons
  
  integer DD__EFT_type     ! ==1  if we want to consider interference between operators
                           ! ==2 if we want to use the relation from DirectDM code
  
  real*8 DD__dp(7)         ! dimensionless coefficients considering interference between operators for protons
  real*8 DD__dn(7)         ! dimensionless coefficients considering interference between operators for neutrons

  real*8 DD__Q5(2)         ! coefficients of dimension-five operators using DirectDM code [GeV-1]
  real*8 DD__Q6(4)         ! coefficients of dimension-six operators using DirectDM code [GeV-2]

contains


!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@


   subroutine Calc_dsigmadQ_EFT(jx,mx,Z,A,v,Q,ans)
    implicit none
    real*8, intent(in) :: jx ! the DM spin
    real*8, intent(in) :: mx
    integer, intent(in) :: Z
    integer, intent(in) :: A
    real*8, intent(in) :: v ! in the unit of [c]
    real*8, intent(in) :: Q ! in the unit of [GeV]
    real*8, intent(out) :: ans ! in the unit of [GeV-3]
    real*8 ProbDen
    real*8 mT ! mass of target nuclear [GeV]
    real*8, parameter :: pi=dacos(-1.0d0)
    real*8, parameter :: atomicmassunit=0.931494028d0 ! [GeV]
       
    if (Z==54) then
            if (A==128) then
                mT=127.9035312*atomicmassunit
            elseif (A==129) then 
                mT=128.9047801*atomicmassunit
            elseif (A==130) then 
                mT=129.9035094*atomicmassunit
            elseif (A==131) then 
                mT=130.905072*atomicmassunit
            elseif (A==132) then 
                mT=131.904144*atomicmassunit
            elseif (A==134) then 
                mT=133.905395*atomicmassunit
            elseif (A==136) then 
                mT=135.907214*atomicmassunit
            else
                write(*,*)'do not prepare this target isotope in the code yet.'
                stop             
            endif
    else
        write(*,*)'do not prepare this target isotope in the code yet.'
        stop  
    endif  

    
    if (DD__EFT_type==1) then
                                                                            ! See Eq. 15 @ 1708.04630
        call Calc_total_transition_prob_rel(jx,mx,Z,A,v,Q,ProbDen) !    the unit of the averaged amplitudes 
                                                                            !    has dimension mass^{-4}.
    elseif (DD__EFT_type==2) then
        call Calc_total_transition_prob_DirectDM(jx,mx,Z,A,v,Q,ProbDen)     ! See Eq. 16 and 17 @ 1708.04630
    else                                                                    ! we follow the notation of 1308.6288
        call Calc_total_transition_prob(jx,mx,Z,A,v,Q,ProbDen) 
    endif 
    
    ans=ProbDen*mT/(2d0*pi*v**2)

    return
  end subroutine 



!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
    subroutine Calc_total_transition_prob(jx,mx,Z,A,v,Q,ans)
    implicit none
    ! we follow the notation of 1308.6288
    real*8, intent(in) :: jx ! the DM spin
    real*8, intent(in) :: mx
    integer, intent(in) :: Z
    integer, intent(in) :: A
    real*8, intent(in) :: v ! relative velocity 
    real*8, intent(in) :: Q ! recoil energy
    real*8, intent(out) :: ans
    real*8 coep(15)
    real*8 coen(15)
    ! coep(k) is dimensionless value of coefficient of operator k for protons
    ! coen(k) is dimensionless value of coefficient of operator k for neutrons
    
    integer k
    do k=1,15,1
        coep(k)=DD__coep(k)
        coen(k)=DD__coen(k)
    enddo
    !set value for coefficients
    
    if (Z==54) then
        if (A==128) then
            ans=(jx*mx*Q*(-4.927901526990747e-7*coen(8)**2 - 4.927901526990747e-7*jx*coen(8)**2&
            + Q**11*((-3.8918442417577287e27 - 3.8918442417577287e27*jx)*coen(5)**2&
            + (-5.3255051094897584e20 - 5.3255051094897584e20*jx)*coen(5)*coep(5)&
            + (-1.821822952657045e13 - 1.821822952657045e13*jx)*coep(5)**2)&
            - 7.192301841366522e-7*coen(8)*coep(8) - 7.192301841366522e-7*jx*coen(8)*coep(8)&
            - 2.624301921111693e-7*coep(8)**2 - 2.624301921111693e-7*jx*coep(8)**2&
            + Q**8*((3.777753227010351e21 + 3.777753227010351e21*jx)*coen(5)**2&
            + (-4.952407276007853e21 - 4.952407276007853e21*jx)*coen(8)**2 + (2.510309925596006e21&
            + 2.510309925596006e21*jx)*coen(5)*coep(5) + 3.924196997071157e20*coep(5)**2&
            + 3.924196997071157e20*jx*coep(5)**2 + (-2.5326061081643214e21&
            - 2.5326061081643214e21*jx)*coen(8)*coep(8) - 2.9428839360482198e20*coep(8)**2& 
            - 2.9428839360482198e20*jx*coep(8)**2) + Q**6*((3.6572808606464125e15&
            + 3.6572808606464125e15*jx)*coen(5)**2 + (-1.8469008674161924e16&
            - 1.8469008674161924e16*jx)*coen(8)**2 + (3.378409276102618e15&
            + 3.378409276102618e15*jx)*coen(5)*coep(5) + 7.544468336339949e14*coep(5)**2&
            + 7.544468336339949e14*jx*coep(5)**2 + (-1.4750277773684082e16&
            - 1.4750277773684082e16*jx)*coen(8)*coep(8) - 2.814531448157589e15*coep(8)**2&
            - 2.814531448157589e15*jx*coep(8)**2) + Q**10*((1.3234888050788993e26&
            + 1.3234888050788993e26*jx)*coen(5)**2 + (-1.4378724345444596e25&
            - 1.4378724345444596e25*jx)*coen(8)**2 + (3.5213721903520443e25&
            + 3.5213721903520443e25*jx)*coen(5)*coep(5) + 2.4092794578722237e18*coep(5)**2&
            + 2.4092794578722237e18*jx*coep(5)**2 + (-1.9675497068460733e18&
            - 1.9675497068460733e18*jx)*coen(8)*coep(8) - 6.730868044870307e10*coep(8)**2&
            - 6.730868044870307e10*jx*coep(8)**2) + Q**4*((3.998507999480764e8&
            + 3.998507999480764e8*jx)*coen(5)**2 + (-5.804966213953338e9&
            - 5.804966213953338e9*jx)*coen(8)**2 + (4.622154133876568e8&
            + 4.622154133876568e8*jx)*coen(5)*coep(5) + 1.3140839706660198e8*coep(5)**2&
            + 1.3140839706660198e8*jx*coep(5)**2 + (-6.052208584637747e9&
            - 6.052208584637747e9*jx)*coen(8)*coep(8) - 1.5399739977610245e9*coep(8)**2&
            - 1.5399739977610245e9*jx*coep(8)**2) + Q**2*((4.413677279630095&
            + 4.413677279630095*jx)*coen(5)**2 + (-215.28744405418348&
            - 215.28744405418348*jx)*coen(8)**2 + (6.022733587374211&
            + 6.022733587374211*jx)*coen(5)*coep(5) + 2.0446506724979843*coep(5)**2&
            + 2.0446506724979843*jx*coep(5)**2 + (-272.0361312800155&
            - 272.0361312800155*jx)*coen(8)*coep(8) - 85.08617270805354*coep(8)**2&
            - 85.08617270805354*jx*coep(8)**2) + Q*((-0.00013338196575027984&
            - 0.00013338196575027984*jx)*coen(5)**2 + (0.01630667750590404&
            + 0.01630667750590404*jx)*coen(8)**2 + (-0.00019467177917750327&
            - 0.00019467177917750327*jx)*coen(5)*coep(5) - 0.00007103115738878494*coep(5)**2&
            - 0.00007103115738878494*jx*coep(5)**2 + (0.022251462463408457&
            + 0.022251462463408457*jx)*coen(8)*coep(8) + 0.007554122564087595*coep(8)**2&
            + 0.007554122564087595*jx*coep(8)**2) + Q**3*((-58271.177563151614&
            - 58271.177563151614*jx)*coen(5)**2 + (1.4772801979254559e6&
            + 1.4772801979254559e6*jx)*coen(8)**2 + (-73631.16682931596 - 73631.16682931596*jx)&
            *coen(5)*coep(5) - 23030.00762456043*coep(5)**2 - 23030.00762456043*jx*coep(5)**2&
            + (1.707691162459056e6 + 1.707691162459056e6*jx)*coen(8)*coep(8)&
            + 485498.6481278154*coep(8)**2 + 485498.6481278154*jx*coep(8)**2)&
            + Q**5*((-1.5712120067542686e12 - 1.5712120067542686e12*jx)*coen(5)**2&
            + (1.3512111503556604e13 + 1.3512111503556604e13*jx)*coen(8)**2&
            + (-1.6381323241307883e12 - 1.6381323241307883e12*jx)*coen(5)*coep(5)&
            - 4.1681993420658734e11*coep(5)**2 - 4.1681993420658734e11*jx*coep(5)**2&
            + (1.2481798522654379e13 + 1.2481798522654379e13*jx)*coen(8)*coep(8)&
            + 2.787363106088049e12*coep(8)**2 + 2.787363106088049e12*jx*coep(8)**2)&
            + Q**9*((-1.3404525517645482e24 - 1.3404525517645482e24*jx)*coen(5)**2&
            + (4.889733380983542e23 + 4.889733380983542e23*jx)*coen(8)**2&
            + (-6.854925556607155e23 - 6.854925556607155e23*jx)*coen(5)*coep(5)&
            - 7.965411691266723e22*coep(5)**2 - 7.965411691266723e22*jx*coep(5)**2&
            + (1.3009986242388382e23 + 1.3009986242388382e23*jx)*coen(8)*coep(8)&
            + 8.901272261667101e15*coep(8)**2 + 8.901272261667101e15*jx*coep(8)**2)&
            + Q**7*((-4.998948678105986e18 - 4.998948678105986e18*jx)*coen(5)**2&
            + (1.3957206126975584e19 + 1.3957206126975584e19*jx)*coen(8)**2&
            + (-3.9924114433716403e18 - 3.9924114433716403e18*jx)*coen(5)*coep(5)&
            - 7.618004036101061e17*coep(5)**2 - 7.618004036101061e17*jx*coep(5)**2&
            + (9.274537262949759e18 + 9.274537262949759e18*jx)*coen(8)*coep(8)&
            + 1.4498254142006367e18*coep(8)**2 + 1.4498254142006367e18*jx*coep(8)**2))&
            + jx*Q*(-0.000029591756937809038*coen(8)**2 - 0.000029591756937809038*jx*coen(8)**2&
            + Q**11*((-2.3370294274576046e29 - 2.3370294274576046e29*jx)*coen(5)**2&
            + (-3.1979342912583516e22 - 3.1979342912583516e22*jx)*coen(5)*coep(5)&
            + (-1.0939938978786755e15 - 1.0939938978786755e15*jx)*coep(5)**2)&
            - 0.00004318934677313694*coen(8)*coep(8) - 0.00004318934677313694*jx*coen(8)*coep(8)&
            - 0.000015758777677601983*coep(8)**2 - 0.000015758777677601983*jx*coep(8)**2&
            + Q**8*((2.268518448520612e23 + 2.268518448520612e23*jx)*coen(5)**2&
            + (-2.9738912509916413e23 - 2.9738912509916413e23*jx)*coen(8)**2&
            + (1.507426249285642e23 + 1.507426249285642e23*jx)*coen(5)*coep(5)&
            + 2.3564570654950067e22*coep(5)**2 + 2.3564570654950067e22*jx*coep(5)**2&
            + (-1.5208149749245144e23 - 1.5208149749245144e23*jx)*coen(8)*coep(8)&
            - 1.767184381724054e22*coep(8)**2 - 1.767184381724054e22*jx*coep(8)**2)&
            + Q**6*((2.1961755057154755e17 + 2.1961755057154755e17*jx)*coen(5)**2&
            + (-1.1090530372302881e18 - 1.1090530372302881e18*jx)*coen(8)**2&
            + (2.0287147701167075e17 + 2.0287147701167075e17*jx)*coen(5)*coep(5)&
            + 4.530408572719588e16*coep(5)**2 + 4.530408572719588e16*jx*coep(5)**2&
            + (-8.857454481452869e17 - 8.857454481452869e17*jx)*coen(8)*coep(8)&
            - 1.690109472592459e17*coep(8)**2 - 1.690109472592459e17*jx*coep(8)**2)&
            + Q**10*((7.94747192396153e27 + 7.94747192396153e27*jx)*coen(5)**2&
            + (-8.634338847391353e26 - 8.634338847391353e26*jx)*coen(8)**2&
            + (2.1145631537830358e27 + 2.1145631537830358e27*jx)*coen(5)*coep(5)&
            + 1.4467580515178797e20*coep(5)**2 + 1.4467580515178797e20*jx*coep(5)**2&
            + (-1.1815019510668024e20 - 1.1815019510668024e20*jx)*coen(8)*coep(8)&
            - 4.041846414205793e12*coep(8)**2 - 4.041846414205793e12*jx*coep(8)**2)&
            + Q**4*((2.401080382520842e10 + 2.401080382520842e10*jx)*coen(5)**2&
            + (-3.485847846078992e11 - 3.485847846078992e11*jx)*coen(8)**2&
            + (2.7755761942404064e10 + 2.7755761942404064e10*jx)*coen(5)*coep(5)&
            + 7.890996450078385e9*coep(5)**2 + 7.890996450078385e9*jx*coep(5)**2&
            + (-3.6343154260001447e11 - 3.6343154260001447e11*jx)*coen(8)*coep(8)&
            - 9.247452690094281e10*coep(8)**2 - 9.247452690094281e10*jx*coep(8)**2)&
            + Q**2*((265.0387077448376 + 265.0387077448376*jx)*coen(5)**2&
            + (-12927.883565286835 - 12927.883565286835*jx)*coen(8)**2&
            + (361.6615864635376 + 361.6615864635376*jx)*coen(5)*coep(5)&
            + 122.78006245030585*coep(5)**2 + 122.78006245030585*jx*coep(5)**2&
            + (-16335.60863797521 - 16335.60863797521*jx)*coen(8)*coep(8)&
            - 5109.37430010437*coep(8)**2 - 5109.37430010437*jx*coep(8)**2)&
            + Q*((-0.00800950808118058 - 0.00800950808118058*jx)*coen(5)**2&
            + (0.9792063306764538 + 0.9792063306764538*jx)*coen(8)**2&
            + (-0.011689925093915799 - 0.011689925093915799*jx)*coen(5)*coep(5)&
            - 0.0042653789507513615*coep(5)**2 - 0.0042653789507513615*jx*coep(5)**2&
            + (1.336187148061899 + 1.336187148061899*jx)*coen(8)*coep(8)&
            + 0.453620587932902*coep(8)**2 + 0.453620587932902*jx*coep(8)**2)&
            + Q**3*((-3.4991497161301374e6 - 3.4991497161301374e6*jx)*coen(5)**2&
            + (8.870980133554643e7 + 8.870980133554643e7*jx)*coen(8)**2&
            + (-4.42150797844966e6 - 4.42150797844966e6*jx)*coen(5)*coep(5)&
            - 1.3829383240903404e6*coep(5)**2 - 1.3829383240903404e6*jx*coep(5)**2&
            + (1.025458433524981e8 + 1.025458433524981e8*jx)*coen(8)*coep(8)&
            + 2.9153906404875617e7*coep(8)**2 + 2.9153906404875617e7*jx*coep(8)**2)&
            + Q**5*((-9.435035084808581e13 - 9.435035084808581e13*jx)*coen(5)**2&
            + (8.113942966185638e14 + 8.113942966185638e14*jx)*coen(8)**2&
            + (-9.836887628971795e13 - 9.836887628971795e13*jx)*coen(5)*coep(5)&
            - 2.5029790291704516e13*coep(5)**2 - 2.5029790291704516e13*jx*coep(5)**2&
            + (7.495246120606699e14 + 7.495246120606699e14*jx)*coen(8)*coep(8)&
            + 1.673795044016285e14*coep(8)**2 + 1.673795044016285e14*jx*coep(8)**2)&
            + Q**9*((-8.049338218555047e25 - 8.049338218555047e25*jx)*coen(5)**2&
            + (2.9362559480590006e25 + 2.9362559480590006e25*jx)*coen(8)**2&
            + (-4.116342215583301e25 - 4.116342215583301e25*jx)*coen(5)*coep(5)&
            - 4.7831825653684546e24*coep(5)**2 - 4.7831825653684546e24*jx*coep(5)**2&
            + (7.812419719435667e24 + 7.812419719435667e24*jx)*coen(8)*coep(8)&
            + 5.345161297599304e17*coep(8)**2 + 5.345161297599304e17*jx*coep(8)**2)&
            + Q**7*((-3.0018390874264705e20 - 3.0018390874264705e20*jx)*coen(5)**2&
            + (8.381219652588565e20 + 8.381219652588565e20*jx)*coen(8)**2 + (-2.397419436668925e20&
            - 2.397419436668925e20*jx)*coen(5)*coep(5) - 4.574566325094794e19*coep(5)**2&
            - 4.574566325094794e19*jx*coep(5)**2 + (5.5693047211407324e20&
            + 5.5693047211407324e20*jx)*coen(8)*coep(8) + 8.706115782610298e19*coep(8)**2&
            + 8.706115782610298e19*jx*coep(8)**2)) + mx**2*((1.4902508966519582e-6&
            - 0.04931316228923792*Q + 651.0525926348478*Q**2 - 4.467455624887381e6*Q**3&
            + 1.755484775415344e10*Q**4 - 4.086209144713348e13*Q**5 + 5.5852286386392056e16*Q**6&
            - 4.2208105887584666e19*Q**7 + 1.4976617010777052e22*Q**8 - 1.4787084351195507e24*Q**9&
            + 4.348282272108474e25*Q**10)*coen(1)**2 + 4.96750298883986e-7*jx*v**2*coen(8)**2&
            + 4.96750298883986e-7*jx**2*v**2*coen(8)**2 + 7.93617378432144e-7*coep(1)**2&
            + coen(1)*(2.175030123751925e-6*coep(1) + Q**9*(-7.280773807460498e22*coen(3)&
            - 3.9343610169475954e23*coep(1) - 7.013635394838858e21*coep(3))&
            + Q**7*(-4.848769473366361e17*coen(3) - 2.8047207105177256e19*coep(1)&
            - 3.64144504028701e17*coep(3)) + Q**5*(-2.3408903524606906e11*coen(3)&
            - 3.774631318896015e13*coep(1) - 2.820219595766543e11*coep(3))&
            + Q**3*(-12236.29998107771*coen(3) - 5.164243384573643e6*coep(1)&
            - 19593.623943536677*coep(3)) + Q*(-0.000036347100177956306*coen(3)&
            - 0.06729083709625452*coep(1) - 0.00007406286378913664*coep(3))&
            + Q**2*(1.065717089900142*coen(3) + 822.6667808626729*coep(1)&
            + 1.935056410769274*coep(3)) + Q**4*(7.147322196189931e7*coen(3)&
            + 1.8302535512491814e10*coep(1) + 1.0021121904771389e8*coep(3))&
            + Q**6*(4.4481045201788556e14*coen(3) + 4.460644060702519e16*coep(1)&
            + 4.420413766759587e14*coep(3)) + Q**10*(3.684849338065312e24*coen(3)&
            + 5.950083821226819e18*coep(1) + 2.5211291533670176e17*coep(3))&
            + Q**8*(2.83124266369752e20*coen(3) + 7.658875695640888e21*coep(1)&
            + 1.2903827935608216e20*coep(3))) + jx*Q**12*((-1.6202675310961138e25&
            - 1.6202675310961138e25*jx)*coen(5)**2 + (-2.217134725745239e18&
            - 2.217134725745239e18*jx)*coen(5)*coep(5) + (-7.584683235582625e10&
            - 7.584683235582625e10*jx)*coep(5)**2) + 7.250100412506416e-7*jx*v**2*coen(8)*coep(8)&
            + 7.250100412506416e-7*jx**2*v**2*coen(8)*coep(8)&
            + 2.64539126144048e-7*jx*v**2*coep(8)**2 + 2.64539126144048e-7*jx**2*v**2*coep(8)**2&
            + Q*(coep(1)*(-0.000026524405378884048*coen(3) - 0.022844486365832087*coep(1)&
            - 0.000054047597003502934*coep(3)) + jx*(-2.0516028763309153e-9*coen(8)**2&
            + 0.00013445384610322287*coen(11)**2 - 0.00001211570005931877*coen(11)*coen(12)&
            + 2.729378745600981e-7*coen(12)**2 - 2.9943267056714867e-9*coen(8)*coep(8)&
            - 1.0925594475101624e-9*coep(8)**2 + v**2*(0.00013445384610322287*coen(5)**2&
            - 0.016437720763079304*coen(8)**2 + 0.00019623619498288658*coen(5)*coep(5)&
            + 0.000071601975952026*coep(5)**2 - 0.02243027903208484*coen(8)*coep(8)&
            - 0.007614828788610695*coep(8)**2) + 0.00019623619498288658*coen(11)*coep(11)&
            - 8.841468459628016e-6*coen(12)*coep(11) + 0.000071601975952026*coep(11)**2&
            - 0.000024687621263045548*coen(11)*coep(12) + 1.112306650460148e-6*coen(12)*coep(12)&
            - 0.00001801586566783431*coep(11)*coep(12) + 1.1332488085905506e-6*coep(12)**2)&
            + jx**2*(-2.0516028763309153e-9*coen(8)**2 + 0.00013445384610322287*coen(11)**2&
            - 0.00001211570005931877*coen(11)*coen(12) + 2.729378745600981e-7*coen(12)**2&
            - 2.9943267056714867e-9*coen(8)*coep(8) - 1.0925594475101624e-9*coep(8)**2&
            + v**2*(0.00013445384610322287*coen(5)**2 - 0.016437720763079304*coen(8)**2&
            + 0.00019623619498288658*coen(5)*coep(5) + 0.000071601975952026*coep(5)**2&
            - 0.02243027903208484*coen(8)*coep(8) - 0.007614828788610695*coep(8)**2)&
            + 0.00019623619498288658*coen(11)*coep(11) - 8.841468459628016e-6*coen(12)*coep(11)&
            + 0.000071601975952026*coep(11)**2 - 0.000024687621263045548*coen(11)*coep(12)&
            + 1.112306650460148e-6*coen(12)*coep(12) - 0.00001801586566783431*coep(11)*coep(12)&
            + 1.1332488085905506e-6*coep(12)**2)) + jx*Q**11*((5.5099993870001984e23&
            + 3.9231197695698223e27*v**2 + jx*(5.5099993870001984e23&
            + 3.9231197695698223e27*v**2))*coen(5)**2 + (-5.986205703078952e22&
            - 5.986205703078952e22*jx)*coen(8)**2 + 3.9231197695698223e27*coen(11)**2&
            + 3.9231197695698223e27*jx*coen(11)**2 - 3.3245553948457355e26*coen(11)*coen(15)&
            - 3.3245553948457355e26*jx*coen(11)*coen(15) + 7.043290303758575e24*coen(15)**2&
            + 7.043290303758575e24*jx*coen(15)**2 + (1.4660311848336806e23&
            + 5.368301781920326e20*v**2 + jx*(1.4660311848336806e23&
            + 5.368301781920326e20*v**2))*coen(5)*coep(5) + 1.0030404703874116e16*coep(5)**2&
            + 1.0030404703874116e16*jx*coep(5)**2 + 1.836463434363168e13*v**2*coep(5)**2&
            + 1.836463434363168e13*jx*v**2*coep(5)**2 + (-8.191378451416512e15&
            - 8.191378451416512e15*jx)*coen(8)*coep(8) - 2.8022208165941894e8*coep(8)**2&
            - 2.8022208165941894e8*jx*coep(8)**2 + 5.368301781920326e20*coen(11)*coep(11)&
            + 5.368301781920326e20*jx*coen(11)*coep(11) - 2.2746204167251534e19*coen(15)*coep(11)&
            - 2.2746204167251534e19*jx*coen(15)*coep(11) + 1.836463434363168e13*coep(11)**2&
            + 1.836463434363168e13*jx*coep(11)**2 - 2.27462041428534e19*coen(11)*coep(15)&
            - 2.27462041428534e19*jx*coen(11)*coep(15) + 9.637867327165193e17*coen(15)*coep(15)&
            + 9.637867327165193e17*jx*coen(15)*coep(15) - 1.55626765691878e12*coep(11)*coep(15)&
            - 1.55626765691878e12*jx*coep(11)*coep(15) + 3.2970558719718483e10*coep(15)**2&
            + 3.2970558719718483e10*jx*coep(15)**2) + Q**2*(0.000221625716568002*coen(3)**2&
            + 257.3098193548749*coep(1)**2 + coen(3)*(0.7206132073473492*coep(1)&
            + 0.0009031936621068109*coep(3)) + 1.2957691388147365*coep(1)*coep(3)&
            + 0.0009201987069713664*coep(3)**2&
            + jx*((-5.553009188295405e-7 - 4.449146347233461*v**2)*coen(5)**2&
            + (0.00006788858562062775 + 217.01753087828257*v**2)*coen(8)**2&
            - 4.449146347233461*coen(11)**2 + 0.355239029966714*coen(11)*coen(12)&
            - 0.006973725540580334*coen(12)**2&
            + 0.003279318552738105*coen(11)*coen(15) - 0.00014775047771200132*coen(12)*coen(15)&
            + (-8.10465022309226e-7 - 6.07113330743384*v**2)*coen(5)*coep(5)&
            - 2.9571964051995707e-7*coep(5)**2 - 2.0610818359776526*v**2*coep(5)**2&
            + (0.00009263814250845091 + 274.2222602875576*v**2)*coen(8)*coep(8)&
            + 0.000031449612975733224*coep(8)**2 + 85.76993978495828*v**2*coep(8)**2&
            - 6.07113330743384*coen(11)*coep(11) + 0.2402044024491164*coen(12)*coep(11)&
            + 0.0023930925502572403*coen(15)*coep(11) - 2.0610818359776526*coep(11)**2&
            + 0.645018803589758*coen(11)*coep(12) - 0.024868127757586202*coen(12)*coep(12)&
            - 0.0003010645540356036*coen(15)*coep(12) + 0.4319230462715788*coep(11)*coep(12)&
            - 0.021717490816843477*coep(12)**2 + 0.006682121052394976*coen(11)*coep(15)&
            - 0.00030106455403560356*coen(12)*coep(15) + 0.004876297881171583*coep(11)*coep(15)&
            - 0.0006134658046475775*coep(12)*coep(15)) + jx**2*((-5.553009188295405e-7&
            - 4.449146347233461*v**2)*coen(5)**2 + (0.00006788858562062775&
            + 217.01753087828257*v**2)*coen(8)**2 - 4.449146347233461*coen(11)**2&
            + 0.355239029966714*coen(11)*coen(12) - 0.006973725540580334*coen(12)**2&
            + 0.003279318552738105*coen(11)*coen(15) - 0.00014775047771200132*coen(12)*coen(15)&
            + (-8.10465022309226e-7 - 6.07113330743384*v**2)*coen(5)*coep(5)&
            - 2.9571964051995707e-7*coep(5)**2 - 2.0610818359776526*v**2*coep(5)**2&
            + (0.00009263814250845091 + 274.2222602875576*v**2)*coen(8)*coep(8)&
            + 0.000031449612975733224*coep(8)**2 + 85.76993978495828*v**2*coep(8)**2&
            - 6.07113330743384*coen(11)*coep(11) + 0.2402044024491164*coen(12)*coep(11)&
            + 0.0023930925502572403*coen(15)*coep(11) - 2.0610818359776526*coep(11)**2&
            + 0.645018803589758*coen(11)*coep(12) - 0.024868127757586202*coen(12)*coep(12)&
            - 0.0003010645540356036*coen(15)*coep(12) + 0.4319230462715788*coep(11)*coep(12)&
            - 0.021717490816843477*coep(12)**2 + 0.006682121052394976*coen(11)*coep(15)&
            - 0.00030106455403560356*coen(12)*coep(15) + 0.004876297881171583*coep(11)*coep(15)&
            - 0.0006134658046475775*coep(12)*coep(15))) + Q**8*(9.563320213700219e17*coen(3)**2&
            + 8.899600368265915e20*coep(1)**2 + 3.172994647274588e19*coep(1)*coep(3)&
            + 2.8281872688243475e17*coep(3)**2 + coen(3)*(8.69959126088991e19*coep(1)&
            + 1.2333315125564534e18*coep(3)) + jx*((-2.0811814989524904e16&
            - 3.808111899860328e21*v**2)*coen(5)**2 + (5.810717620636486e16&
            + 4.992205670259017e21*v**2)*coen(8)**2 - 3.808111899860328e21*coen(11)**2&
            + 9.4374755456584e19*coen(11)*coen(12) - 5.297928398124295e17*coen(12)**2&
            + 4.374670775415495e19*coen(11)*coen(15) - 6.375546809133478e17*coen(12)*coen(15)&
            - 9.999486910551582e16*coen(15)**2 + (-1.6621360544352246e16&
            - 2.530483206698205e21*v**2)*coen(5)*coep(5) - 3.1715566771703485e15*coep(5)**2&
            - 3.95573251717373e20*v**2*coep(5)**2 + (3.86121094772065e16&
            + 2.552958565213629e21*v**2)*coen(8)*coep(8) + 6.035968806722609e15*coep(8)**2&
            + 2.9665334560886384e20*v**2*coep(8)**2 - 2.530483206698205e21*coen(11)*coep(11)&
            + 2.8998637536299704e19*coen(12)*coep(11) + 1.634057000801551e19*coen(15)*coep(11)&
            - 3.95573251717373e20*coep(11)**2 + 4.301275978536072e19*coen(11)*coep(12)&
            - 3.659810732229837e17*coen(12)*coep(12) - 4.111105041854845e17*coen(15)*coep(12)&
            + 1.0576648824248627e19*coep(11)*coep(12) - 2.5039977954217186e10*coep(12)**2&
            + 3.2853950441503367e19*coen(11)*coep(15) - 4.111105041854844e17*coen(12)*coep(15)&
            - 1.8015207009574992e17*coen(15)*coep(15) + 1.1044832706015099e19*coep(11)*coep(15)&
            - 1.8854581792162314e17*coep(12)*coep(15) - 7.118347432573947e16*coep(15)**2)&
            + jx**2*((-2.0811814989524904e16 - 3.808111899860328e21*v**2)*coen(5)**2&
            + (5.810717620636486e16 + 4.992205670259017e21*v**2)*coen(8)**2&
            - 3.808111899860328e21*coen(11)**2 + 9.4374755456584e19*coen(11)*coen(12)&
            - 5.297928398124295e17*coen(12)**2 + 4.374670775415495e19*coen(11)*coen(15)&
            - 6.375546809133478e17*coen(12)*coen(15) - 9.999486910551582e16*coen(15)**2&
            + (-1.6621360544352246e16 - 2.530483206698205e21*v**2)*coen(5)*coep(5)&
            - 3.1715566771703485e15*coep(5)**2 - 3.95573251717373e20*v**2*coep(5)**2&
            + (3.86121094772065e16 + 2.552958565213629e21*v**2)*coen(8)*coep(8)&
            + 6.035968806722609e15*coep(8)**2 + 2.9665334560886384e20*v**2*coep(8)**2&
            - 2.530483206698205e21*coen(11)*coep(11) + 2.8998637536299704e19*coen(12)*coep(11)&
            + 1.634057000801551e19*coen(15)*coep(11) - 3.95573251717373e20*coep(11)**2&
            + 4.301275978536072e19*coen(11)*coep(12) - 3.659810732229837e17*coen(12)*coep(12)&
            - 4.111105041854845e17*coen(15)*coep(12) + 1.0576648824248627e19*coep(11)*coep(12)&
            - 2.5039977954217186e10*coep(12)**2 + 3.2853950441503367e19*coen(11)*coep(15)&
            - 4.111105041854844e17*coen(12)*coep(15) - 1.8015207009574992e17*coen(15)*coep(15)&
            + 1.1044832706015099e19*coep(11)*coep(15) - 1.8854581792162314e17*coep(12)*coep(15)&
            - 7.118347432573947e16*coep(15)**2))&
            + Q**10*(7.80659683211893e22*coen(3)**2 + 2.035487536464488e11*coep(1)**2&
            + 1.7249259417786644e10*coep(1)*coep(3) + 3.654369593671203e8*coep(3)**2&
            + coen(3)*(2.5211291560712426e17*coep(1) + 1.0682357435200404e16*coep(3))&
            + jx*((-5.580623508247359e21 - 1.3341245881064199e26*v**2)*coen(5)**2&
            + (2.035712567301055e21 + 1.4494274240361578e25*v**2)*coen(8)**2&
            - 1.3341245881064199e26*coen(11)**2 + 1.2282831126884372e24*coen(11)*coen(12)&
            + 6.568880738270039e24*coen(11)*coen(15) - 5.204397888079285e22*coen(12)*coen(15)&
            - 3.88129157985931e22*coen(15)**2 + (-2.853868915996289e21&
            - 3.5496705412198447e25*v**2)*coen(5)*coep(5) - 3.316190765472127e20*coep(5)**2&
            - 2.4286408408081853e18*v**2*coep(5)**2 + (5.4163674029843815e20&
            + 1.9833612737422725e18*v**2)*coen(8)*coep(8) + 3.705811829846474e13*coep(8)**2&
            + 6.784958454881626e10*v**2*coep(8)**2 - 3.5496705412198447e25*coen(11)*coep(11)&
            + 8.403763853570808e16*coen(12)*coep(11) + 1.504041999481295e24*coen(15)*coep(11)&
            - 2.4286408408081853e18*coep(11)**2 + 8.403763844556725e16*coen(11)*coep(12)&
            - 3.560785811733468e15*coen(15)*coep(12) + 5.749753139262215e9*coep(11)*coep(12)&
            + 6.327862349355908e23*coen(11)*coep(15) - 3.5607858117334675e15*coen(12)*coep(15)&
            - 2.681197538251279e22*coen(15)*coep(15) + 1.461991612656434e17*coep(11)*coep(15)&
            - 2.4362463957808018e8*coep(12)*coep(15) - 1.8344426026590818e15*coep(15)**2)&
            + jx**2*((-5.580623508247359e21 - 1.3341245881064199e26*v**2)*coen(5)**2&
            + (2.035712567301055e21 + 1.4494274240361578e25*v**2)*coen(8)**2&
            - 1.3341245881064199e26*coen(11)**2 + 1.2282831126884372e24*coen(11)*coen(12)&
            + 6.568880738270039e24*coen(11)*coen(15) - 5.204397888079285e22*coen(12)*coen(15)&
            - 3.88129157985931e22*coen(15)**2 + (-2.853868915996289e21&
            - 3.5496705412198447e25*v**2)*coen(5)*coep(5) - 3.316190765472127e20*coep(5)**2&
            - 2.4286408408081853e18*v**2*coep(5)**2 + (5.4163674029843815e20&
            + 1.9833612737422725e18*v**2)*coen(8)*coep(8) + 3.705811829846474e13*coep(8)**2&
            + 6.784958454881626e10*v**2*coep(8)**2 - 3.5496705412198447e25*coen(11)*coep(11)&
            + 8.403763853570808e16*coen(12)*coep(11) + 1.504041999481295e24*coen(15)*coep(11)&
            - 2.4286408408081853e18*coep(11)**2 + 8.403763844556725e16*coen(11)*coep(12)&
            - 3.560785811733468e15*coen(15)*coep(12) + 5.749753139262215e9*coep(11)*coep(12)&
            + 6.327862349355908e23*coen(11)*coep(15) - 3.5607858117334675e15*coen(12)*coep(15)&
            - 2.681197538251279e22*coen(15)*coep(15) + 1.461991612656434e17*coep(11)*coep(15)&
            - 2.4362463957808018e8*coep(12)*coep(15) - 1.8344426026590818e15*coep(15)**2))&
            + Q**6*(7.246984995608268e11*coen(3)**2 + 8.511448516775234e15*coep(1)**2&
            + 1.818293478568342e14*coep(1)*coep(3) + 8.594102980015076e11*coep(3)**2&
            + coen(3)*(1.9445396560603322e14*coep(1) + 1.6322985465165608e12*coep(3))&
            + jx*((-6.541330127493798e9 - 3.686671396898224e15*v**2)*coen(5)**2&
            + (5.62541411880222e10 + 1.8617428795464016e16*v**2)*coen(8)**2&
            - 3.686671396898224e15*coen(11)**2 + 1.4827015067262853e14*coen(11)*coen(12)&
            - 1.3649210470294968e12*coen(12)**2 + 2.1120048436231574e13*coen(11)*coen(15)&
            - 4.831323330405511e11*coen(12)*coen(15) - 2.4402616655437157e10*coen(15)**2&
            + (-6.819935361106259e9 - 3.40555875247212e15*v**2)*coen(5)*coep(5)&
            - 1.7353207470695937e9*coep(5)**2 - 7.605096977833088e14*v**2*coep(5)**2&
            + (5.1964702643922714e10 + 1.486881353567506e16*v**2)*coen(8)*coep(8)&
            + 1.160445705846113e10*coep(8)**2 + 2.8371495055917445e15*v**2*coep(8)**2&
            - 3.40555875247212e15*coen(11)*coep(11) + 6.48179885353444e13*coen(12)*coep(11)&
            + 1.0552116049588152e13*coen(15)*coep(11) - 7.605096977833088e14*coep(11)**2&
            + 1.473471255586529e14*coen(11)*coep(12) - 2.459059693154382e12*coen(12)*coep(12)&
            - 5.4409951550552026e11*coen(15)*coep(12) + 6.060978261894474e13*coep(11)*coep(12)&
            - 9.716480773164609e11*coep(12)**2 + 2.54446665563756e13*coen(11)*coep(15)&
            - 5.440995155055202e11*coen(12)*coep(15) - 6.523383993205279e10*coen(15)*coep(15)&
            + 1.2228130500726578e13*coep(11)*coep(15) - 5.729401986676716e11*coep(12)*coep(15)&
            - 4.181730996107613e10*coep(15)**2) + jx**2*((-6.541330127493798e9&
            - 3.686671396898224e15*v**2)*coen(5)**2 + (5.62541411880222e10&
            + 1.8617428795464016e16*v**2)*coen(8)**2 - 3.686671396898224e15*coen(11)**2&
            + 1.4827015067262853e14*coen(11)*coen(12) - 1.3649210470294968e12*coen(12)**2&
            + 2.1120048436231574e13*coen(11)*coen(15) - 4.831323330405511e11*coen(12)*coen(15)&
            - 2.4402616655437157e10*coen(15)**2 + (-6.819935361106259e9&
            - 3.40555875247212e15*v**2)*coen(5)*coep(5) - 1.7353207470695937e9*coep(5)**2&
            - 7.605096977833088e14*v**2*coep(5)**2 + (5.1964702643922714e10&
            + 1.486881353567506e16*v**2)*coen(8)*coep(8) + 1.160445705846113e10*coep(8)**2&
            + 2.8371495055917445e15*v**2*coep(8)**2 - 3.40555875247212e15*coen(11)*coep(11)&
            + 6.48179885353444e13*coen(12)*coep(11)&
            + 1.0552116049588152e13*coen(15)*coep(11) - 7.605096977833088e14*coep(11)**2&
            + 1.473471255586529e14*coen(11)*coep(12) - 2.459059693154382e12*coen(12)*coep(12)&
            - 5.4409951550552026e11*coen(15)*coep(12) + 6.060978261894474e13*coep(11)*coep(12)&
            - 9.716480773164609e11*coep(12)**2 + 2.54446665563756e13*coen(11)*coep(15)&
            - 5.440995155055202e11*coen(12)*coep(15) - 6.523383993205279e10*coen(15)*coep(15)&
            + 1.2228130500726578e13*coep(11)*coep(15) - 5.729401986676716e11*coep(12)*coep(15)&
            - 4.181730996107613e10*coep(15)**2)) + Q**4*(55548.44000877604*coen(3)**2&
            + 4.657048478778127e9*coep(1)**2 + 5.474367002464891e7*coep(1)*coep(3)&
            + 129491.19568977116*coep(3)**2 + coen(3)*(4.008457747672484e7*coep(1)&
            + 172132.57594783657*coep(3)) + jx*((-242.59680279925334&
            - 4.0306406955436885e8*v**2)*coen(5)**2 + (6150.269616002941&
            + 5.851615918051146e9*v**2)*coen(8)**2 - 4.0306406955436885e8*coen(11)**2&
            + 2.3824407320633106e7*coen(11)*coen(12) - 333093.5414341318*coen(12)**2&
            + 1.1039869851612817e6*coen(11)*coen(15) - 37032.29333918401*coen(12)*coen(15)&
            - 510.8989813921838*coen(15)**2 + (-306.54409960759295&
            - 4.659298557236264e8*v**2)*coen(5)*coep(5) - 95.87941160286056*coep(5)**2&
            - 1.3246441748311868e8*v**2*coep(5)**2 + (7109.525386407871&
            + 6.100845170830603e9*v**2)*coen(8)*coep(8) + 2021.246604661845*coep(8)**2&
            + 1.552349492926042e9*v**2*coep(8)**2 - 4.659298557236264e8*coen(11)*coep(11)&
            + 1.336152582557495e7*coen(12)*coep(11) + 684722.436518509*coen(15)*coep(11)&
            - 1.3246441748311868e8*coep(11)**2 + 3.3403739682571296e7*coen(11)*coep(12)&
            - 890436.0983547765*coen(12)*coep(12) - 57377.52531594551*coen(15)*coep(12)&
            + 1.8247890008216307e7*coep(11)*coep(12) - 570802.5522369589*coep(12)**2&
            + 1.7677815891453654e6*coen(11)*coep(15) - 57377.5253159455*coen(12)*coep(15)&
            - 1821.8527624223364*coen(15)*coep(15) + 1.0771799704624503e6*coep(11)*coep(15)&
            - 86327.46379318075*coep(12)*coep(15) - 1591.0353615373438*coep(15)**2)&
            + jx**2*((-242.59680279925334 - 4.0306406955436885e8*v**2)*coen(5)**2&
            + (6150.269616002941 + 5.851615918051146e9*v**2)*coen(8)**2&
            - 4.0306406955436885e8*coen(11)**2 + 2.3824407320633106e7*coen(11)*coen(12)&
            - 333093.5414341318*coen(12)**2 + 1.1039869851612817e6*coen(11)*coen(15)&
            - 37032.29333918401*coen(12)*coen(15) - 510.8989813921838*coen(15)**2&
            + (-306.54409960759295 - 4.659298557236264e8*v**2)*coen(5)*coep(5)&
            - 95.87941160286056*coep(5)**2 - 1.3246441748311868e8*v**2*coep(5)**2&
            + (7109.525386407871 + 6.100845170830603e9*v**2)*coen(8)*coep(8)&
            + 2021.246604661845*coep(8)**2 + 1.552349492926042e9*v**2*coep(8)**2&
            - 4.659298557236264e8*coen(11)*coep(11) + 1.336152582557495e7*coen(12)*coep(11)&
            + 684722.436518509*coen(15)*coep(11) - 1.3246441748311868e8*coep(11)**2&
            + 3.3403739682571296e7*coen(11)*coep(12) - 890436.0983547765*coen(12)*coep(12)&
            - 57377.52531594551*coen(15)*coep(12) + 1.8247890008216307e7*coep(11)*coep(12)&
            - 570802.5522369589*coep(12)**2 + 1.7677815891453654e6*coen(11)*coep(15)&
            - 57377.5253159455*coen(12)*coep(15) - 1821.8527624223364*coen(15)*coep(15)&
            + 1.0771799704624503e6*coep(11)*coep(15) - 86327.46379318075*coep(12)*coep(15)&
            - 1591.0353615373438*coep(15)**2)) + Q**3*(-5.662669289012059*coen(3)**2&
            - 1.4682005955943055e6*coep(1)**2 + coen(3)*(-7589.282527448369*coep(1)&
            - 20.19293453817127*coep(3)) - 11939.178114732396*coep(1)*coep(3)&
            - 17.6346154673456*coep(3)**2 + jx*((0.018375190641471834&
            + 58739.45474006574*v**2)*coen(5)**2 + (-0.8962929495249288&
            - 1.48915187496246e6*v**2)*coen(8)**2 + 58739.45474006574*coen(11)**2&
            - 4078.7666603592374*coen(11)*coen(12) + 68.40935874187*coen(12)**2&
            - 96.15143457851781*coen(11)*coen(15) + 3.7751128593413723*coen(12)*coen(15)&
            + 0.019995579304730948*coen(15)**2 + (0.02507407561525923&
            + 74222.87951434549*v**2)*coen(5)*coep(5) + 0.00851236815064848*coep(5)**2&
            + 23215.080715679145*v**2*coep(5)**2 + (-1.13255126212075&
            - 1.7214144615245478e6*v**2)*coen(8)*coep(8) - 0.35423401970945956*coep(8)**2&
            - 489400.1985314351*v**2*coep(8)**2 + 74222.87951434549*coen(11)*coep(11)&
            - 2529.7608424827904*coen(12)*coep(11) - 65.01537257807027*coen(15)*coep(11)&
            + 23215.080715679145*coep(11)**2 - 6531.2079811788935*coen(11)*coep(12)&
            + 211.9857756098522*coen(12)*coep(12) + 6.730978179390423*coen(15)*coep(12)&
            - 3979.726038244132*coep(11)*coep(12) + 159.47179900021882*coep(12)**2&
            - 174.58521745509125*coen(11)*coep(15) + 6.730978179390422*coen(12)*coep(15)&
            + 0.08148819901342878*coen(15)*coep(15) - 116.90725686990871*coep(11)*coep(15)&
            + 11.75641031156373*coep(12)*coep(15) + 0.08302243307450803*coep(15)**2)&
            + jx**2*((0.018375190641471834 + 58739.45474006574*v**2)*coen(5)**2&
            + (-0.8962929495249288 - 1.48915187496246e6*v**2)*coen(8)**2&
            + 58739.45474006574*coen(11)**2 - 4078.7666603592374*coen(11)*coen(12)&
            + 68.40935874187*coen(12)**2 - 96.15143457851781*coen(11)*coen(15)&
            + 3.7751128593413723*coen(12)*coen(15) + 0.019995579304730948*coen(15)**2&
            + (0.02507407561525923 + 74222.87951434549*v**2)*coen(5)*coep(5)&
            + 0.00851236815064848*coep(5)**2 + 23215.080715679145*v**2*coep(5)**2&
            + (-1.13255126212075 - 1.7214144615245478e6*v**2)*coen(8)*coep(8)&
            - 0.35423401970945956*coep(8)**2 - 489400.1985314351*v**2*coep(8)**2&
            + 74222.87951434549*coen(11)*coep(11) - 2529.7608424827904*coen(12)*coep(11)&
            - 65.01537257807027*coen(15)*coep(11) + 23215.080715679145*coep(11)**2&
            - 6531.2079811788935*coen(11)*coep(12) + 211.9857756098522*coen(12)*coep(12)&
            + 6.730978179390423*coen(15)*coep(12) - 3979.726038244132*coep(11)*coep(12)&
            + 159.47179900021882*coep(12)**2 - 174.58521745509125*coen(11)*coep(15)&
            + 6.730978179390422*coen(12)*coep(15) + 0.08148819901342878*coen(15)*coep(15)&
            - 116.90725686990871*coep(11)*coep(15) + 11.75641031156373*coep(12)*coep(15)&
            + 0.08302243307450803*coep(15)**2)) + Q**5*(-2.704721538683268e8*coen(3)**2&
            - 8.429288502197178e12*coep(1)**2 + coen(3)*(-1.169568655730505e11*coep(1)&
            - 7.23034641762187e8*coep(3)) - 1.3553336681120146e11*coep(1)*coep(3)&
            - 4.634920121007442e8*coep(3)**2 + jx*((1.6646741960723263e6&
            + 1.583838535917148e12*v**2)*coen(5)**2 + (-2.4167458128618587e7&
            - 1.3620697149044492e13*v**2)*coen(8)**2 + 1.583838535917148e12*coen(11)**2&
            - 7.802967841535635e10*coen(11)*coen(12) + 8.924851828119571e8*coen(12)**2&
            - 6.448477640749269e9*coen(11)*coen(15) + 1.8031476924555114e8*coen(12)*coen(15)&
            + 5.011707371552997e6*coen(15)**2 + (1.9243129482128152e6&
            + 1.6512966364415254e12*v**2)*coen(5)*coep(5) + 547084.4819427778*coep(5)**2&
            + 4.2016957068613605e11*v**2*coep(5)**2 + (-2.5196787055076983e7&
            - 1.2582104396320049e13*v**2)*coen(8)*coep(8) - 6.411278849580932e6*coep(8)**2&
            - 2.8097628340657256e12*v**2*coep(8)**2 + 1.6512966364415254e12*coen(11)*coep(11)&
            - 3.898562185768351e10*coen(12)*coep(11) - 3.6165223072682223e9*coen(15)*coep(11)&
            + 4.2016957068613605e11*coep(11)**2 - 9.400731985888478e10*coen(11)*coep(12)&
            + 2.0102184116213276e9*coen(12)*coep(12) + 2.410115472540623e8*coen(15)*coep(12)&
            - 4.5177788937067154e10*coep(11)*coep(12) + 1.0583862908329035e9*coep(12)**2&
            - 9.041285500265951e9*coen(11)*coep(15) + 2.4101154725406227e8*coen(12)*coep(15)&
            + 1.5530194900628801e7*coen(15)*coep(15) - 4.939099181994167e9*coep(11)*coep(15)&
            + 3.089946747338294e8*coep(12)*coep(15) + 1.1682991995582728e7*coep(15)**2)&
            + jx**2*((1.6646741960723263e6 + 1.583838535917148e12*v**2)*coen(5)**2&
            + (-2.4167458128618587e7 - 1.3620697149044492e13*v**2)*coen(8)**2&
            + 1.583838535917148e12*coen(11)**2 - 7.802967841535635e10*coen(11)*coen(12)&
            + 8.924851828119571e8*coen(12)**2 - 6.448477640749269e9*coen(11)*coen(15)&
            + 1.8031476924555114e8*coen(12)*coen(15) + 5.011707371552997e6*coen(15)**2&
            + (1.9243129482128152e6 + 1.6512966364415254e12*v**2)*coen(5)*coep(5)&
            + 547084.4819427778*coep(5)**2 + 4.2016957068613605e11*v**2*coep(5)**2&
            + (-2.5196787055076983e7 - 1.2582104396320049e13*v**2)*coen(8)*coep(8)&
            - 6.411278849580932e6*coep(8)**2 - 2.8097628340657256e12*v**2*coep(8)**2&
            + 1.6512966364415254e12*coen(11)*coep(11) - 3.898562185768351e10*coen(12)*coep(11)&
            - 3.6165223072682223e9*coen(15)*coep(11) + 4.2016957068613605e11*coep(11)**2&
            - 9.400731985888478e10*coen(11)*coep(12) + 2.0102184116213276e9*coen(12)*coep(12)&
            + 2.410115472540623e8*coen(15)*coep(12) - 4.5177788937067154e10*coep(11)*coep(12)&
            + 1.0583862908329035e9*coep(12)**2 - 9.041285500265951e9*coen(11)*coep(15)&
            + 2.4101154725406227e8*coen(12)*coep(15) + 1.5530194900628801e7*coen(15)*coep(15)&
            - 4.939099181994167e9*coep(11)*coep(15) + 3.089946747338294e8*coep(12)*coep(15)&
            + 1.1682991995582728e7*coep(15)**2)) + Q**7*(-1.1083167024518324e15*coen(3)**2&
            - 4.3844293796606776e18*coep(1)**2 + coen(3)*(-1.8111455947161254e17*coep(1)&
            - 1.9967579342266388e15*coep(3)) - 1.2241800677738539e17*coep(1)*coep(3)&
            - 7.889788170082858e14*coep(3)**2 + jx*((1.5226132040495771e13&
            + 5.039121087046669e18*v**2)*coen(5)**2 + (-7.689088572764081e13&
            - 1.4069368629194887e19*v**2)*coen(8)**2 + 5.039121087046669e18*coen(11)**2&
            - 1.61625649112212e17*coen(11)*coen(12) + 1.1777479316413488e15*coen(12)**2&
            - 4.013181686055738e16*coen(11)*coen(15) + 7.388778016345548e14*coen(12)*coen(15)&
            + 6.5383956990125734e13*coen(15)**2 + (1.406512315701188e13&
            + 4.024495146464058e18*v**2)*coen(5)*coep(5) + 3.140942012425813e12*coep(5)**2&
            + 7.679223623089495e17*v**2*coep(5)**2 + (-6.140892252295009e13&
            - 9.349069035059085e18*v**2)*coen(8)*coep(8) - 1.171756534284896e13*coep(8)**2&
            - 1.4614764598868923e18*v**2*coep(8)**2 + 4.024495146464058e18*coen(11)*coep(11)&
            - 6.0371519823870856e16*coen(12)*coep(11) - 1.754408175461817e16*coen(15)*coep(11)&
            + 7.679223623089495e17*coep(11)**2 - 1.2138150134290034e17*coen(11)*coep(12)&
            + 1.5188800599404378e15*coen(12)*coep(12)&
            + 6.655859780755462e14*coen(15)*coep(12) - 4.080600225912846e16*coep(11)*coep(12)&
            + 3.482986694705153e14*coep(12)**2 - 3.9881984546610696e16*coen(11)*coep(15)&
            + 6.655859780755461e14*coen(12)*coep(15) + 1.472697101279504e14*coen(15)*coep(15)&
            - 1.6405059851812198e16*coep(11)*coep(15) + 5.259858780055237e14*coep(12)*coep(15)&
            + 7.753796371243256e13*coep(15)**2) + jx**2*((1.5226132040495771e13&
            + 5.039121087046669e18*v**2)*coen(5)**2 + (-7.689088572764081e13&
            - 1.4069368629194887e19*v**2)*coen(8)**2 + 5.039121087046669e18*coen(11)**2&
            - 1.61625649112212e17*coen(11)*coen(12) + 1.1777479316413488e15*coen(12)**2&
            - 4.013181686055738e16*coen(11)*coen(15) + 7.388778016345548e14*coen(12)*coen(15)&
            + 6.5383956990125734e13*coen(15)**2 + (1.406512315701188e13&
            + 4.024495146464058e18*v**2)*coen(5)*coep(5) + 3.140942012425813e12*coep(5)**2&
            + 7.679223623089495e17*v**2*coep(5)**2 + (-6.140892252295009e13&
            - 9.349069035059085e18*v**2)*coen(8)*coep(8) - 1.171756534284896e13*coep(8)**2&
            - 1.4614764598868923e18*v**2*coep(8)**2 + 4.024495146464058e18*coen(11)*coep(11)&
            - 6.0371519823870856e16*coen(12)*coep(11) - 1.754408175461817e16*coen(15)*coep(11)&
            + 7.679223623089495e17*coep(11)**2 - 1.2138150134290034e17*coen(11)*coep(12)&
            + 1.5188800599404378e15*coen(12)*coep(12) + 6.655859780755462e14*coen(15)*coep(12)&
            - 4.080600225912846e16*coep(11)*coep(12) + 3.482986694705153e14*coep(12)**2&
            - 3.9881984546610696e16*coen(11)*coep(15) + 6.655859780755461e14*coen(12)*coep(15)&
            + 1.472697101279504e14*coen(15)*coep(15) - 1.6405059851812198e16*coep(11)*coep(15)&
            + 5.259858780055237e14*coep(12)*coep(15) + 7.753796371243256e13*coep(15)**2))&
            + Q**9*(-4.3019210120716557e20*coen(3)**2 - 2.691841323662372e16*coep(1)**2&
            + coen(3)*(-1.667040403298271e22*coep(1) - 2.971768492522293e20*coep(3))&
            - 1.6204328658521492e15*coep(1)*coep(3) - 2.03324770001036e13*coep(3)**2&
            + jx*((1.5727687219707275e19 + 1.3512246583698655e24*v**2)*coen(5)**2&
            + (-2.06180520381144e19 - 4.9290281170651684e23*v**2)*coen(8)**2&
            + 1.3512246583698655e24*coen(11)**2 - 2.426924602486833e22*coen(11)*coen(12)&
            + 9.614028461594901e19*coen(12)**2 - 2.5544119197706444e22*coen(11)*coen(15)&
            + 2.867947341381103e20*coen(12)*coen(15) + 8.628246338502214e19*coen(15)**2&
            + (1.045101862451336e19 + 6.910012914059743e23*v**2)*coen(5)*coep(5)&
            + 1.6337367543536632e18*coep(5)**2 + 8.029423105755518e22*v**2*coep(5)**2&
            + (-1.0543842947478854e19 - 1.311453672315865e23*v**2)*coen(8)*coep(8)&
            - 1.2251927346428708e18*coep(8)**2 - 8.972804412207906e15*v**2*coep(8)**2&
            + 6.910012914059743e23*coen(11)*coep(11) - 5.55680134432757e21*coen(12)*coep(11)&
            - 7.848970312183693e21*coen(15)*coep(11) + 8.029423105755518e22*coep(11)**2&
            - 2.3378784649462864e21*coen(11)*coep(12) + 1.3155602963432506e13*coen(12)*coep(12)&
            + 9.905894975074312e19*coen(15)*coep(12) - 5.401442886173831e14*coep(11)*coep(12)&
            + 450045.18663231394*coep(12)**2 - 1.1642128847528757e22*coen(11)*coep(15)&
            + 9.90589497507431e19*coen(12)*coep(15) + 1.1127399135009445e20*coen(15)*coep(15)&
            - 2.8627483798162286e21*coep(11)*coep(15) + 1.3554984666735732e13*coep(12)*coep(15)&
            + 2.5516552725981106e19*coep(15)**2) + jx**2*((1.5727687219707275e19&
            + 1.3512246583698655e24*v**2)*coen(5)**2 + (-2.06180520381144e19&
            - 4.9290281170651684e23*v**2)*coen(8)**2 + 1.3512246583698655e24*coen(11)**2&
            - 2.426924602486833e22*coen(11)*coen(12) + 9.614028461594901e19*coen(12)**2&
            - 2.5544119197706444e22*coen(11)*coen(15) + 2.867947341381103e20*coen(12)*coen(15)&
            + 8.628246338502214e19*coen(15)**2 + (1.045101862451336e19&
            + 6.910012914059743e23*v**2)*coen(5)*coep(5) + 1.6337367543536632e18*coep(5)**2&
            + 8.029423105755518e22*v**2*coep(5)**2 + (-1.0543842947478854e19&
            - 1.311453672315865e23*v**2)*coen(8)*coep(8) - 1.2251927346428708e18*coep(8)**2&
            - 8.972804412207906e15*v**2*coep(8)**2 + 6.910012914059743e23*coen(11)*coep(11)&
            - 5.55680134432757e21*coen(12)*coep(11) - 7.848970312183693e21*coen(15)*coep(11)&
            + 8.029423105755518e22*coep(11)**2 - 2.3378784649462864e21*coen(11)*coep(12)&
            + 1.3155602963432506e13*coen(12)*coep(12) + 9.905894975074312e19*coen(15)*coep(12)&
            - 5.401442886173831e14*coep(11)*coep(12) + 450045.18663231394*coep(12)**2&
            - 1.1642128847528757e22*coen(11)*coep(15) + 9.90589497507431e19*coen(12)*coep(15)&
            + 1.1127399135009445e20*coen(15)*coep(15) - 2.8627483798162286e21*coep(11)*coep(15)&
            + 1.3554984666735732e13*coep(12)*coep(15)&
            + 2.5516552725981106e19*coep(15)**2))))/(EXP(15969.932864187107*Q)*mx**2)
        elseif (A==129) then 
            ans=(mx*Q*(-1.6663720693104848e-11*coen(7)**2 - 5.061758422745956e-7*jx*coen(8)**2&
            - 5.061758422745956e-7*jx**2*coen(8)**2 - 1.1109147128736561e-11*jx*coen(12)**2&
            - 1.1109147128736561e-11*jx**2*coen(12)**2 - 9.733714186417128e-13*coen(7)*coep(7)&
            - 1.4214291275006473e-14*coep(7)**2 - 7.289541243862509e-7*jx*coen(8)*coep(8)&
            - 7.289541243862509e-7*jx**2*coen(8)*coep(8) - 2.624454147554222e-7*jx*coep(8)**2&
            - 2.624454147554222e-7*jx**2*coep(8)**2 - 6.489142790944746e-13*jx*coen(12)*coep(12)&
            - 6.489142790944746e-13*jx**2*coen(12)*coep(12) - 9.476194183337636e-15*jx*coep(12)**2&
            - 9.476194183337636e-15*jx**2*coep(12)**2 + jx*Q**12*((-2.2864473606921037e26&
            - 2.2864473606921037e26*jx)*coen(15)**2 + (1.0418335867822516e20&
            + 1.0418335867822516e20*jx)*coen(15)*coep(15) + (-1.1867944580834541e13&
            - 1.1867944580834541e13*jx)*coep(15)**2) + Q*(-4.54562450888429e-9*coen(3)**2&
            + 8.897708831991826e-7*coen(7)**2 - 2.6552179182024184e-10*coen(3)*coep(3)&
            - 3.877455220599396e-12*coep(3)**2 + 4.338205050068525e-8*coen(7)*coep(7)&
            + 5.080474927432823e-10*coep(7)**2 + jx*(-0.0001380775252312418*coen(5)**2&
            + 0.01712874066687467*coen(8)**2 + 4.49208825720669e-7*coen(12)**2&
            - 1.5152081696280958e-9*coen(13)**2 - 1.5152081696280968e-9*coen(14)**2&
            + 3.030416339256193e-9*coen(12)*coen(15) - 0.00019884825212135963*coen(5)*coep(5)&
            - 0.00007159135294737528*coep(5)**2 + 0.02293147009138267*coen(8)*coep(8)&
            + 0.007631014915218425*coep(8)**2 + 2.8257365862352792e-8*coen(12)*coep(12)&
            + 8.850726394008058e-11*coen(15)*coep(12) + 4.421144108115874e-10*coep(12)**2&
            - 8.850726394008046e-11*coen(13)*coep(13) - 1.292485073533128e-12*coep(13)**2&
            - 8.850726394008061e-11*coen(14)*coep(14) - 1.2924850735331318e-12*coep(14)**2&
            + 8.850726394008058e-11*coen(12)*coep(15) + 2.584970147066263e-12*coep(12)*coep(15))&
            + jx**2*(-0.0001380775252312418*coen(5)**2 + 0.01712874066687467*coen(8)**2&
            + 4.49208825720669e-7*coen(12)**2 - 1.5152081696280958e-9*coen(13)**2&
            - 1.5152081696280968e-9*coen(14)**2 + 3.030416339256193e-9*coen(12)*coen(15)&
            - 0.00019884825212135963*coen(5)*coep(5) - 0.00007159135294737528*coep(5)**2&
            + 0.02293147009138267*coen(8)*coep(8) + 0.007631014915218425*coep(8)**2&
            + 2.8257365862352792e-8*coen(12)*coep(12) + 8.850726394008058e-11*coen(15)*coep(12)&
            + 4.421144108115874e-10*coep(12)**2 - 8.850726394008046e-11*coen(13)*coep(13)&
            - 1.292485073533128e-12*coep(13)**2 - 8.850726394008061e-11*coen(14)*coep(14)&
            - 1.2924850735331318e-12*coep(14)**2 + 8.850726394008058e-11*coen(12)*coep(15)&
            + 2.584970147066263e-12*coep(12)*coep(15))) + Q**10*(1.2323529339123864e23*coen(3)**2&
            - 9.218055134112158e21*coen(7)**2 + 7.52395678962366e21*coen(3)*coep(3)&
            - 1.7141746349997242e15*coep(3)**2 + 4.200262647035793e15*coen(7)*coep(7)&
            - 4.784687780505272e8*coep(7)**2 + jx*(2.2915152280882002e26*coen(5)**2&
            - 3.577234485757959e25*coen(8)**2 - 3.1580374070569426e21*coen(12)**2&
            + 2.734854345260092e21*coen(13)**2 + 4.107843113041288e22*coen(14)**2&
            - 8.215686226082574e22*coen(12)*coen(15) - 1.4957308601442229e23*coen(15)**2&
            + 4.6303701557654075e25*coen(5)*coep(5) - 3.6779714095721743e18*coep(5)**2&
            + 5.682898148266636e18*coen(8)*coep(8) - 2.2570040831925992e11*coep(8)**2&
            + 1.4389788698207445e15*coen(12)*coep(12) - 2.5079855965412194e21*coen(15)*coep(12)&
            - 1.6391985914760664e8*coep(12)**2 + 1.290654355870231e20*coen(13)*coep(13)&
            - 2.9404865633699535e13*coep(13)**2 + 2.5079855965412194e21*coen(14)*coep(14)&
            - 5.713915449999081e14*coep(14)**2 - 2.5079855965412194e21*coen(12)*coep(15)&
            - 1.7543682209842075e22*coen(15)*coep(15) + 1.142783089999816e15*coep(12)*coep(15)&
            - 5.117703590622556e20*coep(15)**2) + jx**2*(2.2915152280882002e26*coen(5)**2&
            - 3.577234485757959e25*coen(8)**2 - 3.1580374070569426e21*coen(12)**2&
            + 2.734854345260092e21*coen(13)**2 + 4.107843113041288e22*coen(14)**2&
            - 8.215686226082574e22*coen(12)*coen(15) - 1.4957308601442229e23*coen(15)**2&
            + 4.6303701557654075e25*coen(5)*coep(5) - 3.6779714095721743e18*coep(5)**2&
            + 5.682898148266636e18*coen(8)*coep(8) - 2.2570040831925992e11*coep(8)**2&
            + 1.4389788698207445e15*coen(12)*coep(12) - 2.5079855965412194e21*coen(15)*coep(12)&
            - 1.6391985914760664e8*coep(12)**2 + 1.290654355870231e20*coen(13)*coep(13)&
            - 2.9404865633699535e13*coep(13)**2 + 2.5079855965412194e21*coen(14)*coep(14)&
            - 5.713915449999081e14*coep(14)**2 - 2.5079855965412194e21*coen(12)*coep(15)&
            - 1.7543682209842075e22*coen(15)*coep(15) + 1.142783089999816e15*coep(12)*coep(15)&
            - 5.117703590622556e20*coep(15)**2)) + Q**11*(-2.514553509001452e24*coen(3)**2&
            + 1.1457715346859706e18*coen(3)*coep(3) - 1.3051943466277289e11*coep(3)**2&
            + jx*(-9.758183692562662e27*coen(5)**2 - 2.328290286112454e22*coen(13)**2&
            - 8.381845030004838e23*coen(14)**2 + 1.676369006000967e24*coen(12)*coen(15)&
            + 1.1205608085485048e25*coen(15)**2&
            + 1.550213279495433e21*coen(5)*coep(5) - 6.156784109720102e13*coep(5)**2&
            - 3.8192384489532346e17*coen(15)*coep(12) + 1.0608995692335084e16*coen(13)*coep(13)&
            - 1.208513284096443e9*coep(13)**2 + 3.819238448953235e17*coen(14)*coep(14)&
            - 4.350647822092429e10*coep(14)**2 - 3.8192384489532346e17*coen(12)*coep(15)&
            + 6.841425756904234e23*coen(15)*coep(15) + 8.701295644184857e10*coep(12)*coep(15)&
            - 1.5586743554790698e17*coep(15)**2) + jx**2*(-9.758183692562662e27*coen(5)**2&
            - 2.328290286112454e22*coen(13)**2 - 8.381845030004838e23*coen(14)**2&
            + 1.676369006000967e24*coen(12)*coen(15) + 1.1205608085485048e25*coen(15)**2&
            + 1.550213279495433e21*coen(5)*coep(5) - 6.156784109720102e13*coep(5)**2&
            - 3.8192384489532346e17*coen(15)*coep(12) + 1.0608995692335084e16*coen(13)*coep(13)&
            - 1.208513284096443e9*coep(13)**2 + 3.819238448953235e17*coen(14)*coep(14)&
            - 4.350647822092429e10*coep(14)**2 - 3.8192384489532346e17*coen(12)*coep(15)&
            + 6.841425756904234e23*coen(15)*coep(15) + 8.701295644184857e10*coep(12)*coep(15)&
            - 1.5586743554790698e17*coep(15)**2)) + Q**8*(3.356839924254847e18*coen(3)**2&
            - 6.030197664567674e18*coen(7)**2 + 4.1413567941062714e17*coen(3)*coep(3)&
            + 1.2819092022892714e16*coep(3)**2 - 7.072921627057046e17*coen(7)*coep(7)&
            - 2.063256503054704e16*coep(7)**2 + jx*(4.4555671190680607e21*coen(5)**2&
            - 6.261571313452386e21*coen(8)**2 - 2.3146736660083476e18*coen(12)**2&
            + 1.645550627078832e17*coen(13)**2 + 1.118946641418282e18*coen(14)**2&
            - 2.2378932828365637e18*coen(12)*coen(15) - 2.7140396125967507e17*coen(15)**2&
            + 2.4799802994074274e21*coen(5)*coep(5) + 3.075722052050665e20*coep(5)**2&
            - 2.468284516800017e21*coen(8)*coep(8) - 2.0136367080169724e20*coep(8)**2&
            - 2.643146507782135e17*coen(12)*coep(12) - 1.3804522647020902e17*coen(15)*coep(12)&
            - 7.533221826975273e15*coep(12)**2 + 2.0016472561402016e16*coen(13)*coep(13)&
            + 5.765716512567056e14*coep(13)**2 + 1.3804522647020904e17*coen(14)*coep(14)&
            + 4.273030674297571e15*coep(14)**2 - 1.3804522647020902e17*coen(12)*coep(15)&
            - 3.2147430174550976e16*coen(15)*coep(15) - 8.546061348595141e15*coep(12)*coep(15)&
            - 9.533063202794228e14*coep(15)**2) + jx**2*(4.4555671190680607e21*coen(5)**2&
            - 6.261571313452386e21*coen(8)**2 - 2.3146736660083476e18*coen(12)**2&
            + 1.645550627078832e17*coen(13)**2 + 1.118946641418282e18*coen(14)**2&
            - 2.2378932828365637e18*coen(12)*coen(15) - 2.7140396125967507e17*coen(15)**2&
            + 2.4799802994074274e21*coen(5)*coep(5) + 3.075722052050665e20*coep(5)**2&
            - 2.468284516800017e21*coen(8)*coep(8) - 2.0136367080169724e20*coep(8)**2&
            - 2.643146507782135e17*coen(12)*coep(12) - 1.3804522647020902e17*coen(15)*coep(12)&
            - 7.533221826975273e15*coep(12)**2 + 2.0016472561402016e16*coen(13)*coep(13)&
            + 5.765716512567056e14*coep(13)**2 + 1.3804522647020904e17*coen(14)*coep(14)&
            + 4.273030674297571e15*coep(14)**2 - 1.3804522647020902e17*coen(12)*coep(15)&
            - 3.2147430174550976e16*coen(15)*coep(15) - 8.546061348595141e15*coep(12)*coep(15)&
            - 9.533063202794228e14*coep(15)**2)) + Q**6*(1.4177163359755999e12*coen(3)**2&
            - 1.0941938666590729e13*coen(7)**2 + 1.4950769873401642e11*coen(3)*coep(3)&
            + 3.79391936066388e9*coep(3)**2 - 1.2960577569532632e12*coen(7)*coep(7)&
            - 3.84335557909949e10*coep(7)**2 + jx*(4.1075790470398065e15*coen(5)**2&
            - 2.094233850145866e16*coen(8)**2 - 4.199331987497135e12*coen(12)**2&
            + 7.806153547880905e10*coen(13)**2 + 4.725721119918665e11*coen(14)**2&
            - 9.451442239837329e11*coen(12)*coen(15) - 3.487923750170789e10*coen(15)**2&
            + 3.4880291911530215e15*coen(5)*coep(5) + 7.028939126704784e14*coep(5)**2&
            - 1.4809027453692494e16*coen(8)*coep(8) - 2.4232269138993545e15*coep(8)**2&
            - 5.0807854360212274e11*coen(12)*coep(12) - 4.983589957800546e10*coen(15)*coep(12)&
            - 1.5402464858262484e10*coep(12)**2 + 1.1379650021710482e10*coen(13)*coep(13)&
            + 4.1807775943135786e8*coep(13)**2 + 4.983589957800547e10*coen(14)*coep(14)&
            + 1.2646397868879597e9*coep(14)**2 - 4.983589957800546e10*coen(12)*coep(15)&
            - 3.0311408258167033e9*coen(15)*coep(15) - 2.5292795737759194e9*coep(12)*coep(15)&
            - 5.91073469176418e7*coep(15)**2) + jx**2*(4.1075790470398065e15*coen(5)**2&
            - 2.094233850145866e16*coen(8)**2 - 4.199331987497135e12*coen(12)**2&
            + 7.806153547880905e10*coen(13)**2 + 4.725721119918665e11*coen(14)**2&
            - 9.451442239837329e11*coen(12)*coen(15) - 3.487923750170789e10*coen(15)**2&
            + 3.4880291911530215e15*coen(5)*coep(5) + 7.028939126704784e14*coep(5)**2&
            - 1.4809027453692494e16*coen(8)*coep(8) - 2.4232269138993545e15*coep(8)**2&
            - 5.0807854360212274e11*coen(12)*coep(12) - 4.983589957800546e10*coen(15)*coep(12)&
            - 1.5402464858262484e10*coep(12)**2 + 1.1379650021710482e10*coen(13)*coep(13)&
            + 4.1807775943135786e8*coep(13)**2 + 4.983589957800547e10*coen(14)*coep(14)&
            + 1.2646397868879597e9*coep(14)**2 - 4.983589957800546e10*coen(12)*coep(15)&
            - 3.0311408258167033e9*coen(15)*coep(15)&
            - 2.5292795737759194e9*coep(12)*coep(15) - 5.91073469176418e7*coep(15)**2))&
            + Q**4*(59748.44310196677*coen(3)**2 - 1.406193467883785e6*coen(7)**2&
            + 4004.2723584374276*coen(3)*coep(3) + 57.881585908121664*coep(3)**2&
            - 122203.6585315433*coen(7)*coep(7) - 2382.975405797183*coep(7)**2&
            + jx*(4.377170355407007e8*coen(5)**2 - 6.38600242297184e9*coen(8)**2&
            - 558716.4064766124*coen(12)**2 + 4725.567614820544*coen(13)**2&
            + 19916.147700655587*coen(14)**2 - 39832.29540131117*coen(12)*coen(15)&
            - 478.54690201904896*coen(15)**2 + 4.861194417201269e8*coen(5)*coep(5)&
            + 1.3206482146281828e8*coep(5)**2 - 6.2816271630867195e9*coen(8)*coep(8)&
            - 1.4925080455817416e9*coep(8)**2 - 53673.19840277987*coen(12)*coep(12)&
            - 1334.7574528124753*coen(15)*coep(12) - 1262.012288074372*coep(12)**2&
            + 625.8113597945546*coen(13)*coep(13) + 20.291885562920804*coep(13)**2&
            + 1334.7574528124755*coen(14)*coep(14) + 19.29386196937389*coep(14)**2&
            - 1334.7574528124753*coen(12)*coep(15) - 25.12141203896377*coen(15)*coep(15)&
            - 38.587723938747764*coep(12)*coep(15) - 0.35296907058175686*coep(15)**2)&
            + jx**2*(4.377170355407007e8*coen(5)**2 - 6.38600242297184e9*coen(8)**2&
            - 558716.4064766124*coen(12)**2 + 4725.567614820544*coen(13)**2&
            + 19916.147700655587*coen(14)**2 - 39832.29540131117*coen(12)*coen(15)&
            - 478.54690201904896*coen(15)**2 + 4.861194417201269e8*coen(5)*coep(5)&
            + 1.3206482146281828e8*coep(5)**2 - 6.2816271630867195e9*coen(8)*coep(8)&
            - 1.4925080455817416e9*coep(8)**2 - 53673.19840277987*coen(12)*coep(12)&
            - 1334.7574528124753*coen(15)*coep(12) - 1262.012288074372*coep(12)**2&
            + 625.8113597945546*coen(13)*coep(13) + 20.291885562920804*coep(13)**2&
            + 1334.7574528124755*coen(14)*coep(14) + 19.29386196937389*coep(14)**2&
            - 1334.7574528124753*coen(12)*coep(15) - 25.12141203896377*coen(15)*coep(15)&
            - 38.587723938747764*coep(12)*coep(15) - 0.35296907058175686*coep(15)**2))&
            + Q**2*(0.00024271676226758865*coen(3)**2 - 0.01929312610868451*coen(7)**2&
            + 0.00001183400247960038*coen(3)*coep(3) + 1.3858808469147372e-7*coep(3)**2&
            - 0.0010127963809839072*coen(7)*coep(7) - 0.000014230322592137327*coep(7)**2&
            + jx*(4.6724753022226295*coen(5)**2 - 230.39431298335882*coen(8)**2&
            - 0.008550959702100477*coen(12)**2 + 0.000041632151021917706*coen(13)**2&
            + 0.00008090558742252954*coen(14)**2 - 0.00016181117484505904*coen(12)*coen(15)&
            - 4.133271025583822e-7*coen(15)**2 + 6.255376838815352*coen(5)*coep(5)&
            + 2.0816316514853503*coep(5)**2 - 283.22581278962514*coen(8)*coep(8)&
            - 85.97167654076347*coep(8)**2 - 0.0005789255455917842*coen(12)*coep(12)&
            - 3.944667493200125e-6*coen(15)*coep(12) - 0.000011056040275069782*coep(12)**2&
            + 3.7635374861098304e-6*coen(13)*coep(13) + 7.440645536416372e-8*coep(13)**2&
            + 3.944667493200126e-6*coen(14)*coep(14) + 4.619602823049124e-8*coep(14)**2&
            - 3.944667493200125e-6*coen(12)*coep(15) - 2.4143514860207325e-8*coen(15)*coep(15)&
            - 9.239205646098246e-8*coep(12)*coep(15) - 3.5257142962377863e-10*coep(15)**2)&
            + jx**2*(4.6724753022226295*coen(5)**2 - 230.39431298335882*coen(8)**2&
            - 0.008550959702100477*coen(12)**2 + 0.000041632151021917706*coen(13)**2&
            + 0.00008090558742252954*coen(14)**2 - 0.00016181117484505904*coen(12)*coen(15)&
            - 4.133271025583822e-7*coen(15)**2 + 6.255376838815352*coen(5)*coep(5)&
            + 2.0816316514853503*coep(5)**2 - 283.22581278962514*coen(8)*coep(8)&
            - 85.97167654076347*coep(8)**2 - 0.0005789255455917842*coen(12)*coep(12)&
            - 3.944667493200125e-6*coen(15)*coep(12) - 0.000011056040275069782*coep(12)**2&
            + 3.7635374861098304e-6*coen(13)*coep(13) + 7.440645536416372e-8*coep(13)**2&
            + 3.944667493200126e-6*coen(14)*coep(14) + 4.619602823049124e-8*coep(14)**2&
            - 3.944667493200125e-6*coen(12)*coep(15) - 2.4143514860207325e-8*coen(15)*coep(15)&
            - 9.239205646098246e-8*coep(12)*coep(15) - 3.5257142962377863e-10*coep(15)**2))&
            + Q**3*(-5.262888673411343*coen(3)**2 + 219.03071090740306*coen(7)**2&
            - 0.27627635728524474*coen(3)*coep(3) - 0.0038818283344676475*coep(3)**2&
            + 14.67918787170109*coen(7)*coep(7) + 0.21218703369839756*coep(7)**2&
            + jx*(-62848.27110898843*coen(5)**2 + 1.6046187731978707e6*coen(8)**2&
            + 90.33360776138642*coen(12)**2 - 0.5782831983491713*coen(13)**2&
            - 1.7542962244704474*coen(14)**2 + 3.5085924489408944*coen(12)*coen(15)&
            + 0.022069886303705637*coen(15)**2 - 77259.94811578383*coen(5)*coep(5)&
            - 23451.842907764956*coep(5)**2 + 1.7820562575021207e6*coen(8)*coep(8)&
            + 484133.98289717396*coep(8)**2 + 7.187212896499146*coen(12)*coep(12)&
            + 0.09209211909508154*coen(15)*coep(12) + 0.14511665867179846*coep(12)**2&
            - 0.06583048398540124*coen(13)*coep(13) - 0.0017219867715013144*coep(13)**2&
            - 0.09209211909508155*coen(14)*coep(14) - 0.0012939427781558824*coep(14)**2&
            + 0.09209211909508154*coen(12)*coep(15) + 0.0010760488348745048*coen(15)*coep(15)&
            + 0.0025878855563117647*coep(12)*coep(15) + 0.000012601615329793761*coep(15)**2)&
            + jx**2*(-62848.27110898843*coen(5)**2 + 1.6046187731978707e6*coen(8)**2&
            + 90.33360776138642*coen(12)**2 - 0.5782831983491713*coen(13)**2&
            - 1.7542962244704474*coen(14)**2 + 3.5085924489408944*coen(12)*coen(15)&
            + 0.022069886303705637*coen(15)**2 - 77259.94811578383*coen(5)*coep(5)&
            - 23451.842907764956*coep(5)**2 + 1.7820562575021207e6*coen(8)*coep(8)&
            + 484133.98289717396*coep(8)**2 + 7.187212896499146*coen(12)*coep(12)&
            + 0.09209211909508154*coen(15)*coep(12) + 0.14511665867179846*coep(12)**2&
            - 0.06583048398540124*coen(13)*coep(13) - 0.0017219867715013144*coep(13)**2&
            - 0.09209211909508155*coen(14)*coep(14) - 0.0012939427781558824*coep(14)**2&
            + 0.09209211909508154*coen(12)*coep(15) + 0.0010760488348745048*coen(15)*coep(15)&
            + 0.0025878855563117647*coep(12)*coep(15) + 0.000012601615329793761*coep(15)**2))&
            + Q**5*(-3.835894521738135e8*coen(3)**2 + 5.197180057124412e9*coen(7)**2&
            - 3.3335409031799413e7*coen(3)*coep(3) - 650041.584838991*coep(3)**2&
            + 5.480775036088517e8*coen(7)*coep(7) + 1.3908058713319272e7*coep(7)**2&
            + jx*(-1.7420100625946453e12*coen(5)**2 + 1.5057897948003045e13*coen(8)**2&
            + 2.0185576894315498e9*coen(12)**2 - 2.454668844455154e7*coen(13)**2&
            - 1.2786315072460449e8*coen(14)**2 + 2.5572630144920895e8*coen(12)*coen(15)&
            + 5.432840046828186e6*coen(15)**2 - 1.7135379855481733e12*coen(5)*coep(5)&
            - 4.0713483360955597e11*coep(5)**2 + 1.278670160660424e13*coen(8)*coep(8)&
            + 2.5767257754643174e12*coep(8)**2 + 2.2440894944376916e8*coen(12)*coep(12)&
            + 1.1111803010599801e7*coen(15)*coep(12) + 6.168643085773541e6*coep(12)**2&
            - 3.529477326680407e6*coen(13)*coep(13) - 127578.36160665838*coep(13)**2&
            - 1.1111803010599803e7*coen(14)*coep(14) - 216680.52827966365*coep(14)**2&
            + 1.1111803010599801e7*coen(12)*coep(15) + 364102.7296761461*coen(15)*coep(15)&
            + 433361.05655932723*coep(12)*coep(15) + 5263.089405675545*coep(15)**2)&
            + jx**2*(-1.7420100625946453e12*coen(5)**2 + 1.5057897948003045e13*coen(8)**2&
            + 2.0185576894315498e9*coen(12)**2 - 2.454668844455154e7*coen(13)**2&
            - 1.2786315072460449e8*coen(14)**2 + 2.5572630144920895e8*coen(12)*coen(15)&
            + 5.432840046828186e6*coen(15)**2 - 1.7135379855481733e12*coen(5)*coep(5)&
            - 4.0713483360955597e11*coep(5)**2 + 1.278670160660424e13*coen(8)*coep(8)&
            + 2.5767257754643174e12*coep(8)**2 + 2.2440894944376916e8*coen(12)*coep(12)&
            + 1.1111803010599801e7*coen(15)*coep(12) + 6.168643085773541e6*coep(12)**2&
            - 3.529477326680407e6*coen(13)*coep(13) - 127578.36160665838*coep(13)**2&
            - 1.1111803010599803e7*coen(14)*coep(14) - 216680.52827966365*coep(14)**2&
            + 1.1111803010599801e7*coen(12)*coep(15) + 364102.7296761461*coen(15)*coep(15)&
            + 433361.05655932723*coep(12)*coep(15) + 5263.089405675545*coep(15)**2))&
            + Q**7*(-2.9848042639207335e15*coen(3)**2 + 1.2305777302968578e16*coen(7)**2&
            - 3.535460065274682e14*coen(3)*coep(3) - 1.048412394714512e13*coep(3)**2&
            + 1.5181723165342888e15*coen(7)*coep(7) + 4.699327201162144e13*coep(7)**2&
            + jx*(-5.712770210135123e18*coen(5)**2 + 1.633358097581965e19*coen(8)**2&
            + 4.705165124149709e15*coen(12)**2 - 1.5058290944346022e14*coen(13)**2&
            - 9.949347546402444e14*coen(14)**2 + 1.9898695092804885e15*coen(12)*coen(15)&
            + 1.2891090855683275e14*coen(15)**2 - 4.039690738101438e18*coen(5)*coep(5)&
            - 6.610216201575433e17*coep(5)**2 + 9.091313845426085e18*coen(8)*coep(8)&
            + 1.1275232502117832e18*coep(8)**2 + 5.794354548626915e14*coen(12)*coep(12)&
            + 1.1784866884248938e14*coen(15)*coep(12) + 1.777806734096111e13*coep(12)**2&
            - 2.074788607886676e13*coen(13)*coep(13) - 7.068639862313677e11*coep(13)**2&
            - 1.1784866884248939e14*coen(14)*coep(14) - 3.494707982381706e12*coep(14)**2&
            + 1.1784866884248938e14*coen(12)*coep(15) + 1.3594520138459488e13*coen(15)*coep(15)&
            + 6.989415964763411e12*coep(12)*coep(15) + 3.449756339571135e11*coep(15)**2)&
            + jx**2*(-5.712770210135123e18*coen(5)**2 + 1.633358097581965e19*coen(8)**2&
            + 4.705165124149709e15*coen(12)**2 - 1.5058290944346022e14*coen(13)**2&
            - 9.949347546402444e14*coen(14)**2 + 1.9898695092804885e15*coen(12)*coen(15)&
            + 1.2891090855683275e14*coen(15)**2 - 4.039690738101438e18*coen(5)*coep(5)&
            - 6.610216201575433e17*coep(5)**2 + 9.091313845426085e18*coen(8)*coep(8)&
            + 1.1275232502117832e18*coep(8)**2 + 5.794354548626915e14*coen(12)*coep(12)&
            + 1.1784866884248938e14*coen(15)*coep(12) + 1.777806734096111e13*coep(12)**2&
            - 2.074788607886676e13*coen(13)*coep(13) - 7.068639862313677e11*coep(13)**2&
            - 1.1784866884248939e14*coen(14)*coep(14) - 3.494707982381706e12*coep(14)**2&
            + 1.1784866884248938e14*coen(12)*coep(15) + 1.3594520138459488e13*coen(15)*coep(15)&
            + 6.989415964763411e12*coep(12)*coep(15) + 3.449756339571135e11*coep(15)**2))&
            + Q**9*(-1.644951616886967e21*coen(3)**2 + 4.517659794800054e20*coen(7)**2&
            - 1.929391790074333e20*coen(3)*coep(3) - 5.628268440841747e18*coep(3)**2&
            + 2.758193383642502e19*coen(7)*coep(7) - 6.2839610445191455e12*coep(7)**2&
            + jx*(-1.708067036813294e24*coen(5)**2 + 8.400423231225195e23*coen(8)**2&
            + 1.6061431171601118e20*coen(12)**2 - 8.30926421507007e19*coen(13)**2&
            - 5.4831720562898895e20*coen(14)**2 + 1.0966344112579778e21*coen(12)*coen(15)&
            + 3.0523262907721023e20*coen(15)**2 - 6.733126893508465e23*coen(5)*coep(5)&
            - 5.492912741713486e22*coep(5)**2 + 1.6974388190348264e23*coen(8)*coep(8)&
            - 1.348300726699904e16*coep(8)**2 + 9.6671165352447e18*coen(12)*coep(12)&
            + 6.431305966914442e19*coen(15)*coep(12) - 2.2024484337963184e12*coep(12)**2&
            - 7.788194112178774e18*coen(13)*coep(13) - 1.7886561637620022e17*coep(13)**2&
            - 6.431305966914444e19*coen(14)*coep(14) - 1.8760894802805821e18*coep(14)**2&
            + 6.431305966914442e19*coen(12)*coep(15) + 3.765676203617082e19*coen(15)*coep(15)&
            + 3.752178960561164e18*coep(12)*coep(15) + 1.1656216110450378e18*coep(15)**2)&
            + jx**2*(-1.708067036813294e24*coen(5)**2 + 8.400423231225195e23*coen(8)**2&
            + 1.6061431171601118e20*coen(12)**2 - 8.30926421507007e19*coen(13)**2&
            - 5.4831720562898895e20*coen(14)**2 + 1.0966344112579778e21*coen(12)*coen(15)&
            + 3.0523262907721023e20*coen(15)**2 - 6.733126893508465e23*coen(5)*coep(5)&
            - 5.492912741713486e22*coep(5)**2 + 1.6974388190348264e23*coen(8)*coep(8)&
            - 1.348300726699904e16*coep(8)**2 + 9.6671165352447e18*coen(12)*coep(12)&
            + 6.431305966914442e19*coen(15)*coep(12) - 2.2024484337963184e12*coep(12)**2&
            - 7.788194112178774e18*coen(13)*coep(13) - 1.7886561637620022e17*coep(13)**2&
            - 6.431305966914444e19*coen(14)*coep(14) - 1.8760894802805821e18*coep(14)**2&
            + 6.431305966914442e19*coen(12)*coep(15) + 3.765676203617082e19*coen(15)*coep(15)&
            + 3.752178960561164e18*coep(12)*coep(15) + 1.1656216110450378e18*coep(15)**2)))&
            + Q*(-1.0084641139693765e-9*coen(7)**2 - 0.00003063302498243218*jx*coen(8)**2&
            - 0.00003063302498243218*jx**2*coen(8)**2 - 6.723094093129173e-10*jx*coen(12)**2&
            - 6.723094093129173e-10*jx**2*coen(12)**2 - 5.890702102741092e-11*coen(7)*coep(7)&
            - 8.602282119552956e-13*coep(7)**2 - 0.00004411524225065081*jx*coen(8)*coep(8)&
            - 0.00004411524225065081*jx**2*coen(8)*coep(8) - 0.000015882814380474272*jx*coep(8)**2&
            - 0.000015882814380474272*jx**2*coep(8)**2 - 3.9271347351607245e-11*jx*coen(12)*coep(12)&
            - 3.9271347351607245e-11*jx**2*coen(12)*coep(12) - 5.734854746368629e-13*jx*coep(12)**2&
            - 5.734854746368629e-13*jx**2*coep(12)**2 + jx*Q**12*((-1.38372465201729e28&
            - 1.38372465201729e28*jx)*coen(15)**2 + (6.305025176235953e21&
            + 6.305025176235953e21*jx)*coen(15)*coep(15) + (-7.182307263047969e14&
            - 7.182307263047969e14*jx)*coep(15)**2) + Q*(-2.750945768483924e-7*coen(3)**2&
            + 0.00005384763834480861*coen(7)**2 - 1.6068992241232152e-8*coen(3)*coep(3)&
            - 2.3465794437587433e-10*coep(3)**2 + 2.6254185320359428e-6*coen(7)*coep(7)&
            + 3.074629454367401e-8*coep(7)**2 + jx*(-0.00835625078611802*coen(5)**2&
            + 1.0366064457128443*coen(8)**2 + 0.000027185464084564643*coen(12)**2&
            - 9.169819228279741e-8*coen(13)**2 - 9.169819228279746e-8*coen(14)**2&
            + 1.833963845655949e-7*coen(12)*coen(15) - 0.012034006695329598*coen(5)*coep(5)&
            - 0.0043326044433652615*coep(5)**2 + 1.3877791817100267*coen(8)*coep(8)&
            + 0.4618179119113026*coep(8)**2 + 1.7100946392648957e-6*coen(12)*coep(12)&
            + 5.356330747077382e-9*coen(15)*coep(12) + 2.6756120423735134e-8*coep(12)**2&
            - 5.356330747077374e-9*coen(13)*coep(13) - 7.821931479195785e-11*coep(13)**2&
            - 5.356330747077382e-9*coen(14)*coep(14) - 7.821931479195809e-11*coep(14)**2&
            + 5.356330747077382e-9*coen(12)*coep(15) + 1.5643862958391616e-10*coep(12)*coep(15))&
            + jx**2*(-0.00835625078611802*coen(5)**2 + 1.0366064457128443*coen(8)**2&
            + 0.000027185464084564643*coen(12)**2 - 9.169819228279741e-8*coen(13)**2&
            - 9.169819228279746e-8*coen(14)**2 + 1.833963845655949e-7*coen(12)*coen(15)&
            - 0.012034006695329598*coen(5)*coep(5) - 0.0043326044433652615*coep(5)**2&
            + 1.3877791817100267*coen(8)*coep(8) + 0.4618179119113026*coep(8)**2&
            + 1.7100946392648957e-6*coen(12)*coep(12) + 5.356330747077382e-9*coen(15)*coep(12)&
            + 2.6756120423735134e-8*coep(12)**2 - 5.356330747077374e-9*coen(13)*coep(13)&
            - 7.821931479195785e-11*coep(13)**2 - 5.356330747077382e-9*coen(14)*coep(14)&
            - 7.821931479195809e-11*coep(14)**2 + 5.356330747077382e-9*coen(12)*coep(15)&
            + 1.5643862958391616e-10*coep(12)*coep(15))) + Q**10*(7.458020525450586e24*coen(3)**2&
            - 5.578632752281926e23*coen(7)**2 + 4.553389100269383e23*coen(3)*coep(3)&
            - 1.037393530719148e17*coep(3)**2 + 2.5419377981619213e17*coen(7)*coep(7)&
            - 2.8956233797077072e10*coep(7)**2 + jx*(1.3867916515772581e28*coen(5)**2&
            - 2.1648902262466046e27*coen(8)**2 - 1.9111982577262152e23*coen(12)**2&
            + 1.6550940302721413e23*coen(13)**2 + 2.486006841816862e24*coen(14)**2&
            - 4.972013683633723e24*coen(12)*coen(15) - 9.0519453871796e24*coen(15)**2&
            + 2.8022326000797577e27*coen(5)*coep(5) - 2.225854745809357e20*coep(5)**2&
            + 3.4392072163339305e20*coen(8)*coep(8) - 1.36590600916871e13*coep(8)**2&
            + 8.7084906048317e16*coen(12)*coep(12) - 1.5177963667564606e23*coen(15)*coep(12)&
            - 9.920191208298237e9*coep(12)**2 + 7.810852242452425e21*coen(13)*coep(13)&
            - 1.7795396546671338e15*coep(13)**2 + 1.517796366756461e23*coen(14)*coep(14)&
            - 3.4579784357304924e16*coep(14)**2 - 1.5177963667564606e23*coen(12)*coep(15)&
            - 1.0617181037383451e24*coen(15)*coep(15) + 6.915956871460984e16*coep(12)*coep(15)&
            - 3.097159699280492e22*coep(15)**2) + jx**2*(1.3867916515772581e28*coen(5)**2&
            - 2.1648902262466046e27*coen(8)**2 - 1.9111982577262152e23*coen(12)**2&
            + 1.6550940302721413e23*coen(13)**2 + 2.486006841816862e24*coen(14)**2&
            - 4.972013683633723e24*coen(12)*coen(15) - 9.0519453871796e24*coen(15)**2&
            + 2.8022326000797577e27*coen(5)*coep(5) - 2.225854745809357e20*coep(5)**2&
            + 3.4392072163339305e20*coen(8)*coep(8) - 1.36590600916871e13*coep(8)**2&
            + 8.7084906048317e16*coen(12)*coep(12) - 1.5177963667564606e23*coen(15)*coep(12)&
            - 9.920191208298237e9*coep(12)**2 + 7.810852242452425e21*coen(13)*coep(13)&
            - 1.7795396546671338e15*coep(13)**2 + 1.517796366756461e23*coen(14)*coep(14)&
            - 3.4579784357304924e16*coep(14)**2 - 1.5177963667564606e23*coen(12)*coep(15)&
            - 1.0617181037383451e24*coen(15)*coep(15) + 6.915956871460984e16*coep(12)*coep(15)&
            - 3.097159699280492e22*coep(15)**2)) + Q**11*(-1.521771171748588e26*coen(3)**2&
            + 6.934042503584046e19*coen(3)*coep(3) - 7.898846149494147e12*coep(3)**2&
            + jx*(-5.905510691584361e29*coen(5)**2 - 1.409047381248692e24*coen(13)**2&
            - 5.072570572495292e25*coen(14)**2 + 1.0145141144990582e26*coen(12)*coen(15)&
            + 6.781470859681828e26*coen(15)**2 + 9.381665056452869e22*coen(5)*coep(5)&
            - 3.7259961004259695e15*coep(5)**2 - 2.3113475011946815e19*coen(15)*coep(12)&
            + 6.420409726023914e17*coen(13)*coep(13) - 7.31374643581751e10*coep(13)**2&
            + 2.3113475011946815e19*coen(14)*coep(14) - 2.6329487164980493e12*coep(14)**2&
            - 2.3113475011946815e19*coen(12)*coep(15) + 4.1403312569194225e25*coen(15)*coep(15)&
            + 5.265897432996097e12*coep(12)*coep(15) - 9.432870256373174e18*coep(15)**2)&
            + jx**2*(-5.905510691584361e29*coen(5)**2 - 1.409047381248692e24*coen(13)**2&
            - 5.072570572495292e25*coen(14)**2 + 1.0145141144990582e26*coen(12)*coen(15)&
            + 6.781470859681828e26*coen(15)**2 + 9.381665056452869e22*coen(5)*coep(5)&
            - 3.7259961004259695e15*coep(5)**2 - 2.3113475011946815e19*coen(15)*coep(12)&
            + 6.420409726023914e17*coen(13)*coep(13) - 7.31374643581751e10*coep(13)**2&
            + 2.3113475011946815e19*coen(14)*coep(14) - 2.6329487164980493e12*coep(14)**2&
            - 2.3113475011946815e19*coen(12)*coep(15) + 4.1403312569194225e25*coen(15)*coep(15)&
            + 5.265897432996097e12*coep(12)*coep(15) - 9.432870256373174e18*coep(15)**2))&
            + Q**8*(2.0315106465697366e20*coen(3)**2 - 3.649387826918361e20*coen(7)**2&
            + 2.506288833638194e19*coen(3)*coep(3) + 7.757927846274819e17*coep(3)**2&
            - 4.280429186956036e19*coen(7)*coep(7) - 1.2486527946340224e18*coep(7)**2&
            + jx*(2.6964443474027375e23*coen(5)**2 - 3.789411790423061e23*coen(8)**2&
            - 1.4008068010196753e20*coen(12)**2 + 9.958632802909792e18*coen(13)**2&
            + 6.771702155232455e19*coen(14)**2 - 1.3543404310464905e20*coen(12)*coen(15)&
            - 1.6424972571267944e19*coen(15)**2 + 1.500847968688216e23*coen(5)*coep(5)&
            + 1.8613822033879852e22*coep(5)**2 - 1.493769851344806e23*coen(8)*coep(8)&
            - 1.218623617141403e22*coep(8)**2 - 1.5995937822965953e19*coen(12)*coep(12)&
            - 8.354296112127312e18*coen(15)*coep(12) - 4.558996165975636e17*coep(12)**2&
            + 1.2113677754320008e18*coen(13)*coep(13) + 3.4893276845731604e16*coep(13)**2&
            + 8.354296112127313e18*coen(14)*coep(14) + 2.585975948758273e17*coep(14)**2&
            - 8.354296112127312e18*coen(12)*coep(15) - 1.9455156675054915e18*coen(15)*coep(15)&
            - 5.1719518975165446e17*coep(12)*coep(15) - 5.769271048930835e16*coep(15)**2)&
            + jx**2*(2.6964443474027375e23*coen(5)**2 - 3.789411790423061e23*coen(8)**2&
            - 1.4008068010196753e20*coen(12)**2 + 9.958632802909792e18*coen(13)**2&
            + 6.771702155232455e19*coen(14)**2 - 1.3543404310464905e20*coen(12)*coen(15)&
            - 1.6424972571267944e19*coen(15)**2 + 1.500847968688216e23*coen(5)*coep(5)&
            + 1.8613822033879852e22*coep(5)**2 - 1.493769851344806e23*coen(8)*coep(8)&
            - 1.218623617141403e22*coep(8)**2 - 1.5995937822965953e19*coen(12)*coep(12)&
            - 8.354296112127312e18*coen(15)*coep(12) - 4.558996165975636e17*coep(12)**2&
            + 1.2113677754320008e18*coen(13)*coep(13) + 3.4893276845731604e16*coep(13)**2&
            + 8.354296112127313e18*coen(14)*coep(14) + 2.585975948758273e17*coep(14)**2&
            - 8.354296112127312e18*coen(12)*coep(15) - 1.9455156675054915e18*coen(15)*coep(15)&
            - 5.1719518975165446e17*coep(12)*coep(15) - 5.769271048930835e16*coep(15)**2))&
            + Q**6*(8.579812845825814e13*coen(3)**2 - 6.621901966393725e14*coen(7)**2&
            + 9.04798824417332e12*coen(3)*coep(3) + 2.296024757607889e11*coep(3)**2&
            - 7.843552839071739e13*coen(7)*coep(7) - 2.3259428372137803e12*coep(7)**2&
            + jx*(2.4858470329175667e17*coen(5)**2 - 1.2673998340634202e18*coen(8)**2&
            - 2.5413745765595288e14*coen(12)**2 + 4.724170469581868e12*coen(13)**2&
            + 2.859937615275271e13*coen(14)**2 - 5.7198752305505414e13*coen(12)*coen(15)&
            - 2.1108406694335596e12*coen(15)**2 + 2.1109044807807862e17*coen(5)*coep(5)&
            + 4.253811618128051e16*coep(5)**2 - 8.962207795534973e17*coen(8)*coep(8)&
            - 1.466501646108023e17*coep(8)**2 - 3.0748173696440992e13*coen(12)*coep(12)&
            - 3.015996081391106e12*coen(15)*coep(12) - 9.32134747233212e11*coep(12)**2&
            + 6.886798505434868e11*coen(13)*coep(13) + 2.5301457279568054e10*coep(13)**2&
            + 3.0159960813911064e12*coen(14)*coep(14) + 7.653415858692964e10*coep(14)**2&
            - 3.015996081391106e12*coen(12)*coep(15) - 1.8344022943738455e11*coen(15)*coep(15)&
            - 1.5306831717385922e11*coep(12)*coep(15) - 3.577090575158571e9*coep(15)**2)&
            + jx**2*(2.4858470329175667e17*coen(5)**2 - 1.2673998340634202e18*coen(8)**2&
            - 2.5413745765595288e14*coen(12)**2 + 4.724170469581868e12*coen(13)**2&
            + 2.859937615275271e13*coen(14)**2 - 5.7198752305505414e13*coen(12)*coen(15)&
            - 2.1108406694335596e12*coen(15)**2 + 2.1109044807807862e17*coen(5)*coep(5)&
            + 4.253811618128051e16*coep(5)**2 - 8.962207795534973e17*coen(8)*coep(8)&
            - 1.466501646108023e17*coep(8)**2 - 3.0748173696440992e13*coen(12)*coep(12)&
            - 3.015996081391106e12*coen(15)*coep(12) - 9.32134747233212e11*coep(12)**2&
            + 6.886798505434868e11*coen(13)*coep(13) + 2.5301457279568054e10*coep(13)**2&
            + 3.0159960813911064e12*coen(14)*coep(14) + 7.653415858692964e10*coep(14)**2&
            - 3.015996081391106e12*coen(12)*coep(15) - 1.8344022943738455e11*coen(15)*coep(15)&
            - 1.5306831717385922e11*coep(12)*coep(15) - 3.577090575158571e9*coep(15)**2))&
            + Q**4*(3.615888782797873e6*coen(3)**2 - 8.510078125863746e7*coen(7)**2&
            + 242332.73291207931*coen(3)*coep(3) + 3502.909303570442*coep(3)**2&
            - 7.395587485802179e6*coen(7)*coep(7) - 144214.2019466547*coep(7)**2&
            + jx*(2.6489997674919468e10*coen(5)**2 - 3.8647156861872797e11*coen(8)**2&
            - 3.381270342887676e7*coen(12)**2 + 285984.47162249224*coen(13)**2&
            + 1.205296260932624e6*coen(14)**2 - 2.410592521865248e6*coen(12)*coen(15)&
            - 28960.961745903507*coen(15)**2 + 2.9419240822994946e10*coen(5)*coep(5)&
            + 7.992370708549715e9*coep(5)**2 - 3.801549298608589e11*coen(8)*coep(8)&
            - 9.032441382689265e10*coep(8)**2 - 3.248223819159364e6*coen(12)*coep(12)&
            - 80777.57763735975*coen(15)*coep(12) - 76375.14618436959*coep(12)**2&
            + 37873.192313426596*coen(13)*coep(13) + 1228.0353692825877*coep(13)**2&
            + 80777.57763735976*coen(14)*coep(14) + 1167.6364345234806*coep(14)**2&
            - 80777.57763735975*coen(12)*coep(15) - 1520.311279822159*coen(15)*coep(15)&
            - 2335.2728690469607*coep(12)*coep(15) - 21.361174228641165*coep(15)**2)&
            + jx**2*(2.6489997674919468e10*coen(5)**2 - 3.8647156861872797e11*coen(8)**2&
            - 3.381270342887676e7*coen(12)**2 + 285984.47162249224*coen(13)**2&
            + 1.205296260932624e6*coen(14)**2 - 2.410592521865248e6*coen(12)*coen(15)&
            - 28960.961745903507*coen(15)**2 + 2.9419240822994946e10*coen(5)*coep(5)&
            + 7.992370708549715e9*coep(5)**2 - 3.801549298608589e11*coen(8)*coep(8)&
            - 9.032441382689265e10*coep(8)**2 - 3.248223819159364e6*coen(12)*coep(12)&
            - 80777.57763735975*coen(15)*coep(12) - 76375.14618436959*coep(12)**2&
            + 37873.192313426596*coen(13)*coep(13) + 1228.0353692825877*coep(13)**2&
            + 80777.57763735976*coen(14)*coep(14) + 1167.6364345234806*coep(14)**2&
            - 80777.57763735975*coen(12)*coep(15) - 1520.311279822159*coen(15)*coep(15)&
            - 2335.2728690469607*coep(12)*coep(15) - 21.361174228641165*coep(15)**2))&
            + Q**2*(0.014688865056828607*coen(3)**2 - 1.1675919013059726*coen(7)**2&
            + 0.0007161765997578048*coen(3)*coep(3) + 8.38714910127668e-6*coep(3)**2&
            - 0.06129296234561536*coen(7)*coep(7) - 0.000861198403926457*coep(7)**2&
            + jx*(282.7714021664736*coen(5)**2 - 13943.128367633177*coen(8)**2&
            - 0.5174916309737947*coen(12)**2 + 0.002519517163434572*coen(13)**2&
            + 0.004896288352276202*coen(14)**2 - 0.009792576704552402*coen(12)*coen(15)&
            - 0.00002501395444257197*coen(15)**2 + 378.5662984564279*coen(5)*coep(5)&
            + 125.97731669220887*coep(5)**2 - 17140.413813244693*coen(8)*coep(8)&
            - 5202.880689485963*coep(8)**2 - 0.03503573110362041*coen(12)*coep(12)&
            - 0.00023872553325260154*coen(15)*coep(12) - 0.0006690954598525829*coep(12)**2&
            + 0.0002277638089487872*coen(13)*coep(13) + 4.50297034284018e-6*coep(13)**2&
            + 0.00023872553325260157*coen(14)*coep(14) + 2.795716367092227e-6*coep(14)**2&
            - 0.00023872553325260154*coen(12)*coep(15) - 1.4611303663821113e-6*coen(15)*coep(15)&
            - 5.591432734184453e-6*coep(12)*coep(15) - 2.1337109576829556e-8*coep(15)**2)&
            + jx**2*(282.7714021664736*coen(5)**2 - 13943.128367633177*coen(8)**2&
            - 0.5174916309737947*coen(12)**2 + 0.002519517163434572*coen(13)**2&
            + 0.004896288352276202*coen(14)**2 - 0.009792576704552402*coen(12)*coen(15)&
            - 0.00002501395444257197*coen(15)**2&
            + 378.5662984564279*coen(5)*coep(5) + 125.97731669220887*coep(5)**2&
            - 17140.413813244693*coen(8)*coep(8) - 5202.880689485963*coep(8)**2&
            - 0.03503573110362041*coen(12)*coep(12) - 0.00023872553325260154*coen(15)*coep(12)&
            - 0.0006690954598525829*coep(12)**2 + 0.0002277638089487872*coen(13)*coep(13)&
            + 4.50297034284018e-6*coep(13)**2 + 0.00023872553325260157*coen(14)*coep(14)&
            + 2.795716367092227e-6*coep(14)**2 - 0.00023872553325260154*coen(12)*coep(15)&
            - 1.4611303663821113e-6*coen(15)*coep(15) - 5.591432734184453e-6*coep(12)*coep(15)&
            - 2.1337109576829556e-8*coep(15)**2)) + Q**3*(-318.5023597489461*coen(3)**2&
            + 13255.419715400956*coen(7)**2 - 16.71984288452681*coen(3)*coep(3)&
            - 0.2349225988599271*coep(3)**2 + 888.363077097809*coen(7)*coep(7)&
            + 12.841250335105958*coep(7)**2 + jx*(-3.8034858604332465e6*coen(5)**2&
            + 9.710919182900137e7*coen(8)**2 + 5466.858415986206*coen(12)**2&
            - 34.99685718375506*coen(13)**2 - 106.16745324964867*coen(14)**2&
            + 212.33490649929732*coen(12)*coen(15) + 1.3356373853458072*coen(15)**2&
            - 4.675659569482782e6*coen(5)*coep(5) - 1.4192713868946617e6*coep(5)**2&
            + 1.0784745003011745e8*coen(8)*coep(8) + 2.929908374585788e7*coep(8)**2&
            + 434.95965991415113*coen(12)*coep(12) + 5.573280961508934*coen(15)*coep(12)&
            + 8.782248892962219*coep(12)**2 - 3.9839650416118015*coen(13)*coep(13)&
            - 0.10421213219852027*coep(13)**2 - 5.573280961508935*coen(14)*coep(14)&
            - 0.07830753295330903*coep(14)**2 + 5.573280961508934*coen(12)*coep(15)&
            + 0.06512090875950148*coen(15)*coep(15) + 0.15661506590661803*coep(12)*coep(15)&
            + 0.0007626314118071983*coep(15)**2) + jx**2*(-3.8034858604332465e6*coen(5)**2&
            + 9.710919182900137e7*coen(8)**2 + 5466.858415986206*coen(12)**2&
            - 34.99685718375506*coen(13)**2 - 106.16745324964867*coen(14)**2&
            + 212.33490649929732*coen(12)*coen(15) + 1.3356373853458072*coen(15)**2&
            - 4.675659569482782e6*coen(5)*coep(5) - 1.4192713868946617e6*coep(5)**2&
            + 1.0784745003011745e8*coen(8)*coep(8) + 2.929908374585788e7*coep(8)**2&
            + 434.95965991415113*coen(12)*coep(12) + 5.573280961508934*coen(15)*coep(12)&
            + 8.782248892962219*coep(12)**2 - 3.9839650416118015*coen(13)*coep(13)&
            - 0.10421213219852027*coep(13)**2 - 5.573280961508935*coen(14)*coep(14)&
            - 0.07830753295330903*coep(14)**2 + 5.573280961508934*coen(12)*coep(15)&
            + 0.06512090875950148*coen(15)*coep(15) + 0.15661506590661803*coep(12)*coep(15)&
            + 0.0007626314118071983*coep(15)**2)) + Q**5*(-2.3214275139316837e10*coen(3)**2&
            + 3.145257699630063e11*coen(7)**2 - 2.0174104182489507e9*coen(3)*coep(3)&
            - 3.9339570253908224e7*coep(3)**2 + 3.3168852517562458e10*coen(7)*coep(7)&
            + 8.416954631965959e8*coep(7)**2 + jx*(-1.0542391262157681e14*coen(5)**2&
            + 9.112820595137321e14*coen(8)**2 + 1.221601723444016e11*coen(12)**2&
            - 1.4855298446858842e9*coen(13)**2 - 7.738091713105611e9*coen(14)**2&
            + 1.547618342621122e10*coen(12)*coen(15) + 3.2878756941893375e8*coen(15)**2&
            - 1.0370082397406852e14*coen(5)*coep(5) - 2.46392073417326e13*coep(5)**2&
            + 7.738325637941496e14*coen(8)*coep(8) + 1.5593969221837144e14*coep(8)**2&
            + 1.358090288090652e10*coen(12)*coep(12) + 6.724701394163166e8*coen(15)*coep(12)&
            + 3.7331729800668186e8*coep(12)**2 - 2.1359882889171064e8*coen(13)*coep(13)&
            - 7.72085669034047e6*coep(13)**2 - 6.724701394163167e8*coen(14)*coep(14)&
            - 1.3113190084636072e7*coep(14)**2 + 6.724701394163166e8*coen(12)*coep(15)&
            + 2.203496706642596e7*coen(15)*coep(15) + 2.622638016927214e7*coep(12)*coep(15)&
            + 318514.5077733094*coep(15)**2) + jx**2*(-1.0542391262157681e14*coen(5)**2&
            + 9.112820595137321e14*coen(8)**2 + 1.221601723444016e11*coen(12)**2&
            - 1.4855298446858842e9*coen(13)**2 - 7.738091713105611e9*coen(14)**2&
            + 1.547618342621122e10*coen(12)*coen(15) + 3.2878756941893375e8*coen(15)**2&
            - 1.0370082397406852e14*coen(5)*coep(5) - 2.46392073417326e13*coep(5)**2&
            + 7.738325637941496e14*coen(8)*coep(8) + 1.5593969221837144e14*coep(8)**2&
            + 1.358090288090652e10*coen(12)*coep(12) + 6.724701394163166e8*coen(15)*coep(12)&
            + 3.7331729800668186e8*coep(12)**2 - 2.1359882889171064e8*coen(13)*coep(13)&
            - 7.72085669034047e6*coep(13)**2 - 6.724701394163167e8*coen(14)*coep(14)&
            - 1.3113190084636072e7*coep(14)**2 + 6.724701394163166e8*coen(12)*coep(15)&
            + 2.203496706642596e7*coen(15)*coep(15) + 2.622638016927214e7*coep(12)*coep(15)&
            + 318514.5077733094*coep(15)**2)) + Q**7*(-1.806360081774746e17*coen(3)**2&
            + 7.447277251639054e17*coen(7)**2 - 2.1396089552056876e16*coen(3)*coep(3)&
            - 6.344839163967558e14*coep(3)**2 + 9.18775781377623e16*coen(7)*coep(7)&
            + 2.8439643999392805e15*coep(7)**2 + jx*(-3.457285353239518e20*coen(5)**2&
            + 9.884845389627046e20*coen(8)**2 + 2.8474974259311965e17*coen(12)**2&
            - 9.113058430802066e15*coen(13)**2 - 6.0212002725824856e16*coen(14)**2&
            + 1.204240054516497e17*coen(12)*coen(15) + 7.801500491576661e15*coen(15)**2&
            - 2.444762016801844e20*coen(5)*coep(5) - 4.000406600445559e19*coep(5)**2&
            + 5.5019307697222785e20*coen(8)*coep(8) + 6.823606542896483e19*coep(8)**2&
            + 3.5066590070267816e16*coen(12)*coep(12) + 7.132029850685624e15*coen(15)*coep(12)&
            + 1.075902750608918e15*coep(12)**2 - 1.255631856570886e15*coen(13)*coep(13)&
            - 4.277837925275843e13*coep(13)**2 - 7.132029850685625e15*coen(14)*coep(14)&
            - 2.114946387989186e14*coep(14)**2 + 7.132029850685624e15*coen(12)*coep(15)&
            + 8.227205651582469e14*coen(15)*coep(15) + 4.229892775978371e14*coep(12)*coep(15)&
            + 2.0877423082561473e13*coep(15)**2) + jx**2*(-3.457285353239518e20*coen(5)**2&
            + 9.884845389627046e20*coen(8)**2 + 2.8474974259311965e17*coen(12)**2&
            - 9.113058430802066e15*coen(13)**2 - 6.0212002725824856e16*coen(14)**2&
            + 1.204240054516497e17*coen(12)*coen(15) + 7.801500491576661e15*coen(15)**2&
            - 2.444762016801844e20*coen(5)*coep(5) - 4.000406600445559e19*coep(5)**2&
            + 5.5019307697222785e20*coen(8)*coep(8) + 6.823606542896483e19*coep(8)**2&
            + 3.5066590070267816e16*coen(12)*coep(12) + 7.132029850685624e15*coen(15)*coep(12)&
            + 1.075902750608918e15*coep(12)**2 - 1.255631856570886e15*coen(13)*coep(13)&
            - 4.277837925275843e13*coep(13)**2 - 7.132029850685625e15*coen(14)*coep(14)&
            - 2.114946387989186e14*coep(14)**2 + 7.132029850685624e15*coen(12)*coep(15)&
            + 8.227205651582469e14*coen(15)*coep(15) + 4.229892775978371e14*coep(12)*coep(15)&
            + 2.0877423082561473e13*coep(15)**2)) + Q**9*(-9.955007680444508e22*coen(3)**2&
            + 2.7340219306863808e22*coen(7)**2 - 1.1676398194085234e22*coen(3)*coep(3)&
            - 3.406146112808927e20*coep(3)**2 + 1.6692184764847767e21*coen(7)*coep(7)&
            - 3.802961729670179e14*coep(7)**2 + jx*(-1.0336973012233498e26*coen(5)**2&
            + 5.083813829375242e25*coen(8)**2 + 9.720144290615138e21*coen(12)**2&
            - 5.028645720073436e21*coen(13)**2 - 3.3183358934815026e22*coen(14)**2&
            + 6.636671786963004e22*coen(12)*coen(15) + 1.8472234293044833e22*coen(15)**2&
            - 4.074790361623755e25*coen(5)*coep(5) - 3.324230814475483e24*coep(5)**2&
            + 1.027265258570672e25*coen(8)*coep(8) - 8.159719685402013e17*coep(8)**2&
            + 5.8503981739133397e20*coen(12)*coep(12) + 3.892132731361743e21*coen(15)*coep(12)&
            - 1.332889724484336e14*coep(12)**2 - 4.713301680584322e20*coen(13)*coep(13)&
            - 1.0824686674750198e19*coep(13)**2 - 3.8921327313617436e21*coen(14)*coep(14)&
            - 1.1353820376029757e20*coep(14)**2 + 3.892132731361743e21*coen(12)*coep(15)&
            + 2.278932410183534e21*coen(15)*coep(15) + 2.270764075205951e20*coep(12)*coep(15)&
            + 7.054172275538002e19*coep(15)**2) + jx**2*(-1.0336973012233498e26*coen(5)**2&
            + 5.083813829375242e25*coen(8)**2 + 9.720144290615138e21*coen(12)**2&
            - 5.028645720073436e21*coen(13)**2 - 3.3183358934815026e22*coen(14)**2&
            + 6.636671786963004e22*coen(12)*coen(15) + 1.8472234293044833e22*coen(15)**2&
            - 4.074790361623755e25*coen(5)*coep(5) - 3.324230814475483e24*coep(5)**2&
            + 1.027265258570672e25*coen(8)*coep(8) - 8.159719685402013e17*coep(8)**2&
            + 5.8503981739133397e20*coen(12)*coep(12) + 3.892132731361743e21*coen(15)*coep(12)&
            - 1.332889724484336e14*coep(12)**2 - 4.713301680584322e20*coen(13)*coep(13)&
            - 1.0824686674750198e19*coep(13)**2 - 3.8921327313617436e21*coen(14)*coep(14)&
            - 1.1353820376029757e20*coep(14)**2 + 3.892132731361743e21*coen(12)*coep(15)&
            + 2.278932410183534e21*coen(15)*coep(15) + 2.270764075205951e20*coep(12)*coep(15)&
            + 7.054172275538002e19*coep(15)**2))) + mx**2*((1.53070689425152e-6&
            - 0.051798365782948275*Q + 696.7265796312969*Q**2 - 4.852465909360666e6*Q**3&
            + 1.9311664285722683e10*Q**4 - 4.553600997307194e13*Q**5 + 6.333092030208232e16*Q**6&
            - 4.9393753947544764e19*Q**7 + 1.8935376953745533e22*Q**8 - 2.5403396766002464e24*Q**9&
            + 1.0817777207813652e26*Q**10)*coen(1)**2 + 1.679737225585191e-11*jx*coen(4)**2&
            + 1.679737225585191e-11*jx**2*coen(4)**2 + 1.6797372255851918e-11*v**2*coen(7)**2&
            + 5.102356314171732e-7*jx*v**2*coen(8)**2 + 5.102356314171732e-7*jx**2*v**2*coen(8)**2&
            + 1.1198248170567939e-11*jx*v**2*coen(12)**2 + 1.1198248170567939e-11*jx**2*v**2*coen(12)**2&
            + 7.93651083634472e-7*coep(1)**2 + coen(1)*(2.2044021278790997e-6*coep(1)&
            + Q**9*(-1.3437746337252342e23*coen(3) - 5.133159439594954e23*coep(1)&
            - 9.87562484516437e21*coep(3)) + Q**10*(9.146455640424528e24*coen(3)&
            - 1.7185433693933365e19*coep(1) - 7.265161969972417e17*coep(3))&
            + Q**7*(-8.642727034420256e17*coen(3) - 2.749269249686679e19*coep(1)&
            - 3.4529640026938054e17*coep(3)) + Q**5*(-4.059264519220922e11*coen(3)&
            - 3.866777248003885e13*coep(1) - 2.5479218952417624e11*coep(3))&
            + Q**3*(-20543.276978353017*coen(3) - 5.389047780401083e6*coep(1)&
            - 17133.729377848744*coep(3)) + Q*(-0.00005972894174333287*coen(3)&
            - 0.069346176629978*coep(1) - 0.00006263624134403193*coep(3))&
            + Q**2*(1.768322995570412*coen(3) + 856.4922859986677*coep(1)&
            + 1.6660424564796952*coep(3)) + Q**4*(1.2185700022026448e8*coen(3)&
            + 1.8996027077163895e10*coep(1) + 8.900562360780627e7*coep(3))&
            + Q**6*(7.833938825938404e14*coen(3) + 4.478341028418698e16*coep(1)&
            + 4.075310570227685e14*coep(3)) + Q**8*(5.093745170837302e20*coen(3)&
            + 7.464244263143673e21*coep(1) + 1.2929456296227575e20*coep(3)))&
            + 9.81178355257532e-13*jx*coen(4)*coep(4) + 9.81178355257532e-13*jx**2*coen(4)*coep(4)&
            + 1.4328297161040827e-14*jx*coep(4)**2 + 1.4328297161040827e-14*jx**2*coep(4)**2&
            + 9.811783552575326e-13*v**2*coen(7)*coep(7) + 1.4328297161040843e-14*v**2*coep(7)**2&
            + 7.348007092930332e-7*jx*v**2*coen(8)*coep(8)&
            + 7.348007092930332e-7*jx**2*v**2*coen(8)*coep(8)&
            + 2.6455036121149064e-7*jx*v**2*coep(8)**2&
            + 2.6455036121149064e-7*jx**2*v**2*coep(8)**2&
            + 6.541189035050211e-13*jx*v**2*coen(12)*coep(12)&
            + 6.541189035050211e-13*jx**2*v**2*coen(12)*coep(12)&
            + 9.552198107360545e-15*jx*v**2*coep(12)**2&
            + 9.552198107360545e-15*jx**2*v**2*coep(12)**2 + jx*Q**13*((-9.445234508170356e23&
            - 9.445234508170356e23*jx)*coen(15)**2 + (4.3037783046393664e17&
            + 4.3037783046393664e17*jx)*coen(15)*coep(15) + (-4.902606621217846e10&
            - 4.902606621217846e10*jx)*coep(15)**2) + Q**9*((-1.140750952423546e21&
            + 1.658144970177582e21*v**2)*coen(3)**2 - 2.4910536779303856e16*coen(7)**2&
            - 4.5538937369462676e20*v**2*coen(7)**2 + 2.5127726291736455e20*coen(10)**2&
            + 4.077344364380505e16*coep(1)**2 + 2.508137883590833e15*coep(1)*coep(3)&
            + 8.611733897839828e13*coep(3)**2 + 5.673410032358566e18*v**2*coep(3)**2&
            + coen(3)*(-2.170049725842853e22*coep(1) + (-4.174917412058482e20&
            + 1.9448664990330293e20*v**2)*coep(3)) - 2.9217993195015765e15*coen(7)*coep(7)&
            - 2.780315505278577e19*v**2*coen(7)*coep(7) - 8.523240839430572e13*coep(7)**2&
            + 6.334361626076493e12*v**2*coep(7)**2 + 2.355197823687104e19*coen(10)*coep(10)&
            + 5.409006303052059e17*coep(10)**2 + jx*(-3.1369897854161e20*coen(4)**2&
            + (1.8643466939794825e19 + 1.7217666080525859e24*v**2)*coen(5)**2&
            - 4.5248292796368085e19*coen(6)**2 - 2.510423735825313e19*coen(8)**2&
            - 8.467798922000819e23*v**2*coen(8)**2 + 5.431844077382938e19*coen(8)*coen(9)&
            + 1.1054299801183878e21*coen(9)**2 + 1.7217666080525859e24*coen(11)**2&
            - 4.4792487790841145e22*coen(11)*coen(12) + 2.362366239596244e20*coen(12)**2&
            - 1.619025206434011e20*v**2*coen(12)**2 + 6.797712396545894e14*coen(13)**2&
            + 8.375908763912151e19*v**2*coen(13)**2 + 4.622329650802085e15*coen(14)**2&
            + 5.527149900591939e20*v**2*coen(14)**2 - 4.631669264625848e22*coen(11)*coen(15)&
            + 7.605006349490305e20*coen(12)*coen(15)&
            - 1.1054299801183877e21*v**2*coen(12)*coen(15) + 2.395835964436545e20*coen(15)**2&
            - 3.076807508759899e20*v**2*coen(15)**2 + 7.850659412290345e18*coen(6)*coep(4)&
            + 4.3315670720534194e12*coep(4)**2 + 1.3183995884846807e12*coep(4)*coep(5)&
            + 1.2705701799626865e18*coep(5)**2 + 5.536968711294582e22*v**2*coep(5)**2&
            + coen(5)*(-3.395559793450838e18*coep(4) + (9.806652258266556e18&
            + 6.787130014904042e23*v**2)*coep(5)) - 5.504000887633566e18*coen(6)*coep(6)&
            + 3.6060042020347046e17*coep(4)*coep(6) - 1.5854196440289178e17*coep(6)**2&
            + coen(4)*(-5.431844077382938e19*coen(5) + 1.67518175278243e20*coen(6)&
            - 1.90123701051072e19*coep(4) - 2.391246949334205e18*coep(5)&
            + 7.850659412290345e18*coep(6)) - 1.019639782172036e19*coen(8)*coep(8)&
            - 1.711053146531651e23*v**2*coen(8)*coep(8) + 2.391246949334205e18*coen(9)*coep(8)&
            - 8.318263192257554e17*coep(8)**2 + 1.3591147881268346e16*v**2*coep(8)**2&
            + 3.395559793450838e18*coen(8)*coep(9) + 1.2965776660220194e20*coen(9)*coep(9)&
            - 1.3183995884846807e12*coep(8)*coep(9) + 3.7822733549057106e18*coep(9)**2&
            + 6.787130014904042e23*coen(11)*coep(11) - 7.233499086142845e21*coen(12)*coep(11)&
            - 1.1345805082744605e22*coen(15)*coep(11) + 5.536968711294582e22*coep(11)**2&
            - 3.291874948388124e21*coen(11)*coep(12) - 1.1294053799116222e15*coen(12)*coep(12)&
            - 9.744651754178613e18*v**2*coen(12)*coep(12)&
            + 1.3916391373528272e20*coen(15)*coep(12)&
            - 6.482888330110095e19*v**2*coen(15)*coep(12) + 8.360459611969444e14*coep(11)*coep(12)&
            - 3.1119475817812793e13*coep(12)**2 + 2.2201131966945894e12*v**2*coep(12)**2&
            + 8.268735183633144e13*coen(13)*coep(13) + 7.850659412290347e18*v**2*coen(13)*coep(13)&
            + 2.3817974340918784e12*coep(13)**2 + 1.8030021010173526e17*v**2*coep(13)**2&
            + 5.702600283568004e14*coen(14)*coep(14) + 6.482888330110097e19*v**2*coen(14)*coep(14)&
            + 1.7651741069223227e13*coep(14)**2 + 1.8911366774528553e18*v**2*coep(14)**2&
            - 1.1756568757780362e22*coen(11)*coep(15) + 1.3916391373528274e20*coen(12)*coep(15)&
            - 6.482888330110095e19*v**2*coen(12)*coep(15)&
            + 1.6327574023812714e20*coen(15)*coep(15)&
            - 3.795878852754205e19*v**2*coen(15)*coep(15)&
            - 2.1305026141782493e21*coep(11)*coep(15) - 5.741155931893219e13*coep(12)*coep(15)&
            - 3.78227335490571e18*v**2*coep(12)*coep(15) + 2.0494250919524917e19*coep(15)**2&
            - 1.1749704925317747e18*v**2*coep(15)**2) + jx**2*(-3.1369897854161e20*coen(4)**2&
            + (1.8643466939794825e19 + 1.7217666080525859e24*v**2)*coen(5)**2&
            - 4.5248292796368085e19*coen(6)**2 - 2.510423735825313e19*coen(8)**2&
            - 8.467798922000819e23*v**2*coen(8)**2&
            + 5.431844077382938e19*coen(8)*coen(9) + 1.1054299801183878e21*coen(9)**2&
            + 1.7217666080525859e24*coen(11)**2 - 4.4792487790841145e22*coen(11)*coen(12)&
            + 2.362366239596244e20*coen(12)**2 - 1.619025206434011e20*v**2*coen(12)**2&
            + 6.797712396545894e14*coen(13)**2 + 8.375908763912151e19*v**2*coen(13)**2&
            + 4.622329650802085e15*coen(14)**2 + 5.527149900591939e20*v**2*coen(14)**2&
            - 4.631669264625848e22*coen(11)*coen(15) + 7.605006349490305e20*coen(12)*coen(15)&
            - 1.1054299801183877e21*v**2*coen(12)*coen(15) + 2.395835964436545e20*coen(15)**2&
            - 3.076807508759899e20*v**2*coen(15)**2 + 7.850659412290345e18*coen(6)*coep(4)&
            + 4.3315670720534194e12*coep(4)**2 + 1.3183995884846807e12*coep(4)*coep(5)&
            + 1.2705701799626865e18*coep(5)**2 + 5.536968711294582e22*v**2*coep(5)**2&
            + coen(5)*(-3.395559793450838e18*coep(4) + (9.806652258266556e18&
            + 6.787130014904042e23*v**2)*coep(5)) - 5.504000887633566e18*coen(6)*coep(6)&
            + 3.6060042020347046e17*coep(4)*coep(6) - 1.5854196440289178e17*coep(6)**2&
            + coen(4)*(-5.431844077382938e19*coen(5) + 1.67518175278243e20*coen(6)&
            - 1.90123701051072e19*coep(4) - 2.391246949334205e18*coep(5)&
            + 7.850659412290345e18*coep(6)) - 1.019639782172036e19*coen(8)*coep(8)&
            - 1.711053146531651e23*v**2*coen(8)*coep(8) + 2.391246949334205e18*coen(9)*coep(8)&
            - 8.318263192257554e17*coep(8)**2 + 1.3591147881268346e16*v**2*coep(8)**2&
            + 3.395559793450838e18*coen(8)*coep(9) + 1.2965776660220194e20*coen(9)*coep(9)&
            - 1.3183995884846807e12*coep(8)*coep(9) + 3.7822733549057106e18*coep(9)**2&
            + 6.787130014904042e23*coen(11)*coep(11) - 7.233499086142845e21*coen(12)*coep(11)&
            - 1.1345805082744605e22*coen(15)*coep(11) + 5.536968711294582e22*coep(11)**2&
            - 3.291874948388124e21*coen(11)*coep(12) - 1.1294053799116222e15*coen(12)*coep(12)&
            - 9.744651754178613e18*v**2*coen(12)*coep(12)&
            + 1.3916391373528272e20*coen(15)*coep(12)&
            - 6.482888330110095e19*v**2*coen(15)*coep(12) + 8.360459611969444e14*coep(11)*coep(12)&
            - 3.1119475817812793e13*coep(12)**2 + 2.2201131966945894e12*v**2*coep(12)**2&
            + 8.268735183633144e13*coen(13)*coep(13) + 7.850659412290347e18*v**2*coen(13)*coep(13)&
            + 2.3817974340918784e12*coep(13)**2 + 1.8030021010173526e17*v**2*coep(13)**2&
            + 5.702600283568004e14*coen(14)*coep(14) + 6.482888330110097e19*v**2*coen(14)*coep(14)&
            + 1.7651741069223227e13*coep(14)**2 + 1.8911366774528553e18*v**2*coep(14)**2&
            - 1.1756568757780362e22*coen(11)*coep(15) + 1.3916391373528274e20*coen(12)*coep(15)&
            - 6.482888330110095e19*v**2*coen(12)*coep(15)&
            + 1.6327574023812714e20*coen(15)*coep(15)&
            - 3.795878852754205e19*v**2*coen(15)*coep(15)&
            - 2.1305026141782493e21*coep(11)*coep(15) - 5.741155931893219e13*coep(12)*coep(15)&
            - 3.78227335490571e18*v**2*coep(12)*coep(15) + 2.0494250919524917e19*coep(15)**2&
            - 1.1749704925317747e18*v**2*coep(15)**2)) + Q**7*((-3.0782086472810935e15&
            + 3.0087439207185215e15*v**2)*coen(3)**2 - 4.5200767993487785e10*coen(7)**2&
            - 1.240447593082302e16*v**2*coen(7)**2 + 4.553719841819589e14*coen(10)**2&
            - 3.4096996900768123e18*coep(1)**2 - 9.724824200148963e16*coep(1)*coep(3)&
            - 6.08899395208321e14*coep(3)**2 + 1.0568211983387213e13*v**2*coep(3)**2&
            + coen(3)*(-2.7820059137165734e17*coep(1) + (-2.907223283637582e15&
            + 3.563816263236482e14*v**2)*coep(3)) - 5.353969507896883e9*coen(7)*coep(7)&
            - 1.5303488349938248e15*v**2*coen(7)*coep(7) - 1.58767681981059e8*coep(7)**2&
            - 4.7370182088225766e13*v**2*coep(7)**2 + 6.2742884210656516e13*coen(10)*coep(10)&
            + 2.1376001907959297e12*coep(10)**2 + jx*(-8.877728265212693e15*coen(4)**2&
            + (1.6825713395092525e13 + 5.758589549060756e18*v**2)*coen(5)**2&
            - 2.1464858967903484e13*coen(6)**2 - 8.683632812815472e13*coen(8)**2&
            - 1.6464584649181587e19*v**2*coen(8)**2 - 3.3995997527437824e13*coen(8)*coen(9)&
            + 2.005829280479014e15*coen(9)**2 + 5.758589549060756e18*coen(11)**2&
            - 2.8809090114734186e17*coen(11)*coen(12) + 3.219686685159497e15*coen(12)**2&
            - 4.742902954938353e15*v**2*coen(12)**2 + 3.2246948752934754e8*coen(13)**2&
            + 1.5179066139398628e14*v**2*coen(13)**2 + 1.9521789552300973e9*coen(14)**2&
            + 1.002914640239507e15*v**2*coen(14)**2 - 7.123287966738561e16*coen(11)*coen(15)&
            + 2.0521390981873952e15*coen(12)*coen(15) - 2.0058292804790138e15*v**2*coen(12)*coen(15)&
            + 1.8075501209541475e14*coen(15)**2 - 1.2994483997593617e14*v**2*coen(15)**2&
            + 2.09142947368855e13*coen(6)*coep(4) - 3.3710717229840418e13*coep(4)**2&
            - 7.832624889155275e12*coep(4)*coep(5) + 2.418749543684889e12*coep(5)**2&
            + 6.663233516358086e17*v**2*coep(5)**2 + coen(5)*(2.5269055454688433e12*coep(4)&
            + (1.469629437264373e13 + 4.0720911239519713e18*v**2)*coep(5))&
            - 3.129102973979085e12*coen(6)*coep(6) + 1.4250667938639526e12*coep(4)*coep(6)&
            - 1.1496033339297171e11*coep(6)**2 + coen(4)*(3.3995997527437824e13*coen(5)&
            + 3.035813227879725e14*coen(6) - 1.0941991018986939e15*coep(4) - 1.2653997080860997e14*coep(5)&
            + 2.09142947368855e13*coep(6)) - 6.040645568988054e13*coen(8)*coep(8)&
            - 9.164230832288931e18*v**2*coen(8)*coep(8) + 1.2653997080860997e14*coen(9)*coep(8)&
            - 9.164297413642045e12*coep(8)**2 - 1.1365665633589373e18*v**2*coep(8)**2&
            - 2.5269055454688433e12*coen(8)*coep(9) + 2.375877508824321e14*coen(9)*coep(9)&
            + 7.832624889155275e12*coep(8)*coep(9) + 7.045474655591475e12*coep(9)**2&
            + 4.0720911239519713e18*coen(11)*coep(11) - 9.273353045721912e16*coen(12)*coep(11)&
            - 2.8224988593919784e16*coen(15)*coep(11) + 6.663233516358086e17*coep(11)**2&
            - 1.150988000897935e17*coen(11)*coep(12) + 2.1942112553207018e15*coen(12)*coep(12)&
            - 5.84082823567419e14*v**2*coen(12)*coep(12) + 9.690744278791938e14*coen(15)*coep(12)&
            - 1.1879387544121603e14*v**2*coen(15)*coep(12)&
            - 3.2416080667163212e16*coep(11)*coep(12) + 2.7541581292407494e14*coep(12)**2&
            - 1.7920656533765164e13*v**2*coep(12)**2 + 4.700893837478346e7*coen(13)*coep(13)&
            + 2.09142947368855e13*v**2*coen(13)*coep(13) + 1.7270646805025497e6*coep(13)**2&
            + 7.125333969319764e11*v**2*coep(13)**2 + 2.0587036751084712e8*coen(14)*coep(14)&
            + 1.1879387544121605e14*v**2*coen(14)*coep(14) + 5.224182966496848e6*coep(14)**2&
            + 3.5227373277957373e12*v**2*coep(14)**2 - 3.705621321615051e16*coen(11)*coep(15)&
            + 9.690744278791939e14*coen(12)*coep(15)&
            - 1.1879387544121603e14*v**2*coen(12)*coep(15)&
            + 2.1252476443367344e14*coen(15)*coep(15)&
            - 1.3703555142992025e13*v**2*coen(15)*coep(15)&
            - 1.3626053990275848e16*coep(11)*coep(15) + 4.059329301388807e14*coep(12)*coep(15)&
            - 7.045474655591474e12*v**2*coep(12)*coep(15) + 5.854689351684096e13*coep(15)**2&
            - 3.477425149818961e11*v**2*coep(15)**2) + jx**2*(-8.877728265212693e15*coen(4)**2&
            + (1.6825713395092525e13 + 5.758589549060756e18*v**2)*coen(5)**2&
            - 2.1464858967903484e13*coen(6)**2 - 8.683632812815472e13*coen(8)**2&
            - 1.6464584649181587e19*v**2*coen(8)**2 - 3.3995997527437824e13*coen(8)*coen(9)&
            + 2.005829280479014e15*coen(9)**2 + 5.758589549060756e18*coen(11)**2&
            - 2.8809090114734186e17*coen(11)*coen(12) + 3.219686685159497e15*coen(12)**2&
            - 4.742902954938353e15*v**2*coen(12)**2 + 3.2246948752934754e8*coen(13)**2&
            + 1.5179066139398628e14*v**2*coen(13)**2 + 1.9521789552300973e9*coen(14)**2&
            + 1.002914640239507e15*v**2*coen(14)**2 - 7.123287966738561e16*coen(11)*coen(15)&
            + 2.0521390981873952e15*coen(12)*coen(15)&
            - 2.0058292804790138e15*v**2*coen(12)*coen(15) + 1.8075501209541475e14*coen(15)**2&
            - 1.2994483997593617e14*v**2*coen(15)**2 + 2.09142947368855e13*coen(6)*coep(4)&
            - 3.3710717229840418e13*coep(4)**2 - 7.832624889155275e12*coep(4)*coep(5)&
            + 2.418749543684889e12*coep(5)**2 + 6.663233516358086e17*v**2*coep(5)**2&
            + coen(5)*(2.5269055454688433e12*coep(4) + (1.469629437264373e13&
            + 4.0720911239519713e18*v**2)*coep(5)) - 3.129102973979085e12*coen(6)*coep(6)&
            + 1.4250667938639526e12*coep(4)*coep(6) - 1.1496033339297171e11*coep(6)**2&
            + coen(4)*(3.3995997527437824e13*coen(5) + 3.035813227879725e14*coen(6)&
            - 1.0941991018986939e15*coep(4) - 1.2653997080860997e14*coep(5)&
            + 2.09142947368855e13*coep(6)) - 6.040645568988054e13*coen(8)*coep(8)&
            - 9.164230832288931e18*v**2*coen(8)*coep(8) + 1.2653997080860997e14*coen(9)*coep(8)&
            - 9.164297413642045e12*coep(8)**2 - 1.1365665633589373e18*v**2*coep(8)**2&
            - 2.5269055454688433e12*coen(8)*coep(9) + 2.375877508824321e14*coen(9)*coep(9)&
            + 7.832624889155275e12*coep(8)*coep(9) + 7.045474655591475e12*coep(9)**2&
            + 4.0720911239519713e18*coen(11)*coep(11) - 9.273353045721912e16*coen(12)*coep(11)&
            - 2.8224988593919784e16*coen(15)*coep(11) + 6.663233516358086e17*coep(11)**2&
            - 1.150988000897935e17*coen(11)*coep(12) + 2.1942112553207018e15*coen(12)*coep(12)&
            - 5.84082823567419e14*v**2*coen(12)*coep(12) + 9.690744278791938e14*coen(15)*coep(12)&
            - 1.1879387544121603e14*v**2*coen(15)*coep(12)&
            - 3.2416080667163212e16*coep(11)*coep(12) + 2.7541581292407494e14*coep(12)**2&
            - 1.7920656533765164e13*v**2*coep(12)**2 + 4.700893837478346e7*coen(13)*coep(13)&
            + 2.09142947368855e13*v**2*coen(13)*coep(13) + 1.7270646805025497e6*coep(13)**2&
            + 7.125333969319764e11*v**2*coep(13)**2 + 2.0587036751084712e8*coen(14)*coep(14)&
            + 1.1879387544121605e14*v**2*coen(14)*coep(14) + 5.224182966496848e6*coep(14)**2&
            + 3.5227373277957373e12*v**2*coep(14)**2 - 3.705621321615051e16*coen(11)*coep(15)&
            + 9.690744278791939e14*coen(12)*coep(15)&
            - 1.1879387544121603e14*v**2*coen(12)*coep(15)&
            + 2.1252476443367344e14*coen(15)*coep(15)&
            - 1.3703555142992025e13*v**2*coen(15)*coep(15)&
            - 1.3626053990275848e16*coep(11)*coep(15) + 4.059329301388807e14*coep(12)*coep(15)&
            - 7.045474655591474e12*v**2*coep(12)*coep(15) + 5.854689351684096e13*coep(15)**2&
            - 3.477425149818961e11*v**2*coep(15)**2)) + Q**5*((-7.257006841754656e8&
            + 3.86666035770029e8*v**2)*coen(3)**2 - 5808.936298450048*coen(7)**2&
            - 5.238864099320342e9*v**2*coen(7)**2 + 7.423069632140946e7*coen(10)**2&
            - 7.792177302209391e12*coep(1)**2 - 1.137958717671074e11*coep(1)*coep(3)&
            - 3.382720664477291e8*coep(3)**2 + 655255.2508181778*v**2*coep(3)**2&
            + coen(3)*(-1.8998370101994055e11*coep(1) + (-1.0118665160906883e9&
            + 3.3602776061886296e7*v**2)*coep(3)) - 504.8190622842121*coen(7)*coep(7)&
            - 5.524733655062578e8*v**2*coen(7)*coep(7) - 9.843988504569705*coep(7)**2&
            - 1.4019608457583904e7*v**2*coep(7)**2 + 1.067335662005629e7*coen(10)*coep(10)&
            + 385804.8159530401*coep(10)**2 + jx*(-3.781035587614457e9*coen(4)**2&
            + (1.827273487349365e6 + 1.7559818742611257e12*v**2)*coen(5)**2&
            - 1.2994061899147588e6*coen(6)**2 - 2.6389798325709715e7*coen(8)**2&
            - 1.517866999102398e13*v**2*coen(8)**2 - 1.8740956839656262e6*coen(8)*coen(9)&
            + 2.577773571800193e8*coen(9)**2 + 1.7559818742611257e12*coen(11)**2&
            - 1.3530881730736407e11*coen(11)*coen(12) + 2.4291096614600477e9*coen(12)**2&
            - 2.0347475545076766e9*v**2*coen(12)**2 + 19.52115542808063*coen(13)**2&
            + 2.474356544046982e7*v**2*coen(13)**2 + 82.27291332659784*coen(14)**2&
            + 1.2888867859000964e8*v**2*coen(14)**2 - 1.1080281868653606e10*coen(11)*coen(15)&
            + 4.8380045611697704e8*coen(12)*coen(15) - 2.5777735718001926e8*v**2*coen(12)*coen(15)&
            + 1.3279153877851443e7*coen(15)**2 - 5.476414202671658e6*v**2*coen(15)**2&
            + 3.5577855400187615e6*coen(6)*coep(4) - 1.0891321577516124e7*coep(4)**2&
            - 2.5414741322700004e6*coep(4)*coep(5) + 374774.28477669245*coep(5)**2&
            + 4.104002632073529e11*v**2*coep(5)**2 + coen(5)*(-94004.12946695184*coep(4)&
            + (2.0121210065380377e6 + 1.7272814365944778e12*v**2)*coep(5))&
            - 172081.5827681047*coen(6)*coep(6) + 257203.21063535998*coep(4)*coep(6)&
            - 5579.732183453805*coep(6)**2 + coen(4)*(1.8740956839656262e6*coen(5)&
            + 4.948713088093963e7*coen(6) - 4.103666124960807e8*coep(4)&
            - 4.901400005162571e7*coep(5) + 3.5577855400187615e6*coep(6))&
            - 2.66195992465369e7*coen(8)*coep(8) - 1.2889257493346283e13*v**2*coen(8)*coep(8)&
            + 4.901400005162571e7*coen(9)*coep(8) - 4.6892243196876515e6*coep(8)**2&
            - 2.5973924340697964e12*v**2*coep(8)**2 + 94004.12946695184*coen(8)*coep(9)&
            + 2.2401850707924195e7*coen(9)*coep(9) + 2.5414741322700004e6*coep(8)*coep(9)&
            + 436836.8338787852*coep(9)**2 + 1.7272814365944778e12*coen(11)*coep(11)&
            - 6.3327900339980194e10*coen(12)*coep(11) - 5.946380723405121e9*coen(15)*coep(11)&
            + 4.104002632073529e11*coep(11)**2 - 8.493072984139209e10*coen(11)*coep(12)&
            + 2.8560537378217945e9*coen(12)*coep(12) - 2.262088239939948e8*v**2*coen(12)*coep(12)&
            + 3.37288838696896e8*coen(15)*coep(12) - 1.1200925353962095e7*v**2*coen(15)*coep(12)&
            - 3.793195725570247e10*coep(11)*coep(12) + 7.8679337549622e8*coep(12)**2&
            - 6.218118758321489e6*v**2*coep(12)**2 + 2.5852049571555904*coen(13)*coep(13)&
            + 3.557785540018762e6*v**2*coen(13)*coep(13) + 0.08382507336478885*coep(13)**2&
            + 128601.60531768002*v**2*coep(13)**2 + 5.5138366052415115*coen(14)*coep(14)&
            + 1.1200925353962097e7*v**2*coen(14)*coep(14) + 0.0797022726181825*coep(14)**2&
            + 218418.4169393926*v**2*coep(14)**2 - 8.093153414963021e9*coen(11)*coep(15)&
            + 3.3728883869689614e8*coen(12)*coep(15) - 1.1200925353962095e7*v**2*coen(12)*coep(15)&
            + 2.1443107528298095e7*coen(15)*coep(15) - 367023.01979129587*v**2*coen(15)*coep(15)&
            - 4.2119656817909837e9*coep(11)*coep(15) + 2.255147109651527e8*coep(12)*coep(15)&
            - 436836.8338787851*v**2*coep(12)*coep(15) + 8.410368571110846e6*coep(15)**2&
            - 5305.302074556699*v**2*coep(15)**2) + jx**2*(-3.781035587614457e9*coen(4)**2&
            + (1.827273487349365e6 + 1.7559818742611257e12*v**2)*coen(5)**2&
            - 1.2994061899147588e6*coen(6)**2 - 2.6389798325709715e7*coen(8)**2&
            - 1.517866999102398e13*v**2*coen(8)**2 - 1.8740956839656262e6*coen(8)*coen(9)&
            + 2.577773571800193e8*coen(9)**2 + 1.7559818742611257e12*coen(11)**2&
            - 1.3530881730736407e11*coen(11)*coen(12) + 2.4291096614600477e9*coen(12)**2&
            - 2.0347475545076766e9*v**2*coen(12)**2 + 19.52115542808063*coen(13)**2&
            + 2.474356544046982e7*v**2*coen(13)**2 + 82.27291332659784*coen(14)**2&
            + 1.2888867859000964e8*v**2*coen(14)**2 - 1.1080281868653606e10*coen(11)*coen(15)&
            + 4.8380045611697704e8*coen(12)*coen(15) - 2.5777735718001926e8*v**2*coen(12)*coen(15)&
            + 1.3279153877851443e7*coen(15)**2 - 5.476414202671658e6*v**2*coen(15)**2&
            + 3.5577855400187615e6*coen(6)*coep(4) - 1.0891321577516124e7*coep(4)**2&
            - 2.5414741322700004e6*coep(4)*coep(5) + 374774.28477669245*coep(5)**2&
            + 4.104002632073529e11*v**2*coep(5)**2 + coen(5)*(-94004.12946695184*coep(4)&
            + (2.0121210065380377e6 + 1.7272814365944778e12*v**2)*coep(5))&
            - 172081.5827681047*coen(6)*coep(6) + 257203.21063535998*coep(4)*coep(6)&
            - 5579.732183453805*coep(6)**2 + coen(4)*(1.8740956839656262e6*coen(5)&
            + 4.948713088093963e7*coen(6) - 4.103666124960807e8*coep(4)&
            - 4.901400005162571e7*coep(5) + 3.5577855400187615e6*coep(6))&
            - 2.66195992465369e7*coen(8)*coep(8) - 1.2889257493346283e13*v**2*coen(8)*coep(8)&
            + 4.901400005162571e7*coen(9)*coep(8) - 4.6892243196876515e6*coep(8)**2&
            - 2.5973924340697964e12*v**2*coep(8)**2 + 94004.12946695184*coen(8)*coep(9)&
            + 2.2401850707924195e7*coen(9)*coep(9) + 2.5414741322700004e6*coep(8)*coep(9)&
            + 436836.8338787852*coep(9)**2 + 1.7272814365944778e12*coen(11)*coep(11)&
            - 6.3327900339980194e10*coen(12)*coep(11) - 5.946380723405121e9*coen(15)*coep(11)&
            + 4.104002632073529e11*coep(11)**2 - 8.493072984139209e10*coen(11)*coep(12)&
            + 2.8560537378217945e9*coen(12)*coep(12) - 2.262088239939948e8*v**2*coen(12)*coep(12)&
            + 3.37288838696896e8*coen(15)*coep(12) - 1.1200925353962095e7*v**2*coen(15)*coep(12)&
            - 3.793195725570247e10*coep(11)*coep(12) + 7.8679337549622e8*coep(12)**2&
            - 6.218118758321489e6*v**2*coep(12)**2 + 2.5852049571555904*coen(13)*coep(13)&
            + 3.557785540018762e6*v**2*coen(13)*coep(13) + 0.08382507336478885*coep(13)**2&
            + 128601.60531768002*v**2*coep(13)**2 + 5.5138366052415115*coen(14)*coep(14)&
            + 1.1200925353962097e7*v**2*coen(14)*coep(14) + 0.0797022726181825*coep(14)**2&
            + 218418.4169393926*v**2*coep(14)**2 - 8.093153414963021e9*coen(11)*coep(15)&
            + 3.3728883869689614e8*coen(12)*coep(15) - 1.1200925353962095e7*v**2*coen(12)*coep(15)&
            + 2.1443107528298095e7*coen(15)*coep(15) - 367023.01979129587*v**2*coen(15)*coep(15)&
            - 4.2119656817909837e9*coep(11)*coep(15) + 2.255147109651527e8*coep(12)*coep(15)&
            - 436836.8338787851*v**2*coep(12)*coep(15) + 8.410368571110846e6*coep(15)**2&
            - 5305.302074556699*v**2*coep(15)**2)) + Q**3*((-14.783382841050953&
            + 5.305099732317073*v**2)*coen(3)**2 - 0.00007969923280327314*coen(7)**2&
            - 220.7874492338255*v**2*coen(7)**2 + 1.7487639761019902*coen(10)**2&
            - 1.4640509551621976e6*coep(1)**2 - 10109.933282440223*coep(1)*coep(3)&
            - 12.403874069091797*coep(3)**2 + 0.003912962583101095*v**2*coep(3)**2&
            + coen(3)*(-12360.401760116809*coep(1)&
            + (-27.331103999021416 + 0.27849223497434444*v**2)*coep(3))&
            - 4.183826617606281e-6*coen(7)*coep(7) - 14.796922466215896*v**2*coen(7)*coep(7)&
            - 5.878496759661522e-8*coep(7)**2 - 0.2138888822333523*v**2*coep(7)**2&
            + 0.1990754344094176*coen(10)*coep(10) + 0.00520739395839648*coep(10)**2&
            + jx*(-164.65394590872774*coen(4)**2 + (0.019506314771713877&
            + 63352.34638751512*v**2)*coen(5)**2 - 0.011447741128004214*coen(6)**2&
            - 0.9370884732539911*coen(8)**2 - 1.617488636453555e6*v**2*coen(8)**2&
            + 0.21049228357761307*coen(8)*coen(9) + 3.536733154878048*coen(9)**2&
            + 63352.34638751512*coen(11)**2 - 6847.758992784339*coen(11)*coen(12)&
            + 178.4543839755624*coen(12)**2 - 91.05812949745255*v**2*coen(12)**2&
            + 1.719809676101829e-7*coen(13)**2 + 0.58292132536733*v**2*coen(13)**2&
            + 3.342181671725676e-7*coen(14)**2 + 1.768366577439024*v**2*coen(14)**2&
            - 160.79106813991407*coen(11)*coen(15) + 9.855588560700635*coen(12)*coen(15)&
            - 3.5367331548780476*v**2*coen(12)*coen(15) + 0.05298072809366238*coen(15)**2&
            - 0.02224689807967479*v**2*coen(15)**2 + 0.06635847813647251*coen(6)*coep(4)&
            - 0.21757686257316383*coep(4)**2 - 0.053672297695249636*coep(4)*coep(5)&
            + 0.003695595887633939*coep(5)**2 + 23639.93868251102*v**2*coep(5)**2&
            + coen(5)*(-0.0016838317857852403*coep(4) + (0.02452554413693266&
            + 77879.61241486829*v**2)*coep(5)) + coen(4)*(-0.21049228357761307*coen(5)&
            + 1.16584265073466*coen(6) - 12.177165508623125*coep(4) - 1.7267575786065361*coep(5)&
            + 0.06635847813647251*coep(6)) - 0.0010348733324839028*coen(6)*coep(6)&
            + 0.003471595972264319*coep(4)*coep(6) - 0.00002045980854587395*coep(6)**2&
            - 1.1306583988797647*coen(8)*coep(8) - 1.7963492601336942e6*v**2*coen(8)*coep(8)&
            + 1.7267575786065361*coen(9)*coep(8) - 0.20926103998474352*coep(8)**2&
            - 488016.9850540659*v**2*coep(8)**2 + 0.0016838317857852403*coen(8)*coep(9)&
            + 0.18566148998289628*coen(9)*coep(9) + 0.053672297695249636*coep(8)*coep(9)&
            + 0.0026086417220673964*coep(9)**2 + 77879.61241486829*coen(11)*coep(11)&
            - 4120.133920038937*coen(12)*coep(11) - 106.46606622436211*coen(15)*coep(11)&
            + 23639.93868251102*coep(11)**2 - 5711.2431259495825*coen(11)*coep(12)&
            + 288.16720961509253*coen(12)*coep(12) - 7.244858019884494*v**2*coen(12)*coep(12)&
            + 9.110367999673803*coen(15)*coep(12) - 0.09283074499144812*v**2*coen(15)*coep(12)&
            - 3369.9777608134077*coep(11)*coep(12) + 113.02431084665966*coep(12)**2&
            - 0.1462805684953797*v**2*coep(12)**2 + 1.5547042432604754e-8*coen(13)*coep(13)&
            + 0.06635847813647253*v**2*coen(13)*coep(13) + 3.073704787253463e-10*coep(13)**2&
            + 0.00173579798613216*v**2*coep(13)**2 + 1.6295284190909016e-8*coen(14)*coep(14)&
            + 0.09283074499144814*v**2*coen(14)*coep(14) + 1.9083418559479573e-10*coep(14)**2&
            + 0.0013043208610336982*v**2*coep(14)**2 - 151.49084574190263*coen(11)*coep(15)&
            + 9.110367999673803*coen(12)*coep(15) - 0.09283074499144812*v**2*coen(12)*coep(15)&
            + 0.1111191217713696*coen(15)*coep(15) - 0.0010846792968836677*v**2*coen(15)*coep(15)&
            - 99.31602864574891*coep(11)*coep(15)&
            + 8.269249379394532*coep(12)*coep(15) - 0.002608641722067396*v**2*coep(12)*coep(15)&
            + 0.058263916387388454*coep(15)**2 - 0.000012702686729931984*v**2*coep(15)**2)&
            + jx**2*(-164.65394590872774*coen(4)**2 + (0.019506314771713877&
            + 63352.34638751512*v**2)*coen(5)**2 - 0.011447741128004214*coen(6)**2&
            - 0.9370884732539911*coen(8)**2 - 1.617488636453555e6*v**2*coen(8)**2&
            + 0.21049228357761307*coen(8)*coen(9) + 3.536733154878048*coen(9)**2&
            + 63352.34638751512*coen(11)**2 - 6847.758992784339*coen(11)*coen(12)&
            + 178.4543839755624*coen(12)**2 - 91.05812949745255*v**2*coen(12)**2&
            + 1.719809676101829e-7*coen(13)**2 + 0.58292132536733*v**2*coen(13)**2&
            + 3.342181671725676e-7*coen(14)**2 + 1.768366577439024*v**2*coen(14)**2&
            - 160.79106813991407*coen(11)*coen(15) + 9.855588560700635*coen(12)*coen(15)&
            - 3.5367331548780476*v**2*coen(12)*coen(15) + 0.05298072809366238*coen(15)**2&
            - 0.02224689807967479*v**2*coen(15)**2 + 0.06635847813647251*coen(6)*coep(4)&
            - 0.21757686257316383*coep(4)**2 - 0.053672297695249636*coep(4)*coep(5)&
            + 0.003695595887633939*coep(5)**2 + 23639.93868251102*v**2*coep(5)**2&
            + coen(5)*(-0.0016838317857852403*coep(4) + (0.02452554413693266&
            + 77879.61241486829*v**2)*coep(5)) + coen(4)*(-0.21049228357761307*coen(5)&
            + 1.16584265073466*coen(6) - 12.177165508623125*coep(4) - 1.7267575786065361*coep(5)&
            + 0.06635847813647251*coep(6)) - 0.0010348733324839028*coen(6)*coep(6)&
            + 0.003471595972264319*coep(4)*coep(6) - 0.00002045980854587395*coep(6)**2&
            - 1.1306583988797647*coen(8)*coep(8) - 1.7963492601336942e6*v**2*coen(8)*coep(8)&
            + 1.7267575786065361*coen(9)*coep(8) - 0.20926103998474352*coep(8)**2&
            - 488016.9850540659*v**2*coep(8)**2 + 0.0016838317857852403*coen(8)*coep(9)&
            + 0.18566148998289628*coen(9)*coep(9) + 0.053672297695249636*coep(8)*coep(9)&
            + 0.0026086417220673964*coep(9)**2 + 77879.61241486829*coen(11)*coep(11)&
            - 4120.133920038937*coen(12)*coep(11) - 106.46606622436211*coen(15)*coep(11)&
            + 23639.93868251102*coep(11)**2 - 5711.2431259495825*coen(11)*coep(12)&
            + 288.16720961509253*coen(12)*coep(12) - 7.244858019884494*v**2*coen(12)*coep(12)&
            + 9.110367999673803*coen(15)*coep(12) - 0.09283074499144812*v**2*coen(15)*coep(12)&
            - 3369.9777608134077*coep(11)*coep(12) + 113.02431084665966*coep(12)**2&
            - 0.1462805684953797*v**2*coep(12)**2 + 1.5547042432604754e-8*coen(13)*coep(13)&
            + 0.06635847813647253*v**2*coen(13)*coep(13) + 3.073704787253463e-10*coep(13)**2&
            + 0.00173579798613216*v**2*coep(13)**2 + 1.6295284190909016e-8*coen(14)*coep(14)&
            + 0.09283074499144814*v**2*coen(14)*coep(14) + 1.9083418559479573e-10*coep(14)**2&
            + 0.0013043208610336982*v**2*coep(14)**2 - 151.49084574190263*coen(11)*coep(15)&
            + 9.110367999673803*coen(12)*coep(15) - 0.09283074499144812*v**2*coen(12)*coep(15)&
            + 0.1111191217713696*coen(15)*coep(15) - 0.0010846792968836677*v**2*coen(15)*coep(15)&
            - 99.31602864574891*coep(11)*coep(15) + 8.269249379394532*coep(12)*coep(15)&
            - 0.002608641722067396*v**2*coep(12)*coep(15) + 0.058263916387388454*coep(15)**2&
            - 0.000012702686729931984*v**2*coep(15)**2)) + Q**2*((0.0005826632094300399&
            - 0.0002446634748403333*v**2)*coen(3)**2 + 3.6756125659565724e-9*coen(7)**2&
            + 0.01944786684769419*v**2*coen(7)**2 - 0.0001258981864985939*coen(10)**2&
            + 259.98364007249074*coep(1)**2 + 1.0922423696457289*coep(1)*coep(3)&
            + 0.0006407658357624605*coep(3)**2 - 1.3969963201264332e-7*v**2*coep(3)**2&
            + coen(3)*(1.1708759406252582*coep(1) + (0.0012220485910630304&
            - 0.00001192891723207854*v**2)*coep(3)) + 1.7920974148306205e-10*coen(7)*coep(7)&
            + 0.0010209195259619124*v**2*coen(7)*coep(7) + 2.098726519028954e-12*coep(7)**2&
            + 0.000014344457057534424*v**2*coep(7)**2 - 0.000011381168944916047*coen(10)*coep(10)&
            - 2.25009699575818e-7*coep(10)**2 + jx*(0.01510216505493029*coen(4)**2&
            + (-5.67317292927238e-7 - 4.709950943919957*v**2)*coen(5)**2&
            + 4.1664219732023206e-7*coen(6)**2 + 0.00007150783802584855*coen(8)**2&
            + 232.24219321043225*v**2*coen(8)**2 - 2.4271707944589406e-6*coen(8)*coen(9)&
            - 0.00016310898322688882*coen(9)**2 - 4.709950943919957*coen(11)**2&
            + 0.5894409985234708*coen(11)*coen(12) - 0.018064709149315423*coen(12)**2&
            + 0.008619542772365561*v**2*coen(12)**2 - 6.259272239051621e-12*coen(13)**2&
            - 0.00004196606216619796*v**2*coen(13)**2 - 6.259272239051626e-12*coen(14)**2&
            - 0.00008155449161344441*v**2*coen(14)**2 + 0.005431066816319519*coen(11)*coen(15)&
            - 0.0003884421396200266*coen(12)*coen(15)&
            + 0.0001631089832268888*v**2*coen(12)*coen(15) + 4.166421973202324e-7*v**2*coen(15)**2&
            - 3.793722981638681e-6*coen(6)*coep(4) + 0.000015926201058617647*coep(4)**2&
            + 2.7103722768573817e-6*coep(4)*coep(5) - 4.2431113347148636e-8*coep(5)**2&
            - 2.0983274019968134*v**2*coep(5)**2 + coen(5)*(1.6658411439011793e-7*coep(4)&
            + (-8.772643464838825e-7 - 6.305548160423342*v**2)*coep(5))&
            + coen(4)*(2.4271707944589406e-6*coen(5) - 0.00008393212433239591*coen(6)&
            + 0.0009238753332173328*coep(4) + 0.0001225337750099137*coep(5)&
            - 3.793722981638681e-6*coep(6)) + 2.4337158197772885e-8*coen(6)*coep(6)&
            - 1.500064663838786e-7*coep(4)*coep(6) + 3.5539923281473496e-10*coep(6)**2&
            + 0.00008990773798968537*coen(8)*coep(8) + 285.49742866622256*v**2*coen(8)*coep(8)&
            - 0.0001225337750099137*coen(9)*coep(8) + 0.000013547616514685248*coep(8)**2&
            + 86.6612133574969*v**2*coep(8)**2 - 1.6658411439011793e-7*coen(8)*coep(9)&
            - 7.95261148805236e-6*coen(9)*coep(9) - 2.7103722768573817e-6*coep(8)*coep(9)&
            - 9.313308800842884e-8*coep(9)**2 - 6.305548160423342*coen(11)*coep(11)&
            + 0.3902919802084195*coen(12)*coep(11) + 0.003910694885973735*coen(15)*coep(11)&
            - 2.0983274019968134*coep(11)**2 + 0.5553474854932317*coen(11)*coep(12)&
            - 0.033397529250860834*coen(12)*coep(12) + 0.0005835688245633622*v**2*coen(12)*coep(12)&
            - 0.0004073495303543434*coen(15)*coep(12) + 3.976305744026179e-6*v**2*coen(15)*coep(12)&
            + 0.3640807898819096*coep(11)*coep(12) - 0.015157044067355106*coep(12)**2&
            + 0.000011144715372772842*v**2*coep(12)**2 - 3.6562042842636996e-13*coen(13)*coep(13)&
            - 3.7937229816386824e-6*v**2*coen(13)*coep(13) - 5.3392108769715615e-15*coep(13)**2&
            - 7.500323319193932e-8*v**2*coep(13)**2 - 3.656204284263705e-13*coen(14)*coep(14)&
            - 3.97630574402618e-6*v**2*coen(14)*coep(14) - 5.339210876971578e-15*coep(14)**2&
            - 4.656654400421442e-8*v**2*coep(14)**2 + 0.005695423389960273*coen(11)*coep(15)&
            - 0.00040734953035434344*coen(12)*coep(15) + 3.976305744026179e-6*v**2*coen(12)*coep(15)&
            + 2.4337158197772924e-8*v**2*coen(15)*coep(15) + 0.004101047524888795*coep(11)*coep(15)&
            - 0.00042717722384164037*coep(12)*coep(15)&
            + 9.313308800842883e-8*v**2*coep(12)*coep(15)&
            + 3.5539923281473605e-10*v**2*coep(15)**2) + jx**2*(0.01510216505493029*coen(4)**2&
            + (-5.67317292927238e-7 - 4.709950943919957*v**2)*coen(5)**2&
            + 4.1664219732023206e-7*coen(6)**2 + 0.00007150783802584855*coen(8)**2&
            + 232.24219321043225*v**2*coen(8)**2 - 2.4271707944589406e-6*coen(8)*coen(9)&
            - 0.00016310898322688882*coen(9)**2 - 4.709950943919957*coen(11)**2&
            + 0.5894409985234708*coen(11)*coen(12) - 0.018064709149315423*coen(12)**2&
            + 0.008619542772365561*v**2*coen(12)**2 - 6.259272239051621e-12*coen(13)**2&
            - 0.00004196606216619796*v**2*coen(13)**2 - 6.259272239051626e-12*coen(14)**2&
            - 0.00008155449161344441*v**2*coen(14)**2 + 0.005431066816319519*coen(11)*coen(15)&
            - 0.0003884421396200266*coen(12)*coen(15)&
            + 0.0001631089832268888*v**2*coen(12)*coen(15) + 4.166421973202324e-7*v**2*coen(15)**2&
            - 3.793722981638681e-6*coen(6)*coep(4) + 0.000015926201058617647*coep(4)**2&
            + 2.7103722768573817e-6*coep(4)*coep(5) - 4.2431113347148636e-8*coep(5)**2&
            - 2.0983274019968134*v**2*coep(5)**2 + coen(5)*(1.6658411439011793e-7*coep(4)&
            + (-8.772643464838825e-7 - 6.305548160423342*v**2)*coep(5))&
            + coen(4)*(2.4271707944589406e-6*coen(5) - 0.00008393212433239591*coen(6)&
            + 0.0009238753332173328*coep(4) + 0.0001225337750099137*coep(5)&
            - 3.793722981638681e-6*coep(6)) + 2.4337158197772885e-8*coen(6)*coep(6)&
            - 1.500064663838786e-7*coep(4)*coep(6) + 3.5539923281473496e-10*coep(6)**2&
            + 0.00008990773798968537*coen(8)*coep(8) + 285.49742866622256*v**2*coen(8)*coep(8)&
            - 0.0001225337750099137*coen(9)*coep(8) + 0.000013547616514685248*coep(8)**2&
            + 86.6612133574969*v**2*coep(8)**2 - 1.6658411439011793e-7*coen(8)*coep(9)&
            - 7.95261148805236e-6*coen(9)*coep(9) - 2.7103722768573817e-6*coep(8)*coep(9)&
            - 9.313308800842884e-8*coep(9)**2 - 6.305548160423342*coen(11)*coep(11)&
            + 0.3902919802084195*coen(12)*coep(11) + 0.003910694885973735*coen(15)*coep(11)&
            - 2.0983274019968134*coep(11)**2 + 0.5553474854932317*coen(11)*coep(12)&
            - 0.033397529250860834*coen(12)*coep(12)&
            + 0.0005835688245633622*v**2*coen(12)*coep(12)&
            - 0.0004073495303543434*coen(15)*coep(12)&
            + 3.976305744026179e-6*v**2*coen(15)*coep(12) + 0.3640807898819096*coep(11)*coep(12)&
            - 0.015157044067355106*coep(12)**2 + 0.000011144715372772842*v**2*coep(12)**2&
            - 3.6562042842636996e-13*coen(13)*coep(13)&
            - 3.7937229816386824e-6*v**2*coen(13)*coep(13) - 5.3392108769715615e-15*coep(13)**2&
            - 7.500323319193932e-8*v**2*coep(13)**2 - 3.656204284263705e-13*coen(14)*coep(14)&
            - 3.97630574402618e-6*v**2*coen(14)*coep(14) - 5.339210876971578e-15*coep(14)**2&
            - 4.656654400421442e-8*v**2*coep(14)**2 + 0.005695423389960273*coen(11)*coep(15)&
            - 0.00040734953035434344*coen(12)*coep(15)&
            + 3.976305744026179e-6*v**2*coen(12)*coep(15)&
            + 2.4337158197772924e-8*v**2*coen(15)*coep(15)&
            + 0.004101047524888795*coep(11)*coep(15) - 0.00042717722384164037*coep(12)*coep(15)&
            + 9.313308800842883e-8*v**2*coep(12)*coep(15)&
            + 3.5539923281473605e-10*v**2*coep(15)**2)) + Q**4*((146039.41273336&
            - 60227.656174446216*v**2)*coen(3)**2 + 0.9048082473175623*coen(7)**2&
            + 1.4174718587046992e6*v**2*coen(7)**2 - 14290.407252058138*coen(10)**2&
            + 4.5134361703861475e9*coep(1)**2&
            + 4.632170093666738e7*coep(1)*coep(3) + 92494.24310420487*coep(3)**2&
            - 58.34582583108798*v**2*coep(3)**2 + coen(3)*(6.539618086537028e7*coep(1)&
            + (235823.6721568607 - 4036.388670768052*v**2)*coep(3))&
            + 0.06063921445177653*coen(7)*coep(7) + 123183.7943749692*v**2*coen(7)*coep(7)&
            + 0.0008765372548387714*coep(7)**2 + 2402.0880873428127*v**2*coep(7)**2&
            - 1892.4920609284495*coen(10)*coep(10) - 61.36391059073005*coep(10)**2&
            + jx*(1.0356882173845996e6*coen(4)**2 + (-255.6243236109252&
            - 4.412277500394882e8*v**2)*coen(5)**2 + 159.01259461444678*coen(6)**2&
            + 6698.568032531819*coen(8)**2 + 6.437221428574226e9*v**2*coen(8)**2&
            - 1486.5713971440234*coen(8)*coen(9) - 40151.77078296414*coen(9)**2&
            - 4.412277500394882e8*coen(11)**2 + 4.061900007342149e7*coen(11)*coen(12)&
            - 886777.4940036393*coen(12)**2 + 563197.5978163667*v**2*coen(12)**2&
            - 0.0023888677755911124*coen(13)**2 - 4763.46908401938*v**2*coen(13)**2&
            - 0.00724693667642784*coen(14)**2 - 20075.88539148207*v**2*coen(14)**2&
            + 1.867970646040247e6*coen(11)*coen(15) - 97359.60848890666*coen(12)*coen(15)&
            + 40151.77078296414*v**2*coen(12)*coen(15) - 1344.2317516020844*coen(15)**2&
            + 482.3850929297427*v**2*coen(15)**2 - 630.8306869761498*coen(6)*coep(4)&
            + 2072.8303012049096*coep(4)**2 + 518.2906927974954*coep(4)*coep(5)&
            - 57.083416700741076*coep(5)**2 - 1.331240488765208e8*v**2*coep(5)**2&
            + coen(5)*(87.3985664061606*coep(4) + (-308.4274288904989&
            - 4.9001836827225715e8*v**2)*coep(5)) + coen(4)*(1486.5713971440234*coen(5)&
            - 9526.938168038756*coen(6) + 95164.94969719752*coep(4) + 12335.272006377847*coep(5)&
            - 630.8306869761498*coep(6)) + 18.101643093083375*coen(6)*coep(6)&
            - 40.90927372715335*coep(4)*coep(6) + 0.47350084735272846*coep(6)**2&
            + 7376.197129381577*coen(8)*coep(8) + 6.332009025721297e9*v**2*coen(8)*coep(8)&
            - 12335.272006377847*coen(9)*coep(8) + 1373.878108997126*coep(8)**2&
            + 1.5044787234620492e9*v**2*coep(8)**2 - 87.3985664061606*coen(8)*coep(9)&
            - 2690.9257805120346*coen(9)*coep(9) - 518.2906927974954*coep(8)*coep(9)&
            - 38.89721722072532*coep(9)**2 - 4.9001836827225715e8*coen(11)*coep(11)&
            + 2.179872695512343e7*coen(12)*coep(11) + 1.1239135647877278e6*coen(15)*coep(11)&
            - 1.331240488765208e8*coep(11)**2 + 2.9668541202602092e7*coen(11)*coep(12)&
            - 1.2364608970412225e6*coen(12)*coep(12) + 54103.68490554112*v**2*coen(12)*coep(12)&
            - 78607.89071895355*coen(15)*coep(12) + 1345.462890256017*v**2*coen(15)*coep(12)&
            + 1.5440566978889126e7*coep(11)*coep(12) - 413355.1006656142*coep(12)**2&
            + 1272.134272090639*v**2*coep(12)**2 - 0.0002719434392927615*coen(13)*coep(13)&
            - 630.8306869761498*v**2*coen(13)*coep(13) - 7.1134674501641804e-6*coep(13)**2&
            - 20.454636863576678*v**2*coep(13)**2 - 0.0003804293403650028*coen(14)*coep(14)&
            - 1345.4628902560173*v**2*coen(14)*coep(14) - 5.34523260405886e-6*coep(14)**2&
            - 19.44860861036266*v**2*coep(14)**2 + 1.5579453837254737e6*coen(11)*coep(15)&
            - 78607.89071895357*coen(12)*coep(15) + 1345.462890256017*v**2*coen(12)*coep(15)&
            - 2485.177999977405*coen(15)*coep(15) + 25.32289861205604*v**2*coen(15)*coep(15)&
            + 919281.7010121285*coep(11)*coep(15) - 61662.828736136566*coep(12)*coep(15)&
            + 38.89721722072532*v**2*coep(12)*coep(15) - 1127.866439354256*coep(15)**2&
            + 0.35580006305657386*v**2*coep(15)**2) + jx**2*(1.0356882173845996e6*coen(4)**2&
            + (-255.6243236109252 - 4.412277500394882e8*v**2)*coen(5)**2&
            + 159.01259461444678*coen(6)**2 + 6698.568032531819*coen(8)**2&
            + 6.437221428574226e9*v**2*coen(8)**2 - 1486.5713971440234*coen(8)*coen(9)&
            - 40151.77078296414*coen(9)**2 - 4.412277500394882e8*coen(11)**2&
            + 4.061900007342149e7*coen(11)*coen(12) - 886777.4940036393*coen(12)**2&
            + 563197.5978163667*v**2*coen(12)**2 - 0.0023888677755911124*coen(13)**2&
            - 4763.46908401938*v**2*coen(13)**2 - 0.00724693667642784*coen(14)**2&
            - 20075.88539148207*v**2*coen(14)**2 + 1.867970646040247e6*coen(11)*coen(15)&
            - 97359.60848890666*coen(12)*coen(15) + 40151.77078296414*v**2*coen(12)*coen(15)&
            - 1344.2317516020844*coen(15)**2 + 482.3850929297427*v**2*coen(15)**2&
            - 630.8306869761498*coen(6)*coep(4) + 2072.8303012049096*coep(4)**2&
            + 518.2906927974954*coep(4)*coep(5) - 57.083416700741076*coep(5)**2&
            - 1.331240488765208e8*v**2*coep(5)**2 + coen(5)*(87.3985664061606*coep(4)&
            + (-308.4274288904989 - 4.9001836827225715e8*v**2)*coep(5))&
            + coen(4)*(1486.5713971440234*coen(5) - 9526.938168038756*coen(6)&
            + 95164.94969719752*coep(4) + 12335.272006377847*coep(5) - 630.8306869761498*coep(6))&
            + 18.101643093083375*coen(6)*coep(6) - 40.90927372715335*coep(4)*coep(6)&
            + 0.47350084735272846*coep(6)**2 + 7376.197129381577*coen(8)*coep(8)&
            + 6.332009025721297e9*v**2*coen(8)*coep(8) - 12335.272006377847*coen(9)*coep(8)&
            + 1373.878108997126*coep(8)**2 + 1.5044787234620492e9*v**2*coep(8)**2&
            - 87.3985664061606*coen(8)*coep(9) - 2690.9257805120346*coen(9)*coep(9)&
            - 518.2906927974954*coep(8)*coep(9)&
            - 38.89721722072532*coep(9)**2 - 4.9001836827225715e8*coen(11)*coep(11)&
            + 2.179872695512343e7*coen(12)*coep(11) + 1.1239135647877278e6*coen(15)*coep(11)&
            - 1.331240488765208e8*coep(11)**2 + 2.9668541202602092e7*coen(11)*coep(12)&
            - 1.2364608970412225e6*coen(12)*coep(12) + 54103.68490554112*v**2*coen(12)*coep(12)&
            - 78607.89071895355*coen(15)*coep(12) + 1345.462890256017*v**2*coen(15)*coep(12)&
            + 1.5440566978889126e7*coep(11)*coep(12) - 413355.1006656142*coep(12)**2&
            + 1272.134272090639*v**2*coep(12)**2 - 0.0002719434392927615*coen(13)*coep(13)&
            - 630.8306869761498*v**2*coen(13)*coep(13) - 7.1134674501641804e-6*coep(13)**2&
            - 20.454636863576678*v**2*coep(13)**2 - 0.0003804293403650028*coen(14)*coep(14)&
            - 1345.4628902560173*v**2*coen(14)*coep(14) - 5.34523260405886e-6*coep(14)**2&
            - 19.44860861036266*v**2*coep(14)**2 + 1.5579453837254737e6*coen(11)*coep(15)&
            - 78607.89071895357*coen(12)*coep(15) + 1345.462890256017*v**2*coen(12)*coep(15)&
            - 2485.177999977405*coen(15)*coep(15) + 25.32289861205604*v**2*coen(15)*coep(15)&
            + 919281.7010121285*coep(11)*coep(15) - 61662.828736136566*coep(12)*coep(15)&
            + 38.89721722072532*v**2*coep(12)*coep(15) - 1127.866439354256*coep(15)**2&
            + 0.35580006305657386*v**2*coep(15)**2)) + Q**6*((1.98787935269388e12&
            - 1.4290871460920698e12*v**2)*coen(3)**2 + 2.1469370021213718e7*coen(7)**2&
            + 1.102969861103564e13*v**2*coen(7)**2 - 2.3606288675556903e11*coen(10)**2&
            + 7.327987299380721e15*coep(1)**2 + 1.498544968239281e14*coep(1)*coep(3)&
            + 6.438779176151414e11*coep(3)**2 - 3.824348534365699e9*v**2*coep(3)**2&
            + coen(3)*(3.104083887104379e14*coep(1) + (2.337271792667193e12&
            - 1.5070682694472797e11*v**2)*coep(3)) + 2.264089101387055e6*coen(7)*coep(7)&
            + 1.3064528030427563e12*v**2*coen(7)*coep(7) + 57453.706723840194*coep(7)**2&
            + 3.874181256557698e10*v**2*coep(7)**2 - 3.44127619052829e10*coen(10)*coep(10)&
            - 1.264292870673268e9*coep(10)**2 + jx*(7.909578981496115e12*coen(4)**2&
            + (-7.1987592845033865e9 - 4.140523931850025e15*v**2)*coen(5)**2&
            + 6.749690514791338e9*coen(6)**2 + 6.168107108936564e10*coen(8)**2&
            + 2.1110306767360776e16*v**2*coen(8)**2 + 2.551950102557239e10*coen(8)*coen(9)&
            - 9.527247640613798e11*coen(9)**2 - 4.140523931850025e15*coen(11)**2&
            + 2.611312941979468e14*coen(11)*coen(12) - 3.7614481730372e12*coen(12)**2&
            + 4.233012777817569e12*v**2*coen(12)**2 - 101401.51605659723*coen(13)**2&
            - 7.868762891852301e10*v**2*coen(13)**2 - 528198.2276564871*coen(14)**2&
            - 4.763623820306899e11*v**2*coen(14)**2 + 3.691030878085968e13*coen(11)*coen(15)&
            - 1.32525290179592e12*coen(12)*coen(15) + 9.527247640613796e11*v**2*coen(12)*coen(15)&
            - 6.59869200653582e10*coen(15)**2 + 3.51589868257264e10*v**2*coen(15)**2&
            - 1.1470920635094297e10*coen(6)*coep(4) + 2.843993802599409e10*coep(4)**2&
            + 6.439642140693483e9*coep(4)*coep(5) - 1.2791532808185155e9*coep(5)**2&
            - 7.085314813506041e14*v**2*coep(5)**2 + coen(5)*(-1.5191273396588457e9*coep(4)&
            + (-7.2614456867249775e9 - 3.5160049692454795e15*v**2)*coep(5))&
            + coen(4)*(-2.551950102557239e10*coen(5) - 1.57375257837046e11*coen(6)&
            + 9.476378610634171e11*coep(4) + 1.0895298289805785e11*coep(5)&
            - 1.1470920635094297e10*coep(6)) + 9.705129752177888e8*coen(6)*coep(6)&
            - 8.428619137821785e8*coep(4)*coep(6) + 3.508067734571411e7*coep(6)**2&
            + 5.387487333604955e10*coen(8)*coep(8) + 1.4927803428062324e16*v**2*coen(8)*coep(8)&
            - 1.0895298289805785e11*coen(9)*coep(8) + 8.866849152138308e9*coep(8)**2&
            + 2.4426624331269065e15*v**2*coep(8)**2 + 1.5191273396588457e9*coen(8)*coep(9)&
            - 1.0047121796315195e11*coen(9)*coep(9) - 6.439642140693483e9*coep(8)*coep(9)&
            - 2.549565689577132e9*coep(9)**2 - 3.5160049692454795e15*coen(11)*coep(11)&
            + 1.034694629034793e14*coen(12)*coep(11) + 1.7274944844743418e13*coen(15)*coep(11)&
            - 7.085314813506041e14*coep(11)**2 + 1.3584368567425617e14*coen(11)*coep(12)&
            - 3.552511823645772e12*coen(12)*coep(12) + 5.121535933824984e11*v**2*coen(12)*coep(12)&
            - 7.790905975557307e11*coen(15)*coep(12)&
            + 5.0235608981575966e10*v**2*coen(15)*coep(12)&
            + 4.995149894130936e13*coep(11)*coep(12) - 7.440509986239044e11*coep(12)**2&
            + 1.55260005041351e10*v**2*coep(12)**2 - 14580.148056273494*coen(13)*coep(13)&
            - 1.14709206350943e10*v**2*coen(13)*coep(13) - 527.0217737172363*coep(13)**2&
            - 4.214309568910893e8*v**2*coep(13)**2 - 45902.47168950317*coen(14)*coep(14)&
            - 5.023560898157597e10*v**2*coen(14)*coep(14) - 895.099724638384*coep(14)**2&
            - 1.274782844788566e9*v**2*coep(14)**2 + 2.316788754651945e13*coen(11)*coep(15)&
            - 7.79090597555731e11*coen(12)*coep(15) + 5.0235608981575966e10*v**2*coen(12)*coep(15)&
            - 9.200756781696042e10*coen(15)*coep(15) + 3.055452125540088e9*v**2*coen(15)*coep(15)&
            + 1.0347295045746814e13*coep(11)*coep(15) - 4.2925194507676086e11*coep(12)*coep(15)&
            + 2.5495656895771317e9*v**2*coep(12)*coep(15) - 3.075859275837855e10*coep(15)**2&
            + 5.958141807082939e7*v**2*coep(15)**2)&
            + jx**2*(7.909578981496115e12*coen(4)**2 + (-7.1987592845033865e9&
            - 4.140523931850025e15*v**2)*coen(5)**2 + 6.749690514791338e9*coen(6)**2&
            + 6.168107108936564e10*coen(8)**2 + 2.1110306767360776e16*v**2*coen(8)**2&
            + 2.551950102557239e10*coen(8)*coen(9) - 9.527247640613798e11*coen(9)**2&
            - 4.140523931850025e15*coen(11)**2 + 2.611312941979468e14*coen(11)*coen(12)&
            - 3.7614481730372e12*coen(12)**2 + 4.233012777817569e12*v**2*coen(12)**2&
            - 101401.51605659723*coen(13)**2 - 7.868762891852301e10*v**2*coen(13)**2&
            - 528198.2276564871*coen(14)**2 - 4.763623820306899e11*v**2*coen(14)**2&
            + 3.691030878085968e13*coen(11)*coen(15) - 1.32525290179592e12*coen(12)*coen(15)&
            + 9.527247640613796e11*v**2*coen(12)*coen(15) - 6.59869200653582e10*coen(15)**2&
            + 3.51589868257264e10*v**2*coen(15)**2 - 1.1470920635094297e10*coen(6)*coep(4)&
            + 2.843993802599409e10*coep(4)**2 + 6.439642140693483e9*coep(4)*coep(5)&
            - 1.2791532808185155e9*coep(5)**2 - 7.085314813506041e14*v**2*coep(5)**2&
            + coen(5)*(-1.5191273396588457e9*coep(4) + (-7.2614456867249775e9&
            - 3.5160049692454795e15*v**2)*coep(5)) + coen(4)*(-2.551950102557239e10*coen(5)&
            - 1.57375257837046e11*coen(6) + 9.476378610634171e11*coep(4)&
            + 1.0895298289805785e11*coep(5) - 1.1470920635094297e10*coep(6))&
            + 9.705129752177888e8*coen(6)*coep(6) - 8.428619137821785e8*coep(4)*coep(6)&
            + 3.508067734571411e7*coep(6)**2 + 5.387487333604955e10*coen(8)*coep(8)&
            + 1.4927803428062324e16*v**2*coen(8)*coep(8) - 1.0895298289805785e11*coen(9)*coep(8)&
            + 8.866849152138308e9*coep(8)**2 + 2.4426624331269065e15*v**2*coep(8)**2&
            + 1.5191273396588457e9*coen(8)*coep(9) - 1.0047121796315195e11*coen(9)*coep(9)&
            - 6.439642140693483e9*coep(8)*coep(9) - 2.549565689577132e9*coep(9)**2&
            - 3.5160049692454795e15*coen(11)*coep(11) + 1.034694629034793e14*coen(12)*coep(11)&
            + 1.7274944844743418e13*coen(15)*coep(11) - 7.085314813506041e14*coep(11)**2&
            + 1.3584368567425617e14*coen(11)*coep(12) - 3.552511823645772e12*coen(12)*coep(12)&
            + 5.121535933824984e11*v**2*coen(12)*coep(12) - 7.790905975557307e11*coen(15)*coep(12)&
            + 5.0235608981575966e10*v**2*coen(15)*coep(12)&
            + 4.995149894130936e13*coep(11)*coep(12) - 7.440509986239044e11*coep(12)**2&
            + 1.55260005041351e10*v**2*coep(12)**2 - 14580.148056273494*coen(13)*coep(13)&
            - 1.14709206350943e10*v**2*coen(13)*coep(13) - 527.0217737172363*coep(13)**2&
            - 4.214309568910893e8*v**2*coep(13)**2 - 45902.47168950317*coen(14)*coep(14)&
            - 5.023560898157597e10*v**2*coen(14)*coep(14) - 895.099724638384*coep(14)**2&
            - 1.274782844788566e9*v**2*coep(14)**2 + 2.316788754651945e13*coen(11)*coep(15)&
            - 7.79090597555731e11*coen(12)*coep(15) + 5.0235608981575966e10*v**2*coen(12)*coep(15)&
            - 9.200756781696042e10*coen(15)*coep(15) + 3.055452125540088e9*v**2*coen(15)*coep(15)&
            + 1.0347295045746814e13*coep(11)*coep(15) - 4.2925194507676086e11*coep(12)*coep(15)&
            + 2.5495656895771317e9*v**2*coep(12)*coep(15) - 3.075859275837855e10*coep(15)**2&
            + 5.958141807082939e7*v**2*coep(15)**2)) + Q**12*(-1.03875330716873e22*coen(3)**2&
            + 4.733142351730946e15*coen(3)*coep(3) - 5.391712442006738e8*coep(3)**2&
            + jx*(-4.031071737516795e25*coen(5)**2 + 6.40218288318327e24*coen(6)**2&
            - 9.618086177488239e19*coen(13)**2 - 3.4625110238957666e21*coen(14)**2&
            + 6.925022047791531e21*coen(12)*coen(15) + 4.628997719064858e22*coen(15)**2&
            + 2.3047858379459775e26*v**2*coen(15)**2 + 6.403877130187704e18*coen(5)*coep(5)&
            - 2.543346098065456e11*coep(5)**2 - 2.9171934030030223e18*coen(6)*coep(6)&
            + 3.32309210225694e11*coep(6)**2 - 1.577714117243648e15*coen(15)*coep(12)&
            + 4.3825392148954734e13*coen(13)*coep(13) - 4.992326335942763e6*coep(13)**2&
            + 1.5777141172436488e15*coen(14)*coep(14) - 1.797237480668913e8*coep(14)**2&
            - 1.577714117243648e15*coen(12)*coep(15) + 2.8261691808482014e21*coen(15)*coep(15)&
            - 1.0501896250020622e20*v**2*coen(15)*coep(15) + 3.594474961337825e8*coep(12)*coep(15)&
            - 6.4388295407399e14*coep(15)**2 + 1.196313156632455e13*v**2*coep(15)**2)&
            + jx**2*(-4.031071737516795e25*coen(5)**2 + 6.40218288318327e24*coen(6)**2&
            - 9.618086177488239e19*coen(13)**2 - 3.4625110238957666e21*coen(14)**2&
            + 6.925022047791531e21*coen(12)*coen(15) + 4.628997719064858e22*coen(15)**2&
            + 2.3047858379459775e26*v**2*coen(15)**2 + 6.403877130187704e18*coen(5)*coep(5)&
            - 2.543346098065456e11*coep(5)**2 - 2.9171934030030223e18*coen(6)*coep(6)&
            + 3.32309210225694e11*coep(6)**2 - 1.577714117243648e15*coen(15)*coep(12)&
            + 4.3825392148954734e13*coen(13)*coep(13) - 4.992326335942763e6*coep(13)**2&
            + 1.5777141172436488e15*coen(14)*coep(14) - 1.797237480668913e8*coep(14)**2&
            - 1.577714117243648e15*coen(12)*coep(15) + 2.8261691808482014e21*coen(15)*coep(15)&
            - 1.0501896250020622e20*v**2*coen(15)*coep(15) + 3.594474961337825e8*coep(12)*coep(15)&
            - 6.4388295407399e14*coep(15)**2 + 1.196313156632455e13*v**2*coep(15)**2))&
            + Q**8*((2.6348552059130706e18 - 3.383763497329019e18*v**2)*coen(3)**2&
            + 5.0834737956388125e13*coen(7)**2 + 6.078562934028574e18*v**2*coen(7)**2&
            - 4.9762463542126624e17*coen(10)**2 + 6.089361312915843e20*coep(1)**2&
            + 2.343051021650045e19*coep(1)*coep(3) + 2.253884845546953e17*coep(3)**2&
            - 1.292190769733972e16*v**2*coep(3)**2 + coen(3)*(1.247771301178177e20*coep(1)&
            + (1.7956485358417108e18 - 4.174572593723852e17*v**2)*coep(3))&
            + 6.271517026807061e12*coen(7)*coep(7) + 7.129650076006318e17*v**2*coen(7)*coep(7)&
            + 1.9412757192085388e11*coep(7)**2 + 2.0798048754776416e16*v**2*coep(7)**2&
            - 6.053104472677084e16*coen(10)*coep(10) - 1.7435881523753968e15*coep(10)**2&
            + jx*(4.35942617734361e18*coen(4)**2 + (-2.3687707485651084e16&
            - 4.4913030461971266e21*v**2)*coen(5)**2 + 4.140631995456834e16*coen(6)**2&
            + 6.8344740140475864e16*coen(8)**2 + 6.311792317915176e21*v**2*coen(8)**2&
            - 2.830095693497678e16*coen(8)*coen(9) - 2.2558423315526792e18*coen(9)**2&
            - 4.4913030461971266e21*coen(11)**2 + 1.6979150569457674e20*coen(11)*coen(12)&
            - 1.3939501038762522e18*coen(12)**2 + 2.333238532667419e18*v**2*coen(12)**2&
            - 6.220527605698026e11*coen(13)**2 - 1.658748784737554e17*v**2*coen(13)**2&
            - 4.1100408605346025e12*coen(14)**2 - 1.1279211657763396e18*v**2*coen(14)**2&
            + 7.858707458915754e19*coen(11)*coen(15) - 1.7565701372753805e18*coen(12)*coen(15)&
            + 2.255842331552679e18*v**2*coen(12)*coen(15) - 2.7989708757601165e17*coen(15)**2&
            + 2.735807598406252e17*v**2*coen(15)**2 - 2.0177014908923604e16*coen(6)*coep(4)&
            + 1.4526325037210776e16*coep(4)**2 + 3.577532516150488e15*coep(4)*coep(5)&
            - 2.4998891722539085e15*coep(5)**2 - 3.1003909160098575e20*v**2*coep(5)**2&
            + coen(5)*(1.9258773390388725e15*coep(4) + (-1.6478016556792888e16&
            - 2.499871009813711e21*v**2)*coep(5)) + coen(4)*(2.830095693497678e16*coen(5)&
            - 3.317497569475108e17*coen(6) + 5.040895919967638e17*coep(4)&
            + 6.110859733228135e16*coep(5) - 2.0177014908923604e16*coep(6))&
            + 5.705120272530381e15*coen(6)*coep(6) - 1.1623921015835975e15*coep(4)*coep(6)&
            + 1.9436891269023575e14*coep(6)**2 + 3.595002487485918e16*coen(8)*coep(8)&
            + 2.488081421047891e21*v**2*coen(8)*coep(8) - 6.110859733228135e16*coen(9)*coep(8)&
            + 4.65775968923638e15*coep(8)**2 + 2.0297871043052808e20*v**2*coep(8)**2&
            - 1.9258773390388725e15*coen(8)*coep(9) - 2.7830483958159008e17*coen(9)*coep(9)&
            - 3.577532516150488e15*coep(8)*coep(9) - 8.614605131559811e15*coep(9)**2&
            - 2.499871009813711e21*coen(11)*coep(11) + 4.1592376705939235e19*coen(12)*coep(11)&
            + 2.5296379878482088e19*coen(15)*coep(11) - 3.1003909160098575e20*coep(11)**2&
            + 4.3098187654091915e19*coen(11)*coep(12) - 5.101581106474501e17*coen(12)*coep(12)&
            + 2.6643458946321984e17*v**2*coen(12)*coep(12)&
            - 5.985495119472369e17*coen(15)*coep(12)&
            + 1.3915241979079502e17*v**2*coen(15)*coep(12)&
            + 7.810170072166816e18*coep(11)*coep(12) + 1.1396337306346587e11*coep(12)**2&
            + 7.593642118951973e15*v**2*coep(12)**2 - 8.57087956332309e10*coen(13)*coep(13)&
            - 2.017701490892361e16*v**2*coen(13)*coep(13) - 2.9200305373811035e9*coep(13)**2&
            - 5.811960507917988e14*v**2*coep(13)**2 - 4.8682875137614606e11*coen(14)*coep(14)&
            - 1.3915241979079504e17*v**2*coen(14)*coep(14) - 1.4436517104499851e10*coep(14)**2&
            - 4.3073025657799055e15*v**2*coep(14)**2 + 3.1397305335766225e19*coen(11)*coep(15)&
            - 5.98549511947237e17*coen(12)*coep(15) + 1.3915241979079502e17*v**2*coen(12)*coep(15)&
            - 2.6434963424005405e17*coen(15)*coep(15)&
            + 3.240526900660178e16*v**2*coen(15)*coep(15) + 8.842642857282089e18*coep(11)*coep(15)&
            - 1.5025898970313018e17*coep(12)*coep(15) + 8.61460513155981e15*v**2*coep(12)*coep(15)&
            - 5.5366346753700376e16*coep(15)**2 + 9.609523245439279e14*v**2*coep(15)**2)&
            + jx**2*(4.35942617734361e18*coen(4)**2 + (-2.3687707485651084e16&
            - 4.4913030461971266e21*v**2)*coen(5)**2 + 4.140631995456834e16*coen(6)**2&
            + 6.8344740140475864e16*coen(8)**2 + 6.311792317915176e21*v**2*coen(8)**2&
            - 2.830095693497678e16*coen(8)*coen(9) - 2.2558423315526792e18*coen(9)**2&
            - 4.4913030461971266e21*coen(11)**2 + 1.6979150569457674e20*coen(11)*coen(12)&
            - 1.3939501038762522e18*coen(12)**2 + 2.333238532667419e18*v**2*coen(12)**2&
            - 6.220527605698026e11*coen(13)**2 - 1.658748784737554e17*v**2*coen(13)**2&
            - 4.1100408605346025e12*coen(14)**2 - 1.1279211657763396e18*v**2*coen(14)**2&
            + 7.858707458915754e19*coen(11)*coen(15) - 1.7565701372753805e18*coen(12)*coen(15)&
            + 2.255842331552679e18*v**2*coen(12)*coen(15) - 2.7989708757601165e17*coen(15)**2&
            + 2.735807598406252e17*v**2*coen(15)**2 - 2.0177014908923604e16*coen(6)*coep(4)&
            + 1.4526325037210776e16*coep(4)**2 + 3.577532516150488e15*coep(4)*coep(5)&
            - 2.4998891722539085e15*coep(5)**2 - 3.1003909160098575e20*v**2*coep(5)**2&
            + coen(5)*(1.9258773390388725e15*coep(4) + (-1.6478016556792888e16&
            - 2.499871009813711e21*v**2)*coep(5)) + coen(4)*(2.830095693497678e16*coen(5)&
            - 3.317497569475108e17*coen(6) + 5.040895919967638e17*coep(4)&
            + 6.110859733228135e16*coep(5) - 2.0177014908923604e16*coep(6))&
            + 5.705120272530381e15*coen(6)*coep(6) - 1.1623921015835975e15*coep(4)*coep(6)&
            + 1.9436891269023575e14*coep(6)**2 + 3.595002487485918e16*coen(8)*coep(8)&
            + 2.488081421047891e21*v**2*coen(8)*coep(8) - 6.110859733228135e16*coen(9)*coep(8)&
            + 4.65775968923638e15*coep(8)**2 + 2.0297871043052808e20*v**2*coep(8)**2&
            - 1.9258773390388725e15*coen(8)*coep(9) - 2.7830483958159008e17*coen(9)*coep(9)&
            - 3.577532516150488e15*coep(8)*coep(9) - 8.614605131559811e15*coep(9)**2&
            - 2.499871009813711e21*coen(11)*coep(11) + 4.1592376705939235e19*coen(12)*coep(11)&
            + 2.5296379878482088e19*coen(15)*coep(11) - 3.1003909160098575e20*coep(11)**2&
            + 4.3098187654091915e19*coen(11)*coep(12) - 5.101581106474501e17*coen(12)*coep(12)&
            + 2.6643458946321984e17*v**2*coen(12)*coep(12) - 5.985495119472369e17*coen(15)*coep(12)&
            + 1.3915241979079502e17*v**2*coen(15)*coep(12) + 7.810170072166816e18*coep(11)*coep(12)&
            + 1.1396337306346587e11*coep(12)**2 + 7.593642118951973e15*v**2*coep(12)**2&
            - 8.57087956332309e10*coen(13)*coep(13) - 2.017701490892361e16*v**2*coen(13)*coep(13)&
            - 2.9200305373811035e9*coep(13)**2 - 5.811960507917988e14*v**2*coep(13)**2&
            - 4.8682875137614606e11*coen(14)*coep(14) - 1.3915241979079504e17*v**2*coen(14)*coep(14)&
            - 1.4436517104499851e10*coep(14)**2 - 4.3073025657799055e15*v**2*coep(14)**2&
            + 3.1397305335766225e19*coen(11)*coep(15) - 5.98549511947237e17*coen(12)*coep(15)&
            + 1.3915241979079502e17*v**2*coen(12)*coep(15)&
            - 2.6434963424005405e17*coen(15)*coep(15)&
            + 3.240526900660178e16*v**2*coen(15)*coep(15) + 8.842642857282089e18*coep(11)*coep(15)&
            - 1.5025898970313018e17*coep(12)*coep(15) + 8.61460513155981e15*v**2*coep(12)*coep(15)&
            - 5.5366346753700376e16*coep(15)**2 + 9.609523245439279e14*v**2*coep(15)**2))&
            + Q**11*((5.0908071000203946e20 + 2.5347215142314137e24*v**2)*coen(3)**2&
            - 3.807946508970936e19*coen(7)**2 + 7.040893095087259e22*coen(10)**2&
            + (3.108120376137792e19 - 1.1549612084078333e18*v**2)*coen(3)*coep(3)&
            - 7.081195786037603e12*coep(3)**2 + 1.3156626728393904e11*v**2*coep(3)**2&
            + 1.7351138879992691e13*coen(7)*coep(7) - 1.976537877590575e6*coep(7)**2&
            - 3.2082255791520652e16*coen(10)*coep(10) + 3.6546185362149878e9*coep(10)**2&
            + jx*((9.466169692087275e23 + 9.836449316675265e27*v**2)*coen(5)**2&
            + 4.693928730058172e22*coen(4)*coen(6) - 7.520126584584255e23*coen(6)**2&
            - 1.4777431219090299e23*coen(8)**2 + 1.6898143428209422e24*coen(9)**2&
            + 9.836449316675265e27*coen(11)**2 - 1.304574266962265e19*coen(12)**2&
            + 1.129758816264686e19*coen(13)**2 + 2.346964365029086e22*v**2*coen(13)**2&
            + 1.696935700006798e20*coen(14)**2 + 8.449071714104711e23*v**2*coen(14)**2&
            - 8.316740639590024e26*coen(11)*coen(15) - 3.393871400013595e20*coen(12)*coen(15)&
            - 1.6898143428209422e24*v**2*coen(12)*coen(15) + 1.7578940717473217e25*coen(15)**2&
            - 1.129548279352525e25*v**2*coen(15)**2 - 1.0694085263840214e16*coen(6)*coep(4)&
            + (1.9127898036366378e23 - 1.562646782865515e21*v**2)*coen(5)*coep(5)&
            - 1.519357194702245e16*coep(5)**2 + 6.206164667214738e13*v**2*coep(5)**2&
            - 1.0694085263840214e16*coen(4)*coep(6) - 3.548958338461764e22*coen(6)*coep(6)&
            + 2.436412357476658e9*coep(4)*coep(6) + 8.085560832567189e15*coep(6)**2&
            + 2.347585455900359e16*coen(8)*coep(8) - 9.323605353065829e8*coep(8)**2&
            - 7.699741389385555e17*coen(9)*coep(9) + 8.771084485595934e10*coep(9)**2&
            - 1.562646782865515e21*coen(11)*coep(11) + 6.606107339134314e19*coen(15)*coep(11)&
            + 6.206164667214738e13*coep(11)**2 + 5.944371653342919e12*coen(12)*coep(12)&
            - 1.0360401253792637e19*coen(15)*coep(12)&
            + 3.849870694692777e17*v**2*coen(15)*coep(12)&
            - 677147.2358439697*coep(12)**2 + 5.331648245991473e17*coen(13)*coep(13)&
            - 1.0694085263840216e16*v**2*coen(13)*coep(13) - 1.2147047702312346e11*coep(13)**2&
            + 1.2182061787383292e9*v**2*coep(13)**2 + 1.036040125379264e19*coen(14)*coep(14)&
            - 3.8498706946927776e17*v**2*coen(14)*coep(14) - 2.360398595345867e12*coep(14)**2&
            + 4.385542242797967e10*v**2*coep(14)**2 + 6.606107347399659e19*coen(11)*coep(15)&
            - 1.0360401253792637e19*coen(12)*coep(15)&
            + 3.849870694692777e17*v**2*coen(12)*coep(15) - 7.526508043314753e19*coen(15)*coep(15)&
            - 6.896297490574531e23*v**2*coen(15)*coep(15) - 5.247326581644765e12*coep(11)*coep(15)&
            + 4.720797190691733e12*coep(12)*coep(15) - 8.771084485595934e10*v**2*coep(12)*coep(15)&
            - 2.11410543936928e18*coep(15)**2 + 1.571175721006023e17*v**2*coep(15)**2)&
            + jx**2*((9.466169692087275e23 + 9.836449316675265e27*v**2)*coen(5)**2&
            + 4.693928730058172e22*coen(4)*coen(6) - 7.520126584584255e23*coen(6)**2&
            - 1.4777431219090299e23*coen(8)**2 + 1.6898143428209422e24*coen(9)**2&
            + 9.836449316675265e27*coen(11)**2 - 1.304574266962265e19*coen(12)**2&
            + 1.129758816264686e19*coen(13)**2 + 2.346964365029086e22*v**2*coen(13)**2&
            + 1.696935700006798e20*coen(14)**2&
            + 8.449071714104711e23*v**2*coen(14)**2 - 8.316740639590024e26*coen(11)*coen(15)&
            - 3.393871400013595e20*coen(12)*coen(15)&
            - 1.6898143428209422e24*v**2*coen(12)*coen(15) + 1.7578940717473217e25*coen(15)**2&
            - 1.129548279352525e25*v**2*coen(15)**2 - 1.0694085263840214e16*coen(6)*coep(4)&
            + (1.9127898036366378e23 - 1.562646782865515e21*v**2)*coen(5)*coep(5)&
            - 1.519357194702245e16*coep(5)**2 + 6.206164667214738e13*v**2*coep(5)**2&
            - 1.0694085263840214e16*coen(4)*coep(6) - 3.548958338461764e22*coen(6)*coep(6)&
            + 2.436412357476658e9*coep(4)*coep(6) + 8.085560832567189e15*coep(6)**2&
            + 2.347585455900359e16*coen(8)*coep(8) - 9.323605353065829e8*coep(8)**2&
            - 7.699741389385555e17*coen(9)*coep(9) + 8.771084485595934e10*coep(9)**2&
            - 1.562646782865515e21*coen(11)*coep(11) + 6.606107339134314e19*coen(15)*coep(11)&
            + 6.206164667214738e13*coep(11)**2 + 5.944371653342919e12*coen(12)*coep(12)&
            - 1.0360401253792637e19*coen(15)*coep(12)&
            + 3.849870694692777e17*v**2*coen(15)*coep(12) - 677147.2358439697*coep(12)**2&
            + 5.331648245991473e17*coen(13)*coep(13)&
            - 1.0694085263840216e16*v**2*coen(13)*coep(13) - 1.2147047702312346e11*coep(13)**2&
            + 1.2182061787383292e9*v**2*coep(13)**2 + 1.036040125379264e19*coen(14)*coep(14)&
            - 3.8498706946927776e17*v**2*coen(14)*coep(14) - 2.360398595345867e12*coep(14)**2&
            + 4.385542242797967e10*v**2*coep(14)**2 + 6.606107347399659e19*coen(11)*coep(15)&
            - 1.0360401253792637e19*coen(12)*coep(15)&
            + 3.849870694692777e17*v**2*coen(12)*coep(15) - 7.526508043314753e19*coen(15)*coep(15)&
            - 6.896297490574531e23*v**2*coen(15)*coep(15) - 5.247326581644765e12*coep(11)*coep(15)&
            + 4.720797190691733e12*coep(12)*coep(15) - 8.771084485595934e10*v**2*coep(12)*coep(15)&
            - 2.11410543936928e18*coep(15)**2 + 1.571175721006023e17*v**2*coep(15)**2))&
            + Q**10*((1.933269395376351e23 - 1.242237034738769e23*v**2)*coen(3)**2&
            + 1.8662295456083901e18*coen(7)**2 + 9.291988650933075e21*v**2*coen(7)**2&
            - 8.270367827614228e21*coen(10)**2 + 6.825319230905807e11*coep(1)**2&
            + 5.7708231974154976e10*coep(1)*coep(3) - 2.3250179918228796e16*coep(3)**2&
            + 1.7279231923002498e15*v**2*coep(3)**2 + coen(3)*(-7.265161960882486e17*coep(1)&
            + (-8.277385929023072e17 - 7.584302771262124e21*v**2)*coep(3))&
            + 1.139400091830738e17*coen(7)*coep(7) - 4.23395090172172e15*v**2*coen(7)*coep(7)&
            - 2.5958824474194004e10*coep(7)**2 + 4.8230634236703634e8*v**2*coep(7)**2&
            - 3.903018191758281e20*coen(10)*coep(10) + 8.892212308628084e13*coep(10)**2&
            + jx*(6.280696032575134e21*coen(4)**2 + (-6.848076651917395e21&
            - 2.3098943522305905e26*v**2)*coen(5)**2 + 2.284828032263649e22*coen(6)**2&
            + 3.470185614191741e21*coen(8)**2 + 3.6059257359378833e25*v**2*coen(8)**2&
            - 2.269626056500374e21*coen(8)*coen(9) - 8.281580231591792e22*coen(9)**2&
            - 2.3098943522305905e26*coen(11)**2 + 3.048818546808176e24*coen(11)*coen(12)&
            + 6.63492134394423e17*coen(12)**2 + 3.183366482264109e21*v**2*coen(12)**2&
            - 3.432528141733744e17*coen(13)**2 - 2.756789275871409e21*v**2*coen(13)**2&
            - 2.2650793020937068e18*coen(14)**2 - 4.140790115795896e22*v**2*coen(14)**2&
            + 1.2218749585751158e25*coen(11)*coen(15) - 1.2888462635842341e23*coen(12)*coen(15)&
            + 8.28158023159179e22*v**2*coen(12)*coen(15) - 1.0372684434985768e23*coen(15)**2&
            + 1.5077273866456546e23*v**2*coen(15)**2 - 1.3010060639194268e20*coen(6)*coep(4)&
            + 3.260033610450577e8*coep(4)**2 + 1.1780632578189947e8*coep(4)*coep(5)&
            - 2.269103145357077e20*coep(5)**2 + 3.7074706214037647e18*v**2*coep(5)**2&
            + coen(5)*(-5.170844288281559e14*coep(4) + (-2.781431391845452e21&
            - 4.667508092653147e25*v**2)*coep(5)) + coen(4)*(2.269626056500374e21*coen(5)&
            - 5.513578551742817e21*coen(6) - 2.8618371835741125e15*coep(4)&
            - 5.1708442898022125e14*coep(5) - 1.3010060639194268e20*coep(6))&
            + 2.141547526668305e21*coen(6)*coep(6) + 5.928141539085388e13*coep(4)*coep(6)&
            + 4.9183316804785455e19*coep(6)**2 + 7.012060712476934e20*coen(8)*coep(8)&
            - 5.728477897977788e18*v**2*coen(8)*coep(8) + 5.1708442898022125e14*coen(9)*coep(8)&
            - 5.5697833985393984e13*coep(8)**2 + 2.2751064103019357e11*v**2*coep(8)**2&
            + 5.170844288281559e14*coen(8)*coep(9) - 5.056201847508083e21*coen(9)*coep(9)&
            - 1.1780632578189947e8*coep(8)*coep(9) + 1.151948794866833e15*coep(9)**2&
            - 4.667508092653147e25*coen(11)*coep(11) - 2.4217206536274954e17*coen(12)*coep(11)&
            + 1.9731950226799298e24*coen(15)*coep(11) + 3.7074706214037647e18*coep(11)**2&
            - 2.421720656657473e17*coen(11)*coep(12) + 3.993452211624878e16*coen(12)*coep(12)&
            - 1.450520216333539e15*v**2*coen(12)*coep(12)&
            + 2.7591286430076902e17*coen(15)*coep(12)&
            + 2.528100923754041e21*v**2*coen(15)*coep(12)&
            + 1.9236077324718327e10*coep(11)*coep(12) - 9.098237863242044e9*coep(12)**2&
            + 1.6523458025604564e8*v**2*coep(12)**2 - 3.2172758948805744e16*coen(13)*coep(13)&
            - 1.301006063919427e20*v**2*coen(13)*coep(13) - 7.388876390358972e14*coep(13)**2&
            + 2.9640707695426945e13*v**2*coep(13)**2 - 2.6567501222907994e17*coen(14)*coep(14)&
            - 2.5281009237540413e21*v**2*coen(14)*coep(14) - 7.75006037934663e15*coep(14)**2&
            + 5.759743974334165e14*v**2*coep(14)**2 + 8.979763716135105e23*coen(11)*coep(15)&
            + 2.7591286430076902e17*coen(12)*coep(15)&
            + 2.528100923754041e21*v**2*coen(12)*coep(15) - 3.79619239110935e22*coen(15)*coep(15)&
            + 1.7684391514016473e22*v**2*coen(15)*coep(15)&
            - 2.2806137247265994e17*coep(11)*coep(15) + 1.550011994548586e16*coep(12)*coep(15)&
            - 1.1519487948668328e15*v**2*coep(12)*coep(15) + 7.830525845329002e15*coep(15)**2&
            + 5.158750190908289e20*v**2*coep(15)**2) + jx**2*(6.280696032575134e21*coen(4)**2&
            + (-6.848076651917395e21 - 2.3098943522305905e26*v**2)*coen(5)**2&
            + 2.284828032263649e22*coen(6)**2 + 3.470185614191741e21*coen(8)**2&
            + 3.6059257359378833e25*v**2*coen(8)**2 - 2.269626056500374e21*coen(8)*coen(9)&
            - 8.281580231591792e22*coen(9)**2 - 2.3098943522305905e26*coen(11)**2&
            + 3.048818546808176e24*coen(11)*coen(12) + 6.63492134394423e17*coen(12)**2&
            + 3.183366482264109e21*v**2*coen(12)**2 - 3.432528141733744e17*coen(13)**2&
            - 2.756789275871409e21*v**2*coen(13)**2 - 2.2650793020937068e18*coen(14)**2&
            - 4.140790115795896e22*v**2*coen(14)**2 + 1.2218749585751158e25*coen(11)*coen(15)&
            - 1.2888462635842341e23*coen(12)*coen(15) + 8.28158023159179e22*v**2*coen(12)*coen(15)&
            - 1.0372684434985768e23*coen(15)**2 + 1.5077273866456546e23*v**2*coen(15)**2&
            - 1.3010060639194268e20*coen(6)*coep(4) + 3.260033610450577e8*coep(4)**2&
            + 1.1780632578189947e8*coep(4)*coep(5) - 2.269103145357077e20*coep(5)**2&
            + 3.7074706214037647e18*v**2*coep(5)**2 + coen(5)*(-5.170844288281559e14*coep(4)&
            + (-2.781431391845452e21 - 4.667508092653147e25*v**2)*coep(5))&
            + coen(4)*(2.269626056500374e21*coen(5) - 5.513578551742817e21*coen(6)&
            - 2.8618371835741125e15*coep(4) - 5.1708442898022125e14*coep(5)&
            - 1.3010060639194268e20*coep(6)) + 2.141547526668305e21*coen(6)*coep(6)&
            + 5.928141539085388e13*coep(4)*coep(6) + 4.9183316804785455e19*coep(6)**2&
            + 7.012060712476934e20*coen(8)*coep(8) - 5.728477897977788e18*v**2*coen(8)*coep(8)&
            + 5.1708442898022125e14*coen(9)*coep(8) - 5.5697833985393984e13*coep(8)**2&
            + 2.2751064103019357e11*v**2*coep(8)**2 + 5.170844288281559e14*coen(8)*coep(9)&
            - 5.056201847508083e21*coen(9)*coep(9) - 1.1780632578189947e8*coep(8)*coep(9)&
            + 1.151948794866833e15*coep(9)**2 - 4.667508092653147e25*coen(11)*coep(11)&
            - 2.4217206536274954e17*coen(12)*coep(11) + 1.9731950226799298e24*coen(15)*coep(11)&
            + 3.7074706214037647e18*coep(11)**2 - 2.421720656657473e17*coen(11)*coep(12)&
            + 3.993452211624878e16*coen(12)*coep(12) - 1.450520216333539e15*v**2*coen(12)*coep(12)&
            + 2.7591286430076902e17*coen(15)*coep(12)&
            + 2.528100923754041e21*v**2*coen(15)*coep(12)&
            + 1.9236077324718327e10*coep(11)*coep(12) - 9.098237863242044e9*coep(12)**2&
            + 1.6523458025604564e8*v**2*coep(12)**2 - 3.2172758948805744e16*coen(13)*coep(13)&
            - 1.301006063919427e20*v**2*coen(13)*coep(13) - 7.388876390358972e14*coep(13)**2&
            + 2.9640707695426945e13*v**2*coep(13)**2 - 2.6567501222907994e17*coen(14)*coep(14)&
            - 2.5281009237540413e21*v**2*coen(14)*coep(14) - 7.75006037934663e15*coep(14)**2&
            + 5.759743974334165e14*v**2*coep(14)**2 + 8.979763716135105e23*coen(11)*coep(15)&
            + 2.7591286430076902e17*coen(12)*coep(15)&
            + 2.528100923754041e21*v**2*coen(12)*coep(15) - 3.79619239110935e22*coen(15)*coep(15)&
            + 1.7684391514016473e22*v**2*coen(15)*coep(15)&
            - 2.2806137247265994e17*coep(11)*coep(15) + 1.550011994548586e16*coep(12)*coep(15)&
            - 1.1519487948668328e15*v**2*coep(12)*coep(15) + 7.830525845329002e15*coep(15)**2&
            + 5.158750190908289e20*v**2*coep(15)**2)) + Q*(-6.883725050087478e-14*coen(7)**2&
            + 4.582082742340226e-9*coen(10)**2 - 0.0000430084318459125*coen(3)*coep(1)&
            - 0.023076658673339592*coep(1)**2 - 0.00004510186248577933*coep(1)*coep(3)&
            - 4.020963469650364e-15*coen(7)*coep(7) - 5.871874278323054e-17*coep(7)**2&
            + 2.676514123938064e-10*coen(10)*coep(10) + 3.908554394622946e-12*coep(10)**2&
            + jx*(-7.517808105084695e-7*coen(4)**2 - 2.079717956296438e-9*coen(8)**2&
            + 3.054721828226819e-9*coen(9)**2 + 0.00013918497760441077*coen(11)**2&
            - 0.000019909647247777624*coen(11)*coen(12) + 7.11991399509963e-7*coen(12)**2&
            + 1.08414195361282e-11*coen(5)*coep(4) + 8.92171374646021e-11*coen(6)*coep(4)&
            - 6.163678256690818e-10*coep(4)**2 - 9.838028118614964e-11*coep(4)*coep(5)&
            + coen(4)*(3.712013391946718e-10*coen(5) + 3.054721828226817e-9*coen(6)&
            - 4.3060669966944657e-8*coep(4) - 3.368460375963748e-9*coep(5)&
            + 8.92171374646021e-11*coep(6)) + 2.6057029297486296e-12*coep(4)*coep(6)&
            + 3.368460375963748e-9*coen(9)*coep(8) - 1.5554743251063287e-10*coep(8)**2&
            + coen(8)*(-3.712013391946718e-10*coen(9) - 3.215947119093353e-9*coep(8)&
            - 1.08414195361282e-11*coep(9)) + 1.7843427492920455e-10*coen(9)*coep(9)&
            + 9.838028118614964e-11*coep(8)*coep(9) + 2.6057029297486377e-12*coep(9)**2&
            + 0.0002004431168058439*coen(11)*coep(11) - 0.000014336143948637498*coen(12)*coep(11)&
            + 0.00007216555221396242*coep(11)**2 - 0.000020878747114677313*coen(11)*coep(12)&
            + 1.493295097061526e-6*coen(12)*coep(12) - 0.000015033954161926446*coep(11)*coep(12)&
            + 7.829905363347159e-7*coep(12)**2) + jx**2*(-7.517808105084695e-7*coen(4)**2&
            - 2.079717956296438e-9*coen(8)**2 + 3.054721828226819e-9*coen(9)**2&
            + 0.00013918497760441077*coen(11)**2 - 0.000019909647247777624*coen(11)*coen(12)&
            + 7.11991399509963e-7*coen(12)**2 + 1.08414195361282e-11*coen(5)*coep(4)&
            + 8.92171374646021e-11*coen(6)*coep(4) - 6.163678256690818e-10*coep(4)**2&
            - 9.838028118614964e-11*coep(4)*coep(5) + coen(4)*(3.712013391946718e-10*coen(5)&
            + 3.054721828226817e-9*coen(6) - 4.3060669966944657e-8*coep(4)&
            - 3.368460375963748e-9*coep(5) + 8.92171374646021e-11*coep(6))&
            + 2.6057029297486296e-12*coep(4)*coep(6) + 3.368460375963748e-9*coen(9)*coep(8)&
            - 1.5554743251063287e-10*coep(8)**2 + coen(8)*(-3.712013391946718e-10*coen(9)&
            - 3.215947119093353e-9*coep(8) - 1.08414195361282e-11*coep(9))&
            + 1.7843427492920455e-10*coen(9)*coep(9) + 9.838028118614964e-11*coep(8)*coep(9)&
            + 2.6057029297486377e-12*coep(9)**2 + 0.0002004431168058439*coen(11)*coep(11)&
            - 0.000014336143948637498*coen(12)*coep(11) + 0.00007216555221396242*coep(11)**2&
            - 0.000020878747114677313*coen(11)*coep(12) + 1.493295097061526e-6*coen(12)*coep(12)&
            - 0.000015033954161926446*coep(11)*coep(12) + 7.829905363347159e-7*coep(12)**2)&
            + v**2*(4.58208274234023e-9*coen(3)**2 - 8.969073007626154e-7*coen(7)**2&
            + 2.6765141239380685e-10*coen(3)*coep(3) + 3.908554394622957e-12*coep(3)**2&
            - 4.37299967337848e-8*coen(7)*coep(7) - 5.121222934798888e-10*coep(7)**2&
            + jx*(0.00013918497760441077*coen(5)**2 - 0.01726612192764942*coen(8)**2&
            - 4.5281171025426447e-7*coen(12)**2 + 1.5273609141134086e-9*coen(13)**2&
            + 1.5273609141134094e-9*coen(14)**2 - 3.0547218282268185e-9*coen(12)*coen(15)&
            + 0.0002004431168058439*coen(5)*coep(5) + 0.00007216555221396242*coep(5)**2&
            - 0.02311539220999266*coen(8)*coep(8) - 0.007692219557779864*coep(8)**2&
            - 2.8484004389016395e-8*coen(12)*coep(12) - 8.921713746460225e-11*coen(15)*coep(12)&
            - 4.4566039450911886e-10*coep(12)**2 + 8.921713746460212e-11*coen(13)*coep(13)&
            + 1.3028514648743148e-12*coep(13)**2 + 8.921713746460227e-11*coen(14)*coep(14)&
            + 1.3028514648743188e-12*coep(14)**2 - 8.921713746460225e-11*coen(12)*coep(15)&
            - 2.6057029297486373e-12*coep(12)*coep(15)) + jx**2*(0.00013918497760441077*coen(5)**2&
            - 0.01726612192764942*coen(8)**2 - 4.5281171025426447e-7*coen(12)**2&
            + 1.5273609141134086e-9*coen(13)**2 + 1.5273609141134094e-9*coen(14)**2&
            - 3.0547218282268185e-9*coen(12)*coen(15) + 0.0002004431168058439*coen(5)*coep(5)&
            + 0.00007216555221396242*coep(5)**2 - 0.02311539220999266*coen(8)*coep(8)&
            - 0.007692219557779864*coep(8)**2 - 2.8484004389016395e-8*coen(12)*coep(12)&
            - 8.921713746460225e-11*coen(15)*coep(12) - 4.4566039450911886e-10*coep(12)**2&
            + 8.921713746460212e-11*coen(13)*coep(13) + 1.3028514648743148e-12*coep(13)**2&
            + 8.921713746460227e-11*coen(14)*coep(14) + 1.3028514648743188e-12*coep(14)**2&
            - 8.921713746460225e-11*coen(12)*coep(15)&
            - 2.6057029297486373e-12*coep(12)*coep(15))))))/(EXP(16131.57550319467*Q)*mx**2)
        elseif (A==130) then 
            ans=(jx*mx*Q*(-5.197619426408239e-7*coen(8)**2 - 5.197619426408239e-7*jx*coen(8)**2&
            + Q**11*((-1.086635923898136e28 - 1.086635923898136e28*jx)*coen(5)**2&
            + (1.878287480030493e21 + 1.878287480030493e21*jx)*coen(5)*coep(5)&
            + (-8.116710896560652e13 - 8.116710896560652e13*jx)*coep(5)**2)&
            - 7.386569077537429e-7*coen(8)*coep(8) - 7.386569077537429e-7*jx*coen(8)*coep(8)&
            - 2.624345794731274e-7*coep(8)**2 - 2.624345794731274e-7*jx*coep(8)**2&
            + Q**8*((5.258113036383434e21 + 5.258113036383434e21*jx)*coen(5)**2&
            + (-7.416314320502635e21 - 7.416314320502635e21*jx)*coen(8)**2&
            + (3.0676666726908056e21 + 3.0676666726908056e21*jx)*coen(5)*coep(5)&
            + 4.10085522126442e20*coep(5)**2 + 4.10085522126442e20*jx*coep(5)**2&
            + (-3.192186763745401e21 - 3.192186763745401e21*jx)*coen(8)*coep(8)&
            - 2.9601238595275384e20*coep(8)**2 - 2.9601238595275384e20*jx*coep(8)**2)&
            + Q**9*((-2.0387387137387517e24 - 2.0387387137387517e24*jx)*coen(5)**2&
            + (9.673048148874592e23 + 9.673048148874592e23*jx)*coen(8)**2 + (-8.775295187719457e23&
            - 8.775295187719457e23*jx)*coen(5)*coep(5) - 8.13735617056688e22*coep(5)**2&
            - 8.13735617056688e22*jx*coep(5)**2 + (2.163416374530104e23&
            + 2.163416374530104e23*jx)*coen(8)*coep(8) - 1.8697704407113296e16*coep(8)**2&
            - 1.8697704407113296e16*jx*coep(8)**2) + Q**6*((4.650418371595078e15&
            + 4.650418371595078e15*jx)*coen(5)**2 + (-2.404437814371188e16&
            - 2.404437814371188e16*jx)*coen(8)**2 + (3.9612640417163745e15&
            + 3.9612640417163745e15*jx)*coen(5)*coep(5) + 8.07366131875136e14*coep(5)**2&
            + 8.07366131875136e14*jx*coep(5)**2 + (-1.7346278517118156e16&
            - 1.7346278517118156e16*jx)*coen(8)*coep(8) - 2.9422932266581325e15*coep(8)**2&
            - 2.9422932266581325e15*jx*coep(8)**2) + Q**10*((2.6591129891098026e26&
            + 2.6591129891098026e26*jx)*coen(5)**2 + (-3.952852569714353e25&
            - 3.952852569714353e25*jx)*coen(8)**2 + (5.9472138397606774e25&
            + 5.9472138397606774e25*jx)*coen(5)*coep(5) - 5.13998358017841e18*coep(5)**2&
            - 5.13998358017841e18*jx*coep(5)**2 + (6.832641300378019e18&
            + 6.832641300378019e18*jx)*coen(8)*coep(8) - 2.95261373377031e11*coep(8)**2&
            - 2.95261373377031e11*jx*coep(8)**2) + Q**4*((4.752035948755416e8&
            + 4.752035948755416e8*jx)*coen(5)**2 + (-7.021313866890324e9&
            - 7.021313866890324e9*jx)*coen(8)**2 + (5.210909026645227e8&
            + 5.210909026645227e8*jx)*coen(5)*coep(5) + 1.3995490737335664e8*coep(5)**2&
            + 1.3995490737335664e8*jx*coep(5)**2 + (-6.857692145485332e9&
            - 6.857692145485332e9*jx)*coen(8)*coep(8) - 1.6238949763956695e9*coep(8)**2&
            - 1.6238949763956695e9*jx*coep(8)**2) + Q**2*((4.900998289996101&
            + 4.900998289996101*jx)*coen(5)**2 + (-243.6557816232855&
            - 243.6557816232855*jx)*coen(8)**2 + (6.464203867256294&
            + 6.464203867256294*jx)*coen(5)*coep(5) + 2.1187064774498228*coep(5)**2&
            + 2.1187064774498228*jx*coep(5)**2 + (-295.0598896170864&
            - 295.0598896170864*jx)*coen(8)*coep(8) - 88.23287393171078*coep(8)**2&
            - 88.23287393171078*jx*coep(8)**2) + Q*((-0.00014288213101492313&
            - 0.00014288213101492313*jx)*coen(5)**2 + (0.01782834826155886&
            + 0.01782834826155886*jx)*coen(8)**2 + (-0.00020305617708851994&
            - 0.00020305617708851994*jx)*coen(5)*coep(5) - 0.00007214305029069375*coep(5)**2&
            - 0.00007214305029069375*jx*coep(5)**2 + (0.02351481697400316&
            + 0.02351481697400316*jx)*coen(8)*coep(8) + 0.00770721283888187*coep(8)**2&
            + 0.00770721283888187*jx*coep(8)**2) + Q**3*((-66980.7741897326&
            - 66980.7741897326*jx)*coen(5)**2 + (1.7286468354578067e6&
            + 1.7286468354578067e6*jx)*coen(8)**2 + (-81111.72124552929&
            - 81111.72124552929*jx)*coen(5)*coep(5) - 24255.14455498671*coep(5)**2&
            - 24255.14455498671*jx*coep(5)**2 + (1.8955709712440176e6&
            + 1.8955709712440176e6*jx)*coen(8)*coep(8) + 509113.58909460035*coep(8)**2&
            + 509113.58909460035*jx*coep(8)**2) + Q**5*((-1.9301534135583772e12&
            - 1.9301534135583772e12*jx)*coen(5)**2 + (1.6916814368203838e13&
            + 1.6916814368203838e13*jx)*coen(8)**2 + (-1.8851739367696526e12&
            - 1.8851739367696526e12*jx)*coen(5)*coep(5) - 4.464073948795831e11*coep(5)**2&
            - 4.464073948795831e11*jx*coep(5)**2 + (1.4409879521048746e13&
            + 1.4409879521048746e13*jx)*coen(8)*coep(8) + 2.9369536004610664e12*coep(8)**2&
            + 2.9369536004610664e12*jx*coep(8)**2) + Q**7*((-6.609779797741501e18&
            - 6.609779797741501e18*jx)*coen(5)**2 + (1.9127423611355947e19&
            + 1.9127423611355947e19*jx)*coen(8)**2 + (-4.768477713299886e18&
            - 4.768477713299886e18*jx)*coen(5)*coep(5) - 8.088339907298716e17*coep(5)**2&
            - 8.088339907298716e17*jx*coep(5)**2 + (1.1159242781770625e19&
            + 1.1159242781770625e19*jx)*coen(8)*coep(8) + 1.4917669978414185e18*coep(8)**2&
            + 1.4917669978414185e18*jx*coep(8)**2)) + jx*Q*(-0.00003169907503395692*coen(8)**2&
            - 0.00003169907503395692*jx*coen(8)**2 + Q**11*((-6.627140400320388e29&
            - 6.627140400320388e29*jx)*coen(5)**2 + (1.1455239578010608e23&
            + 1.1455239578010608e23*jx)*coen(5)*coep(5)&
            + (-4.950193668119541e15 - 4.950193668119541e15*jx)*coep(5)**2)&
            - 0.000045048971119874795*coen(8)*coep(8) - 0.000045048971119874795*jx*coen(8)*coep(8)&
            - 0.00001600527615384166*coep(8)**2 - 0.00001600527615384166*jx*coep(8)**2&
            + Q**8*((3.206801152667812e23 + 3.206801152667812e23*jx)*coen(5)**2&
            + (-4.523038045582321e23 - 4.523038045582321e23*jx)*coen(8)**2&
            + (1.870898733807316e23 + 1.870898733807316e23*jx)*coen(5)*coep(5)&
            + 2.5010164596080367e22*coep(5)**2 + 2.5010164596080367e22*jx*coep(5)**2&
            + (-1.9468406484754012e23 - 1.9468406484754012e23*jx)*coen(8)*coep(8)&
            - 1.8053108670523045e22*coep(8)**2 - 1.8053108670523045e22*jx*coep(8)**2)&
            + Q**9*((-1.2433794427711058e26 - 1.2433794427711058e26*jx)*coen(5)**2&
            + (5.89936765128156e25 + 5.89936765128156e25*jx)*coen(8)**2 + (-5.351848948141742e25&
            - 5.351848948141742e25*jx)*coen(5)*coep(5) - 4.962784741765583e24*coep(5)**2&
            - 4.962784741765583e24*jx*coep(5)**2 + (1.3194174555660214e25&
            + 1.3194174555660214e25*jx)*coen(8)*coep(8) - 1.1403296131156154e18*coep(8)**2&
            - 1.1403296131156154e18*jx*coep(8)**2) + Q**6*((2.836182275129617e17&
            + 2.836182275129617e17*jx)*coen(5)**2 + (-1.4664108400276938e18&
            - 1.4664108400276938e18*jx)*coen(8)**2 + (2.4158830377170493e17&
            + 2.4158830377170493e17*jx)*coen(5)*coep(5) + 4.92393872936386e16*coep(5)**2&
            + 4.92393872936386e16*jx*coep(5)**2 + (-1.0579092833928765e18&
            - 1.0579092833928765e18*jx)*coen(8)*coep(8) - 1.7944363777359366e17*coep(8)**2&
            - 1.7944363777359366e17*jx*coep(8)**2) + Q**10*((1.6217313206367212e28&
            + 1.6217313206367212e28*jx)*coen(5)**2 + (-2.4107530760891664e27&
            - 2.4107530760891664e27*jx)*coen(8)**2 + (3.627067745508955e27&
            + 3.627067745508955e27*jx)*coen(5)*coep(5) - 3.134756737931752e20*coep(5)**2&
            - 3.134756737931752e20*jx*coep(5)**2 + (4.1670694118223846e20&
            + 4.1670694118223846e20*jx)*coen(8)*coep(8) - 1.8007306155878887e13*coep(8)**2&
            - 1.8007306155878887e13*jx*coep(8)**2) + Q**4*((2.8981564779119175e10&
            + 2.8981564779119175e10*jx)*coen(5)**2 + (-4.2821364329346967e11&
            - 4.2821364329346967e11*jx)*coen(8)**2 + (3.1780125222615063e10&
            + 3.1780125222615063e10*jx)*coen(5)*coep(5) + 8.535525105315917e9*coep(5)**2&
            + 8.535525105315917e9*jx*coep(5)**2 + (-4.1823473410737286e11&
            - 4.1823473410737286e11*jx)*coen(8)*coep(8) - 9.903758717402663e10*coep(8)**2&
            - 9.903758717402663e10*jx*coep(8)**2) + Q**2*((298.90051539082947&
            + 298.90051539082947*jx)*coen(5)**2 + (-14860.000839790817&
            - 14860.000839790817*jx)*coen(8)**2 + (394.23679691098937&
            + 394.23679691098937*jx)*coen(5)*coep(5) + 129.21499266063702*coep(5)**2&
            + 129.21499266063702*jx*coep(5)**2 + (-17995.01812880219&
            - 17995.01812880219*jx)*coen(8)*coep(8) - 5381.118280827518*coep(8)**2&
            - 5381.118280827518*jx*coep(8)**2) + Q*((-0.008714049684056207&
            - 0.008714049684056207*jx)*coen(5)**2 + (1.087309598704508&
            + 1.087309598704508*jx)*coen(8)**2 + (-0.012383925150297988&
            - 0.012383925150297988*jx)*coen(5)*coep(5) - 0.004399837265352738*coep(5)**2&
            - 0.004399837265352738*jx*coep(5)**2 + (1.434114132869073&
            + 1.434114132869073*jx)*coen(8)*coep(8) + 0.470045030309619*coep(8)**2&
            + 0.470045030309619*jx*coep(8)**2) + Q**3*((-4.0850020224356716e6&
            - 4.0850020224356716e6*jx)*coen(5)**2 + (1.0542616003391336e8&
            + 1.0542616003391336e8*jx)*coen(8)**2 + (-4.946815699571542e6&
            - 4.946815699571542e6*jx)*coen(5)*coep(5) - 1.4792649944732718e6*coep(5)**2&
            - 1.4792649944732718e6*jx*coep(5)**2 + (1.1560647581151934e8&
            + 1.1560647581151934e8*jx)*coen(8)*coep(8) + 3.1049656655352976e7*coep(8)**2&
            + 3.1049656655352976e7*jx*coep(8)**2) + Q**5*((-1.1771557873700597e14&
            - 1.1771557873700597e14*jx)*coen(5)**2 + (1.0317172613074178e15&
            + 1.0317172613074178e15*jx)*coen(8)**2 + (-1.1497238480004781e14&
            - 1.1497238480004781e14*jx)*coen(5)*coep(5) - 2.7225351348549656e13*coep(5)**2&
            - 2.7225351348549656e13*jx*coep(5)**2 + (8.788251210682741e14&
            + 8.788251210682741e14*jx)*coen(8)*coep(8) + 1.791179863597674e14*coep(8)**2&
            + 1.791179863597674e14*jx*coep(8)**2) + Q**7*((-4.031151351751234e20&
            - 4.031151351751234e20*jx)*coen(5)**2 + (1.1665371904338209e21&
            + 1.1665371904338209e21*jx)*coen(8)**2 + (-2.908183928658652e20&
            - 2.908183928658652e20*jx)*coen(5)*coep(5) - 4.932890859975638e19*coep(5)**2&
            - 4.932890859975638e19*jx*coep(5)**2 + (6.805763278169367e20&
            + 6.805763278169367e20*jx)*coen(8)*coep(8) + 9.09794082989131e19*coep(8)**2&
            + 9.09794082989131e19*jx*coep(8)**2)) + mx**2*((1.571798541596684e-6&
            - 0.05391424322877117*Q + 736.8330975931669*Q**2 - 5.227555832778462e6*Q**3&
            + 2.123297223340024e10*Q**4 - 5.115769734372269e13*Q**5&
            + 7.271197715605338e16*Q**6 - 5.784274312982354e19*Q**7 + 2.2427482808305685e22*Q**8&
            - 2.9252012750195165e24*Q**9 + 1.19537184131952e26*Q**10)*coen(1)**2&
            + 5.239328471988946e-7*jx*v**2*coen(8)**2 + 5.239328471988946e-7*jx**2*v**2*coen(8)**2&
            + 7.936215706455649e-7*coep(1)**2&
            + coen(1)*(2.233753099445298e-6*coep(1) + Q**9*(-1.5061865690447188e23*coen(3)&
            - 6.54233106232376e23*coep(1) - 1.0912396604717182e22*coep(3))&
            + Q**10*(1.0084116071074022e25*coen(3) - 2.0662412443322892e19*coep(1)&
            - 8.715370320049313e17*coep(3)) + Q**7*(-8.997935028572886e17*coen(3)&
            - 3.37463751974455e19*coep(1) - 4.000948479891762e17*coep(3))&
            + Q**5*(-4.092776101994281e11*coen(3) - 4.357654102316605e13*coep(1)&
            - 2.9552467135223615e11*coep(3)) + Q**3*(-20266.038998937798*coen(3)&
            - 5.732346760434668e6*coep(1) - 19636.655361783545*coep(3))&
            + Q*(-0.00005686989688585233*coen(3) - 0.07111054502732674*coep(1)&
            - 0.00007012831198805887*coep(3)) + Q**2*(1.716569187178262*coen(3)&
            + 892.28292057601*coep(1) + 1.888306959760294*coep(3))&
            + Q**4*(1.2162951771835327e8*coen(3) + 2.0738168050987015e10*coep(1)&
            + 1.0280888687049219e8*coep(3)) + Q**6*(7.998843051597404e14*coen(3)&
            + 5.245642868119204e16*coep(1) + 4.7314729492042494e14*coep(3))&
            + Q**8*(5.46941376927486e20*coen(3) + 9.653408778384788e21*coep(1)&
            + 1.4882409621821597e20*coep(3))) + jx*Q**12*((-4.454325545332009e25&
            - 4.454325545332009e25*jx)*coen(5)**2 + (7.699454545698791e18&
            + 7.699454545698791e18*jx)*coen(5)*coep(5) + (-3.327192843112188e11&
            - 3.327192843112188e11*jx)*coep(5)**2) + 7.445843664817658e-7*jx*v**2*coen(8)*coep(8)&
            + 7.445843664817658e-7*jx**2*v**2*coen(8)*coep(8)&
            + 2.645405235485216e-7*jx*v**2*coep(8)**2 + 2.645405235485216e-7*jx**2*v**2*coep(8)**2&
            + Q*(coep(1)*(-0.00004041017505489685*coen(3) - 0.023307181434599845*coep(1)&
            - 0.000049831237947028466*coep(3)) + jx*(-2.1306022078591277e-9*coen(8)**2&
            + 0.00014402870925127705*coen(11)**2 - 0.00001895663229528411*coen(11)*coen(12)&
            + 6.237539547613185e-7*coen(12)**2 - 3.027893944128318e-9*coen(8)*coep(8)&
            - 1.0757688252493266e-9*coep(8)**2 + v**2*(0.00014402870925127705*coen(5)**2&
            - 0.01797141440959039*coen(8)**2 + 0.00020468563062307426*coen(5)*coep(5)&
            + 0.00007272197258685446*coep(5)**2 - 0.023703515009108906*coen(8)*coep(8)&
            - 0.007769060478199948*coep(8)**2) + 0.00020468563062307426*coen(11)*coep(11)&
            - 0.000013470058351632287*coen(12)*coep(11) + 0.00007272197258685446*coep(11)**2&
            - 0.000023376103996019625*coen(11)*coep(12) + 1.5383467999277948e-6*coen(12)*coep(12)&
            - 0.00001661041264900949*coep(11)*coep(12) + 9.48495339702351e-7*coep(12)**2)&
            + jx**2*(-2.1306022078591277e-9*coen(8)**2 + 0.00014402870925127705*coen(11)**2&
            - 0.00001895663229528411*coen(11)*coen(12) + 6.237539547613185e-7*coen(12)**2&
            - 3.027893944128318e-9*coen(8)*coep(8) - 1.0757688252493266e-9*coep(8)**2&
            + v**2*(0.00014402870925127705*coen(5)**2 - 0.01797141440959039*coen(8)**2&
            + 0.00020468563062307426*coen(5)*coep(5) + 0.00007272197258685446*coep(5)**2&
            - 0.023703515009108906*coen(8)*coep(8) - 0.007769060478199948*coep(8)**2)&
            + 0.00020468563062307426*coen(11)*coep(11) - 0.000013470058351632287*coen(12)*coep(11)&
            + 0.00007272197258685446*coep(11)**2 - 0.000023376103996019625*coen(11)*coep(12)&
            + 1.5383467999277948e-6*coen(12)*coep(12) - 0.00001661041264900949*coep(11)*coep(12)&
            + 9.48495339702351e-7*coep(12)**2)) + jx*Q**11*((1.090020553786356e24&
            + 1.095355790352618e28*v**2 + jx*(1.090020553786356e24&
            + 1.095355790352618e28*v**2))*coen(5)**2 + (-1.6203488022967725e23&
            - 1.6203488022967725e23*jx)*coen(8)**2 + 1.095355790352618e28*coen(11)**2&
            + 1.095355790352618e28*jx*coen(11)**2 - 9.24038407734781e26*coen(11)*coen(15)&
            - 9.24038407734781e26*jx*coen(11)*coen(15) + 1.9487891205974223e25*coen(15)**2&
            + 1.9487891205974223e25*jx*coen(15)**2 + (2.437875092051656e23&
            - 1.8933600684005112e21*v**2 + jx*(2.437875092051656e23&
            - 1.8933600684005112e21*v**2))*coen(5)*coep(5) - 2.1069761877228364e16*coep(5)**2&
            - 2.1069761877228364e16*jx*coep(5)**2 + 8.181844612013153e13*v**2*coep(5)**2&
            + 8.181844612013153e13*jx*v**2*coep(5)**2 + (2.800828503551053e16&
            + 2.800828503551053e16*jx)*coen(8)*coep(8) - 1.2103320431972122e9*coep(8)**2&
            - 1.2103320431972122e9*jx*coep(8)**2 - 1.8933600684005112e21*coen(11)*coep(11)&
            - 1.8933600684005112e21*jx*coen(11)*coep(11) + 7.986160470791925e19*coen(15)*coep(11)&
            + 7.986160470791925e19*jx*coen(15)*coep(11) + 8.181844612013153e13*coep(11)**2&
            + 8.181844612013153e13*jx*coep(11)**2 + 7.98616046919381e19*coen(11)*coep(15)&
            + 7.98616046919381e19*jx*coen(11)*coep(15) - 3.368548862782144e18*coen(15)*coep(15)&
            - 3.368548862782144e18*jx*coen(15)*coep(15) - 6.902176199453235e12*coep(11)*coep(15)&
            - 6.902176199453235e12*jx*coep(11)*coep(15) + 1.455663073164157e11*coep(15)**2&
            + 1.455663073164157e11*jx*coep(15)**2) + Q**2*(0.000514408349132658*coen(3)**2&
            + 266.8227340041774*coep(1)**2 + coen(3)*(1.1200777321761246*coep(1)&
            + 0.001268670814355273*coep(3)) + 1.21887345733249*coep(1)*coep(3)&
            + 0.0007822217689072734*coep(3)**2 + jx*((-5.857007965171126e-7&
            - 4.940327056552155*v**2)*coen(5)**2&
            + (0.00007308176119159993 + 245.61103253105563*v**2)*coen(8)**2&
            - 4.940327056552155*coen(11)**2 + 0.5721897290594207*coen(11)*coen(12)&
            - 0.016259582004823983*coen(12)**2 + 0.005211162643911064*coen(11)*coen(15)&
            - 0.00034293889942177186*coen(12)*coen(15) + (-8.323655576360668e-7&
            - 6.51607680207959*v**2)*coen(5)*coep(5) - 2.957279662494694e-7*coep(5)**2&
            - 2.135708342686647*v**2*coep(5)**2 + (0.00009639166866981644&
            + 297.42764019200337*v**2)*coen(8)*coep(8) + 0.00003159331867879656*coep(8)**2&
            + 88.94091133472578*v**2*coep(8)**2 - 6.51607680207959*coen(11)*coep(11)&
            + 0.3733592440587083*coen(12)*coep(11) + 0.003702907974365833*coen(15)*coep(11)&
            - 2.135708342686647*coep(11)**2 + 0.6294356532534313*coen(11)*coep(12)&
            - 0.03508909529451164*coen(12)*coep(12) - 0.0004228902714517576*coen(15)*coep(12)&
            + 0.40629115244416336*coep(11)*coep(12) - 0.018544910030557284*coep(12)**2&
            + 0.006426071783570032*coen(11)*coep(15) - 0.0004228902714517576*coen(12)*coep(15)&
            + 0.004566188790716783*coep(11)*coep(15) - 0.0005214811792715156*coep(12)*coep(15))&
            + jx**2*((-5.857007965171126e-7 - 4.940327056552155*v**2)*coen(5)**2&
            + (0.00007308176119159993 + 245.61103253105563*v**2)*coen(8)**2&
            - 4.940327056552155*coen(11)**2 + 0.5721897290594207*coen(11)*coen(12)&
            - 0.016259582004823983*coen(12)**2 + 0.005211162643911064*coen(11)*coen(15)&
            - 0.00034293889942177186*coen(12)*coen(15) + (-8.323655576360668e-7&
            - 6.51607680207959*v**2)*coen(5)*coep(5) - 2.957279662494694e-7*coep(5)**2&
            - 2.135708342686647*v**2*coep(5)**2 + (0.00009639166866981644&
            + 297.42764019200337*v**2)*coen(8)*coep(8) + 0.00003159331867879656*coep(8)**2&
            + 88.94091133472578*v**2*coep(8)**2 - 6.51607680207959*coen(11)*coep(11)&
            + 0.3733592440587083*coen(12)*coep(11) + 0.003702907974365833*coen(15)*coep(11)&
            - 2.135708342686647*coep(11)**2 + 0.6294356532534313*coen(11)*coep(12)&
            - 0.03508909529451164*coen(12)*coep(12) - 0.0004228902714517576*coen(15)*coep(12)&
            + 0.40629115244416336*coep(11)*coep(12) - 0.018544910030557284*coep(12)**2&
            + 0.006426071783570032*coen(11)*coep(15) - 0.0004228902714517576*coen(12)*coep(15)&
            + 0.004566188790716783*coep(11)*coep(15) - 0.0005214811792715156*coep(12)*coep(15)))&
            + Q**8*(2.503054034367967e18*coen(3)**2 + 8.951633399150473e20*coep(1)**2&
            + 2.9862085013068722e19*coep(1)*coep(3) + 2.4904511215290832e17*coep(3)**2&
            + coen(3)*(1.4406123271868126e20*coep(1) + 1.8886512546240353e18*coep(3))&
            + jx*((-2.7094733714012e16 - 5.300307521648889e21*v**2)*coen(5)**2&
            + (7.840691600072322e16 + 7.475827602768562e21*v**2)*coen(8)**2&
            - 5.300307521648889e21*coen(11)**2 + 1.8231379230916202e20*coen(11)*coen(12)&
            - 1.3929136827947515e18*coen(12)**2 + 8.245083156622134e19*coen(11)*coen(15)&
            - 1.6687026895786445e18*coen(12)*coen(15) - 2.6054464707164256e17*coen(15)**2&
            + (-1.9546889278703036e16 - 3.0922836056712366e21*v**2)*coen(5)*coep(5)&
            - 3.3155630396576475e15*coep(5)**2 - 4.133763124539246e20*v**2*coep(5)**2&
            + (4.574384032058043e16 + 3.2178029261282623e21*v**2)*coen(8)*coep(8)&
            + 6.115034207898294e15*coep(8)**2 + 2.983877799716825e20*v**2*coep(8)**2&
            - 3.0922836056712366e21*coen(11)*coep(11) + 4.80204109062271e19*coen(12)*coep(11)&
            + 2.7406221103060386e19*coen(15)*coep(11) - 4.133763124539246e20*coep(11)**2&
            + 4.960803207273866e19*coen(11)*coep(12) - 5.581238673686815e17*coen(12)*coep(12)&
            - 6.295504182080115e17*coen(15)*coep(12) + 9.954028337689573e18*coep(11)*coep(12)&
            + 4.823682317311459e10*coep(12)**2 + 3.6661915003069956e19*coen(11)*coep(15)&
            - 6.295504182080115e17*coen(12)*coep(15) - 2.7725435845974864e17*coen(15)*coep(15)&
            + 1.0732822523546573e19*coep(11)*coep(15) - 1.6603007476860547e17*coep(12)*coep(15)&
            - 6.40138914179535e16*coep(15)**2) + jx**2*((-2.7094733714012e16&
            - 5.300307521648889e21*v**2)*coen(5)**2 + (7.840691600072322e16&
            + 7.475827602768562e21*v**2)*coen(8)**2 - 5.300307521648889e21*coen(11)**2&
            + 1.8231379230916202e20*coen(11)*coen(12) - 1.3929136827947515e18*coen(12)**2&
            + 8.245083156622134e19*coen(11)*coen(15) - 1.6687026895786445e18*coen(12)*coen(15)&
            - 2.6054464707164256e17*coen(15)**2 + (-1.9546889278703036e16&
            - 3.0922836056712366e21*v**2)*coen(5)*coep(5) - 3.3155630396576475e15*coep(5)**2&
            - 4.133763124539246e20*v**2*coep(5)**2 + (4.574384032058043e16&
            + 3.2178029261282623e21*v**2)*coen(8)*coep(8) + 6.115034207898294e15*coep(8)**2&
            + 2.983877799716825e20*v**2*coep(8)**2 - 3.0922836056712366e21*coen(11)*coep(11)&
            + 4.80204109062271e19*coen(12)*coep(11) + 2.7406221103060386e19*coen(15)*coep(11)&
            - 4.133763124539246e20*coep(11)**2 + 4.960803207273866e19*coen(11)*coep(12)&
            - 5.581238673686815e17*coen(12)*coep(12) - 6.295504182080115e17*coen(15)*coep(12)&
            + 9.954028337689573e18*coep(11)*coep(12) + 4.823682317311459e10*coep(12)**2&
            + 3.6661915003069956e19*coen(11)*coep(15) - 6.295504182080115e17*coen(12)*coep(15)&
            - 2.7725435845974864e17*coen(15)*coep(15)&
            + 1.0732822523546573e19*coep(11)*coep(15) - 1.6603007476860547e17*coep(12)*coep(15)&
            - 6.40138914179535e16*coep(15)**2)) + Q**6*(1.822332565097005e12*coen(3)**2&
            + 8.897712247098668e15*coep(1)**2 + 1.7550570331684394e14*coep(1)*coep(3)&
            + 7.627920911150547e11*coep(3)**2 + coen(3)*(3.191570291877634e14*coep(1)&
            + 2.446583073552944e12*coep(3)) + jx*((-7.912062786936547e9&
            - 4.687736323510664e15*v**2)*coen(5)**2 + (6.934521188625243e10&
            + 2.4237325718684456e16*v**2)*coen(8)**2 - 4.687736323510664e15*coen(11)**2&
            + 2.666281017199135e14*coen(11)*coen(12) - 3.4477460622225703e12*coen(12)**2&
            + 3.750335959886431e13*coen(11)*coen(15) - 1.2148883767313364e12*coen(12)*coen(15)&
            - 6.1087029903460365e10*coen(15)**2 + (-7.72768343036517e9&
            - 3.9930517754688125e15*v**2)*coen(5)*coep(5) - 1.8299080850410411e9*coep(5)**2&
            - 8.138449576667291e14*v**2*coep(5)**2 + (5.906881324658007e10&
            + 1.7485476227064012e16*v**2)*coen(8)*coep(8) + 1.2039126592703096e10*coep(8)**2&
            + 2.9659040823662225e15*v**2*coep(8)**2 - 3.9930517754688125e15*coen(11)*coep(11)&
            + 1.0638567639592112e14*coen(12)*coep(11) + 1.7428719130138465e13*coen(15)*coep(11)&
            - 8.138449576667291e14*coep(11)**2 + 1.57715764973475e14*coen(11)*coep(12)&
            - 3.668863027344394e12*coen(12)*coep(12) - 8.155276911843143e11*coen(15)*coep(12)&
            + 5.850190110561465e13*coep(11)*coep(12) - 8.470856897056304e11*coep(12)**2&
            + 2.7079829787558203e13*coen(11)*coep(15) - 8.155276911843143e11*coen(12)*coep(15)&
            - 9.808941950630452e10*coen(15)*coep(15) + 1.2017281514872785e13*coep(11)*coep(15)&
            - 5.0852806074336957e11*coep(12)*coep(15) - 3.762984544915445e10*coep(15)**2)&
            + jx**2*((-7.912062786936547e9 - 4.687736323510664e15*v**2)*coen(5)**2&
            + (6.934521188625243e10 + 2.4237325718684456e16*v**2)*coen(8)**2&
            - 4.687736323510664e15*coen(11)**2 + 2.666281017199135e14*coen(11)*coen(12)&
            - 3.4477460622225703e12*coen(12)**2 + 3.750335959886431e13*coen(11)*coen(15)&
            - 1.2148883767313364e12*coen(12)*coen(15) - 6.1087029903460365e10*coen(15)**2&
            + (-7.72768343036517e9 - 3.9930517754688125e15*v**2)*coen(5)*coep(5)&
            - 1.8299080850410411e9*coep(5)**2 - 8.138449576667291e14*v**2*coep(5)**2&
            + (5.906881324658007e10 + 1.7485476227064012e16*v**2)*coen(8)*coep(8)&
            + 1.2039126592703096e10*coep(8)**2 + 2.9659040823662225e15*v**2*coep(8)**2&
            - 3.9930517754688125e15*coen(11)*coep(11) + 1.0638567639592112e14*coen(12)*coep(11)&
            + 1.7428719130138465e13*coen(15)*coep(11) - 8.138449576667291e14*coep(11)**2&
            + 1.57715764973475e14*coen(11)*coep(12) - 3.668863027344394e12*coen(12)*coep(12)&
            - 8.155276911843143e11*coen(15)*coep(12) + 5.850190110561465e13*coep(11)*coep(12)&
            - 8.470856897056304e11*coep(12)**2 + 2.7079829787558203e13*coen(11)*coep(15)&
            - 8.155276911843143e11*coen(12)*coep(15) - 9.808941950630452e10*coen(15)*coep(15)&
            + 1.2017281514872785e13*coep(11)*coep(15) - 5.0852806074336957e11*coep(12)*coep(15)&
            - 3.762984544915445e10*coep(15)**2)) + Q**4*(134199.28738206218*coen(3)**2&
            + 4.910778466457921e9*coep(1)**2 + 5.2790137229142725e7*coep(1)*coep(3)&
            + 113907.9593101974*coep(3)**2 + coen(3)*(6.439547690233156e7*coep(1)&
            + 251126.6481669652*coep(3)) + jx*((-274.56682312613225&
            - 4.790169345552566e8*v**2)*coen(5)**2 + (7086.049327740483&
            + 7.077657411133414e9*v**2)*coen(8)**2 - 4.790169345552566e8*coen(11)**2&
            + 4.0543172572784424e7*coen(11)*coen(12) - 808354.9946992971*coen(12)**2&
            + 1.8570391569952401e6*coen(11)*coen(15) - 89466.19158804143*coen(12)*coen(15)&
            - 1228.7294303363972*coen(15)**2 + (-332.4922396028563&
            - 5.2527247165368307e8*v**2)*coen(5)*coep(5) - 99.42641101853158*coep(5)**2&
            - 1.4107799568205816e8*v**2*coep(5)**2 + (7770.302835113656&
            + 6.912722683662336e9*v**2)*coen(8)*coep(8) + 2086.9525988470145*coep(8)**2&
            + 1.6369261554859734e9*v**2*coep(8)**2 - 5.2527247165368307e8*coen(11)*coep(11)&
            + 2.1465158967443857e7*coen(12)*coep(11) + 1.1004413120673664e6*coen(15)*coep(11)&
            - 1.4107799568205816e8*coep(11)**2 + 3.426962895683073e7*coen(11)*coep(12)&
            - 1.2980017576625438e6*coen(12)*coep(12) - 83708.88272232172*coen(15)*coep(12)&
            + 1.7596712409714244e7*coep(11)*coep(12) - 497949.786831321*coep(12)**2&
            + 1.7993668087367178e6*coen(11)*coep(15) - 83708.88272232172*coen(12)*coep(15)&
            - 2651.667432745395*coen(15)*coep(15) + 1.0450650234182627e6*coep(11)*coep(15)&
            - 75938.63954013158*coep(12)*coep(15) - 1401.4306598242138*coep(15)**2)&
            + jx**2*((-274.56682312613225 - 4.790169345552566e8*v**2)*coen(5)**2&
            + (7086.049327740483 + 7.077657411133414e9*v**2)*coen(8)**2&
            - 4.790169345552566e8*coen(11)**2 + 4.0543172572784424e7*coen(11)*coen(12)&
            - 808354.9946992971*coen(12)**2 + 1.8570391569952401e6*coen(11)*coen(15)&
            - 89466.19158804143*coen(12)*coen(15) - 1228.7294303363972*coen(15)**2&
            + (-332.4922396028563 - 5.2527247165368307e8*v**2)*coen(5)*coep(5)&
            - 99.42641101853158*coep(5)**2 - 1.4107799568205816e8*v**2*coep(5)**2&
            + (7770.302835113656 + 6.912722683662336e9*v**2)*coen(8)*coep(8)&
            + 2086.9525988470145*coep(8)**2 + 1.6369261554859734e9*v**2*coep(8)**2&
            - 5.2527247165368307e8*coen(11)*coep(11) + 2.1465158967443857e7*coen(12)*coep(11)&
            + 1.1004413120673664e6*coen(15)*coep(11) - 1.4107799568205816e8*coep(11)**2&
            + 3.426962895683073e7*coen(11)*coep(12) - 1.2980017576625438e6*coen(12)*coep(12)&
            - 83708.88272232172*coen(15)*coep(12) + 1.7596712409714244e7*coep(11)*coep(12)&
            - 497949.786831321*coep(12)**2 + 1.7993668087367178e6*coen(11)*coep(15)&
            - 83708.88272232172*coen(12)*coep(15) - 2651.667432745395*coen(15)*coep(15)&
            + 1.0450650234182627e6*coep(11)*coep(15) - 75938.63954013158*coep(12)*coep(15)&
            - 1401.4306598242138*coep(15)**2)) + Q**3*(-13.409237204578709*coen(3)**2&
            - 1.5395971332186065e6*coep(1)**2 + coen(3)*(-12009.217179073565*coep(1)&
            - 28.93789040570522*coep(3)) - 11404.890651465485*coep(1)*coep(3)&
            - 15.293941594779984*coep(3)**2 + jx*((0.020090116110319748&
            + 67518.27105791736*v**2)*coen(5)**2 + (-0.998790991981026&
            - 1.7425186109261538e6*v**2)*coen(8)**2 + 67518.27105791736*coen(11)**2&
            - 6755.346332979267*coen(11)*coen(12) + 162.7254619250458*coen(12)**2&
            - 157.29448642868016*coen(11)*coen(15) + 8.939491469719137*coen(12)*coen(15)&
            + 0.04713681085261371*coen(15)**2 + (0.0264979905255303&
            + 81762.61393331834*v**2)*coen(5)*coep(5) + 0.008684977348908104*coep(5)**2&
            + 24449.783455378398*v**2*coep(5)**2 + (-1.20950612327394&
            - 1.9107822534782225e6*v**2)*coen(8)*coep(8) - 0.36168318720973963*coep(8)**2&
            - 513199.0444062021*v**2*coep(8)**2 + 81762.61393331834*coen(11)*coep(11)&
            - 4003.072393024522*coen(12)*coep(11) - 102.63614945369991*coen(15)*coep(11)&
            + 24449.783455378398*coep(11)**2 - 6545.551787261183*coen(11)*coep(12)&
            + 304.5075769166869*coen(12)*coep(12) + 9.645963468568404*coen(15)*coep(12)&
            - 3801.6302171551615*coep(11)*coep(12) + 138.12089212456416*coep(12)**2&
            - 173.03134395849608*coen(11)*coep(15) + 9.645963468568404*coen(12)*coep(15)&
            + 0.11625218819120321*coen(15)*coep(15) - 111.68910401323303*coep(11)*coep(15)&
            + 10.195961063186653*coep(12)*coep(15) + 0.07167737387611973*coep(15)**2)&
            + jx**2*((0.020090116110319748 + 67518.27105791736*v**2)*coen(5)**2&
            + (-0.998790991981026 - 1.7425186109261538e6*v**2)*coen(8)**2&
            + 67518.27105791736*coen(11)**2 - 6755.346332979267*coen(11)*coen(12)&
            + 162.7254619250458*coen(12)**2 - 157.29448642868016*coen(11)*coen(15)&
            + 8.939491469719137*coen(12)*coen(15) + 0.04713681085261371*coen(15)**2&
            + (0.0264979905255303 + 81762.61393331834*v**2)*coen(5)*coep(5)&
            + 0.008684977348908104*coep(5)**2 + 24449.783455378398*v**2*coep(5)**2&
            + (-1.20950612327394 - 1.9107822534782225e6*v**2)*coen(8)*coep(8)&
            - 0.36168318720973963*coep(8)**2 - 513199.0444062021*v**2*coep(8)**2&
            + 81762.61393331834*coen(11)*coep(11) - 4003.072393024522*coen(12)*coep(11)&
            - 102.63614945369991*coen(15)*coep(11) + 24449.783455378398*coep(11)**2&
            - 6545.551787261183*coen(11)*coep(12) + 304.5075769166869*coen(12)*coep(12)&
            + 9.645963468568404*coen(15)*coep(12) - 3801.6302171551615*coep(11)*coep(12)&
            + 138.12089212456416*coep(12)**2 - 173.03134395849608*coen(11)*coep(15)&
            + 9.645963468568404*coen(12)*coep(15) + 0.11625218819120321*coen(15)*coep(15)&
            - 111.68910401323303*coep(11)*coep(15) + 10.195961063186653*coep(12)*coep(15)&
            + 0.07167737387611973*coep(15)**2)) + Q**5*(-6.666483717855073e8*coen(3)**2&
            - 8.881564822709385e12*coep(1)**2 + coen(3)*(-1.9020121372369354e11*coep(1)&
            - 1.070458850374706e9*coep(3)) - 1.31145697668372e11*coep(1)*coep(3)&
            - 4.1065796190883505e8*coep(3)**2 + jx*((1.9479491385618441e6&
            + 1.9456422075810469e12*v**2)*coen(5)**2 + (-2.878168946125809e7&
            - 1.7052565781240893e13*v**2)*coen(8)**2 + 1.9456422075810469e12*coen(11)**2&
            - 1.3642587006647604e11*coen(11)*coen(12) + 2.2096980857448344e9*coen(12)**2&
            - 1.1145284831498987e10*coen(11)*coen(15) + 4.4443224785700476e8*coen(12)*coen(15)&
            + 1.229709128273209e7*coen(15)**2 + (2.1360498655815516e6&
            + 1.9003017865034724e12*v**2)*coen(5)*coep(5) + 573701.5548605745*coep(5)**2&
            + 4.499896553055766e11*v**2*coep(5)**2 + (-2.8110973173128292e7&
            - 1.452551367438868e13*v**2)*coen(8)*coep(8) - 6.656651705703798e6*coep(8)**2&
            - 2.960521607569795e12*v**2*coep(8)**2 + 1.9003017865034724e12*coen(11)*coep(11)&
            - 6.3400404574564514e10*coen(12)*coep(11) - 5.900754565175763e9*coen(15)*coep(11)&
            + 4.499896553055766e11*coep(11)**2 - 9.850822378407872e10*coen(11)*coep(12)&
            + 2.9666428827483935e9*coen(12)*coep(12) + 3.568196167915686e8*coen(15)*coep(12)&
            - 4.3715232556124e10*coep(11)*coep(12) + 9.249355775346704e8*coep(12)**2&
            - 9.420692845582922e9*coen(11)*coep(15) + 3.568196167915686e8*coen(12)*coep(15)&
            + 2.3011503088268213e7*coen(15)*coep(15) - 4.8373217846273985e9*coep(11)*coep(15)&
            + 2.7377197460589e8*coep(12)*coep(15) + 1.0437734810613163e7*coep(15)**2)&
            + jx**2*((1.9479491385618441e6 + 1.9456422075810469e12*v**2)*coen(5)**2&
            + (-2.878168946125809e7 - 1.7052565781240893e13*v**2)*coen(8)**2&
            + 1.9456422075810469e12*coen(11)**2 - 1.3642587006647604e11*coen(11)*coen(12)&
            + 2.2096980857448344e9*coen(12)**2 - 1.1145284831498987e10*coen(11)*coen(15)&
            + 4.4443224785700476e8*coen(12)*coen(15) + 1.229709128273209e7*coen(15)**2&
            + (2.1360498655815516e6 + 1.9003017865034724e12*v**2)*coen(5)*coep(5)&
            + 573701.5548605745*coep(5)**2 + 4.499896553055766e11*v**2*coep(5)**2&
            + (-2.8110973173128292e7 - 1.452551367438868e13*v**2)*coen(8)*coep(8)&
            - 6.656651705703798e6*coep(8)**2 - 2.960521607569795e12*v**2*coep(8)**2&
            + 1.9003017865034724e12*coen(11)*coep(11) - 6.3400404574564514e10*coen(12)*coep(11)&
            - 5.900754565175763e9*coen(15)*coep(11) + 4.499896553055766e11*coep(11)**2&
            - 9.850822378407872e10*coen(11)*coep(12) + 2.9666428827483935e9*coen(12)*coep(12)&
            + 3.568196167915686e8*coen(15)*coep(12) - 4.3715232556124e10*coep(11)*coep(12)&
            + 9.249355775346704e8*coep(12)**2 - 9.420692845582922e9*coen(11)*coep(15)&
            + 3.568196167915686e8*coen(12)*coep(15) + 2.3011503088268213e7*coen(15)*coep(15)&
            - 4.8373217846273985e9*coep(11)*coep(15) + 2.7377197460589e8*coep(12)*coep(15)&
            + 1.0437734810613163e7*coep(15)**2)) + Q**7*(-2.8433476798959765e15*coen(3)**2&
            - 4.511213690821386e18*coep(1)**2 + coen(3)*(-2.9908660977661734e17*coep(1)&
            - 3.0257022960475725e15*coep(3)) - 1.171282786426621e17*coep(1)*coep(3)&
            - 6.98589480498144e14*coep(3)**2 + jx*((1.906294177608936e13&
            + 6.662820927582926e18*v**2)*coen(5)**2 + (-9.856243975714389e13&
            - 1.9280914376607846e19*v**2)*coen(8)**2 + 6.662820927582926e18*coen(11)**2&
            - 2.9993116761909626e17*coen(11)*coen(12) + 3.0351176367000615e15*coen(12)**2&
            - 7.329584611109366e16*coen(11)*coen(15) + 1.89556511993065e15*coen(12)*coen(15)&
            + 1.669859083282178e14*coen(15)**2 + (1.623796823274953e13&
            + 4.806743049404353e18*v**2)*coen(5)*coep(5) + 3.309546009436398e12*coep(5)**2&
            + 8.153245955662266e17*v**2*coep(5)**2 + (-7.110566641130698e13&
            - 1.124879173248183e19*v**2)*coen(8)*coep(8) - 1.2061014727311045e13*coep(8)**2&
            - 1.503737896940462e18*v**2*coep(8)**2 + 4.806743049404353e18*coen(11)*coep(11)&
            - 9.969553659220581e16*coen(12)*coep(11) - 2.9245335038732224e16*coen(15)*coep(11)&
            + 8.153245955662266e17*coep(11)**2 - 1.3336494932972539e17*coen(11)*coep(12)&
            + 2.2901138584219715e15*coen(12)*coep(12) + 1.0085674320158574e15*coen(15)*coep(12)&
            - 3.904275954755404e16*coep(11)*coep(12) + 3.0198357760187156e14*coep(12)**2&
            - 4.33559342178142e16*coen(11)*coep(15) + 1.0085674320158574e15*coen(12)*coep(15)&
            + 2.241878922994135e14*coen(15)*coep(15) - 1.6082124550952016e16*coep(11)*coep(15)&
            + 4.657263203320959e14*coep(12)*coep(15) + 6.989697305532578e13*coep(15)**2)&
            + jx**2*((1.906294177608936e13 + 6.662820927582926e18*v**2)*coen(5)**2&
            + (-9.856243975714389e13 - 1.9280914376607846e19*v**2)*coen(8)**2&
            + 6.662820927582926e18*coen(11)**2 - 2.9993116761909626e17*coen(11)*coen(12)&
            + 3.0351176367000615e15*coen(12)**2 - 7.329584611109366e16*coen(11)*coen(15)&
            + 1.89556511993065e15*coen(12)*coen(15) + 1.669859083282178e14*coen(15)**2&
            + (1.623796823274953e13 + 4.806743049404353e18*v**2)*coen(5)*coep(5)&
            + 3.309546009436398e12*coep(5)**2 + 8.153245955662266e17*v**2*coep(5)**2&
            + (-7.110566641130698e13 - 1.124879173248183e19*v**2)*coen(8)*coep(8)&
            - 1.2061014727311045e13*coep(8)**2 - 1.503737896940462e18*v**2*coep(8)**2&
            + 4.806743049404353e18*coen(11)*coep(11) - 9.969553659220581e16*coen(12)*coep(11)&
            - 2.9245335038732224e16*coen(15)*coep(11) + 8.153245955662266e17*coep(11)**2&
            - 1.3336494932972539e17*coen(11)*coep(12) + 2.2901138584219715e15*coen(12)*coep(12)&
            + 1.0085674320158574e15*coen(15)*coep(12) - 3.904275954755404e16*coep(11)*coep(12)&
            + 3.0198357760187156e14*coep(12)**2 - 4.33559342178142e16*coen(11)*coep(15)&
            + 1.0085674320158574e15*coen(12)*coep(15) + 2.241878922994135e14*coen(15)*coep(15)&
            - 1.6082124550952016e16*coep(11)*coep(15) + 4.657263203320959e14*coep(12)*coep(15)&
            + 6.989697305532578e13*coep(15)**2)) + Q**10*(2.1267314784377652e23*coen(3)**2&
            + 8.928922223621841e11*coep(1)**2 + coen(3)*(-8.715370321793352e17*coep(1)&
            - 3.676128333956584e16*coep(3)) + 7.532408323688248e10*coep(1)*coep(3)&
            + 1.588578490600674e9*coep(3)**2 + jx*((-8.35717440694068e21&
            - 2.680451424221754e26*v**2)*coen(5)**2 + (3.965164828730406e21&
            + 3.9845728043984e25*v**2)*coen(8)**2 - 2.680451424221754e26*coen(11)**2&
            + 3.361372023691341e24*coen(11)*coen(12) + 1.3801648346788235e25*coen(11)*coen(15)&
            - 1.4178209856251763e23*coen(12)*coen(15) - 1.0526187176646891e23*coen(15)**2&
            + (-3.5971589621541015e21 - 5.994938113658161e25*v**2)*coen(5)*coep(5)&
            - 3.335655730209314e20*coep(5)**2 + 5.181230118611081e18*v**2*coep(5)**2&
            + (8.868251647423316e20 - 6.887470814440963e18*v**2)*coen(8)*coep(8)&
            - 7.664541595578523e13*coep(8)**2&
            + 2.9763074078739465e11*v**2*coep(8)**2 - 5.994938113658161e25*coen(11)*coep(11)&
            - 2.905123440597784e17*coen(12)*coep(11) + 2.528655431393664e24*coen(15)*coep(11)&
            + 5.181230118611081e18*coep(11)**2 - 2.905123440016438e17*coen(11)*coep(12)&
            + 1.2253761113188612e16*coen(15)*coep(12) + 2.5108027745627495e10*coep(11)*coep(12)&
            + 9.999362871394775e23*coen(11)*coep(15) + 1.2253761113188612e16*coen(12)*coep(15)&
            - 4.217717413680161e22*coen(15)*coep(15) - 3.049646961244186e17*coep(11)*coep(15)&
            - 1.0590523270671158e9*coep(12)*coep(15) + 3.6452354212521615e15*coep(15)**2)&
            + jx**2*((-8.35717440694068e21 - 2.680451424221754e26*v**2)*coen(5)**2&
            + (3.965164828730406e21 + 3.9845728043984e25*v**2)*coen(8)**2&
            - 2.680451424221754e26*coen(11)**2 + 3.361372023691341e24*coen(11)*coen(12)&
            + 1.3801648346788235e25*coen(11)*coen(15) - 1.4178209856251763e23*coen(12)*coen(15)&
            - 1.0526187176646891e23*coen(15)**2 + (-3.5971589621541015e21&
            - 5.994938113658161e25*v**2)*coen(5)*coep(5) - 3.335655730209314e20*coep(5)**2&
            + 5.181230118611081e18*v**2*coep(5)**2 + (8.868251647423316e20&
            - 6.887470814440963e18*v**2)*coen(8)*coep(8) - 7.664541595578523e13*coep(8)**2&
            + 2.9763074078739465e11*v**2*coep(8)**2 - 5.994938113658161e25*coen(11)*coep(11)&
            - 2.905123440597784e17*coen(12)*coep(11) + 2.528655431393664e24*coen(15)*coep(11)&
            + 5.181230118611081e18*coep(11)**2 - 2.905123440016438e17*coen(11)*coep(12)&
            + 1.2253761113188612e16*coen(15)*coep(12) + 2.5108027745627495e10*coep(11)*coep(12)&
            + 9.999362871394775e23*coen(11)*coep(15) + 1.2253761113188612e16*coen(12)*coep(15)&
            - 4.217717413680161e22*coen(15)*coep(15) - 3.049646961244186e17*coep(11)*coep(15)&
            - 1.0590523270671158e9*coep(12)*coep(15) + 3.6452354212521615e15*coep(15)**2))&
            + Q**9*(-1.1487324811029375e21*coen(3)**2 + 5.654324048618456e16*coep(1)**2&
            + coen(3)*(-2.7595449129041295e22*coep(1) - 4.6028337781765105e20*coep(3))&
            + 3.3281077578120905e15*coep(1)*coep(3) + 3.978078918213955e13*coep(3)**2&
            + jx*((2.1553996792399032e19 + 2.05509886613976e24*v**2)*coen(5)**2&
            + (-3.040087080088403e19 - 9.75067091673172e23*v**2)*coen(8)**2&
            + 2.05509886613976e24*coen(11)**2 - 5.0206218968157305e22*coen(11)*coen(12)&
            + 2.5788017877774434e20*coen(12)**2 - 5.011791172357212e22*coen(11)*coen(15)&
            + 7.658216540686249e20*coen(12)*coen(15) + 2.293624992106179e20*coen(15)**2&
            + (1.2574944122693329e19 + 8.845713807657101e23*v**2)*coen(5)*coep(5)&
            + 1.6810178798670572e18*coep(5)**2 + 8.202655556993706e22*v**2*coep(5)**2&
            + (-1.3085375455114056e19 - 2.1807770207745867e23*v**2)*coen(8)*coep(8)&
            - 1.213410585354099e18*coep(8)**2 + 1.8847746828728184e16*v**2*coep(8)**2&
            + 8.845713807657101e23*coen(11)*coep(11) - 9.198483043013765e21*coen(12)*coep(11)&
            - 1.3200771506342055e22*coen(15)*coep(11) + 8.202655556993706e22*coep(11)**2&
            - 3.6374655349057273e21*coen(11)*coep(12) - 4.457547375313358e13*coen(12)*coep(12)&
            + 1.5342779260588363e20*coen(15)*coep(12) + 1.1093692526040301e15*coep(11)*coep(12)&
            + 1.9262558969574643e6*coep(12)**2 - 1.363720726068571e22*coen(11)*coep(15)&
            + 1.5342779260588363e20*coen(12)*coep(15) + 1.7306289275022354e20*coen(15)*coep(15)&
            - 2.7363542121721904e21*coep(11)*coep(15) - 2.6520526121426355e13*coep(12)*coep(15)&
            + 2.2820765574884266e19*coep(15)**2) + jx**2*((2.1553996792399032e19&
            + 2.05509886613976e24*v**2)*coen(5)**2 + (-3.040087080088403e19&
            - 9.75067091673172e23*v**2)*coen(8)**2 + 2.05509886613976e24*coen(11)**2&
            - 5.0206218968157305e22*coen(11)*coen(12) + 2.5788017877774434e20*coen(12)**2&
            - 5.011791172357212e22*coen(11)*coen(15) + 7.658216540686249e20*coen(12)*coen(15)&
            + 2.293624992106179e20*coen(15)**2 + (1.2574944122693329e19&
            + 8.845713807657101e23*v**2)*coen(5)*coep(5) + 1.6810178798670572e18*coep(5)**2&
            + 8.202655556993706e22*v**2*coep(5)**2 + (-1.3085375455114056e19&
            - 2.1807770207745867e23*v**2)*coen(8)*coep(8) - 1.213410585354099e18*coep(8)**2&
            + 1.8847746828728184e16*v**2*coep(8)**2 + 8.845713807657101e23*coen(11)*coep(11)&
            - 9.198483043013765e21*coen(12)*coep(11) - 1.3200771506342055e22*coen(15)*coep(11)&
            + 8.202655556993706e22*coep(11)**2 - 3.6374655349057273e21*coen(11)*coep(12)&
            - 4.457547375313358e13*coen(12)*coep(12) + 1.5342779260588363e20*coen(15)*coep(12)&
            + 1.1093692526040301e15*coep(11)*coep(12) + 1.9262558969574643e6*coep(12)**2&
            - 1.363720726068571e22*coen(11)*coep(15) + 1.5342779260588363e20*coen(12)*coep(15)&
            + 1.7306289275022354e20*coen(15)*coep(15) - 2.7363542121721904e21*coep(11)*coep(15)&
            - 2.6520526121426355e13*coep(12)*coep(15)&
            + 2.2820765574884266e19*coep(15)**2))))/(EXP(16293.284150525782*Q)*mx**2)
        elseif (A==131) then 
            ans=(mx*Q*(-5.929722744083378e-12*coen(7)**2 - 5.335632262545173e-7*jx*coen(8)**2&
            - 5.335632262545173e-7*jx**2*coen(8)**2 - 3.953148496055584e-12*jx*coen(12)**2&
            - 3.953148496055584e-12*jx**2*coen(12)**2 - 3.0447164277699145e-13*coen(7)*coep(7)&
            - 3.9084028569387184e-15*coep(7)**2 - 7.484219837778057e-7*jx*coen(8)*coep(8)&
            - 7.484219837778057e-7*jx**2*coen(8)*coep(8) - 2.624503705652277e-7*jx*coep(8)**2&
            - 2.624503705652277e-7*jx**2*coep(8)**2 - 2.0298109518466006e-13*jx*coen(12)*coep(12)&
            - 2.0298109518466006e-13*jx**2*coen(12)*coep(12) - 2.60560190462579e-15*jx*coep(12)**2&
            - 2.60560190462579e-15*jx**2*coep(12)**2 + jx*Q**12*((-1.0822953402971453e26&
            - 1.0822953402971453e26*jx)*coen(15)**2 + (-3.951450509848723e20&
            - 3.951450509848723e20*jx)*coen(15)*coep(15) + (-3.6602525805863394e14&
            - 3.6602525805863394e14*jx)*coep(15)**2) + Q*(-1.642643878078864e-9*coen(3)**2&
            + 4.5756606811353367e-7*coen(7)**2 - 8.43443279966628e-11*coen(3)*coep(3)&
            - 1.0827005415088352e-12*coep(3)**2 + 2.141364312935171e-8*coen(7)*coep(7)&
            + 2.481686052408006e-10*coep(7)**2 + jx*(-0.00014780697260247263*coen(5)**2&
            + 0.018700213355870555*coen(8)**2 + 9.781715176202178e-8*coen(12)**2&
            - 5.47547959359621e-10*coen(13)**2 - 5.475479593596214e-10*coen(14)**2&
            + 1.0950959187192424e-9*coen(12)*coen(15) - 0.00020732685876399972*coen(5)*coep(5)&
            - 0.00007270365126913575*coep(5)**2 + 0.024214700007108826*coen(8)*coep(8)&
            + 0.007784511335633302*coep(8)**2 + 5.9695473376525264e-9*coen(12)*coep(12)&
            + 2.8114775998887595e-11*coen(15)*coep(12) + 8.878494485868688e-11*coep(12)**2&
            - 2.811477599888735e-11*coen(13)*coep(13) - 3.6090018050293884e-13*coep(13)**2&
            - 2.8114775998887598e-11*coen(14)*coep(14) - 3.6090018050294505e-13*coep(14)**2&
            + 2.8114775998887595e-11*coen(12)*coep(15) + 7.2180036100589e-13*coep(12)*coep(15))&
            + jx**2*(-0.00014780697260247263*coen(5)**2 + 0.018700213355870555*coen(8)**2&
            + 9.781715176202178e-8*coen(12)**2 - 5.47547959359621e-10*coen(13)**2&
            - 5.475479593596214e-10*coen(14)**2 + 1.0950959187192424e-9*coen(12)*coen(15)&
            - 0.00020732685876399972*coen(5)*coep(5) - 0.00007270365126913575*coep(5)**2&
            + 0.024214700007108826*coen(8)*coep(8) + 0.007784511335633302*coep(8)**2&
            + 5.9695473376525264e-9*coen(12)*coep(12) + 2.8114775998887595e-11*coen(15)*coep(12)&
            + 8.878494485868688e-11*coep(12)**2 - 2.811477599888735e-11*coen(13)*coep(13)&
            - 3.6090018050293884e-13*coep(13)**2 - 2.8114775998887598e-11*coen(14)*coep(14)&
            - 3.6090018050294505e-13*coep(14)**2 + 2.8114775998887595e-11*coen(12)*coep(15)&
            + 7.2180036100589e-13*coep(12)*coep(15))) + Q**10*(-1.0621505734520131e23*coen(3)**2&
            - 4.2310580075597194e21*coen(7)**2 + 3.036772382941275e21*coen(3)*coep(3)&
            + 5.472649348000547e15*coep(3)**2 - 1.5447554561753174e16*coen(7)*coep(7)&
            - 1.4309163510330791e10*coep(7)**2 + jx*(4.1173144297095706e26*coen(5)**2&
            - 7.869291376035453e25*coen(8)**2 - 1.4822008129109945e21*coen(12)**2&
            + 8.267705870100582e21*coen(13)**2 - 3.5405019115067097e22*coen(14)**2&
            + 7.0810038230134186e22*coen(12)*coen(15) - 2.251990922736729e23*coen(15)**2&
            + 7.071096991603849e25*coen(5)*coep(5) + 1.020643391771307e19*coep(5)**2&
            - 2.2774337869698142e19*coen(8)*coep(8) - 1.667729141054265e12*coep(8)**2&
            - 5.379008137376198e15*coen(12)*coep(12) - 1.0122574609804249e21*coen(15)*coep(12)&
            - 4.959851785585747e9*coep(12)**2 + 8.104658205325251e19*coen(13)*coep(13)&
            + 1.1276199554992227e14*coep(13)**2 + 1.012257460980425e21*coen(14)*coep(14)&
            + 1.8242164493335155e15*coep(14)**2 - 1.0122574609804249e21*coen(12)*coep(15)&
            + 1.2766269139058638e22*coen(15)*coep(15) - 3.6484328986670305e15*coep(12)*coep(15)&
            - 1.8367895349095298e20*coep(15)**2) + jx**2*(4.1173144297095706e26*coen(5)**2&
            - 7.869291376035453e25*coen(8)**2 - 1.4822008129109945e21*coen(12)**2&
            + 8.267705870100582e21*coen(13)**2 - 3.5405019115067097e22*coen(14)**2&
            + 7.0810038230134186e22*coen(12)*coen(15) - 2.251990922736729e23*coen(15)**2&
            + 7.071096991603849e25*coen(5)*coep(5) + 1.020643391771307e19*coep(5)**2&
            - 2.2774337869698142e19*coen(8)*coep(8) - 1.667729141054265e12*coep(8)**2&
            - 5.379008137376198e15*coen(12)*coep(12) - 1.0122574609804249e21*coen(15)*coep(12)&
            - 4.959851785585747e9*coep(12)**2 + 8.104658205325251e19*coen(13)*coep(13)&
            + 1.1276199554992227e14*coep(13)**2 + 1.012257460980425e21*coen(14)*coep(14)&
            + 1.8242164493335155e15*coep(14)**2 - 1.0122574609804249e21*coen(12)*coep(15)&
            + 1.2766269139058638e22*coen(15)*coep(15) - 3.6484328986670305e15*coep(12)*coep(15)&
            - 1.8367895349095298e20*coep(15)**2)) + Q**8*(5.116207719835333e18*coen(3)**2&
            - 8.803793079235684e18*coen(7)**2 - 2.7984245908342134e17*coen(3)*coep(3)&
            + 3.7997698942439195e15*coep(3)**2 + 4.99076576017104e17*coen(7)*coep(7)&
            - 7.18063062874055e15*coep(7)**2 + jx*(6.19531162098637e21*coen(5)**2&
            - 9.274958681987389e21*coen(8)**2 - 6.078376007371354e18*coen(12)**2&
            + 1.3676421541798897e18*coen(13)**2 + 1.705402573278444e18*coen(14)**2&
            - 3.4108051465568876e18*coen(12)*coen(15) - 3.8340374395745965e17*coen(15)**2&
            + 3.06115591266851e21*coen(5)*coep(5) + 3.264102173913091e20*coep(5)**2&
            - 3.134840123867459e21*coen(8)*coep(8) - 2.0710264125614418e20*coep(8)**2&
            + 1.0176983898941035e17*coen(12)*coep(12) + 9.328081969447376e16*coen(15)*coep(12)&
            - 2.8369061948681465e15*coep(12)**2 + 3.5756282921865544e16*coen(13)*coep(13)&
            + 3.147133764607181e14*coep(13)**2 - 9.328081969447378e16*coen(14)*coep(14)&
            + 1.266589964747973e15*coep(14)**2 + 9.328081969447376e16*coen(12)*coep(15)&
            + 1.8552657845785884e16*coen(15)*coep(15) - 2.5331799294959455e15*coep(12)*coep(15)&
            - 2.1449453501168556e14*coep(15)**2) + jx**2*(6.19531162098637e21*coen(5)**2&
            - 9.274958681987389e21*coen(8)**2 - 6.078376007371354e18*coen(12)**2&
            + 1.3676421541798897e18*coen(13)**2 + 1.705402573278444e18*coen(14)**2&
            - 3.4108051465568876e18*coen(12)*coen(15) - 3.8340374395745965e17*coen(15)**2&
            + 3.06115591266851e21*coen(5)*coep(5) + 3.264102173913091e20*coep(5)**2&
            - 3.134840123867459e21*coen(8)*coep(8) - 2.0710264125614418e20*coep(8)**2&
            + 1.0176983898941035e17*coen(12)*coep(12) + 9.328081969447376e16*coen(15)*coep(12)&
            - 2.8369061948681465e15*coep(12)**2 + 3.5756282921865544e16*coen(13)*coep(13)&
            + 3.147133764607181e14*coep(13)**2 - 9.328081969447378e16*coen(14)*coep(14)&
            + 1.266589964747973e15*coep(14)**2 + 9.328081969447376e16*coen(12)*coep(15)&
            + 1.8552657845785884e16*coen(15)*coep(15) - 2.5331799294959455e15*coep(12)*coep(15)&
            - 2.1449453501168556e14*coep(15)**2)) + Q**5*(-3.9042474101024735e8*coen(3)**2&
            + 6.1774946968413315e9*coen(7)**2 + 6.634483585857287e6*coen(3)*coep(3)&
            + 29675.786334083783*coep(3)**2 - 2.223315898564204e8*coen(7)*coep(7)&
            + 1.5098377353881334e6*coep(7)**2 + jx*(-2.1365636317070002e12*coen(5)**2&
            + 1.883608789397678e13*coen(8)**2 + 2.9689735761794443e9*coen(12)**2&
            - 3.551539180791698e7*coen(13)**2 - 1.3014158033674909e8*coen(14)**2&
            + 2.6028316067349815e8*coen(12)*coen(15) + 4.6669982558925925e6*coen(15)**2&
            - 1.9760065662262327e12*coen(5)*coep(5) - 4.383482085101162e11*coep(5)**2&
            + 1.4829552897807066e13*coen(8)*coep(8) + 2.7390307013326226e12*coep(8)**2&
            - 4.408684243406016e7*coen(12)*coep(12) - 2.211494528619095e6*coen(15)*coep(12)&
            + 951810.528036578*coep(12)**2 - 1.1017586258551693e6*coen(13)*coep(13)&
            - 25006.071578759475*coep(13)**2 + 2.2114945286190957e6*coen(14)*coep(14)&
            + 9891.928778027926*coep(14)**2 - 2.211494528619095e6*coen(12)*coep(15)&
            + 36160.42930701504*coen(15)*coep(15) - 19783.857556055853*coep(12)*coep(15)&
            - 719.5409460541729*coep(15)**2) + jx**2*(-2.1365636317070002e12*coen(5)**2&
            + 1.883608789397678e13*coen(8)**2 + 2.9689735761794443e9*coen(12)**2&
            - 3.551539180791698e7*coen(13)**2 - 1.3014158033674909e8*coen(14)**2&
            + 2.6028316067349815e8*coen(12)*coen(15) + 4.6669982558925925e6*coen(15)**2&
            - 1.9760065662262327e12*coen(5)*coep(5) - 4.383482085101162e11*coep(5)**2&
            + 1.4829552897807066e13*coen(8)*coep(8) + 2.7390307013326226e12*coep(8)**2&
            - 4.408684243406016e7*coen(12)*coep(12) - 2.211494528619095e6*coen(15)*coep(12)&
            + 951810.528036578*coep(12)**2 - 1.1017586258551693e6*coen(13)*coep(13)&
            - 25006.071578759475*coep(13)**2 + 2.2114945286190957e6*coen(14)*coep(14)&
            + 9891.928778027926*coep(14)**2 - 2.211494528619095e6*coen(12)*coep(15)&
            + 36160.42930701504*coen(15)*coep(15) - 19783.857556055853*coep(12)*coep(15)&
            - 719.5409460541729*coep(15)**2)) + Q**4*(50541.70183910392*coen(3)**2&
            - 1.4093806317465093e6*coen(7)**2 + 391.60281110060265*coen(3)*coep(3)&
            - 7.792337164596042*coep(3)**2 + 23949.59049816459*coen(7)*coep(7)&
            + 107.12558426210926*coep(7)**2 + jx*(5.1919845373320913e8*coen(5)**2&
            - 7.712706405926243e9*coen(8)**2 - 597999.3159509723*coen(12)**2&
            + 1614.8043857017124*coen(13)**2 + 16847.23394636797*coen(14)**2&
            - 33694.467892735935*coen(12)*coen(15) - 336.30496184797835*coen(15)**2&
            + 5.482290023721673e8*coen(5)*coep(5) + 1.4100517834243697e8*coep(5)**2&
            - 7.13311706485856e9*coen(8)*coep(8) - 1.5823778827037067e9*coep(8)**2&
            + 4005.9968629604723*coen(12)*coep(12) - 130.53427036686753*coen(15)*coep(12)&
            - 54.56001603198321*coep(12)**2 + 5.362275377575171*coen(13)*coep(13)&
            + 2.1686003270069807*coep(13)**2 + 130.53427036686753*coen(14)*coep(14)&
            - 2.5974457215320137*coep(14)**2 - 130.53427036686753*coen(12)*coep(15)&
            - 11.07640012604248*coen(15)*coep(15) + 5.1948914430640265*coep(12)*coep(15)&
            - 0.09040405473489828*coep(15)**2) + jx**2*(5.1919845373320913e8*coen(5)**2&
            - 7.712706405926243e9*coen(8)**2 - 597999.3159509723*coen(12)**2&
            + 1614.8043857017124*coen(13)**2 + 16847.23394636797*coen(14)**2&
            - 33694.467892735935*coen(12)*coen(15) - 336.30496184797835*coen(15)**2&
            + 5.482290023721673e8*coen(5)*coep(5) + 1.4100517834243697e8*coep(5)**2&
            - 7.13311706485856e9*coen(8)*coep(8) - 1.5823778827037067e9*coep(8)**2&
            + 4005.9968629604723*coen(12)*coep(12) - 130.53427036686753*coen(15)*coep(12)&
            - 54.56001603198321*coep(12)**2 + 5.362275377575171*coen(13)*coep(13)&
            + 2.1686003270069807*coep(13)**2 + 130.53427036686753*coen(14)*coep(14)&
            - 2.5974457215320137*coep(14)**2 - 130.53427036686753*coen(12)*coep(15)&
            - 11.07640012604248*coen(15)*coep(15) + 5.1948914430640265*coep(12)*coep(15)&
            - 0.09040405473489828*coep(15)**2)) + Q**2*(0.0001267543413144013*coen(3)**2&
            - 0.013147296757447823*coen(7)**2 + 5.931978831369882e-6*coen(3)*coep(3)&
            + 6.874733570585968e-8*coep(3)**2 - 0.0004330138891234689*coen(7)*coep(7)&
            - 3.5341998201428214e-6*coep(7)**2 + jx*(5.180308138089475*coen(5)**2&
            - 260.2760645269219*coen(8)**2 - 0.004028907165403274*coen(12)**2&
            - 0.000015154270314651174*coen(13)**2 + 0.00004225144710480043*coen(14)**2&
            - 0.00008450289420960084*coen(12)*coen(15) - 1.516810046969052e-7*coen(15)**2&
            + 6.707923868090086*coen(5)*coep(5) + 2.156454937471148*coep(5)**2&
            - 307.0331456716213*coen(8)*coep(8) - 89.21441312908415*coep(8)**2&
            - 0.00011202478670214563*coen(12)*coep(12) - 1.9773262771232933e-6*coen(15)*coep(12)&
            - 1.3721131377842126e-6*coep(12)**2 - 3.236502433017998e-7*coen(13)*coep(13)&
            + 1.679308323631748e-9*coep(13)**2 + 1.9773262771232937e-6*coen(14)*coep(14)&
            + 2.291577856861989e-8*coep(14)**2 - 1.9773262771232933e-6*coen(12)*coep(15)&
            - 7.78831771252911e-9*coen(15)*coep(15) - 4.5831557137239776e-8*coep(12)*coep(15)&
            - 9.997608618248472e-11*coep(15)**2) + jx**2*(5.180308138089475*coen(5)**2&
            - 260.2760645269219*coen(8)**2 - 0.004028907165403274*coen(12)**2&
            - 0.000015154270314651174*coen(13)**2 + 0.00004225144710480043*coen(14)**2&
            - 0.00008450289420960084*coen(12)*coen(15) - 1.516810046969052e-7*coen(15)**2&
            + 6.707923868090086*coen(5)*coep(5) + 2.156454937471148*coep(5)**2&
            - 307.0331456716213*coen(8)*coep(8) - 89.21441312908415*coep(8)**2&
            - 0.00011202478670214563*coen(12)*coep(12) - 1.9773262771232933e-6*coen(15)*coep(12)&
            - 1.3721131377842126e-6*coep(12)**2 - 3.236502433017998e-7*coen(13)*coep(13)&
            + 1.679308323631748e-9*coep(13)**2 + 1.9773262771232937e-6*coen(14)*coep(14)&
            + 2.291577856861989e-8*coep(14)**2 - 1.9773262771232933e-6*coen(12)*coep(15)&
            - 7.78831771252911e-9*coen(15)*coep(15) - 4.5831557137239776e-8*coep(12)*coep(15)&
            - 9.997608618248472e-11*coep(15)**2)) + Q**3*(-3.6420465954257946*coen(3)**2&
            + 182.44872362141206*coen(7)**2 - 0.11995292946900468*coen(3)*coep(3)&
            - 0.0009790393158360792*coep(3)**2 + 1.4136333058057673*coen(7)*coep(7)&
            - 0.028129285678470683*coep(7)**2 + jx*(-72101.32791321549*coen(5)**2&
            + 1.8742363581541448e6*coen(8)**2 + 66.64546714035653*coen(12)**2&
            + 0.0979330476484076*coen(13)**2 - 1.2140155318085981*coen(14)**2&
            + 2.428031063617196*coen(12)*coen(15) + 0.011704439469100658*coen(15)**2&
            - 85053.91210879407*coen(5)*coep(5) - 24714.0576191547*coep(5)**2&
            + 1.9790327214042107e6*coen(8)*coep(8) + 509009.66679920803*coep(8)**2&
            + 0.4905681924712965*coen(12)*coep(12) + 0.03998430982300155*coen(15)*coep(12)&
            - 0.0015480740065123306*coep(12)**2 + 0.008951352969668005*coen(13)*coep(13)&
            - 0.00005375451097482094*coep(13)**2 - 0.03998430982300156*coen(14)*coep(14)&
            - 0.00032634643861202635*coep(14)**2 + 0.03998430982300155*coen(12)*coep(15)&
            + 0.0005477562854556592*coen(15)*coep(15) + 0.0006526928772240526*coep(12)*coep(15)&
            + 6.348098385327306e-6*coep(15)**2) + jx**2*(-72101.32791321549*coen(5)**2&
            + 1.8742363581541448e6*coen(8)**2 + 66.64546714035653*coen(12)**2&
            + 0.0979330476484076*coen(13)**2 - 1.2140155318085981*coen(14)**2&
            + 2.428031063617196*coen(12)*coen(15) + 0.011704439469100658*coen(15)**2&
            - 85053.91210879407*coen(5)*coep(5) - 24714.0576191547*coep(5)**2&
            + 1.9790327214042107e6*coen(8)*coep(8) + 509009.66679920803*coep(8)**2&
            + 0.4905681924712965*coen(12)*coep(12) + 0.03998430982300155*coen(15)*coep(12)&
            - 0.0015480740065123306*coep(12)**2 + 0.008951352969668005*coen(13)*coep(13)&
            - 0.00005375451097482094*coep(13)**2 - 0.03998430982300156*coen(14)*coep(14)&
            - 0.00032634643861202635*coep(14)**2 + 0.03998430982300155*coen(12)*coep(15)&
            + 0.0005477562854556592*coen(15)*coep(15) + 0.0006526928772240526*coep(12)*coep(15)&
            + 6.348098385327306e-6*coep(15)**2)) + Q**6*(1.7112813336434773e12*coen(3)**2&
            - 1.4988547216272855e13*coen(7)**2 - 6.159000019783296e10*coen(3)*coep(3)&
            + 4.182532337456114e8*coep(3)**2 + 7.25286052865907e11*coen(7)*coep(7)&
            - 8.385315783488683e9*coep(7)**2 + jx*(5.217947921236113e15*coen(5)**2&
            - 2.7259356177418416e16*coen(8)**2 - 8.054639137163456e12*coen(12)**2&
            + 2.5203398512517493e11*coen(13)**2 + 5.704271112144924e11*coen(14)**2&
            - 1.1408542224289846e12*coen(12)*coen(15) - 3.605164683913316e10*coen(15)**2&
            + 4.1080629455289985e15*coen(5)*coep(5) + 7.587626281352532e14*coep(5)**2&
            - 1.7546843327563532e16*coen(8)*coep(8) - 2.564623376925098e15*coep(8)**2&
            + 1.491328954019982e11*coen(12)*coep(12)&
            + 2.0530000065944317e10*coen(15)*coep(12) - 3.855875495357437e9*coep(12)**2&
            + 8.317121833079992e9*coen(13)*coep(13) + 1.2425153717844915e8*coep(13)**2&
            - 2.053000006594432e10*coen(14)*coep(14) + 1.3941774458187047e8*coep(14)**2&
            + 2.0530000065944317e10*coen(12)*coep(15) + 6.12625261858277e8*coen(15)*coep(15)&
            - 2.788354891637409e8*coep(12)*coep(15) + 2.740248903851825e6*coep(15)**2)&
            + jx**2*(5.217947921236113e15*coen(5)**2 - 2.7259356177418416e16*coen(8)**2&
            - 8.054639137163456e12*coen(12)**2 + 2.5203398512517493e11*coen(13)**2&
            + 5.704271112144924e11*coen(14)**2 - 1.1408542224289846e12*coen(12)*coen(15)&
            - 3.605164683913316e10*coen(15)**2 + 4.1080629455289985e15*coen(5)*coep(5)&
            + 7.587626281352532e14*coep(5)**2 - 1.7546843327563532e16*coen(8)*coep(8)&
            - 2.564623376925098e15*coep(8)**2 + 1.491328954019982e11*coen(12)*coep(12)&
            + 2.0530000065944317e10*coen(15)*coep(12) - 3.855875495357437e9*coep(12)**2&
            + 8.317121833079992e9*coen(13)*coep(13) + 1.2425153717844915e8*coep(13)**2&
            - 2.053000006594432e10*coen(14)*coep(14) + 1.3941774458187047e8*coep(14)**2&
            + 2.0530000065944317e10*coen(12)*coep(15) + 6.12625261858277e8*coen(15)*coep(15)&
            - 2.788354891637409e8*coep(12)*coep(15) + 2.740248903851825e6*coep(15)**2))&
            + Q**7*(-4.152107339364738e15*coen(3)**2 + 1.846881949558272e16*coen(7)**2&
            + 2.0091777407045212e14*coen(3)*coep(3) - 2.322888983510977e12*coep(3)**2&
            - 1.0101935157898646e15*coen(7)*coep(7) + 1.3716656583247379e13*coep(7)**2&
            + jx*(-7.551350455615502e18*coen(5)**2 + 2.236423896615478e19*coen(8)**2&
            + 1.1093276794302414e16*coen(12)**2 - 8.472496006281736e14*coen(13)**2&
            - 1.384035779788246e15*coen(14)**2 + 2.7680715595764915e15*coen(12)*coen(15)&
            + 1.580189567988884e14*coen(15)**2 - 4.860803112656614e18*coen(5)*coep(5)&
            - 7.104485439706958e17*coep(5)**2 + 1.1050359777169117e19*coen(8)*coep(8)&
            + 1.1782968395012764e18*coep(8)**2 - 2.0765581547859944e14*coen(12)*coep(12)&
            - 6.697259135681736e13*coen(15)*coep(12) + 5.708291683177621e12*coep(12)**2&
            - 2.565999577269855e13*coen(13)*coep(13) - 2.938531540933456e11*coep(13)**2&
            + 6.697259135681738e13*coen(14)*coep(14) - 7.742963278369923e11*coep(14)**2&
            - 6.697259135681736e13*coen(12)*coep(15) - 5.68719320965404e12*coen(15)*coep(15)&
            + 1.5485926556739841e12*coep(12)*coep(15) + 3.862131747415678e10*coep(15)**2)&
            + jx**2*(-7.551350455615502e18*coen(5)**2 + 2.236423896615478e19*coen(8)**2&
            + 1.1093276794302414e16*coen(12)**2 - 8.472496006281736e14*coen(13)**2&
            - 1.384035779788246e15*coen(14)**2 + 2.7680715595764915e15*coen(12)*coen(15)&
            + 1.580189567988884e14*coen(15)**2 - 4.860803112656614e18*coen(5)*coep(5)&
            - 7.104485439706958e17*coep(5)**2 + 1.1050359777169117e19*coen(8)*coep(8)&
            + 1.1782968395012764e18*coep(8)**2 - 2.0765581547859944e14*coen(12)*coep(12)&
            - 6.697259135681736e13*coen(15)*coep(12) + 5.708291683177621e12*coep(12)**2&
            - 2.565999577269855e13*coen(13)*coep(13) - 2.938531540933456e11*coep(13)**2&
            + 6.697259135681738e13*coen(14)*coep(14) - 7.742963278369923e11*coep(14)**2&
            - 6.697259135681736e13*coen(12)*coep(15) - 5.68719320965404e12*coen(15)*coep(15)&
            + 1.5485926556739841e12*coep(12)*coep(15) + 3.862131747415678e10*coep(15)**2))&
            + Q**9*(-2.4388150052899384e21*coen(3)**2 - 3.834220245233959e20*coen(7)**2&
            + 1.3825352679516942e20*coen(3)*coep(3) - 1.989168710259752e18*coep(3)**2&
            + 1.096233852512895e19*coen(7)*coep(7) + 1.975552567558646e13*coep(7)**2&
            + jx*(-2.569336671533722e24*coen(5)**2 + 1.486294950731821e24*coen(8)**2&
            - 9.796203894796201e19*coen(12)**2 - 8.708852715184443e20*coen(13)**2&
            - 8.12938335096646e20*coen(14)**2 + 1.6258766701932918e21*coen(12)*coen(15)&
            + 4.7242834404879335e20*coen(15)**2 - 8.684092259397657e23*coen(5)*coep(5)&
            - 5.737129718801991e22*coep(5)**2 + 2.5525705976983426e23*coen(8)*coep(8)&
            + 3.684384920280423e16*coep(8)**2 + 3.9466800666235817e18*coen(12)*coep(12)&
            - 4.608450893172313e19*coen(15)*coep(12) + 6.992230813204382e12*coep(12)**2&
            - 1.789236400288496e19*coen(13)*coep(13) - 1.2281972993235816e17*coep(13)**2&
            + 4.608450893172314e19*coen(14)*coep(14) - 6.630562367532506e17*coep(14)**2&
            - 4.608450893172313e19*coen(12)*coep(15) - 2.584052813703546e19*coen(15)*coep(15)&
            + 1.326112473506501e18*coep(12)*coep(15) + 3.5086906107125184e17*coep(15)**2)&
            + jx**2*(-2.569336671533722e24*coen(5)**2 + 1.486294950731821e24*coen(8)**2&
            - 9.796203894796201e19*coen(12)**2 - 8.708852715184443e20*coen(13)**2&
            - 8.12938335096646e20*coen(14)**2 + 1.6258766701932918e21*coen(12)*coen(15)&
            + 4.7242834404879335e20*coen(15)**2 - 8.684092259397657e23*coen(5)*coep(5)&
            - 5.737129718801991e22*coep(5)**2 + 2.5525705976983426e23*coen(8)*coep(8)&
            + 3.684384920280423e16*coep(8)**2 + 3.9466800666235817e18*coen(12)*coep(12)&
            - 4.608450893172313e19*coen(15)*coep(12) + 6.992230813204382e12*coep(12)**2&
            - 1.789236400288496e19*coen(13)*coep(13) - 1.2281972993235816e17*coep(13)**2&
            + 4.608450893172314e19*coen(14)*coep(14) - 6.630562367532506e17*coep(14)**2&
            - 4.608450893172313e19*coen(12)*coep(15) - 2.584052813703546e19*coen(15)*coep(15)&
            + 1.326112473506501e18*coep(12)*coep(15) + 3.5086906107125184e17*coep(15)**2))&
            + Q**11*(-1.1720820405725202e24*coen(3)**2 - 4.2792609414111063e18*coen(3)*coep(3)&
            - 3.9639053721571167e12*coep(3)**2 + jx*(-2.1799405910776054e28*coen(5)**2&
            - 1.990327685354452e22*coen(13)**2 - 3.906940135241733e23*coen(14)**2&
            + 7.813880270483465e23*coen(12)*coen(15) - 9.807851127720847e24*coen(15)**2&
            - 6.308916671742666e21*coen(5)*coep(5) - 4.619921001500354e14*coep(5)**2&
            + 1.4264203138037018e18*coen(15)*coep(12) - 6.366533915767474e16*coen(13)*coep(13)&
            - 5.266972926479529e10*coep(13)**2 - 1.4264203138037018e18*coen(14)*coep(14)&
            - 1.3213017907190388e12*coep(14)**2 + 1.4264203138037018e18*coen(12)*coep(15)&
            + 2.804142104246365e23*coen(15)*coep(15) + 2.642603581438077e12*coep(12)*coep(15)&
            + 5.053420053708884e17*coep(15)**2) + jx**2*(-2.1799405910776054e28*coen(5)**2&
            - 1.990327685354452e22*coen(13)**2 - 3.906940135241733e23*coen(14)**2&
            + 7.813880270483465e23*coen(12)*coen(15) - 9.807851127720847e24*coen(15)**2&
            - 6.308916671742666e21*coen(5)*coep(5) - 4.619921001500354e14*coep(5)**2&
            + 1.4264203138037018e18*coen(15)*coep(12) - 6.366533915767474e16*coen(13)*coep(13)&
            - 5.266972926479529e10*coep(13)**2 - 1.4264203138037018e18*coen(14)*coep(14)&
            - 1.3213017907190388e12*coep(14)**2 + 1.4264203138037018e18*coen(12)*coep(15)&
            + 2.804142104246365e23*coen(15)*coep(15) + 2.642603581438077e12*coep(12)*coep(15)&
            + 5.053420053708884e17*coep(15)**2))) + Q*(-3.644218796141472e-10*coen(7)**2&
            - 0.00003279109702029023*jx*coen(8)**2 - 0.00003279109702029023*jx**2*coen(8)**2&
            - 2.4294791974276473e-10*jx*coen(12)**2 - 2.4294791974276473e-10*jx**2*coen(12)**2&
            - 1.871185772736329e-11*coen(7)*coep(7) - 2.401979952327571e-13*coep(7)**2&
            - 0.00004599563214738758*jx*coen(8)*coep(8)&
            - 0.00004599563214738758*jx**2*coen(8)*coep(8) - 0.00001612936413295901*jx*coep(8)**2&
            - 0.00001612936413295901*jx**2*coep(8)**2&
            - 1.2474571818242135e-11*jx*coen(12)*coep(12)&
            - 1.2474571818242135e-11*jx**2*coen(12)*coep(12)&
            - 1.6013199682183667e-13*jx*coep(12)**2 - 1.6013199682183667e-13*jx**2*coep(12)**2&
            + jx*Q**12*((-6.651442558629904e27 - 6.651442558629904e27*jx)*coen(15)**2&
            + (-2.4284356691687908e22 - 2.4284356691687908e22*jx)*coen(15)*coep(15)&
            + (-2.249474693586198e16 - 2.249474693586198e16*jx)*coep(15)**2)&
            + Q*(-1.0095166256861918e-7*coen(3)**2 + 0.0000281205536558969*coen(7)**2&
            - 5.183533846334553e-9*coen(3)*coep(3) - 6.653932796260884e-11*coep(3)**2&
            + 1.316014325690232e-6*coen(7)*coep(7) + 1.5251652309260515e-8*coep(7)**2&
            + jx*(-0.0090837459187472*coen(5)**2 + 1.149255571372479*coen(8)**2&
            + 6.011530697482647e-6*coen(12)**2 - 3.365055418953971e-8*coen(13)**2&
            - 3.365055418953973e-8*coen(14)**2 + 6.730110837907944e-8*coen(12)*coen(15)&
            - 0.012741648610917118*coen(5)*coep(5) - 0.004468134918575442*coep(5)**2&
            + 1.4881583628320854*coen(8)*coep(8) + 0.47841128080393*coep(8)**2&
            + 3.668693723334011e-7*coen(12)*coep(12) + 1.7278446154448508e-9*coen(15)*coep(12)&
            + 5.456440019750463e-9*coep(12)**2 - 1.7278446154448357e-9*coen(13)*coep(13)&
            - 2.2179775987535895e-11*coep(13)**2 - 1.727844615444851e-9*coen(14)*coep(14)&
            - 2.217977598753628e-11*coep(14)**2 + 1.7278446154448508e-9*coen(12)*coep(15)&
            + 4.4359551975072546e-11*coep(12)*coep(15)) + jx**2*(-0.0090837459187472*coen(5)**2&
            + 1.149255571372479*coen(8)**2 + 6.011530697482647e-6*coen(12)**2&
            - 3.365055418953971e-8*coen(13)**2 - 3.365055418953973e-8*coen(14)**2&
            + 6.730110837907944e-8*coen(12)*coen(15) - 0.012741648610917118*coen(5)*coep(5)&
            - 0.004468134918575442*coep(5)**2 + 1.4881583628320854*coen(8)*coep(8)&
            + 0.47841128080393*coep(8)**2 + 3.668693723334011e-7*coen(12)*coep(12)&
            + 1.7278446154448508e-9*coen(15)*coep(12) + 5.456440019750463e-9*coep(12)**2&
            - 1.7278446154448357e-9*coen(13)*coep(13) - 2.2179775987535895e-11*coep(13)**2&
            - 1.727844615444851e-9*coen(14)*coep(14) - 2.217977598753628e-11*coep(14)**2&
            + 1.7278446154448508e-9*coen(12)*coep(15) + 4.4359551975072546e-11*coep(12)*coep(15)))&
            + Q**10*(-6.527639235693484e24*coen(3)**2 - 2.600273534559242e23*coen(7)**2&
            + 1.8663036157230348e23*coen(3)*coep(3) + 3.3633160401258963e17*coep(3)**2&
            - 9.493575183516252e17*coen(7)*coep(7) - 8.793956289683132e11*coep(7)**2&
            + jx*(2.53037035320806e28*coen(5)**2 - 4.8362159214739753e27*coen(8)**2&
            - 9.10913426341214e22*coen(12)**2 + 5.081068784008913e23*coen(13)**2&
            - 2.1758797452311612e24*coen(14)**2 + 4.351759490462322e24*coen(12)*coen(15)&
            - 1.3840019177230135e25*coen(15)**2 + 4.345671067311512e27*coen(5)*coep(5)&
            + 6.272549312970512e20*coep(5)**2 - 1.3996382919798704e21*coen(8)*coep(8)&
            - 1.0249332295960997e14*coep(8)**2 - 3.3057671336123174e17*coen(12)*coep(12)&
            - 6.221012052410114e22*coen(15)*coep(12) - 3.0481669857401465e11*coep(12)**2&
            + 4.980864880675642e21*coen(13)*coep(13) + 6.929993212304391e15*coep(13)**2&
            + 6.221012052410115e22*coen(14)*coep(14) + 1.1211053467086317e17*coep(14)**2&
            - 6.221012052410114e22*coen(12)*coep(15) + 7.84574253485605e23*coen(15)*coep(15)&
            - 2.242210693417263e17*coep(12)*coep(15) - 1.1288323647766054e22*coep(15)**2)&
            + jx**2*(2.53037035320806e28*coen(5)**2 - 4.8362159214739753e27*coen(8)**2&
            - 9.10913426341214e22*coen(12)**2 + 5.081068784008913e23*coen(13)**2&
            - 2.1758797452311612e24*coen(14)**2 + 4.351759490462322e24*coen(12)*coen(15)&
            - 1.3840019177230135e25*coen(15)**2 + 4.345671067311512e27*coen(5)*coep(5)&
            + 6.272549312970512e20*coep(5)**2 - 1.3996382919798704e21*coen(8)*coep(8)&
            - 1.0249332295960997e14*coep(8)**2 - 3.3057671336123174e17*coen(12)*coep(12)&
            - 6.221012052410114e22*coen(15)*coep(12) - 3.0481669857401465e11*coep(12)**2&
            + 4.980864880675642e21*coen(13)*coep(13) + 6.929993212304391e15*coep(13)**2&
            + 6.221012052410115e22*coen(14)*coep(14) + 1.1211053467086317e17*coep(14)**2&
            - 6.221012052410114e22*coen(12)*coep(15) + 7.84574253485605e23*coen(15)*coep(15)&
            - 2.242210693417263e17*coep(12)*coep(15) - 1.1288323647766054e22*coep(15)**2))&
            + Q**8*(3.1442583645569955e20*coen(3)**2 - 5.4105309137266075e20*coen(7)**2&
            - 1.719822651687735e19*coen(3)*coep(3) + 2.3352175923288797e17*coep(3)**2&
            + 3.0671657302193164e19*coen(7)*coep(7) - 4.412986953144722e17*coep(7)**2&
            + jx*(3.80744126353621e23*coen(5)**2 - 5.700094291265014e23*coen(8)**2&
            - 3.7355763586383584e20*coen(12)**2 + 8.40509322232771e19*coen(13)**2&
            + 1.0480861215189983e20*coen(14)**2 - 2.0961722430379966e20*coen(12)*coen(15)&
            - 2.3562773346104705e19*coen(15)**2 + 1.8812889567218064e23*coen(5)*coep(5)&
            + 2.0060132670737685e22*coep(5)**2 - 1.926572926819396e23*coen(8)*coep(8)&
            - 1.272786891679286e22*coep(8)**2 + 6.254450269121817e18*coen(12)*coep(12)&
            + 5.732742172292449e18*coen(15)*coep(12) - 1.7434722202727174e17*coep(12)**2&
            + 2.1974673003730324e18*coen(13)*coep(13) + 1.9341282069885084e16*coep(13)**2&
            - 5.732742172292449e18*coen(14)*coep(14) + 7.784058641096266e16*coep(14)**2&
            + 5.732742172292449e18*coen(12)*coep(15) + 1.1401872795394194e18*coen(15)*coep(15)&
            - 1.5568117282192528e17*coep(12)*coep(15) - 1.3182151171218716e16*coep(15)**2)&
            + jx**2*(3.80744126353621e23*coen(5)**2 - 5.700094291265014e23*coen(8)**2&
            - 3.7355763586383584e20*coen(12)**2 + 8.40509322232771e19*coen(13)**2&
            + 1.0480861215189983e20*coen(14)**2 - 2.0961722430379966e20*coen(12)*coen(15)&
            - 2.3562773346104705e19*coen(15)**2 + 1.8812889567218064e23*coen(5)*coep(5)&
            + 2.0060132670737685e22*coep(5)**2 - 1.926572926819396e23*coen(8)*coep(8)&
            - 1.272786891679286e22*coep(8)**2 + 6.254450269121817e18*coen(12)*coep(12)&
            + 5.732742172292449e18*coen(15)*coep(12) - 1.7434722202727174e17*coep(12)**2&
            + 2.1974673003730324e18*coen(13)*coep(13) + 1.9341282069885084e16*coep(13)**2&
            - 5.732742172292449e18*coen(14)*coep(14) + 7.784058641096266e16*coep(14)**2&
            + 5.732742172292449e18*coen(12)*coep(15) + 1.1401872795394194e18*coen(15)*coep(15)&
            - 1.5568117282192528e17*coep(12)*coep(15) - 1.3182151171218716e16*coep(15)**2))&
            + Q**5*(-2.3994261470114418e10*coen(3)**2 + 3.796491549247534e11*coen(7)**2&
            + 4.077342369910515e8*coen(3)*coep(3) + 1.8237793403891015e6*coep(3)**2&
            - 1.3663791608793491e10*coen(7)*coep(7) + 9.278981989360519e7*coep(7)**2&
            + jx*(-1.3130639798610888e14*coen(5)**2 + 1.1576059878599582e15*coen(8)**2&
            + 1.8246366278012204e11*coen(12)**2 - 2.1826628995070605e9*coen(13)**2&
            - 7.998087156704806e9*coen(14)**2 + 1.5996174313409609e10*coen(12)*coen(15)&
            + 2.868188530847119e8*coen(15)**2 - 1.2143907195535738e14*coen(5)*coep(5)&
            - 2.693948519433584e13*coep(5)**2 + 9.113771038027955e14*coen(8)*coep(8)&
            + 1.683321058301499e14*coep(8)**2 - 2.7094369634910264e9*coen(12)*coep(12)&
            - 1.3591141233035046e8*coen(15)*coep(12) + 5.849524448840679e7*coep(12)**2&
            - 6.771057714559397e7*coen(13)*coep(13) - 1.5367935398986503e6*coep(13)**2&
            + 1.359114123303505e8*coen(14)*coep(14) + 607926.4467963671*coep(14)**2&
            - 1.3591141233035046e8*coen(12)*coep(15) + 2.2223048504022304e6*coen(15)*coep(15)&
            - 1.2158528935927341e6*coep(12)*coep(15) - 44220.695526117226*coep(15)**2)&
            + jx**2*(-1.3130639798610888e14*coen(5)**2 + 1.1576059878599582e15*coen(8)**2&
            + 1.8246366278012204e11*coen(12)**2 - 2.1826628995070605e9*coen(13)**2&
            - 7.998087156704806e9*coen(14)**2 + 1.5996174313409609e10*coen(12)*coen(15)&
            + 2.868188530847119e8*coen(15)**2 - 1.2143907195535738e14*coen(5)*coep(5)&
            - 2.693948519433584e13*coep(5)**2 + 9.113771038027955e14*coen(8)*coep(8)&
            + 1.683321058301499e14*coep(8)**2 - 2.7094369634910264e9*coen(12)*coep(12)&
            - 1.3591141233035046e8*coen(15)*coep(12) + 5.849524448840679e7*coep(12)**2&
            - 6.771057714559397e7*coen(13)*coep(13) - 1.5367935398986503e6*coep(13)**2&
            + 1.359114123303505e8*coen(14)*coep(14) + 607926.4467963671*coep(14)**2&
            - 1.3591141233035046e8*coen(12)*coep(15)&
            + 2.2223048504022304e6*coen(15)*coep(15) - 1.2158528935927341e6*coep(12)*coep(15)&
            - 44220.695526117226*coep(15)**2)) + Q**4*(3.1061320702526704e6*coen(3)**2&
            - 8.661604615920895e7*coen(7)**2 + 24066.661906892492*coen(3)*coep(3)&
            - 478.89223133454067*coep(3)**2 + 1.4718655765210493e6*coen(7)*coep(7)&
            + 6583.597320888943*coep(7)**2 + jx*(3.190828383856634e10*coen(5)**2&
            - 4.739983784510303e11*coen(8)**2 - 3.6751133928524755e7*coen(12)**2&
            + 99240.73600806313*coen(13)**2 + 1.03537735675089e6*coen(14)**2&
            - 2.0707547135017798e6*coen(12)*coen(15) - 20668.23216017822*coen(15)**2&
            + 3.369240892464985e10*coen(5)*coep(5) + 8.665729300438334e9*coep(5)**2&
            - 4.383786629614725e11*coen(8)*coep(8) - 9.724790637979128e10*coep(8)**2&
            + 246195.81210353886*coen(12)*coep(12) - 8022.220635630829*coen(15)*coep(12)&
            - 3353.084866234641*coep(12)**2 + 329.5483712209678*coen(13)*coep(13)&
            + 133.27527127440783*coep(13)**2 + 8022.22063563083*coen(14)*coep(14)&
            - 159.6307437781802*coep(14)**2 - 8022.220635630829*coen(12)*coep(15)&
            - 680.7202844885694*coen(15)*coep(15) + 319.26148755636035*coep(12)*coep(15)&
            - 5.555945357496571*coep(15)**2) + jx**2*(3.190828383856634e10*coen(5)**2&
            - 4.739983784510303e11*coen(8)**2 - 3.6751133928524755e7*coen(12)**2&
            + 99240.73600806313*coen(13)**2 + 1.03537735675089e6*coen(14)**2&
            - 2.0707547135017798e6*coen(12)*coen(15) - 20668.23216017822*coen(15)**2&
            + 3.369240892464985e10*coen(5)*coep(5) + 8.665729300438334e9*coep(5)**2&
            - 4.383786629614725e11*coen(8)*coep(8) - 9.724790637979128e10*coep(8)**2&
            + 246195.81210353886*coen(12)*coep(12) - 8022.220635630829*coen(15)*coep(12)&
            - 3353.084866234641*coep(12)**2 + 329.5483712209678*coen(13)*coep(13)&
            + 133.27527127440783*coep(13)**2 + 8022.22063563083*coen(14)*coep(14)&
            - 159.6307437781802*coep(14)**2 - 8022.220635630829*coen(12)*coep(15)&
            - 680.7202844885694*coen(15)*coep(15) + 319.26148755636035*coep(12)*coep(15)&
            - 5.555945357496571*coep(15)**2)) + Q**2*(0.007789918231360358*coen(3)**2&
            - 0.8079909977198673*coen(7)**2 + 0.00036456053155539377*coen(3)*coep(3)&
            + 4.224992360965248e-6*coep(3)**2 - 0.026611654909305425*coen(7)*coep(7)&
            - 0.0002172006680537504*coep(7)**2 + jx*(318.36524406586744*coen(5)**2&
            - 15995.738206835162*coen(8)**2 - 0.24760380634527054*coen(12)**2&
            - 0.0009313332023417792*coen(13)**2 + 0.002596639410453452*coen(14)**2&
            - 0.005193278820906904*coen(12)*coen(15) - 9.32183159635284e-6*coen(15)**2&
            + 412.2476429032207*coen(5)*coep(5) + 132.52885430445585*coep(5)**2&
            - 18869.27953944202*coen(8)*coep(8) - 5482.833772222108*coep(8)**2&
            - 0.00688468670379301*coen(12)*coep(12) - 0.00012152017718513123*coen(15)*coep(12)&
            - 0.00008432570463998699*coep(12)**2 - 0.00001989051345095394*coen(13)*coep(13)&
            + 1.0320494265270477e-7*coep(13)**2 + 0.00012152017718513125*coen(14)*coep(14)&
            + 1.4083307869884159e-6*coep(14)**2 - 0.00012152017718513123*coen(12)*coep(15)&
            - 4.786452086084422e-7*coen(15)*coep(15) - 2.8166615739768313e-6*coep(12)*coep(15)&
            - 6.144211932917105e-9*coep(15)**2) + jx**2*(318.36524406586744*coen(5)**2&
            - 15995.738206835162*coen(8)**2 - 0.24760380634527054*coen(12)**2&
            - 0.0009313332023417792*coen(13)**2 + 0.002596639410453452*coen(14)**2&
            - 0.005193278820906904*coen(12)*coen(15) - 9.32183159635284e-6*coen(15)**2&
            + 412.2476429032207*coen(5)*coep(5) + 132.52885430445585*coep(5)**2&
            - 18869.27953944202*coen(8)*coep(8) - 5482.833772222108*coep(8)**2&
            - 0.00688468670379301*coen(12)*coep(12) - 0.00012152017718513123*coen(15)*coep(12)&
            - 0.00008432570463998699*coep(12)**2 - 0.00001989051345095394*coen(13)*coep(13)&
            + 1.0320494265270477e-7*coep(13)**2 + 0.00012152017718513125*coen(14)*coep(14)&
            + 1.4083307869884159e-6*coep(14)**2 - 0.00012152017718513123*coen(12)*coep(15)&
            - 4.786452086084422e-7*coen(15)*coep(15) - 2.8166615739768313e-6*coep(12)*coep(15)&
            - 6.144211932917105e-9*coep(15)**2)) + Q**3*(-223.8285874785095*coen(3)**2&
            + 11212.717637035972*coen(7)**2 - 7.371925115037597*coen(3)*coep(3)&
            - 0.0601686390901038*coep(3)**2 + 86.87740196637675*coen(7)*coep(7)&
            - 1.7287363341532074*coep(7)**2 + jx*(-4.431118042918148e6*coen(5)**2&
            + 1.1518459900358935e8*coen(8)**2 + 4095.8182112789364*coen(12)**2&
            + 6.018653289647418*coen(13)**2 - 74.60952915950317*coen(14)**2&
            + 149.21905831900628*coen(12)*coen(15) + 0.7193175828356568*coen(15)**2&
            - 5.227142626550328e6*coen(5)*coep(5) - 1.5188472917137884e6*coep(5)**2&
            + 1.2162504981731781e8*coen(8)*coep(8) + 3.128211343470023e7*coep(8)**2&
            + 30.14875914016105*coen(12)*coep(12) + 2.4573083716791984*coen(15)*coep(12)&
            - 0.09513969937261108*coep(12)**2 + 0.5501216524079402*coen(13)*coep(13)&
            - 0.003303581090149551*coep(13)**2 - 2.4573083716791992*coen(14)*coep(14)&
            - 0.020056213030034596*coep(14)**2 + 2.4573083716791984*coen(12)*coep(15)&
            + 0.03366335724809191*coen(15)*coep(15) + 0.04011242606006919*coep(12)*coep(15)&
            + 0.0003901339144169573*coep(15)**2)&
            + jx**2*(-4.431118042918148e6*coen(5)**2 + 1.1518459900358935e8*coen(8)**2&
            + 4095.8182112789364*coen(12)**2 + 6.018653289647418*coen(13)**2&
            - 74.60952915950317*coen(14)**2 + 149.21905831900628*coen(12)*coen(15)&
            + 0.7193175828356568*coen(15)**2 - 5.227142626550328e6*coen(5)*coep(5)&
            - 1.5188472917137884e6*coep(5)**2 + 1.2162504981731781e8*coen(8)*coep(8)&
            + 3.128211343470023e7*coep(8)**2 + 30.14875914016105*coen(12)*coep(12)&
            + 2.4573083716791984*coen(15)*coep(12) - 0.09513969937261108*coep(12)**2&
            + 0.5501216524079402*coen(13)*coep(13) - 0.003303581090149551*coep(13)**2&
            - 2.4573083716791992*coen(14)*coep(14) - 0.020056213030034596*coep(14)**2&
            + 2.4573083716791984*coen(12)*coep(15) + 0.03366335724809191*coen(15)*coep(15)&
            + 0.04011242606006919*coep(12)*coep(15) + 0.0003901339144169573*coep(15)**2))&
            + Q**6*(1.0516990204596178e14*coen(3)**2 - 9.211483883777929e14*coen(7)**2&
            - 3.7851253095981836e12*coen(3)*coep(3) + 2.570451202770903e10*coep(3)**2&
            + 4.457377149834631e13*coen(7)*coep(7) - 5.1533480920775964e11*coep(7)**2&
            + jx*(3.206784652929903e17*coen(5)**2 - 1.6752732368740664e18*coen(8)**2&
            - 4.950124753990532e14*coen(12)**2 + 1.5489206249584611e13*coen(13)**2&
            + 3.5056634015320594e13*coen(14)**2 - 7.011326803064117e13*coen(12)*coen(15)&
            - 2.215619426289588e12*coen(15)**2 + 2.5246846855979363e17*coen(5)*coep(5)&
            + 4.663113522498467e16*coep(5)**2 - 1.0783731217628996e18*coen(8)*coep(8)&
            - 1.5761358698498432e17*coep(8)**2 + 9.165232912267846e12*coen(12)*coep(12)&
            + 1.2617084365327273e12*coen(15)*coep(12) - 2.369698308370908e11*coep(12)**2&
            + 5.1114382614517975e11*coen(13)*coep(13) + 7.636103858093108e9*coep(13)**2&
            - 1.2617084365327278e12*coen(14)*coep(14) + 8.568170675903009e9*coep(14)**2&
            + 1.2617084365327273e12*coen(12)*coep(15) + 3.764999799497594e10*coen(15)*coep(15)&
            - 1.7136341351806015e10*coep(12)*coep(15) + 1.6840697267822325e8*coep(15)**2)&
            + jx**2*(3.206784652929903e17*coen(5)**2 - 1.6752732368740664e18*coen(8)**2&
            - 4.950124753990532e14*coen(12)**2 + 1.5489206249584611e13*coen(13)**2&
            + 3.5056634015320594e13*coen(14)**2 - 7.011326803064117e13*coen(12)*coen(15)&
            - 2.215619426289588e12*coen(15)**2 + 2.5246846855979363e17*coen(5)*coep(5)&
            + 4.663113522498467e16*coep(5)**2 - 1.0783731217628996e18*coen(8)*coep(8)&
            - 1.5761358698498432e17*coep(8)**2 + 9.165232912267846e12*coen(12)*coep(12)&
            + 1.2617084365327273e12*coen(15)*coep(12) - 2.369698308370908e11*coep(12)**2&
            + 5.1114382614517975e11*coen(13)*coep(13) + 7.636103858093108e9*coep(13)**2&
            - 1.2617084365327278e12*coen(14)*coep(14) + 8.568170675903009e9*coep(14)**2&
            + 1.2617084365327273e12*coen(12)*coep(15) + 3.764999799497594e10*coen(15)*coep(15)&
            - 1.7136341351806015e10*coep(12)*coep(15) + 1.6840697267822325e8*coep(15)**2))&
            + Q**7*(-2.5517529676758826e17*coen(3)**2 + 1.1350348414772397e18*coen(7)**2&
            + 1.2347766672177348e16*coen(3)*coep(3) - 1.4275736084806112e14*coep(3)**2&
            - 6.208327702429078e16*coen(7)*coep(7) + 8.429820397718228e14*coep(7)**2&
            + jx*(-4.6408195550227805e20*coen(5)**2 + 1.3744349191230042e21*coen(8)**2&
            + 6.817574707845132e17*coen(12)**2 - 5.2069262811879144e16*coen(13)**2&
            - 8.505843225586274e16*coen(14)**2 + 1.7011686451172544e17*coen(12)*coen(15)&
            + 9.711341952501232e15*coen(15)**2 - 2.9872948250676475e20*coen(5)*coep(5)&
            - 4.3661905444274954e19*coep(5)**2 + 6.791199275592834e20*coen(8)*coep(8)&
            + 7.241437205861147e19*coep(8)**2 - 1.2761865243198236e16*coen(12)*coep(12)&
            - 4.1159222240591145e15*coen(15)*coep(12) + 3.5081343164737725e14*coep(12)**2&
            - 1.5769816387635128e15*coen(13)*coep(13) - 1.8059279222134383e13*coep(13)**2&
            + 4.1159222240591155e15*coen(14)*coep(14) - 4.75857869493537e13*coep(14)**2&
            - 4.1159222240591145e15*coen(12)*coep(15) - 3.495167866421577e14*coen(15)*coep(15)&
            + 9.517157389870739e13*coep(12)*coep(15) + 2.373543201686837e12*coep(15)**2)&
            + jx**2*(-4.6408195550227805e20*coen(5)**2 + 1.3744349191230042e21*coen(8)**2&
            + 6.817574707845132e17*coen(12)**2 - 5.2069262811879144e16*coen(13)**2&
            - 8.505843225586274e16*coen(14)**2 + 1.7011686451172544e17*coen(12)*coen(15)&
            + 9.711341952501232e15*coen(15)**2 - 2.9872948250676475e20*coen(5)*coep(5)&
            - 4.3661905444274954e19*coep(5)**2 + 6.791199275592834e20*coen(8)*coep(8)&
            + 7.241437205861147e19*coep(8)**2 - 1.2761865243198236e16*coen(12)*coep(12)&
            - 4.1159222240591145e15*coen(15)*coep(12) + 3.5081343164737725e14*coep(12)**2&
            - 1.5769816387635128e15*coen(13)*coep(13) - 1.8059279222134383e13*coep(13)**2&
            + 4.1159222240591155e15*coen(14)*coep(14) - 4.75857869493537e13*coep(14)**2&
            - 4.1159222240591145e15*coen(12)*coep(15) - 3.495167866421577e14*coen(15)*coep(15)&
            + 9.517157389870739e13*coep(12)*coep(15) + 2.373543201686837e12*coep(15)**2))&
            + Q**9*(-1.4988180503814274e23*coen(3)**2 - 2.356389681148182e22*coen(7)**2&
            + 8.496621557601796e21*coen(3)*coep(3) - 1.2224797541939087e20*coep(3)**2&
            + 6.737104216685612e20*coen(7)*coep(7) + 1.2141117064277925e15*coep(7)**2&
            + jx*(-1.579032510645004e26*coen(5)**2 + 9.134295530885455e25*coen(8)**2&
            - 6.020435002609734e21*coen(12)**2 - 5.352183588881906e22*coen(13)**2&
            - 4.996060167938091e22*coen(14)**2 + 9.992120335876182e22*coen(12)*coen(15)&
            + 2.9033941813391385e22*coen(15)**2 - 5.33696660112826e25*coen(5)*coep(5)&
            - 3.525857254965457e24*coep(5)**2 + 1.5687286154975703e25*coen(8)*coep(8)&
            + 2.264305661188486e18*coep(8)**2 + 2.4255039066535313e20*coen(12)*coep(12)&
            - 2.8322071858672645e21*coen(15)*coep(12) + 4.297202425166319e14*coep(12)**2&
            - 1.0996077223303245e21*coen(13)*coep(13) - 7.548109543622627e18*coep(13)**2&
            + 2.8322071858672655e21*coen(14)*coep(14) - 4.074932513979695e19*coep(14)**2&
            - 2.8322071858672645e21*coen(12)*coep(15) - 1.588076583060611e21*coen(15)*coep(15)&
            + 8.14986502795939e19*coep(12)*coep(15) + 2.156329532634868e19*coep(15)**2)&
            + jx**2*(-1.579032510645004e26*coen(5)**2 + 9.134295530885455e25*coen(8)**2&
            - 6.020435002609734e21*coen(12)**2 - 5.352183588881906e22*coen(13)**2&
            - 4.996060167938091e22*coen(14)**2 + 9.992120335876182e22*coen(12)*coen(15)&
            + 2.9033941813391385e22*coen(15)**2 - 5.33696660112826e25*coen(5)*coep(5)&
            - 3.525857254965457e24*coep(5)**2 + 1.5687286154975703e25*coen(8)*coep(8)&
            + 2.264305661188486e18*coep(8)**2 + 2.4255039066535313e20*coen(12)*coep(12)&
            - 2.8322071858672645e21*coen(15)*coep(12) + 4.297202425166319e14*coep(12)**2&
            - 1.0996077223303245e21*coen(13)*coep(13) - 7.548109543622627e18*coep(13)**2&
            + 2.8322071858672655e21*coen(14)*coep(14) - 4.074932513979695e19*coep(14)**2&
            - 2.8322071858672645e21*coen(12)*coep(15) - 1.588076583060611e21*coen(15)*coep(15)&
            + 8.14986502795939e19*coep(12)*coep(15) + 2.156329532634868e19*coep(15)**2))&
            + Q**11*(-7.20324303043699e25*coen(3)**2 - 2.6298975229228912e20*coen(3)*coep(3)&
            - 2.4360900309807144e14*coep(3)**2 + jx*(-1.3397220779678762e30*coen(5)**2&
            - 1.2231920233853443e24*coen(13)**2 - 2.401081010145663e25*coen(14)**2&
            + 4.802162020291325e25*coen(12)*coen(15) - 6.0275930211173256e26*coen(15)**2&
            - 3.877259310546214e23*coen(5)*coep(5) - 2.8392563492374292e16*coep(5)**2&
            + 8.766325076409636e19*coen(15)*coep(12) - 3.9126690341908106e18*coen(13)*coep(13)&
            - 3.236913860196339e12*coep(13)**2 - 8.766325076409637e19*coen(14)*coep(14)&
            - 8.120300103269047e13*coep(14)**2 + 8.766325076409636e19*coen(12)*coep(15)&
            + 1.7233364533852165e25*coen(15)*coep(15) + 1.624060020653809e14*coep(12)*coep(15)&
            + 3.1056710641149694e19*coep(15)**2) + jx**2*(-1.3397220779678762e30*coen(5)**2&
            - 1.2231920233853443e24*coen(13)**2 - 2.401081010145663e25*coen(14)**2&
            + 4.802162020291325e25*coen(12)*coen(15) - 6.0275930211173256e26*coen(15)**2&
            - 3.877259310546214e23*coen(5)*coep(5) - 2.8392563492374292e16*coep(5)**2&
            + 8.766325076409636e19*coen(15)*coep(12) - 3.9126690341908106e18*coen(13)*coep(13)&
            - 3.236913860196339e12*coep(13)**2 - 8.766325076409637e19*coen(14)*coep(14)&
            - 8.120300103269047e13*coep(14)**2 + 8.766325076409636e19*coen(12)*coep(15)&
            + 1.7233364533852165e25*coen(15)*coep(15) + 1.624060020653809e14*coep(12)*coep(15)&
            + 3.1056710641149694e19*coep(15)**2))) + mx**2*((1.61350623422276e-6&
            - 0.056549832046708465*Q + 787.0802035611583*Q**2 - 5.667729520111529e6*Q**3&
            + 2.3323383727265354e10*Q**4 - 5.696071946083088e13*Q**5 + 8.243285700537154e16*Q**6&
            - 6.762992129132477e19*Q**7 + 2.8047666928992123e22*Q**8 - 4.4945866785722843e24*Q**9&
            + 2.3796899916209525e26*Q**10)*coen(1)**2 + 5.977201416193e-12*jx*coen(4)**2&
            + 5.977201416193e-12*jx**2*coen(4)**2 + 5.9772014161930016e-12*v**2*coen(7)**2&
            + 5.378354114075866e-7*jx*v**2*coen(8)**2 + 5.378354114075866e-7*jx**2*v**2*coen(8)**2&
            + 3.984800944128666e-12*jx*v**2*coen(12)**2&
            + 3.984800944128666e-12*jx**2*v**2*coen(12)**2 + 7.936553500017062e-7*coep(1)**2&
            + coen(1)*(2.2632435618395114e-6*coep(1) + Q**9*(-2.4623306500832697e23*coen(3)&
            - 7.719026293456301e23*coep(1) - 1.3678892042808019e22*coep(3))&
            + Q**7*(-1.4177840046301317e18*coen(3) - 3.341651656923171e19*coep(1)&
            - 3.7896557354156826e17*coep(3)) + Q**5*(-6.264239596742513e11*coen(3)&
            - 4.484487474767255e13*coep(1) - 2.6670835544555408e11*coep(3))&
            + Q**3*(-30030.89372198237*coen(3) - 5.98463589054268e6*coep(1)&
            - 17178.927221062128*coep(3)) + Q*(-0.00008238014816207418*coen(3)&
            - 0.07322575376037399*coep(1) - 0.000059426143396818775*coep(3))&
            + Q**2*(2.5128321824035527*coen(3) + 928.4745842245751*coep(1)&
            + 1.6279430180178944*coep(3)) + Q**4*(1.8303986255644053e8*coen(3)&
            + 2.1570693569687515e10*coep(1) + 9.127317114339218e7*coep(3))&
            + Q**6*(1.2438404895646785e15*coen(3) + 5.3062017220895576e16*coep(1)&
            + 4.3568195474535675e14*coep(3)) + Q**10*(2.003006367657984e25*coen(3)&
            + 6.887006885951473e19*coep(1) + 2.898427673935294e18*coep(3))&
            + Q**8*(8.717955246543446e20*coen(3) + 9.4798213862269e21*coep(1)&
            + 1.4846684434359285e20*coep(3))) + 3.069095154934697e-13*jx*coen(4)*coep(4)&
            + 3.069095154934697e-13*jx**2*coen(4)*coep(4) + 3.9396970313420546e-15*jx*coep(4)**2&
            + 3.9396970313420546e-15*jx**2*coep(4)**2&
            + 3.0690951549347063e-13*v**2*coen(7)*coep(7) + 3.9396970313420775e-15*v**2*coep(7)**2&
            + 7.544145206131705e-7*jx*v**2*coen(8)*coep(8)&
            + 7.544145206131705e-7*jx**2*v**2*coen(8)*coep(8)&
            + 2.64551783333902e-7*jx*v**2*coep(8)**2 + 2.64551783333902e-7*jx**2*v**2*coep(8)**2&
            + 2.0460634366231283e-13*jx*v**2*coen(12)*coep(12)&
            + 2.0460634366231283e-13*jx**2*v**2*coen(12)*coep(12)&
            + 2.626464687561362e-15*jx*v**2*coep(12)**2&
            + 2.626464687561362e-15*jx**2*v**2*coep(12)**2 + jx*Q**13*((-4.402666013063324e23&
            - 4.402666013063324e23*jx)*coen(15)**2 + (-1.607409383952108e18&
            - 1.607409383952108e18*jx)*coen(15)*coep(15) + (-1.4889530644519312e12&
            - 1.4889530644519312e12*jx)*coep(15)**2) + Q**11*((-4.3207191756078495e20&
            + 1.1814667793354661e24*v**2)*coen(3)**2 - 1.7211508352302692e19*coen(7)**2&
            + 6.0187920951915045e22*coen(10)**2 + (1.235327739294725e19&
            + 4.313524537851851e18*v**2)*coen(3)*coep(3) + 2.2262174093108516e13*coep(3)**2&
            + 3.9956439494162324e12*v**2*coep(3)**2 - 6.283906150358154e13*coen(7)*coep(7)&
            - 5.820820391317861e7*coep(7)**2 + 1.925253026823359e17*coen(10)*coep(10)&
            + 1.592743508958282e11*coep(10)**2 + jx*((1.67488111884513e24&
            + 2.197395148231311e28*v**2)*coen(5)**2 + 4.0125280634610016e22*coen(4)*coen(6)&
            - 2.3086471065600932e24*coen(6)**2 - 3.2011467109016246e23*coen(8)**2&
            + 7.876445195569773e23*coen(9)**2 + 2.197395148231311e28*coen(11)**2&
            - 6.029440302077294e18*coen(12)**2 + 3.363217625080261e19*coen(13)**2&
            + 2.006264031730501e22*v**2*coen(13)**2 - 1.4402397252026163e20*coen(14)**2&
            + 3.938222597784887e23*v**2*coen(14)**2 - 1.849567166171092e27*coen(11)*coen(15)&
            + 2.8804794504052323e20*coen(12)*coen(15) - 7.87644519556977e23*v**2*coen(12)*coen(15)&
            + 3.891901491580748e25*coen(15)**2 + 9.886381569680834e24*v**2*coen(15)**2&
            + 6.4175100894111944e16*coen(6)*coep(4) + (2.8764494533868505e23&
            + 6.359431510117441e21*v**2)*coen(5)*coep(5) + 4.151872234038724e16*coep(5)**2&
            + 4.65691222754401e14*v**2*coep(5)**2 + 6.4175100894111944e16*coen(4)*coep(6)&
            - 2.2631182107056538e22*coen(6)*coep(6) + 1.0618290059721877e11*coep(4)*coep(6)&
            - 3.148729028904158e16*coep(6)**2 - 9.264366164730264e16*coen(8)*coep(8)&
            - 6.78415043928677e9*coep(8)**2 + 2.8756830252345667e18*coen(9)*coep(9)&
            + 2.6637626329441543e12*coep(9)**2 + 6.359431510117441e21*coen(11)*coep(11)&
            - 2.676395214145822e20*coen(15)*coep(11) + 4.65691222754401e14*coep(11)**2&
            - 2.1881251289426543e13*coen(12)*coep(12) - 4.117759130982416e18*coen(15)*coep(12)&
            - 1.4378415126172833e18*v**2*coen(15)*coep(12) - 2.017616640595954e7*coep(12)**2&
            + 3.296891513760351e17*coen(13)*coep(13) + 6.417510089411195e16*v**2*coen(13)*coep(13)&
            + 4.587041881161004e11*coep(13)**2 + 5.309145029860939e10*v**2*coep(13)**2&
            + 4.1177591309824164e18*coen(14)*coep(14)&
            + 1.4378415126172833e18*v**2*coen(14)*coep(14) + 7.420724697702838e12*coep(14)**2&
            + 1.3318813164720771e12*v**2*coep(14)**2 - 2.6763952156080924e20*coen(11)*coep(15)&
            - 4.117759130982416e18*coen(12)*coep(15)&
            - 1.4378415126172833e18*v**2*coen(12)*coep(15)&
            + 6.319559751776371e19*coen(15)*coep(15) - 2.826594577871569e23*v**2*coen(15)*coep(15)&
            - 3.919764710229431e13*coep(11)*coep(15) - 1.4841449395405674e13*coep(12)*coep(15)&
            - 2.6637626329441543e12*v**2*coep(12)*coep(15) - 7.471862467070195e17*coep(15)**2&
            - 5.093882261491134e17*v**2*coep(15)**2) + jx**2*((1.67488111884513e24&
            + 2.197395148231311e28*v**2)*coen(5)**2 + 4.0125280634610016e22*coen(4)*coen(6)&
            - 2.3086471065600932e24*coen(6)**2 - 3.2011467109016246e23*coen(8)**2&
            + 7.876445195569773e23*coen(9)**2 + 2.197395148231311e28*coen(11)**2&
            - 6.029440302077294e18*coen(12)**2 + 3.363217625080261e19*coen(13)**2&
            + 2.006264031730501e22*v**2*coen(13)**2 - 1.4402397252026163e20*coen(14)**2&
            + 3.938222597784887e23*v**2*coen(14)**2 - 1.849567166171092e27*coen(11)*coen(15)&
            + 2.8804794504052323e20*coen(12)*coen(15) - 7.87644519556977e23*v**2*coen(12)*coen(15)&
            + 3.891901491580748e25*coen(15)**2 + 9.886381569680834e24*v**2*coen(15)**2&
            + 6.4175100894111944e16*coen(6)*coep(4) + (2.8764494533868505e23&
            + 6.359431510117441e21*v**2)*coen(5)*coep(5) + 4.151872234038724e16*coep(5)**2&
            + 4.65691222754401e14*v**2*coep(5)**2 + 6.4175100894111944e16*coen(4)*coep(6)&
            - 2.2631182107056538e22*coen(6)*coep(6) + 1.0618290059721877e11*coep(4)*coep(6)&
            - 3.148729028904158e16*coep(6)**2 - 9.264366164730264e16*coen(8)*coep(8)&
            - 6.78415043928677e9*coep(8)**2 + 2.8756830252345667e18*coen(9)*coep(9)&
            + 2.6637626329441543e12*coep(9)**2 + 6.359431510117441e21*coen(11)*coep(11)&
            - 2.676395214145822e20*coen(15)*coep(11)&
            + 4.65691222754401e14*coep(11)**2 - 2.1881251289426543e13*coen(12)*coep(12)&
            - 4.117759130982416e18*coen(15)*coep(12)&
            - 1.4378415126172833e18*v**2*coen(15)*coep(12) - 2.017616640595954e7*coep(12)**2&
            + 3.296891513760351e17*coen(13)*coep(13) + 6.417510089411195e16*v**2*coen(13)*coep(13)&
            + 4.587041881161004e11*coep(13)**2 + 5.309145029860939e10*v**2*coep(13)**2&
            + 4.1177591309824164e18*coen(14)*coep(14) + 1.4378415126172833e18*v**2*coen(14)*coep(14)&
            + 7.420724697702838e12*coep(14)**2 + 1.3318813164720771e12*v**2*coep(14)**2&
            - 2.6763952156080924e20*coen(11)*coep(15) - 4.117759130982416e18*coen(12)*coep(15)&
            - 1.4378415126172833e18*v**2*coen(12)*coep(15) + 6.319559751776371e19*coen(15)*coep(15)&
            - 2.826594577871569e23*v**2*coen(15)*coep(15) - 3.919764710229431e13*coep(11)*coep(15)&
            - 1.4841449395405674e13*coep(12)*coep(15) - 2.6637626329441543e12*v**2*coep(12)*coep(15)&
            - 7.471862467070195e17*coep(15)**2 - 5.093882261491134e17*v**2*coep(15)**2))&
            + Q**9*((-2.4020638906795263e21 + 2.4583423429024306e21*v**2)*coen(3)**2&
            - 3.5812923822947828e16*coen(7)**2 + 3.8649204471955404e20*v**2*coen(7)**2&
            + 2.6335750774126637e21*coen(10)**2 - 1.1141656219225733e17*coep(1)**2&
            - 6.693512715833374e15*coep(1)*coep(3) - 6.890307342902006e13*coep(3)**2&
            + 2.0050957768429883e18*v**2*coep(3)**2 + coen(3)*(-3.2485848406847572e22*coep(1)&
            + (-5.7568262290559264e20 - 1.3936050837761466e20*v**2)*coep(3))&
            + 2.0301921271722898e15*coen(7)*coep(7) - 1.1050112827377502e19*v**2*coen(7)*coep(7)&
            - 2.9210066092345844e13*coep(7)**2 - 1.991370611106134e13*v**2*coep(7)**2&
            + 5.410687889098761e19*coen(10)*coep(10) + 3.714094041347698e17*coep(10)**2&
            + jx*(2.2757709235894318e20*coen(4)**2 + (2.256697436941291e19&
            + 2.5899090825269174e24*v**2)*coen(5)**2 - 3.818959154649515e20*coen(6)**2&
            - 3.739663363672151e19*coen(8)**2 - 1.4981955595240944e24*v**2*coen(8)**2&
            - 3.1399971502108164e19*coen(8)*coen(9) + 1.638894895268287e21*coen(9)**2&
            + 2.5899090825269174e24*coen(11)**2 - 8.207768833610898e22*coen(11)*coen(12)&
            + 5.075705538073328e20*coen(12)**2 + 9.874641078575851e19*v**2*coen(12)**2&
            - 9.472052455010879e17*coen(13)**2 + 8.778583591375544e20*v**2*coen(13)**2&
            + 6.937402082782992e15*coen(14)**2 + 8.194474476341435e20*v**2*coen(14)**2&
            - 8.050121078251627e22*coen(11)*coen(15) + 1.601375927119684e21*coen(12)*coen(15)&
            - 1.6388948952682867e21*v**2*coen(12)*coen(15) + 4.985130786625518e20*coen(15)**2&
            - 4.762110285705554e20*v**2*coen(15)**2 + 1.8035626296995865e19*coen(6)*coep(4)&
            - 1.3686118913726057e13*coep(4)**2 - 4.499926393480056e12*coep(4)*coep(5)&
            + 1.3278027762937037e18*coep(5)**2 + 5.783066318627197e22*v**2*coep(5)**2&
            + coen(5)*(-1.3222804179023496e18*coep(4) + (1.2246330891909403e19&
            + 8.75362488119942e23*v**2)*coep(5)) - 9.98446732453784e18*coen(6)*coep(6)&
            + 2.476062694231798e17*coep(4)*coep(6) - 8.787953240926738e16*coep(6)**2&
            + coen(4)*(3.1399971502108164e19*coen(5) + 1.7557167182751085e21*coen(6)&
            - 7.661651665115406e18*coep(4) - 1.1419926624854592e18*coep(5)&
            + 1.8035626296995865e19*coep(6)) - 1.275220564219922e19*coen(8)*coep(8)&
            - 2.5730087644854338e23*v**2*coen(8)*coep(8) + 1.1419926624854592e18*coen(9)*coep(8)&
            - 8.424722216973624e17*coep(8)**2 - 3.713885406408578e16*v**2*coep(8)**2&
            + 1.3222804179023496e18*coen(8)*coep(9) - 9.290700558507642e19*coen(9)*coep(9)&
            + 4.499926393480056e12*coep(8)*coep(9) + 1.3367305178953254e18*coep(9)**2&
            + 8.75362488119942e23*coen(11)*coep(11) - 1.0828616135615856e22*coen(12)*coep(11)&
            - 1.7085168678757435e22*coen(15)*coep(11) + 5.783066318627197e22*coep(11)**2&
            - 4.559630680936006e21*coen(11)*coep(12) + 5.607775071814494e14*coen(12)*coep(12)&
            - 3.978280722656239e18*v**2*coen(12)*coep(12)&
            + 1.9189420763519753e20*coen(15)*coep(12) + 4.64535027925382e19*v**2*coen(15)*coep(12)&
            - 2.2311709052777912e15*coep(11)*coep(12) - 1.1540231569292846e13*coep(12)**2&
            - 7.048216876705609e12*v**2*coep(12)**2 - 1.931827129430931e17*coen(13)*coep(13)&
            + 1.8035626296995867e19*v**2*coen(13)*coep(13) + 1.2781255997006555e12*coep(13)**2&
            + 1.2380313471158992e17*v**2*coep(13)**2 - 3.794567704844723e14*coen(14)*coep(14)&
            - 4.645350279253821e19*v**2*coen(14)*coep(14) + 5.152357570671954e12*coep(14)**2&
            + 6.683652589476627e17*v**2*coep(14)**2 - 1.3709362336377348e22*coen(11)*coep(15)&
            + 1.918942076351975e20*coen(12)*coep(15) + 4.64535027925382e19*v**2*coen(12)*coep(15)&
            + 2.2413748857292505e20*coen(15)*coep(15)&
            + 2.6047430553136108e19*v**2*coen(15)*coep(15)&
            - 2.0431755556220772e21*coep(11)*coep(15) + 4.593538228601336e13*coep(12)*coep(15)&
            - 1.3367305178953252e18*v**2*coep(12)*coep(15) + 1.8392465122414574e19*coep(15)**2&
            - 3.536784330812055e17*v**2*coep(15)**2) + jx**2*(2.2757709235894318e20*coen(4)**2&
            + (2.256697436941291e19 + 2.5899090825269174e24*v**2)*coen(5)**2&
            - 3.818959154649515e20*coen(6)**2 - 3.739663363672151e19*coen(8)**2&
            - 1.4981955595240944e24*v**2*coen(8)**2 - 3.1399971502108164e19*coen(8)*coen(9)&
            + 1.638894895268287e21*coen(9)**2 + 2.5899090825269174e24*coen(11)**2&
            - 8.207768833610898e22*coen(11)*coen(12) + 5.075705538073328e20*coen(12)**2&
            + 9.874641078575851e19*v**2*coen(12)**2 - 9.472052455010879e17*coen(13)**2&
            + 8.778583591375544e20*v**2*coen(13)**2 + 6.937402082782992e15*coen(14)**2&
            + 8.194474476341435e20*v**2*coen(14)**2 - 8.050121078251627e22*coen(11)*coen(15)&
            + 1.601375927119684e21*coen(12)*coen(15)&
            - 1.6388948952682867e21*v**2*coen(12)*coen(15) + 4.985130786625518e20*coen(15)**2&
            - 4.762110285705554e20*v**2*coen(15)**2 + 1.8035626296995865e19*coen(6)*coep(4)&
            - 1.3686118913726057e13*coep(4)**2 - 4.499926393480056e12*coep(4)*coep(5)&
            + 1.3278027762937037e18*coep(5)**2 + 5.783066318627197e22*v**2*coep(5)**2&
            + coen(5)*(-1.3222804179023496e18*coep(4) + (1.2246330891909403e19&
            + 8.75362488119942e23*v**2)*coep(5)) - 9.98446732453784e18*coen(6)*coep(6)&
            + 2.476062694231798e17*coep(4)*coep(6) - 8.787953240926738e16*coep(6)**2&
            + coen(4)*(3.1399971502108164e19*coen(5) + 1.7557167182751085e21*coen(6)&
            - 7.661651665115406e18*coep(4) - 1.1419926624854592e18*coep(5)&
            + 1.8035626296995865e19*coep(6)) - 1.275220564219922e19*coen(8)*coep(8)&
            - 2.5730087644854338e23*v**2*coen(8)*coep(8) + 1.1419926624854592e18*coen(9)*coep(8)&
            - 8.424722216973624e17*coep(8)**2 - 3.713885406408578e16*v**2*coep(8)**2&
            + 1.3222804179023496e18*coen(8)*coep(9) - 9.290700558507642e19*coen(9)*coep(9)&
            + 4.499926393480056e12*coep(8)*coep(9) + 1.3367305178953254e18*coep(9)**2&
            + 8.75362488119942e23*coen(11)*coep(11) - 1.0828616135615856e22*coen(12)*coep(11)&
            - 1.7085168678757435e22*coen(15)*coep(11) + 5.783066318627197e22*coep(11)**2&
            - 4.559630680936006e21*coen(11)*coep(12) + 5.607775071814494e14*coen(12)*coep(12)&
            - 3.978280722656239e18*v**2*coen(12)*coep(12)&
            + 1.9189420763519753e20*coen(15)*coep(12) + 4.64535027925382e19*v**2*coen(15)*coep(12)&
            - 2.2311709052777912e15*coep(11)*coep(12) - 1.1540231569292846e13*coep(12)**2&
            - 7.048216876705609e12*v**2*coep(12)**2 - 1.931827129430931e17*coen(13)*coep(13)&
            + 1.8035626296995867e19*v**2*coen(13)*coep(13) + 1.2781255997006555e12*coep(13)**2&
            + 1.2380313471158992e17*v**2*coep(13)**2 - 3.794567704844723e14*coen(14)*coep(14)&
            - 4.645350279253821e19*v**2*coen(14)*coep(14) + 5.152357570671954e12*coep(14)**2&
            + 6.683652589476627e17*v**2*coep(14)**2 - 1.3709362336377348e22*coen(11)*coep(15)&
            + 1.918942076351975e20*coen(12)*coep(15) + 4.64535027925382e19*v**2*coen(12)*coep(15)&
            + 2.2413748857292505e20*coen(15)*coep(15)&
            + 2.6047430553136108e19*v**2*coen(15)*coep(15)&
            - 2.0431755556220772e21*coep(11)*coep(15) + 4.593538228601336e13*coep(12)*coep(15)&
            - 1.3367305178953252e18*v**2*coep(12)*coep(15) + 1.8392465122414574e19*coep(15)**2&
            - 3.536784330812055e17*v**2*coep(15)**2)) + Q**7*((-6.165204433546132e15&
            + 4.185352830163842e15*v**2)*coen(3)**2 - 6.097186688077387e10*coen(7)**2&
            - 1.861669740875455e16*v**2*coen(7)**2 + 2.5621003196802085e15*coen(10)**2&
            - 3.5631940185349094e18*coep(1)**2 - 9.340716201149659e16*coep(1)*coep(3)&
            - 5.4085942093773544e14*coep(3)**2 + 2.341488113547068e12*v**2*coep(3)**2&
            + coen(3)*(-4.093449930699737e17*coep(1) + (-3.8798221824557695e15&
            - 2.025265017509503e14*v**2)*coep(3)) + 2.9503889888548207e9*coen(7)*coep(7)&
            + 1.0182820300043464e15*v**2*coen(7)*coep(7) - 3.4110601269551136e7*coep(7)**2&
            - 1.3826484423175666e13*v**2*coep(7)**2 + 7.759635805491058e13*coen(10)*coep(10)&
            + 8.886180170322792e11*coep(10)**2 + jx*(-1.7387665308546096e16*coen(4)**2&
            + (-7.115423640319576e12 + 7.61181333183088e18*v**2)*coen(5)**2&
            - 7.037714447707681e13*coen(6)**2 - 3.543474419243733e13*coen(8)**2&
            - 2.2543307097108255e19*v**2*coen(8)**2 - 1.194499055770302e15*coen(8)*coen(9)&
            + 2.790235220109228e15*coen(9)**2 + 7.61181333183088e18*coen(11)**2&
            - 4.725946682100439e17*coen(11)*coen(12) + 6.506006251718743e15*coen(12)**2&
            - 1.1182099505627914e16*v**2*coen(12)**2 - 3.4919926265932456e12*coen(13)**2&
            + 8.540334398934028e14*v**2*coen(13)**2 + 2.320438758226966e9*coen(14)**2&
            + 1.395117610054614e15*v**2*coen(14)**2 - 1.1485567727590126e17*coen(11)*coen(15)&
            + 4.1101362890307535e15*coen(12)*coen(15) - 2.790235220109227e15*v**2*coen(12)*coen(15)&
            + 3.604242811469641e14*coen(15)**2 - 1.5928419811973178e14*v**2*coen(15)**2&
            + 2.5865452684970188e13*coen(6)*coep(4) - 1.0362825520915184e13*coep(4)**2&
            - 3.0021433585111577e12*coep(4)*coep(5) + 2.830117097582871e12*coep(5)**2&
            + 7.161370314305185e17*v**2*coep(5)**2 + coen(5)*(-3.2375779780743152e13*coep(4)&
            + (1.1222146378507348e13 + 4.899723056663362e18*v**2)*coep(5))&
            - 2.3224458582020537e12*coen(6)*coep(6) + 5.924120113548527e11*coep(4)*coep(6)&
            - 3.4695592259764404e10*coep(6)**2 + coen(4)*(1.194499055770302e15*coen(5)&
            + 1.7080668797868052e15*coen(6) + 5.487458372893024e14*coep(4)&
            + 1.0801786759820283e14*coep(5) + 2.5865452684970188e13*coep(6))&
            - 5.997766131263943e13*coen(8)*coep(8) - 1.1138838856410567e19*v**2*coen(8)*coep(8)&
            - 1.0801786759820283e14*coen(9)*coep(8) - 1.001544698449788e13*coep(8)**2&
            - 1.1877313395116362e18*v**2*coep(8)**2 + 3.2375779780743152e13*coen(8)*coep(9)&
            - 1.3501766783396686e14*coen(9)*coep(9) + 3.0021433585111577e12*coep(8)*coep(9)&
            + 1.5609920756980452e12*coep(9)**2 + 4.899723056663362e18*coen(11)*coep(11)&
            - 1.3644833102332454e17*coen(12)*coep(11) - 4.178842160416049e16*coen(15)*coep(11)&
            + 7.161370314305185e17*coep(11)**2 - 1.2632185784718939e17*coen(11)*coep(12)&
            + 2.9241676354423705e15*coen(12)*coep(12) + 2.093184939545202e14*v**2*coen(12)*coep(12)&
            + 1.2932740608185898e15*coen(15)*coep(12)&
            + 6.7508833916983414e13*v**2*coen(15)*coep(12)&
            - 3.1135720670498864e16*coep(11)*coep(12) + 2.399611797478974e14*coep(12)**2&
            - 5.753997379856627e12*v**2*coep(12)**2 - 1.3096468179378328e12*coen(13)*coep(13)&
            + 2.586545268497019e13*v**2*coen(13)*coep(13) - 1.317104776978066e11*coep(13)**2&
            + 2.962060056774264e11*v**2*coep(13)**2 - 8.351392653478956e7*coen(14)*coep(14)&
            - 6.750883391698343e13*v**2*coen(14)*coep(14) + 567137.0307480234*coep(14)**2&
            + 7.804960378490226e11*v**2*coep(14)**2 - 4.02306778151914e16*coen(11)*coep(15)&
            + 1.2932740608185895e15*coen(12)*coep(15)&
            + 6.7508833916983414e13*v**2*coen(12)*coep(15)&
            + 2.8472509339730875e14*coen(15)*coep(15)&
            + 5.732729973054092e12*v**2*coen(15)*coep(15)&
            - 1.3395678649171466e16*coep(11)*coep(15) + 3.6057294729182356e14*coep(12)*coep(15)&
            - 1.5609920756980452e12*v**2*coep(12)*coep(15) + 5.3023311456536234e13*coep(15)**2&
            - 3.8930554338667305e10*v**2*coep(15)**2) + jx**2*(-1.7387665308546096e16*coen(4)**2&
            + (-7.115423640319576e12 + 7.61181333183088e18*v**2)*coen(5)**2&
            - 7.037714447707681e13*coen(6)**2 - 3.543474419243733e13*coen(8)**2&
            - 2.2543307097108255e19*v**2*coen(8)**2 - 1.194499055770302e15*coen(8)*coen(9)&
            + 2.790235220109228e15*coen(9)**2 + 7.61181333183088e18*coen(11)**2&
            - 4.725946682100439e17*coen(11)*coen(12) + 6.506006251718743e15*coen(12)**2&
            - 1.1182099505627914e16*v**2*coen(12)**2 - 3.4919926265932456e12*coen(13)**2&
            + 8.540334398934028e14*v**2*coen(13)**2 + 2.320438758226966e9*coen(14)**2&
            + 1.395117610054614e15*v**2*coen(14)**2 - 1.1485567727590126e17*coen(11)*coen(15)&
            + 4.1101362890307535e15*coen(12)*coen(15)&
            - 2.790235220109227e15*v**2*coen(12)*coen(15) + 3.604242811469641e14*coen(15)**2&
            - 1.5928419811973178e14*v**2*coen(15)**2 + 2.5865452684970188e13*coen(6)*coep(4)&
            - 1.0362825520915184e13*coep(4)**2 - 3.0021433585111577e12*coep(4)*coep(5)&
            + 2.830117097582871e12*coep(5)**2 + 7.161370314305185e17*v**2*coep(5)**2&
            + coen(5)*(-3.2375779780743152e13*coep(4) + (1.1222146378507348e13&
            + 4.899723056663362e18*v**2)*coep(5)) - 2.3224458582020537e12*coen(6)*coep(6)&
            + 5.924120113548527e11*coep(4)*coep(6) - 3.4695592259764404e10*coep(6)**2&
            + coen(4)*(1.194499055770302e15*coen(5) + 1.7080668797868052e15*coen(6)&
            + 5.487458372893024e14*coep(4) + 1.0801786759820283e14*coep(5)&
            + 2.5865452684970188e13*coep(6)) - 5.997766131263943e13*coen(8)*coep(8)&
            - 1.1138838856410567e19*v**2*coen(8)*coep(8)&
            - 1.0801786759820283e14*coen(9)*coep(8) - 1.001544698449788e13*coep(8)**2&
            - 1.1877313395116362e18*v**2*coep(8)**2 + 3.2375779780743152e13*coen(8)*coep(9)&
            - 1.3501766783396686e14*coen(9)*coep(9) + 3.0021433585111577e12*coep(8)*coep(9)&
            + 1.5609920756980452e12*coep(9)**2 + 4.899723056663362e18*coen(11)*coep(11)&
            - 1.3644833102332454e17*coen(12)*coep(11) - 4.178842160416049e16*coen(15)*coep(11)&
            + 7.161370314305185e17*coep(11)**2 - 1.2632185784718939e17*coen(11)*coep(12)&
            + 2.9241676354423705e15*coen(12)*coep(12) + 2.093184939545202e14*v**2*coen(12)*coep(12)&
            + 1.2932740608185898e15*coen(15)*coep(12) + 6.7508833916983414e13*v**2*coen(15)*coep(12)&
            - 3.1135720670498864e16*coep(11)*coep(12) + 2.399611797478974e14*coep(12)**2&
            - 5.753997379856627e12*v**2*coep(12)**2 - 1.3096468179378328e12*coen(13)*coep(13)&
            + 2.586545268497019e13*v**2*coen(13)*coep(13) - 1.317104776978066e11*coep(13)**2&
            + 2.962060056774264e11*v**2*coep(13)**2 - 8.351392653478956e7*coen(14)*coep(14)&
            - 6.750883391698343e13*v**2*coen(14)*coep(14) + 567137.0307480234*coep(14)**2&
            + 7.804960378490226e11*v**2*coep(14)**2 - 4.02306778151914e16*coen(11)*coep(15)&
            + 1.2932740608185895e15*coen(12)*coep(15)&
            + 6.7508833916983414e13*v**2*coen(12)*coep(15)&
            + 2.8472509339730875e14*coen(15)*coep(15) + 5.732729973054092e12*v**2*coen(15)*coep(15)&
            - 1.3395678649171466e16*coep(11)*coep(15) + 3.6057294729182356e14*coep(12)*coep(15)&
            - 1.5609920756980452e12*v**2*coep(12)*coep(15) + 5.3023311456536234e13*coep(15)**2&
            - 3.8930554338667305e10*v**2*coep(15)**2)) + Q**6*((3.9032490595647783e12&
            - 1.72498338495919e12*v**2)*coen(3)**2 + 2.512941240252884e7*coen(7)**2&
            + 1.5108558951963379e13*v**2*coen(7)**2 - 7.62155984943187e11*coen(10)**2&
            + 7.755474147178521e15*coep(1)**2 + 1.4506977699510328e14*coep(1)*coep(3)&
            + 5.742209984275323e11*coep(3)**2 - 4.216021438044522e8*v**2*coep(3)**2&
            + coen(3)*(4.525516893814015e14*coep(1) + (3.083457500424011e12&
            + 6.2083144911477844e10*v**2)*coep(3)) - 904422.0166580887*coen(7)*coep(7)&
            - 7.310933427133224e11*v**2*coen(7)*coep(7) + 6141.8644572643225*coep(7)**2&
            + 8.452456133181617e9*v**2*coep(7)**2 - 2.515114848275432e10*coen(10)*coep(10)&
            - 3.7573921886730003e8*coep(10)**2 + jx*(1.31553181107283e13*coen(4)**2&
            + (7.689374040264272e9 - 5.259727486509153e15*v**2)*coen(5)**2&
            + 9.91720167891e9*coen(6)**2 - 2.5685719197478947e10*coen(8)**2&
            + 2.7477619001790508e16*v**2*coen(8)**2 + 8.229016005744971e11*coen(8)*coen(9)&
            - 1.1499889233061265e12*coen(9)**2 - 5.259727486509153e15*coen(11)**2&
            + 4.1461349652155944e14*coen(11)*coen(12) - 7.431124316215356e12*coen(12)**2&
            + 8.119131793407173e12*v**2*coen(12)**2 + 2.4798936278696356e9*coen(13)**2&
            - 2.540519949810623e11*v**2*coen(13)**2 - 529402.5496567748*coen(14)**2&
            - 5.749944616530632e11*v**2*coen(14)**2 + 5.784370966683888e13*coen(11)*coen(15)&
            - 2.602166039709852e12*coen(12)*coen(15)&
            + 1.1499889233061262e12*v**2*coen(12)*coen(15) - 1.2912824014951544e11*coen(15)**2&
            + 3.634030861863964e10*v**2*coen(15)**2 - 8.383716160918102e9*coen(6)*coep(4)&
            + 6.704234466377394e9*coep(4)**2 + 2.2231764473840714e9*coep(4)*coep(5)&
            - 1.5560097725412946e9*coep(5)**2 - 7.648379614324492e14*v**2*coep(5)**2&
            + coen(5)*(1.866122512774217e10*coep(4) + (-4.0826529036635976e9&
            - 4.140955777456272e15*v**2)*coep(5)) + coen(4)*(-8.229016005744971e11*coen(5)&
            - 5.081039899621245e11*coen(6) - 3.940247678596427e11*coep(4)&
            - 9.262143654516617e10*coep(5) - 8.383716160918102e9*coep(6))&
            + 3.0765146990857047e8*coen(6)*coep(6) - 2.5049281257819995e8*coep(4)*coep(6)&
            + 6.982613521063168e6*coep(6)**2 + 4.051043412762964e10*coen(8)*coep(8)&
            + 1.7687339073631858e16*v**2*coen(8)*coep(8) + 9.262143654516617e10*coen(9)*coep(8)&
            + 1.0216340830724285e10*coep(8)**2 + 2.585158049059507e15*v**2*coep(8)**2&
            - 1.866122512774217e10*coen(8)*coep(9) + 4.138876327431856e10*coen(9)*coep(9)&
            - 2.2231764473840714e9*coep(8)*coep(9) - 2.8106809586963475e8*coep(9)**2&
            - 4.140955777456272e15*coen(11)*coep(11) + 1.508505631271338e14*coen(12)*coep(11)&
            + 2.5287135967564246e13*coen(15)*coep(11) - 7.648379614324492e14*coep(11)**2&
            + 1.4522731824845225e14*coen(11)*coep(12) - 4.673272495478505e12*coen(12)*coep(12)&
            - 1.5032698695520193e11*v**2*coen(12)*coep(12)&
            - 1.0278191668080035e12*coen(15)*coep(12)&
            - 2.069438163715928e10*v**2*coen(15)*coep(12)&
            + 4.8356592331701086e13*coep(11)*coep(12) - 6.512852993402308e11*coep(12)**2&
            + 3.8867490886501875e9*v**2*coep(12)**2 + 7.933061105724626e8*coen(13)*coep(13)&
            - 8.383716160918104e9*v**2*coen(13)*coep(13) + 6.07639327861947e7*coep(13)**2&
            - 1.2524640628909999e8*v**2*coep(13)**2 + 8996.132050752092*coen(14)*coep(14)&
            + 2.069438163715928e10*v**2*coen(14)*coep(14) + 40.23934781305596*coep(14)**2&
            - 1.4053404793481737e8*v**2*coep(14)**2 + 2.462773085201777e13*coen(11)*coep(15)&
            - 1.0278191668080033e12*coen(12)*coep(15) - 2.069438163715928e10*v**2*coen(12)*coep(15)&
            - 1.2181688205433876e11*coen(15)*coep(15) - 6.175304884918264e8*v**2*coen(15)*coep(15)&
            + 1.0189640120857281e13*coep(11)*coep(15) - 3.8281399895168823e11*coep(12)*coep(15)&
            + 2.8106809586963475e8*v**2*coep(12)*coep(15) - 2.784740265635589e10*coep(15)**2&
            - 2.7621897912794114e6*v**2*coep(15)**2) + jx**2*(1.31553181107283e13*coen(4)**2&
            + (7.689374040264272e9 - 5.259727486509153e15*v**2)*coen(5)**2&
            + 9.91720167891e9*coen(6)**2 - 2.5685719197478947e10*coen(8)**2&
            + 2.7477619001790508e16*v**2*coen(8)**2 + 8.229016005744971e11*coen(8)*coen(9)&
            - 1.1499889233061265e12*coen(9)**2 - 5.259727486509153e15*coen(11)**2&
            + 4.1461349652155944e14*coen(11)*coen(12) - 7.431124316215356e12*coen(12)**2&
            + 8.119131793407173e12*v**2*coen(12)**2 + 2.4798936278696356e9*coen(13)**2&
            - 2.540519949810623e11*v**2*coen(13)**2 - 529402.5496567748*coen(14)**2&
            - 5.749944616530632e11*v**2*coen(14)**2 + 5.784370966683888e13*coen(11)*coen(15)&
            - 2.602166039709852e12*coen(12)*coen(15) + 1.1499889233061262e12*v**2*coen(12)*coen(15)&
            - 1.2912824014951544e11*coen(15)**2 + 3.634030861863964e10*v**2*coen(15)**2&
            - 8.383716160918102e9*coen(6)*coep(4) + 6.704234466377394e9*coep(4)**2&
            + 2.2231764473840714e9*coep(4)*coep(5) - 1.5560097725412946e9*coep(5)**2&
            - 7.648379614324492e14*v**2*coep(5)**2 + coen(5)*(1.866122512774217e10*coep(4)&
            + (-4.0826529036635976e9 - 4.140955777456272e15*v**2)*coep(5))&
            + coen(4)*(-8.229016005744971e11*coen(5) - 5.081039899621245e11*coen(6)&
            - 3.940247678596427e11*coep(4) - 9.262143654516617e10*coep(5) - 8.383716160918102e9*coep(6))&
            + 3.0765146990857047e8*coen(6)*coep(6) - 2.5049281257819995e8*coep(4)*coep(6)&
            + 6.982613521063168e6*coep(6)**2 + 4.051043412762964e10*coen(8)*coep(8)&
            + 1.7687339073631858e16*v**2*coen(8)*coep(8) + 9.262143654516617e10*coen(9)*coep(8)&
            + 1.0216340830724285e10*coep(8)**2 + 2.585158049059507e15*v**2*coep(8)**2&
            - 1.866122512774217e10*coen(8)*coep(9) + 4.138876327431856e10*coen(9)*coep(9)&
            - 2.2231764473840714e9*coep(8)*coep(9) - 2.8106809586963475e8*coep(9)**2&
            - 4.140955777456272e15*coen(11)*coep(11) + 1.508505631271338e14*coen(12)*coep(11)&
            + 2.5287135967564246e13*coen(15)*coep(11) - 7.648379614324492e14*coep(11)**2&
            + 1.4522731824845225e14*coen(11)*coep(12) - 4.673272495478505e12*coen(12)*coep(12)&
            - 1.5032698695520193e11*v**2*coen(12)*coep(12)&
            - 1.0278191668080035e12*coen(15)*coep(12) - 2.069438163715928e10*v**2*coen(15)*coep(12)&
            + 4.8356592331701086e13*coep(11)*coep(12) - 6.512852993402308e11*coep(12)**2&
            + 3.8867490886501875e9*v**2*coep(12)**2 + 7.933061105724626e8*coen(13)*coep(13)&
            - 8.383716160918104e9*v**2*coen(13)*coep(13) + 6.07639327861947e7*coep(13)**2&
            - 1.2524640628909999e8*v**2*coep(13)**2 + 8996.132050752092*coen(14)*coep(14)&
            + 2.069438163715928e10*v**2*coen(14)*coep(14) + 40.23934781305596*coep(14)**2&
            - 1.4053404793481737e8*v**2*coep(14)**2 + 2.462773085201777e13*coen(11)*coep(15)&
            - 1.0278191668080033e12*coen(12)*coep(15) - 2.069438163715928e10*v**2*coen(12)*coep(15)&
            - 1.2181688205433876e11*coen(15)*coep(15) - 6.175304884918264e8*v**2*coen(15)*coep(15)&
            + 1.0189640120857281e13*coep(11)*coep(15) - 3.8281399895168823e11*coep(12)*coep(15)&
            + 2.8106809586963475e8*v**2*coep(12)*coep(15) - 2.784740265635589e10*coep(15)**2&
            - 2.7621897912794114e6*v**2*coep(15)**2)) + Q**3*((-27.363582477663037&
            + 3.6712080829986715*v**2)*coen(3)**2 - 0.000053481849586251846*coen(7)**2&
            - 183.9095715395181*v**2*coen(7)**2 - 0.2961515620676688*coen(10)**2&
            - 1.5392557624809113e6*coep(1)**2 - 9674.547630153409*coep(1)*coep(3)&
            - 10.81192221344953*coep(3)**2 + 0.000986878381617931*v**2*coep(3)**2&
            + coen(3)*(-17288.665544735115*coep(1) + (-34.587263155039494&
            + 0.12091338007565915*v**2)*coep(3)) - 1.761455918589522e-6*coen(7)*coep(7)&
            - 1.4249521203787696*v**2*coen(7)*coep(7) - 1.4376760993210345e-8*coep(7)**2&
            + 0.028354513937707358*v**2*coep(7)**2 - 0.027069076560381614*coen(10)*coep(10)&
            + 0.0001625547532282451*coep(10)**2 + jx*(-128.48228096418177*coen(4)**2&
            + (-0.06131042765687025 + 72678.63573255253*v**2)*coen(5)**2&
            + 0.0042316288053342805*coen(6)**2 + 2.0106814381536426*coen(8)**2&
            - 1.8892431733705096e6*v**2*coen(8)**2 - 6.504557453817875*coen(8)*coen(9)&
            + 2.4474720553324474*coen(9)**2 + 72678.63573255253*coen(11)**2&
            - 10010.297907327456*coen(11)*coen(12) + 333.0560244575585*coen(12)**2&
            - 67.17909045100909*v**2*coen(12)**2 - 0.030561274784674342*coen(13)**2&
            - 0.0987171873558896*v**2*coen(13)**2 + 1.7187453668605463e-7*coen(14)**2&
            + 1.2237360276662237*v**2*coen(14)**2 - 232.03380546942188*coen(11)*coen(15)&
            + 18.242388318442025*coen(12)*coen(15) - 2.447472055332447*v**2*coen(12)*coen(15)&
            + 0.09736571785764904*coen(15)**2 - 0.011798155696277526*v**2*coen(15)**2&
            - 0.009023025520127204*coen(6)*coep(4) + 0.01101197391966905*coep(4)**2&
            + 0.008001791324375992*coep(4)*coep(5) + 0.0056983576820958445*coep(5)**2&
            + 24911.94050320168*v**2*coep(5)**2 + coen(5)*(0.0456139916304453*coep(4)&
            + (-0.004558308235737714 + 85734.9299200709*v**2)*coep(5))&
            + coen(4)*(6.504557453817875*coen(5) - 0.19743437471177916*coen(6)&
            - 0.9694801609953849*coep(4) + 1.2207139227431507*coep(5)&
            - 0.009023025520127204*coep(6)) + 0.00009037503383354881*coen(6)*coep(6)&
            + 0.00010836983548549672*coep(4)*coep(6) - 4.6892455577009424e-7*coep(6)**2&
            - 0.1937896962604607*coen(8)*coep(8) - 1.9948786301808932e6*v**2*coen(8)*coep(8)&
            - 1.2207139227431507*coen(9)*coep(8) - 0.2735642834672355*coep(8)**2&
            - 513085.2541603037*v**2*coep(8)**2 - 0.0456139916304453*coen(8)*coep(9)&
            + 0.08060892005043943*coen(9)*coep(9) - 0.008001791324375992*coep(8)*coep(9)&
            + 0.0006579189210786206*coep(9)**2 + 85734.9299200709*coen(11)*coep(11)&
            - 5762.888514911705*coen(12)*coep(11) - 148.36507496586304*coen(15)*coep(11)&
            + 24911.94050320168*coep(11)**2 - 5726.3090736873755*coen(11)*coep(12)&
            + 365.0859959711297*coen(12)*coep(12) - 0.4944961208691284*v**2*coen(12)*coep(12)&
            + 11.529087718346496*coen(15)*coep(12) - 0.04030446002521971*v**2*coen(15)*coep(12)&
            - 3224.849210051136*coep(11)*coep(12) + 98.51976084570035*coep(12)**2&
            + 0.0015604692737665993*v**2*coep(12)**2 + 0.0038900052409372833*coen(13)*coep(13)&
            - 0.009023025520127204*v**2*coen(13)*coep(13) + 0.00012884274922386047*coep(13)**2&
            + 0.00005418491774274837*v**2*coep(13)**2 + 8.043559713227309e-9*coen(14)*coep(14)&
            + 0.040304460025219715*v**2*coen(14)*coep(14) + 9.321902784802541e-11*coep(14)**2&
            + 0.0003289594605393103*v**2*coep(14)**2 - 150.32353382101195*coen(11)*coep(15)&
            + 11.529087718346494*coen(12)*coep(15) - 0.04030446002521971*v**2*coen(12)*coep(15)&
            + 0.14000027365862183*coen(15)*coep(15) - 0.0005521421129547751*v**2*coen(15)*coep(15)&
            - 95.06262311867982*coep(11)*coep(15) + 7.207948142299686*coep(12)*coep(15)&
            - 0.0006579189210786203*v**2*coep(12)*coep(15) + 0.05053212587813594*coep(15)**2&
            - 6.398926947599854e-6*v**2*coep(15)**2) + jx**2*(-128.48228096418177*coen(4)**2&
            + (-0.06131042765687025 + 72678.63573255253*v**2)*coen(5)**2&
            + 0.0042316288053342805*coen(6)**2 + 2.0106814381536426*coen(8)**2&
            - 1.8892431733705096e6*v**2*coen(8)**2 - 6.504557453817875*coen(8)*coen(9)&
            + 2.4474720553324474*coen(9)**2 + 72678.63573255253*coen(11)**2&
            - 10010.297907327456*coen(11)*coen(12) + 333.0560244575585*coen(12)**2&
            - 67.17909045100909*v**2*coen(12)**2 - 0.030561274784674342*coen(13)**2&
            - 0.0987171873558896*v**2*coen(13)**2 + 1.7187453668605463e-7*coen(14)**2&
            + 1.2237360276662237*v**2*coen(14)**2 - 232.03380546942188*coen(11)*coen(15)&
            + 18.242388318442025*coen(12)*coen(15) - 2.447472055332447*v**2*coen(12)*coen(15)&
            + 0.09736571785764904*coen(15)**2 - 0.011798155696277526*v**2*coen(15)**2&
            - 0.009023025520127204*coen(6)*coep(4) + 0.01101197391966905*coep(4)**2&
            + 0.008001791324375992*coep(4)*coep(5) + 0.0056983576820958445*coep(5)**2&
            + 24911.94050320168*v**2*coep(5)**2 + coen(5)*(0.0456139916304453*coep(4)&
            + (-0.004558308235737714 + 85734.9299200709*v**2)*coep(5))&
            + coen(4)*(6.504557453817875*coen(5) - 0.19743437471177916*coen(6)&
            - 0.9694801609953849*coep(4) + 1.2207139227431507*coep(5)&
            - 0.009023025520127204*coep(6)) + 0.00009037503383354881*coen(6)*coep(6)&
            + 0.00010836983548549672*coep(4)*coep(6) - 4.6892455577009424e-7*coep(6)**2&
            - 0.1937896962604607*coen(8)*coep(8) - 1.9948786301808932e6*v**2*coen(8)*coep(8)&
            - 1.2207139227431507*coen(9)*coep(8) - 0.2735642834672355*coep(8)**2&
            - 513085.2541603037*v**2*coep(8)**2 - 0.0456139916304453*coen(8)*coep(9)&
            + 0.08060892005043943*coen(9)*coep(9) - 0.008001791324375992*coep(8)*coep(9)&
            + 0.0006579189210786206*coep(9)**2 + 85734.9299200709*coen(11)*coep(11)&
            - 5762.888514911705*coen(12)*coep(11) - 148.36507496586304*coen(15)*coep(11)&
            + 24911.94050320168*coep(11)**2 - 5726.3090736873755*coen(11)*coep(12)&
            + 365.0859959711297*coen(12)*coep(12) - 0.4944961208691284*v**2*coen(12)*coep(12)&
            + 11.529087718346496*coen(15)*coep(12) - 0.04030446002521971*v**2*coen(15)*coep(12)&
            - 3224.849210051136*coep(11)*coep(12) + 98.51976084570035*coep(12)**2&
            + 0.0015604692737665993*v**2*coep(12)**2 + 0.0038900052409372833*coen(13)*coep(13)&
            - 0.009023025520127204*v**2*coen(13)*coep(13) + 0.00012884274922386047*coep(13)**2&
            + 0.00005418491774274837*v**2*coep(13)**2 + 8.043559713227309e-9*coen(14)*coep(14)&
            + 0.040304460025219715*v**2*coen(14)*coep(14) + 9.321902784802541e-11*coep(14)**2&
            + 0.0003289594605393103*v**2*coep(14)**2 - 150.32353382101195*coen(11)*coep(15)&
            + 11.529087718346494*coen(12)*coep(15) - 0.04030446002521971*v**2*coen(12)*coep(15)&
            + 0.14000027365862183*coen(15)*coep(15) - 0.0005521421129547751*v**2*coen(15)*coep(15)&
            - 95.06262311867982*coep(11)*coep(15) + 7.207948142299686*coep(12)*coep(15)&
            - 0.0006579189210786203*v**2*coep(12)*coep(15) + 0.05053212587813594*coep(15)**2&
            - 6.398926947599854e-6*v**2*coep(15)**2)) + Q**2*((0.0010544313092678529&
            - 0.0001277692501169659*v**2)*coen(3)**2 + 1.8613316548710143e-9*coen(7)**2&
            + 0.013252565792580641*v**2*coen(7)**2 + 0.0000458268269337641*coen(10)**2&
            + 269.7862309154658*coep(1)**2 + 1.0294897255727615*coep(1)*coep(3)&
            + 0.0005472424670834484*coep(3)**2 - 6.929778845909263e-8*v**2*coep(3)**2&
            + coen(3)*(1.6067337014307048*coep(1) + (0.0015161462894726786&
            - 5.979475567735314e-6*v**2)*coep(3)) + 8.71084955383619e-11*coen(7)*coep(7)&
            + 0.0004364809862117973*v**2*coen(7)*coep(7) + 1.0095243351071125e-12*coep(7)**2&
            + 3.5624977898240984e-6*v**2*coep(7)**2 + 9.78725031222569e-7*coen(10)*coep(10)&
            - 5.07826311116406e-9*coep(10)**2 + jx*(0.008478688136107661*coen(4)**2&
            + (3.2947361651768065e-6 - 5.221786325541085*v**2)*coen(5)**2&
            + 1.5289549869570943e-7*coen(6)**2 - 0.00022132237070862404*coen(8)**2&
            + 262.36006785371944*v**2*coen(8)**2 + 0.000387358008620917*coen(8)*coen(9)&
            - 0.00008517950007797726*coen(9)**2 - 5.221786325541085*coen(11)**2&
            + 0.8376107274678508*coen(11)*coen(12) - 0.033036602193276345*coen(12)**2&
            + 0.004061166205247448*v**2*coen(12)**2 + 1.4593348807749723e-6*coen(13)**2&
            + 0.000015275608977921366*v**2*coen(13)**2 - 2.227368724079447e-12*coen(14)**2&
            - 0.00004258975003898863*v**2*coen(14)**2 + 0.00760694622069715*coen(11)*coen(15)&
            - 0.0007029542061785686*coen(12)*coen(15)&
            + 0.00008517950007797723*v**2*coen(12)*coen(15) + 1.5289549869570948e-7*v**2*coen(15)**2&
            + 3.262416770741896e-7*coen(6)*coep(4) + 2.5705987679731437e-6*coep(4)**2&
            - 1.6831695224453234e-6*coep(4)*coep(5) - 1.4008698720880845e-7*coep(5)**2&
            - 2.1737214474437314*v**2*coep(5)**2 + coen(5)*(-8.562097086733396e-6*coep(4)&
            + (7.141352818528864e-7 - 6.761633515507775*v**2)*coep(5))&
            + coen(4)*(-0.000387358008620917*coen(5) + 0.000030551217955842725*coen(6)&
            + 0.0002584154195664111*coep(4) - 0.00007632620754507681*coep(5)&
            + 3.262416770741896e-7*coep(6)) + 7.850677960877532e-9*coen(6)*coep(6)&
            - 3.385508740776039e-9*coep(4)*coep(6) + 1.0077658428661407e-10*coep(6)**2&
            - 0.000016454877640069206*coen(8)*coep(8) + 309.4915280748583*v**2*coen(8)*coep(8)&
            + 0.00007632620754507681*coen(9)*coep(8) + 0.000020570302304942877*coep(8)**2&
            + 89.92874363848861*v**2*coep(8)**2 + 8.562097086733396e-6*coen(8)*coep(9)&
            - 3.986317045156876e-6*coen(9)*coep(9) + 1.6831695224453234e-6*coep(8)*coep(9)&
            - 4.619852563939508e-8*coep(9)**2 - 6.761633515507775*coen(11)*coep(11)&
            + 0.5355779004769016*coen(12)*coep(11) + 0.005335080737244719*coen(15)*coep(11)&
            - 2.1737214474437314*coep(11)**2 + 0.5426476726726315*coen(11)*coep(12)&
            - 0.04160440855225874*coen(12)*coep(12) + 0.00011292175749581198*v**2*coen(12)*coep(12)&
            - 0.0005053820964908928*coen(15)*coep(12)&
            + 1.9931585225784375e-6*v**2*coen(15)*coep(12) + 0.3431632418575872*coep(11)*coep(12)&
            - 0.013009394977319325*coep(12)**2 + 1.383099504698444e-6*v**2*coep(12)**2&
            - 4.5111651632158256e-7*coen(13)*coep(13) + 3.262416770741897e-7*v**2*coen(13)*coep(13)&
            + 3.4862746654979156e-8*coep(13)**2 - 1.6927543703880197e-9*v**2*coep(13)**2&
            - 1.143680141145272e-13*coen(14)*coep(14) - 1.993158522578438e-6*v**2*coen(14)*coep(14)&
            - 1.4681047766245532e-15*coep(14)**2 - 2.309926281969754e-8*v**2*coep(14)**2&
            + 0.005487383635602041*coen(11)*coep(15) - 0.0005053820964908927*coen(12)*coep(15)&
            + 1.9931585225784375e-6*v**2*coen(12)*coep(15)&
            + 7.850677960877601e-9*v**2*coen(15)*coep(15) + 0.0038485397270876633*coep(11)*coep(15)&
            - 0.00036482831138896554*coep(12)*coep(15)&
            + 4.6198525639395076e-8*v**2*coep(12)*coep(15)&
            + 1.0077658428661579e-10*v**2*coep(15)**2) + jx**2*(0.008478688136107661*coen(4)**2&
            + (3.2947361651768065e-6 - 5.221786325541085*v**2)*coen(5)**2&
            + 1.5289549869570943e-7*coen(6)**2 - 0.00022132237070862404*coen(8)**2&
            + 262.36006785371944*v**2*coen(8)**2 + 0.000387358008620917*coen(8)*coen(9)&
            - 0.00008517950007797726*coen(9)**2 - 5.221786325541085*coen(11)**2&
            + 0.8376107274678508*coen(11)*coen(12) - 0.033036602193276345*coen(12)**2&
            + 0.004061166205247448*v**2*coen(12)**2 + 1.4593348807749723e-6*coen(13)**2&
            + 0.000015275608977921366*v**2*coen(13)**2 - 2.227368724079447e-12*coen(14)**2&
            - 0.00004258975003898863*v**2*coen(14)**2 + 0.00760694622069715*coen(11)*coen(15)&
            - 0.0007029542061785686*coen(12)*coen(15)&
            + 0.00008517950007797723*v**2*coen(12)*coen(15)&
            + 1.5289549869570948e-7*v**2*coen(15)**2 + 3.262416770741896e-7*coen(6)*coep(4)&
            + 2.5705987679731437e-6*coep(4)**2 - 1.6831695224453234e-6*coep(4)*coep(5)&
            - 1.4008698720880845e-7*coep(5)**2 - 2.1737214474437314*v**2*coep(5)**2&
            + coen(5)*(-8.562097086733396e-6*coep(4) + (7.141352818528864e-7&
            - 6.761633515507775*v**2)*coep(5)) + coen(4)*(-0.000387358008620917*coen(5)&
            + 0.000030551217955842725*coen(6) + 0.0002584154195664111*coep(4)&
            - 0.00007632620754507681*coep(5) + 3.262416770741896e-7*coep(6))&
            + 7.850677960877532e-9*coen(6)*coep(6) - 3.385508740776039e-9*coep(4)*coep(6)&
            + 1.0077658428661407e-10*coep(6)**2 - 0.000016454877640069206*coen(8)*coep(8)&
            + 309.4915280748583*v**2*coen(8)*coep(8) + 0.00007632620754507681*coen(9)*coep(8)&
            + 0.000020570302304942877*coep(8)**2 + 89.92874363848861*v**2*coep(8)**2&
            + 8.562097086733396e-6*coen(8)*coep(9) - 3.986317045156876e-6*coen(9)*coep(9)&
            + 1.6831695224453234e-6*coep(8)*coep(9) - 4.619852563939508e-8*coep(9)**2&
            - 6.761633515507775*coen(11)*coep(11) + 0.5355779004769016*coen(12)*coep(11)&
            + 0.005335080737244719*coen(15)*coep(11) - 2.1737214474437314*coep(11)**2&
            + 0.5426476726726315*coen(11)*coep(12) - 0.04160440855225874*coen(12)*coep(12)&
            + 0.00011292175749581198*v**2*coen(12)*coep(12)&
            - 0.0005053820964908928*coen(15)*coep(12)&
            + 1.9931585225784375e-6*v**2*coen(15)*coep(12) + 0.3431632418575872*coep(11)*coep(12)&
            - 0.013009394977319325*coep(12)**2 + 1.383099504698444e-6*v**2*coep(12)**2&
            - 4.5111651632158256e-7*coen(13)*coep(13) + 3.262416770741897e-7*v**2*coen(13)*coep(13)&
            + 3.4862746654979156e-8*coep(13)**2 - 1.6927543703880197e-9*v**2*coep(13)**2&
            - 1.143680141145272e-13*coen(14)*coep(14) - 1.993158522578438e-6*v**2*coen(14)*coep(14)&
            - 1.4681047766245532e-15*coep(14)**2 - 2.309926281969754e-8*v**2*coep(14)**2&
            + 0.005487383635602041*coen(11)*coep(15) - 0.0005053820964908927*coen(12)*coep(15)&
            + 1.9931585225784375e-6*v**2*coen(12)*coep(15) + 7.850677960877601e-9*v**2*coen(15)*coep(15)&
            + 0.0038485397270876633*coep(11)*coep(15) - 0.00036482831138896554*coep(12)*coep(15)&
            + 4.6198525639395076e-8*v**2*coep(12)*coep(15) + 1.0077658428661579e-10*v**2*coep(15)**2))&
            + Q**4*((276042.5932705731 - 50946.383979069986*v**2)*coen(3)**2&
            + 0.7421826230853387*coen(7)**2 + 1.420665395601466e6*v**2*coen(7)**2&
            - 4883.201868445686*coen(10)**2 + 4.785143452560092e9*coep(1)**2&
            + 4.474786827085899e7*coep(1)*coep(3) + 81883.35992173203*coep(3)**2&
            + 7.854729596278391*v**2*coep(3)**2 + coen(3)*(9.297600501482768e7*coep(1)&
            + (303368.20689611573 - 394.7383340024119*v**2)*coep(3))&
            + 0.005750514742765746*coen(7)*coep(7) - 24141.352373634516*v**2*coen(7)*coep(7)&
            - 0.00011442703799717951*coep(7)**2 - 107.98332765235588*v**2*coep(7)**2&
            - 16.215631673254755*coen(10)*coep(10) - 6.55789225154393*coep(10)**2&
            + jx*(1.0763425660268825e6*coen(4)**2 + (556.9962876334182&
            - 5.2335562164957064e8*v**2)*coen(5)**2 - 27.346503448768356*coen(6)**2&
            - 10082.024182854331*coen(8)**2 + 7.7744612424217825e9*v**2*coen(8)**2&
            + 57480.87335992518*coen(8)*coen(9) - 33964.25598604666*coen(9)**2&
            - 5.2335562164957064e8*coen(11)**2 + 6.1013287518813506e7*coen(11)*coen(12)&
            - 1.6864158408429017e6*coen(12)**2 + 602787.4341597273*v**2*coen(12)**2&
            + 248.5374913251275*coen(13)**2 - 1627.733956148562*v**2*coen(13)**2&
            - 0.004938490190447705*coen(14)**2 - 16982.12799302333*v**2*coen(14)**2&
            + 2.7730393620214597e6*coen(11)*coen(15) - 184028.3955137154*coen(12)*coen(15)&
            + 33964.25598604665*v**2*coen(12)*coen(15) - 2526.741028720585*coen(15)**2&
            + 338.9977206330921*v**2*coen(15)**2 - 5.405210557751584*coen(6)*coep(4)&
            + 19.00242984418099*coep(4)**2 + 68.80310742798851*coep(4)*coep(5)&
            - 75.78241258360035*coep(5)**2 - 1.4213419211208743e8*v**2*coep(5)**2&
            + coen(5)*(279.81102724589647*coep(4) + (-53.68336293878706&
            - 5.526186148663747e8*v**2)*coep(5)) + coen(4)*(-57480.87335992518*coen(5)&
            - 3255.4679122971233*coen(6) - 12085.189920278508*coep(4) - 9548.407052676801*coep(5)&
            - 5.405210557751584*coep(6)) - 2.499546483379038*coen(6)*coep(6)&
            - 4.371928167695952*coep(4)*coep(6) + 0.015010233573423359*coep(6)**2&
            + 2803.623335841746*coen(8)*coep(8) + 7.190231189895839e9*v**2*coen(8)*coep(8)&
            + 9548.407052676801*coen(9)*coep(8) + 1701.419006357814*coep(8)**2&
            + 1.5950478175200305e9*v**2*coep(8)**2 - 279.81102724589647*coen(8)*coep(9)&
            - 263.15888933494125*coen(9)*coep(9) - 68.80310742798851*coep(8)*coep(9)&
            + 5.236486397518926*coep(9)**2 - 5.526186148663747e8*coen(11)*coep(11)&
            + 3.099200167160923e7*coen(12)*coep(11) + 1.596427682646068e6*coen(15)*coep(11)&
            - 1.4213419211208743e8*coep(11)**2 + 3.0424390381130725e7*coen(11)*coep(12)&
            - 1.5881958092970126e6*coen(12)*coep(12) - 4038.0724624003374*v**2*coen(12)*coep(12)&
            - 101122.73563203857*coen(15)*coep(12) + 131.5794446674706*v**2*coen(15)*coep(12)&
            + 1.4915956090286328e7*coep(11)*coep(12) - 362889.4023191433*coep(12)**2&
            + 54.99687239496628*v**2*coep(12)**2 + 12.899563162141893*coen(13)*coep(13)&
            - 5.405210557751585*v**2*coen(13)*coep(13) - 2.640686753652382*coep(13)**2&
            - 2.1859640838479764*v**2*coep(13)**2 - 0.0001626520556442493*coen(14)*coep(14)&
            - 131.57944466747063*v**2*coen(14)*coep(14) - 1.3275437122060896e-6*coep(14)**2&
            + 2.618243198759463*v**2*coep(14)**2 + 1.5862944946735534e6*coen(11)*coep(15)&
            - 101122.73563203856*coen(12)*coep(15) + 131.5794446674706*v**2*coen(12)*coep(15)&
            - 3193.7724878068598*coen(15)*coep(15) + 11.165087707643844*v**2*coen(15)*coep(15)&
            + 893343.4228275741*coep(11)*coep(15) - 54588.90661448801*coep(12)*coep(15)&
            - 5.2364863975189255*v**2*coep(12)*coep(15) - 998.3680856978038*coep(15)**2&
            + 0.09112791058067476*v**2*coep(15)**2) + jx**2*(1.0763425660268825e6*coen(4)**2&
            + (556.9962876334182 - 5.2335562164957064e8*v**2)*coen(5)**2&
            - 27.346503448768356*coen(6)**2 - 10082.024182854331*coen(8)**2&
            + 7.7744612424217825e9*v**2*coen(8)**2 + 57480.87335992518*coen(8)*coen(9)&
            - 33964.25598604666*coen(9)**2 - 5.2335562164957064e8*coen(11)**2&
            + 6.1013287518813506e7*coen(11)*coen(12) - 1.6864158408429017e6*coen(12)**2&
            + 602787.4341597273*v**2*coen(12)**2 + 248.5374913251275*coen(13)**2&
            - 1627.733956148562*v**2*coen(13)**2 - 0.004938490190447705*coen(14)**2&
            - 16982.12799302333*v**2*coen(14)**2 + 2.7730393620214597e6*coen(11)*coen(15)&
            - 184028.3955137154*coen(12)*coen(15) + 33964.25598604665*v**2*coen(12)*coen(15)&
            - 2526.741028720585*coen(15)**2 + 338.9977206330921*v**2*coen(15)**2&
            - 5.405210557751584*coen(6)*coep(4) + 19.00242984418099*coep(4)**2&
            + 68.80310742798851*coep(4)*coep(5) - 75.78241258360035*coep(5)**2&
            - 1.4213419211208743e8*v**2*coep(5)**2 + coen(5)*(279.81102724589647*coep(4)&
            + (-53.68336293878706 - 5.526186148663747e8*v**2)*coep(5))&
            + coen(4)*(-57480.87335992518*coen(5) - 3255.4679122971233*coen(6)&
            - 12085.189920278508*coep(4) - 9548.407052676801*coep(5)&
            - 5.405210557751584*coep(6)) - 2.499546483379038*coen(6)*coep(6)&
            - 4.371928167695952*coep(4)*coep(6) + 0.015010233573423359*coep(6)**2&
            + 2803.623335841746*coen(8)*coep(8) + 7.190231189895839e9*v**2*coen(8)*coep(8)&
            + 9548.407052676801*coen(9)*coep(8) + 1701.419006357814*coep(8)**2&
            + 1.5950478175200305e9*v**2*coep(8)**2 - 279.81102724589647*coen(8)*coep(9)&
            - 263.15888933494125*coen(9)*coep(9) - 68.80310742798851*coep(8)*coep(9)&
            + 5.236486397518926*coep(9)**2 - 5.526186148663747e8*coen(11)*coep(11)&
            + 3.099200167160923e7*coen(12)*coep(11) + 1.596427682646068e6*coen(15)*coep(11)&
            - 1.4213419211208743e8*coep(11)**2 + 3.0424390381130725e7*coen(11)*coep(12)&
            - 1.5881958092970126e6*coen(12)*coep(12) - 4038.0724624003374*v**2*coen(12)*coep(12)&
            - 101122.73563203857*coen(15)*coep(12) + 131.5794446674706*v**2*coen(15)*coep(12)&
            + 1.4915956090286328e7*coep(11)*coep(12) - 362889.4023191433*coep(12)**2&
            + 54.99687239496628*v**2*coep(12)**2 + 12.899563162141893*coen(13)*coep(13)&
            - 5.405210557751585*v**2*coen(13)*coep(13) - 2.640686753652382*coep(13)**2&
            - 2.1859640838479764*v**2*coep(13)**2 - 0.0001626520556442493*coen(14)*coep(14)&
            - 131.57944466747063*v**2*coen(14)*coep(14) - 1.3275437122060896e-6*coep(14)**2&
            + 2.618243198759463*v**2*coep(14)**2 + 1.5862944946735534e6*coen(11)*coep(15)&
            - 101122.73563203856*coen(12)*coep(15) + 131.5794446674706*v**2*coen(12)*coep(15)&
            - 3193.7724878068598*coen(15)*coep(15) + 11.165087707643844*v**2*coen(15)*coep(15)&
            + 893343.4228275741*coep(11)*coep(15) - 54588.90661448801*coep(12)*coep(15)&
            - 5.2364863975189255*v**2*coep(12)*coep(15) - 998.3680856978038*coep(15)**2&
            + 0.09112791058067476*v**2*coep(15)**2)) + Q**5*((-1.3984065677343624e9&
            + 3.935508312276029e8*v**2)*coen(3)**2 - 5733.215302540688*coen(7)**2&
            - 6.226957253157801e9*v**2*coen(7)**2 + 1.0739927954780567e8*coen(10)**2&
            - 8.28288550421874e12*coep(1)**2 - 1.1034967758685011e11*coep(1)*coep(3)&
            - 3.01576097714188e8*coep(3)**2 - 29913.39726291803*v**2*coep(3)**2&
            + coen(3)*(-2.7384944590965488e11*coep(1) + (-1.3192275193130844e9&
            - 6.687605204587927e6*v**2)*coep(3)) + 97.42446833791631*coen(7)*coep(7)&
            + 2.2411177572850636e8*v**2*coen(7)*coep(7) + 0.43577584731248226*coep(7)**2&
            - 1.5219268488038925e6*v**2*coep(7)**2 + 3.3317408770934143e6*coen(10)*coep(10)&
            + 75618.87776445576*coep(10)**2 + jx*(-5.068398255943543e9*coen(4)**2&
            + (-2.7929088791096206e6 + 2.1536708740670886e12*v**2)*coen(5)**2&
            - 450912.6874231017*coen(6)**2 + 2.775760269331504e7*coen(8)**2&
            - 1.8986906486943625e13*v**2*coen(8)**2 - 2.905220465279182e8*coen(8)*coen(9)&
            + 2.6236722081840193e8*coen(9)**2 + 2.1536708740670886e12*coen(11)**2&
            - 2.0880798655808377e11*coen(11)*coen(12) + 4.70568621694362e9*coen(12)**2&
            - 2.9927458382242775e9*v**2*coen(12)**2 - 1.0331421996503815e6*coen(13)**2&
            + 3.5799759849268556e7*v**2*coen(13)**2 + 68.53281313161412*coen(14)**2&
            + 1.3118361040920097e8*v**2*coen(14)**2 - 1.6901819452561462e10*coen(11)*coen(15)&
            + 9.322710451562415e8*coen(12)*coen(15) - 2.6236722081840187e8*v**2*coen(12)*coen(15)&
            + 2.5489650218882978e7*coen(15)**2 - 4.704366424606518e6*v**2*coen(15)**2&
            + 1.1105802923644711e6*coen(6)*coep(4) - 1.4667405253531714e6*coep(4)**2&
            - 698504.1480561965*coep(4)*coep(5) + 471324.8216587087*coep(5)**2&
            + 4.4185801693791473e11*v**2*coep(5)**2 + coen(5)*(-4.367962053367566e6*coep(4)&
            + (776655.9935124571 + 1.9918282448937434e12*v**2)*coep(5))&
            - 1497.344212472177*coen(6)*coep(6) + 50412.58517630382*coep(4)*coep(6)&
            - 605.5528521322482*coep(6)**2 + coen(4)*(2.905220465279182e8*coen(5)&
            + 7.15995196985371e7*coen(6) + 1.1914376643022922e8*coep(4)&
            + 4.0298203499978125e7*coep(5) + 1.1105802923644711e6*coep(6))&
            - 1.4737826075464051e7*coen(8)*coep(8) - 1.4948291582557516e13*v**2*coen(8)*coep(8)&
            - 4.0298203499978125e7*coen(9)*coep(8) - 5.6169853133626925e6*coep(8)**2&
            - 2.7609618347395796e12*v**2*coep(8)**2 + 4.367962053367566e6*coen(8)*coep(9)&
            - 4.458403469725284e6*coen(9)*coep(9) + 698504.1480561965*coep(8)*coep(9)&
            - 19942.26484194535*coep(9)**2 + 1.9918282448937434e12*coen(11)*coep(11)&
            - 9.12831486365516e10*coen(12)*coep(11) - 8.5853629271411e9*coen(15)*coep(11)&
            + 4.4185801693791473e11*coep(11)**2 - 8.8902785148518e10*coen(11)*coep(12)&
            + 3.713152227886788e9*coen(12)*coep(12) + 4.443984118739376e7*v**2*coen(12)*coep(12)&
            + 4.397425064376947e8*coen(15)*coep(12) + 2.2292017348626414e6*v**2*coen(15)*coep(12)&
            - 3.678322586228336e10*coep(11)*coep(12) + 6.911727895463058e8*coep(12)**2&
            - 959431.575751874*v**2*coep(12)**2 - 217376.33006559*coen(13)*coep(13)&
            + 1.1105802923644714e6*v**2*coen(13)*coep(13) - 1771.849564454895*coep(13)**2&
            + 25206.292588151915*v**2*coep(13)**2 + 0.5309999722686071*coen(14)*coep(14)&
            - 2.229201734862642e6*v**2*coen(14)*coep(14) - 0.010566141766651294*coep(14)**2&
            - 9971.132420972675*v**2*coep(14)**2 - 8.428124005243228e9*coen(11)*coep(15)&
            + 4.3974250643769467e8*coen(12)*coep(15) + 2.2292017348626414e6*v**2*coen(12)*coep(15)&
            + 2.801288522069556e7*coen(15)*coep(15) - 36449.96209640625*v**2*coen(15)*coep(15)&
            - 4.1319982425568643e9*coep(11)*coep(15) + 2.0105073180945864e8*coep(12)*coep(15)&
            + 19942.264841945347*v**2*coep(12)*coep(15) + 7.5610730156633025e6*coep(15)**2&
            + 725.3022354300111*v**2*coep(15)**2) + jx**2*(-5.068398255943543e9*coen(4)**2&
            + (-2.7929088791096206e6 + 2.1536708740670886e12*v**2)*coen(5)**2&
            - 450912.6874231017*coen(6)**2 + 2.775760269331504e7*coen(8)**2&
            - 1.8986906486943625e13*v**2*coen(8)**2 - 2.905220465279182e8*coen(8)*coen(9)&
            + 2.6236722081840193e8*coen(9)**2 + 2.1536708740670886e12*coen(11)**2&
            - 2.0880798655808377e11*coen(11)*coen(12) + 4.70568621694362e9*coen(12)**2&
            - 2.9927458382242775e9*v**2*coen(12)**2 - 1.0331421996503815e6*coen(13)**2&
            + 3.5799759849268556e7*v**2*coen(13)**2 + 68.53281313161412*coen(14)**2&
            + 1.3118361040920097e8*v**2*coen(14)**2 - 1.6901819452561462e10*coen(11)*coen(15)&
            + 9.322710451562415e8*coen(12)*coen(15) - 2.6236722081840187e8*v**2*coen(12)*coen(15)&
            + 2.5489650218882978e7*coen(15)**2 - 4.704366424606518e6*v**2*coen(15)**2&
            + 1.1105802923644711e6*coen(6)*coep(4) - 1.4667405253531714e6*coep(4)**2&
            - 698504.1480561965*coep(4)*coep(5) + 471324.8216587087*coep(5)**2&
            + 4.4185801693791473e11*v**2*coep(5)**2 + coen(5)*(-4.367962053367566e6*coep(4)&
            + (776655.9935124571 + 1.9918282448937434e12*v**2)*coep(5))&
            - 1497.344212472177*coen(6)*coep(6) + 50412.58517630382*coep(4)*coep(6)&
            - 605.5528521322482*coep(6)**2 + coen(4)*(2.905220465279182e8*coen(5)&
            + 7.15995196985371e7*coen(6) + 1.1914376643022922e8*coep(4)&
            + 4.0298203499978125e7*coep(5) + 1.1105802923644711e6*coep(6))&
            - 1.4737826075464051e7*coen(8)*coep(8) - 1.4948291582557516e13*v**2*coen(8)*coep(8)&
            - 4.0298203499978125e7*coen(9)*coep(8) - 5.6169853133626925e6*coep(8)**2&
            - 2.7609618347395796e12*v**2*coep(8)**2 + 4.367962053367566e6*coen(8)*coep(9)&
            - 4.458403469725284e6*coen(9)*coep(9) + 698504.1480561965*coep(8)*coep(9)&
            - 19942.26484194535*coep(9)**2 + 1.9918282448937434e12*coen(11)*coep(11)&
            - 9.12831486365516e10*coen(12)*coep(11) - 8.5853629271411e9*coen(15)*coep(11)&
            + 4.4185801693791473e11*coep(11)**2 - 8.8902785148518e10*coen(11)*coep(12)&
            + 3.713152227886788e9*coen(12)*coep(12) + 4.443984118739376e7*v**2*coen(12)*coep(12)&
            + 4.397425064376947e8*coen(15)*coep(12) + 2.2292017348626414e6*v**2*coen(15)*coep(12)&
            - 3.678322586228336e10*coep(11)*coep(12) + 6.911727895463058e8*coep(12)**2&
            - 959431.575751874*v**2*coep(12)**2 - 217376.33006559*coen(13)*coep(13)&
            + 1.1105802923644714e6*v**2*coen(13)*coep(13) - 1771.849564454895*coep(13)**2&
            + 25206.292588151915*v**2*coep(13)**2 + 0.5309999722686071*coen(14)*coep(14)&
            - 2.229201734862642e6*v**2*coen(14)*coep(14) - 0.010566141766651294*coep(14)**2&
            - 9971.132420972675*v**2*coep(14)**2 - 8.428124005243228e9*coen(11)*coep(15)&
            + 4.3974250643769467e8*coen(12)*coep(15) + 2.2292017348626414e6*v**2*coen(12)*coep(15)&
            + 2.801288522069556e7*coen(15)*coep(15) - 36449.96209640625*v**2*coen(15)*coep(15)&
            - 4.1319982425568643e9*coep(11)*coep(15) + 2.0105073180945864e8*coep(12)*coep(15)&
            + 19942.264841945347*v**2*coep(12)*coep(15) + 7.5610730156633025e6*coep(15)**2&
            + 725.3022354300111*v**2*coep(15)**2)) + Q**8*((5.39869483620315e18&
            - 5.157172661917517e18*v**2)*coen(3)**2 + 7.512925619016253e13*coen(7)**2&
            + 8.874284133028558e18*v**2*coen(7)**2 - 4.135778167182891e18*coen(10)**2&
            + 6.262826715711263e20*coep(1)**2 + 2.2126764158699323e19*coep(1)*coep(3)&
            + 1.9918295172480323e17*coep(3)**2 - 3.830194255834996e15*v**2*coep(3)**2&
            + coen(3)*(1.850254604535743e20*coep(1) + (2.42731826696424e18&
            + 2.820831284925315e17*v**2)*coep(3)) - 4.109363214447461e12*coen(7)*coep(7)&
            - 5.030726301553776e17*v**2*coen(7)*coep(7) + 5.579794673729672e10*coep(7)**2&
            + 7.238125189932717e15*v**2*coep(7)**2 - 1.0812773925979389e17*coen(10)*coep(10)&
            - 9.516997610147128e14*coep(10)**2 + jx*(9.085139641679341e18*coen(4)**2&
            + (-9.816085528969434e15 - 6.244916835557868e21*v**2)*coen(5)**2&
            + 2.3658320334037616e17*coen(6)**2 + 8.146373232155304e16*coen(8)**2&
            + 9.34922230966404e21*v**2*coen(8)**2 + 7.099721278903821e17*coen(8)*coen(9)&
            - 3.4381151079450107e18*coen(9)**2 - 6.244916835557868e21*coen(11)**2&
            + 2.9059850821811483e20*coen(11)*coen(12) - 2.8937947879984404e18*coen(12)**2&
            + 6.127044930669823e18*v**2*coen(12)**2 + 2.7202205946197555e15*coen(13)**2&
            - 1.3785927223942966e18*v**2*coen(13)**2 - 5.630115054236808e12*coen(14)**2&
            - 1.7190575539725053e18*v**2*coen(14)**2 + 1.309175440491767e20*coen(11)*coen(15)&
            - 3.599129890802099e18*coen(12)*coen(15)&
            + 3.4381151079450107e18*v**2*coen(12)*coen(15) - 5.69292233771193e17*coen(15)**2&
            + 3.8647361778303123e17*v**2*coen(15)**2 - 3.604257975326462e16*coen(6)*coep(4)&
            + 5.27232940379704e15*coep(4)**2&
            + 1.4659305950826155e15*coep(4)*coep(5) - 2.774465752504926e15*coep(5)**2&
            - 3.2902374998863276e20*v**2*coep(5)**2 + coen(5)*(2.0354957646489256e16*coep(4)&
            + (-1.6614931663541772e16 - 3.085666269075821e21*v**2)*coep(5))&
            + coen(4)*(-7.099721278903821e17*coen(5) - 2.7571854447885926e18*coen(6)&
            - 2.70275576203809e17*coep(4) - 4.938184670788969e16*coep(5)&
            - 3.604257975326462e16*coep(6)) + 7.165213171070882e15*coen(6)*coep(6)&
            - 6.344665073431416e14*coep(4)*coep(6) + 8.205459224239508e13*coep(6)**2&
            + 4.420760201916368e16*coen(8)*coep(8) + 3.159940462075633e21*v**2*coen(8)*coep(8)&
            + 4.938184670788969e16*coen(9)*coep(8) + 4.793188850802034e15*coep(8)**2&
            + 2.0876089052370875e20*v**2*coep(8)**2 - 2.0354957646489256e16*coen(8)*coep(9)&
            + 1.88055418995021e17*coen(9)*coep(9) - 1.4659305950826155e15*coep(8)*coep(9)&
            - 2.5534628372233305e15*coep(9)**2 - 3.085666269075821e21*coen(11)*coep(11)&
            + 6.167515348452476e19*coen(12)*coep(11) + 3.779873449448935e19*coen(15)*coep(11)&
            - 3.2902374998863276e20*coep(11)**2 + 4.948894811453094e19*coen(11)*coep(12)&
            - 6.934095591864535e17*coen(12)*coep(12) - 1.025846994853498e17*v**2*coen(12)*coep(12)&
            - 8.0910608898808e17*coen(15)*coep(12) - 9.402770949751048e16*v**2*coen(15)*coep(12)&
            + 7.375588052899773e18*coep(11)*coep(12) - 7.82964048554742e10*coep(12)**2&
            + 2.859621007152801e15*v**2*coep(12)**2 + 9.429254668559645e14*coen(13)*coep(13)&
            - 3.604257975326463e16*v**2*coen(13)*coep(13) + 7.940841155284895e13*coep(13)**2&
            - 3.172332536715709e14*v**2*coep(13)**2 + 2.7243760625679578e11*coen(14)*coep(14)&
            + 9.40277094975105e16*v**2*coen(14)*coep(14) - 3.1497577414236374e9*coep(14)**2&
            - 1.2767314186116652e15*v**2*coep(14)**2 + 3.499351241460279e19*coen(11)*coep(15)&
            - 8.091060889880799e17*coen(12)*coep(15) - 9.402770949751048e16*v**2*coen(12)*coep(15)&
            - 3.5826105374008326e17*coen(15)*coep(15)&
            - 1.870120704389148e16*v**2*coen(15)*coep(15) + 8.625175772341196e18*coep(11)*coep(15)&
            - 1.3278863448320214e17*coep(12)*coep(15)&
            + 2.5534628372233305e15*v**2*coep(12)*coep(15) - 4.994271823760658e16*coep(15)**2&
            + 2.162119704022841e14*v**2*coep(15)**2) + jx**2*(9.085139641679341e18*coen(4)**2&
            + (-9.816085528969434e15 - 6.244916835557868e21*v**2)*coen(5)**2&
            + 2.3658320334037616e17*coen(6)**2 + 8.146373232155304e16*coen(8)**2&
            + 9.34922230966404e21*v**2*coen(8)**2 + 7.099721278903821e17*coen(8)*coen(9)&
            - 3.4381151079450107e18*coen(9)**2 - 6.244916835557868e21*coen(11)**2&
            + 2.9059850821811483e20*coen(11)*coen(12) - 2.8937947879984404e18*coen(12)**2&
            + 6.127044930669823e18*v**2*coen(12)**2 + 2.7202205946197555e15*coen(13)**2&
            - 1.3785927223942966e18*v**2*coen(13)**2 - 5.630115054236808e12*coen(14)**2&
            - 1.7190575539725053e18*v**2*coen(14)**2 + 1.309175440491767e20*coen(11)*coen(15)&
            - 3.599129890802099e18*coen(12)*coen(15) + 3.4381151079450107e18*v**2*coen(12)*coen(15)&
            - 5.69292233771193e17*coen(15)**2 + 3.8647361778303123e17*v**2*coen(15)**2&
            - 3.604257975326462e16*coen(6)*coep(4) + 5.27232940379704e15*coep(4)**2&
            + 1.4659305950826155e15*coep(4)*coep(5) - 2.774465752504926e15*coep(5)**2&
            - 3.2902374998863276e20*v**2*coep(5)**2 + coen(5)*(2.0354957646489256e16*coep(4)&
            + (-1.6614931663541772e16 - 3.085666269075821e21*v**2)*coep(5))&
            + coen(4)*(-7.099721278903821e17*coen(5) - 2.7571854447885926e18*coen(6)&
            - 2.70275576203809e17*coep(4) - 4.938184670788969e16*coep(5)&
            - 3.604257975326462e16*coep(6)) + 7.165213171070882e15*coen(6)*coep(6)&
            - 6.344665073431416e14*coep(4)*coep(6) + 8.205459224239508e13*coep(6)**2&
            + 4.420760201916368e16*coen(8)*coep(8) + 3.159940462075633e21*v**2*coen(8)*coep(8)&
            + 4.938184670788969e16*coen(9)*coep(8) + 4.793188850802034e15*coep(8)**2&
            + 2.0876089052370875e20*v**2*coep(8)**2 - 2.0354957646489256e16*coen(8)*coep(9)&
            + 1.88055418995021e17*coen(9)*coep(9) - 1.4659305950826155e15*coep(8)*coep(9)&
            - 2.5534628372233305e15*coep(9)**2 - 3.085666269075821e21*coen(11)*coep(11)&
            + 6.167515348452476e19*coen(12)*coep(11) + 3.779873449448935e19*coen(15)*coep(11)&
            - 3.2902374998863276e20*coep(11)**2 + 4.948894811453094e19*coen(11)*coep(12)&
            - 6.934095591864535e17*coen(12)*coep(12) - 1.025846994853498e17*v**2*coen(12)*coep(12)&
            - 8.0910608898808e17*coen(15)*coep(12) - 9.402770949751048e16*v**2*coen(15)*coep(12)&
            + 7.375588052899773e18*coep(11)*coep(12) - 7.82964048554742e10*coep(12)**2&
            + 2.859621007152801e15*v**2*coep(12)**2 + 9.429254668559645e14*coen(13)*coep(13)&
            - 3.604257975326463e16*v**2*coen(13)*coep(13) + 7.940841155284895e13*coep(13)**2&
            - 3.172332536715709e14*v**2*coep(13)**2 + 2.7243760625679578e11*coen(14)*coep(14)&
            + 9.40277094975105e16*v**2*coen(14)*coep(14) - 3.1497577414236374e9*coep(14)**2&
            - 1.2767314186116652e15*v**2*coep(14)**2 + 3.499351241460279e19*coen(11)*coep(15)&
            - 8.091060889880799e17*coen(12)*coep(15) - 9.402770949751048e16*v**2*coen(12)*coep(15)&
            - 3.5826105374008326e17*coen(15)*coep(15) - 1.870120704389148e16*v**2*coen(15)*coep(15)&
            + 8.625175772341196e18*coep(11)*coep(15) - 1.3278863448320214e17*coep(12)*coep(15)&
            + 2.5534628372233305e15*v**2*coep(12)*coep(15) - 4.994271823760658e16*coep(15)**2&
            + 2.162119704022841e14*v**2*coep(15)**2)) + Q**12*(-4.767909065499425e21*coen(3)**2&
            - 1.7407592924319032e16*coen(3)*coep(3) - 1.6124758937060444e10*coep(3)**2&
            + jx*(-8.86777388157241e25*coen(5)**2 + 5.55772583603546e24*coen(6)**2&
            - 8.096448103309047e19*coen(13)**2 - 1.5893030218331414e21*coen(14)**2&
            + 3.1786060436662827e21*coen(12)*coen(15) - 3.989732858809691e22*coen(15)**2&
            + 1.090961166307141e26*v**2*coen(15)**2 - 2.566402346544713e19*coen(5)*coep(5)&
            - 1.8793362974988623e12*coep(5)**2 + 1.7777700772601948e19*coen(6)*coep(6)&
            + 1.4707322681883678e13*coep(6)**2 + 5.80253097477301e15*coen(15)*coep(12)&
            - 2.5898404481967772e14*coen(13)*coep(13) - 2.1425503586451378e8*coep(13)**2&
            - 5.802530974773011e15*coen(14)*coep(14) - 5.374919645686815e9*coep(14)**2&
            + 5.80253097477301e15*coen(12)*coep(15) + 1.1406961370429462e21*coen(15)*coep(15)&
            + 3.9830893623231834e20*v**2*coen(15)*coep(15) + 1.0749839291373629e10*coep(12)*coep(15)&
            + 2.0556792487056612e15*coep(15)**2 + 3.6895598415852556e14*v**2*coep(15)**2)&
            + jx**2*(-8.86777388157241e25*coen(5)**2 + 5.55772583603546e24*coen(6)**2&
            - 8.096448103309047e19*coen(13)**2 - 1.5893030218331414e21*coen(14)**2&
            + 3.1786060436662827e21*coen(12)*coen(15) - 3.989732858809691e22*coen(15)**2&
            + 1.090961166307141e26*v**2*coen(15)**2 - 2.566402346544713e19*coen(5)*coep(5)&
            - 1.8793362974988623e12*coep(5)**2 + 1.7777700772601948e19*coen(6)*coep(6)&
            + 1.4707322681883678e13*coep(6)**2 + 5.80253097477301e15*coen(15)*coep(12)&
            - 2.5898404481967772e14*coen(13)*coep(13) - 2.1425503586451378e8*coep(13)**2&
            - 5.802530974773011e15*coen(14)*coep(14) - 5.374919645686815e9*coep(14)**2&
            + 5.80253097477301e15*coen(12)*coep(15) + 1.1406961370429462e21*coen(15)*coep(15)&
            + 3.9830893623231834e20*v**2*coen(15)*coep(15)&
            + 1.0749839291373629e10*coep(12)*coep(15) + 2.0556792487056612e15*coep(15)**2&
            + 3.6895598415852556e14*v**2*coep(15)**2)) + Q**10*((4.2147717652621485e23&
            + 1.07065510241305e23*v**2)*coen(3)**2 - 1.55972131928945e18*coen(7)**2&
            + 4.264935648132065e21*v**2*coen(7)**2 - 2.5001713588417413e22*coen(10)**2&
            + 5.043247423506715e12*coep(1)**2 + 4.2449465031130023e11*coep(1)*coep(3)&
            - 8.091724579426208e15*coep(3)**2 - 5.516468281056722e15*v**2*coep(3)**2&
            + coen(3)*(2.898427672351715e18*coep(1) + (6.843827385737804e17&
            - 3.0610875029669277e21*v**2)*coep(3)) + 4.459366445671767e16*coen(7)*coep(7)&
            + 1.5571241521428448e16*v**2*coen(7)*coep(7) + 8.036344445336404e10*coep(7)**2&
            + 1.4423735491482498e10*v**2*coep(7)**2 - 2.4508654077106628e20*coen(10)*coep(10)&
            - 3.409946072940374e14*coep(10)**2 + jx*(2.9157138564123853e21*coen(4)**2&
            + (-1.0359565523600974e22 - 4.150281337306634e26*v**2)*coen(5)**2&
            + 2.4318315066188945e23*coen(6)**2 + 6.046094833207032e21*coen(8)**2&
            + 7.932299972069841e25*v**2*coen(8)**2 - 1.0237575902691337e21*coen(8)*coen(9)&
            + 7.137700682753666e22*coen(9)**2 - 4.150281337306634e26*coen(11)**2&
            + 6.676687892193278e24*coen(11)*coen(12) - 3.984994884373851e17*coen(12)**2&
            + 1.4940686403683633e21*v**2*coen(12)**2 + 1.1412500731314954e20*coen(13)**2&
            - 8.333904529472471e21*v**2*coen(13)**2 - 3.3069494484413496e18*coen(14)**2&
            + 3.568850341376833e22*v**2*coen(14)**2 + 2.273705164490211e25*coen(11)*coen(15)&
            - 2.809847843508099e23*coen(12)*coen(15) - 7.137700682753665e22*v**2*coen(12)*coen(15)&
            - 2.2180551070542823e23*coen(15)**2 + 2.2700223793880794e23*v**2*coen(15)**2&
            - 8.169551359035541e19*coen(6)*coep(4) + 9.807476632489513e9*coep(4)**2&
            + 3.483807244022235e9*coep(4)*coep(5) - 2.333805301105345e20*coep(5)**2&
            - 1.0288155770538508e19*v**2*coep(5)**2 + coen(5)*(1.8754814593683745e15*coep(4)&
            + (-3.5325989821467376e21 - 7.127714528377333e25*v**2)*coep(5))&
            + coen(4)*(1.0237575902691337e21*coen(5) - 1.666780905894494e22*coen(6)&
            + 1.0612491135492864e16*coep(4) + 1.8754814593315192e15*coep(5)&
            - 8.169551359035541e19*coep(6)) + 4.996205118298168e21*coen(6)*coep(6)&
            - 2.2732973819602484e14*coep(4)*coep(6) + 3.4295779094204908e19*coep(6)**2&
            + 1.0383594383161432e21*coen(8)*coep(8) + 2.2956689619838243e19*v**2*coen(8)*coep(8)&
            - 1.8754814593315192e15*coen(9)*coep(8) + 1.4987698517770688e14*coep(8)**2&
            + 1.6810824745022383e12*v**2*coep(8)**2 - 1.8754814593683745e15*coen(8)*coep(9)&
            - 2.0407250019779514e21*coen(9)*coep(9) - 3.483807244022235e9*coep(8)*coep(9)&
            - 3.6776455207044805e15*coep(9)**2 - 7.127714528377333e25*coen(11)*coep(11)&
            + 9.661425574505714e17*coen(12)*coep(11) + 2.999728785124676e24*coen(15)*coep(11)&
            - 1.0288155770538508e19*coep(11)**2 + 9.661425579784312e17*coen(11)*coep(12)&
            + 1.605468816763783e16*coen(12)*coep(12) + 5.422077295016715e15*v**2*coen(12)*coep(12)&
            - 2.281275795245934e17*coen(15)*coep(12)&
            + 1.0203625009889756e21*v**2*coen(15)*coep(12) + 1.414982167704334e11*coep(11)*coep(12)&
            + 2.8443675040065453e10*coep(12)**2 + 4.999564801995346e9*v**2*coep(12)**2&
            - 7.27817431522172e16*coen(13)*coep(13) - 8.169551359035541e19*v**2*coen(13)*coep(13)&
            - 4.996180160429972e14*coep(13)**2 - 1.1366486909801244e14*v**2*coep(13)**2&
            + 1.874670375525277e17*coen(14)*coep(14) - 1.0203625009889757e21*v**2*coen(14)*coep(14)&
            - 2.6972445039832955e15*coep(14)**2 - 1.8388227603522402e15*v**2*coep(14)**2&
            + 1.263102803889675e24*coen(11)*coep(15) - 2.281275795245934e17*coen(12)*coep(15)&
            + 1.0203625009889756e21*v**2*coen(12)*coep(15) - 5.315827721039218e22*coen(15)*coep(15)&
            - 1.2868487325755707e22*v**2*coen(15)*coep(15) + 6.180759854512867e17*coep(11)*coep(15)&
            + 5.394483052950804e15*coep(12)*coep(15) + 3.67764552070448e15*v**2*coep(12)*coep(15)&
            - 6.362479137377961e15*coep(15)**2 + 1.8514965173142926e20*v**2*coep(15)**2)&
            + jx**2*(2.9157138564123853e21*coen(4)**2 + (-1.0359565523600974e22&
            - 4.150281337306634e26*v**2)*coen(5)**2 + 2.4318315066188945e23*coen(6)**2&
            + 6.046094833207032e21*coen(8)**2 + 7.932299972069841e25*v**2*coen(8)**2&
            - 1.0237575902691337e21*coen(8)*coen(9) + 7.137700682753666e22*coen(9)**2&
            - 4.150281337306634e26*coen(11)**2 + 6.676687892193278e24*coen(11)*coen(12)&
            - 3.984994884373851e17*coen(12)**2 + 1.4940686403683633e21*v**2*coen(12)**2&
            + 1.1412500731314954e20*coen(13)**2 - 8.333904529472471e21*v**2*coen(13)**2&
            - 3.3069494484413496e18*coen(14)**2 + 3.568850341376833e22*v**2*coen(14)**2&
            + 2.273705164490211e25*coen(11)*coen(15) - 2.809847843508099e23*coen(12)*coen(15)&
            - 7.137700682753665e22*v**2*coen(12)*coen(15) - 2.2180551070542823e23*coen(15)**2&
            + 2.2700223793880794e23*v**2*coen(15)**2 - 8.169551359035541e19*coen(6)*coep(4)&
            + 9.807476632489513e9*coep(4)**2 + 3.483807244022235e9*coep(4)*coep(5)&
            - 2.333805301105345e20*coep(5)**2 - 1.0288155770538508e19*v**2*coep(5)**2&
            + coen(5)*(1.8754814593683745e15*coep(4) + (-3.5325989821467376e21&
            - 7.127714528377333e25*v**2)*coep(5)) + coen(4)*(1.0237575902691337e21*coen(5)&
            - 1.666780905894494e22*coen(6) + 1.0612491135492864e16*coep(4)&
            + 1.8754814593315192e15*coep(5) - 8.169551359035541e19*coep(6))&
            + 4.996205118298168e21*coen(6)*coep(6) - 2.2732973819602484e14*coep(4)*coep(6)&
            + 3.4295779094204908e19*coep(6)**2 + 1.0383594383161432e21*coen(8)*coep(8)&
            + 2.2956689619838243e19*v**2*coen(8)*coep(8) - 1.8754814593315192e15*coen(9)*coep(8)&
            + 1.4987698517770688e14*coep(8)**2 + 1.6810824745022383e12*v**2*coep(8)**2&
            - 1.8754814593683745e15*coen(8)*coep(9) - 2.0407250019779514e21*coen(9)*coep(9)&
            - 3.483807244022235e9*coep(8)*coep(9) - 3.6776455207044805e15*coep(9)**2&
            - 7.127714528377333e25*coen(11)*coep(11) + 9.661425574505714e17*coen(12)*coep(11)&
            + 2.999728785124676e24*coen(15)*coep(11) - 1.0288155770538508e19*coep(11)**2&
            + 9.661425579784312e17*coen(11)*coep(12) + 1.605468816763783e16*coen(12)*coep(12)&
            + 5.422077295016715e15*v**2*coen(12)*coep(12) - 2.281275795245934e17*coen(15)*coep(12)&
            + 1.0203625009889756e21*v**2*coen(15)*coep(12)&
            + 1.414982167704334e11*coep(11)*coep(12) + 2.8443675040065453e10*coep(12)**2&
            + 4.999564801995346e9*v**2*coep(12)**2 - 7.27817431522172e16*coen(13)*coep(13)&
            - 8.169551359035541e19*v**2*coen(13)*coep(13) - 4.996180160429972e14*coep(13)**2&
            - 1.1366486909801244e14*v**2*coep(13)**2 + 1.874670375525277e17*coen(14)*coep(14)&
            - 1.0203625009889757e21*v**2*coen(14)*coep(14) - 2.6972445039832955e15*coep(14)**2&
            - 1.8388227603522402e15*v**2*coep(14)**2 + 1.263102803889675e24*coen(11)*coep(15)&
            - 2.281275795245934e17*coen(12)*coep(15)&
            + 1.0203625009889756e21*v**2*coen(12)*coep(15)&
            - 5.315827721039218e22*coen(15)*coep(15)&
            - 1.2868487325755707e22*v**2*coen(15)*coep(15)&
            + 6.180759854512867e17*coep(11)*coep(15) + 5.394483052950804e15*coep(12)*coep(15)&
            + 3.67764552070448e15*v**2*coep(12)*coep(15) - 6.362479137377961e15*coep(15)**2&
            + 1.8514965173142926e20*v**2*coep(15)**2)) + Q*(-2.4121501608883293e-14*coen(7)**2&
            + 1.655796356440184e-9*coen(10)**2 - 0.00005777676466214965*coen(3)*coep(1)&
            - 0.023540523320155892*coep(1)**2 - 0.00004167812729666628*coep(1)*coep(3)&
            - 1.238559294940497e-15*coen(7)*coep(7) - 1.589898041959544e-17*coep(7)**2&
            + 8.501966424189473e-11*coen(10)*coep(10) + 1.0913696119226905e-12*coep(10)**2&
            + jx*(-2.523436141500831e-7*coen(4)**2 + 1.1893552969772174e-8*coen(8)**2&
            + 1.1038642376267897e-9*coen(9)**2 + 0.00014899044762998738*coen(11)**2&
            - 0.000027460049387358062*coen(11)*coen(12) + 1.2740529163840806e-6*coen(12)**2&
            + 2.0231367393944403e-10*coen(5)*coep(4) + 2.833988808063157e-11*coen(6)*coep(4)&
            - 1.7288105846131693e-10*coep(4)**2 + 4.043987875665763e-11*coep(4)*coep(5)&
            + coen(4)*(7.880300331788053e-9*coen(5) + 1.1038642376267889e-9*coen(6)&
            - 1.3212378193964972e-8*coep(4) + 1.5751698032973473e-9*coep(5)&
            + 2.833988808063157e-11*coep(6)) + 7.275797412817936e-13*coep(4)*coep(6)&
            - 1.5751698032973473e-9*coen(9)*coep(8) - 5.056951207060763e-10*coep(8)**2&
            + coen(8)*(-7.880300331788053e-9*coen(9) + 2.5779320031971947e-9*coep(8)&
            - 2.0231367393944403e-10*coep(9)) + 5.667977616126365e-11*coen(9)*coep(9)&
            - 4.043987875665763e-11*coep(8)*coep(9) + 7.275797412818061e-13*coep(9)**2&
            + 0.00020898690331778533*coen(11)*coep(11) - 0.000019258921554049882*coen(12)*coep(11)&
            + 0.00007328578182881898*coep(11)**2 - 0.000019808714465606256*coen(11)*coep(12)&
            + 1.822732703115931e-6*coen(12)*coep(12)&
            - 0.000013892709098888758*coep(11)*coep(12) + 6.586163371889187e-7*coep(12)**2)&
            + jx**2*(-2.523436141500831e-7*coen(4)**2 + 1.1893552969772174e-8*coen(8)**2&
            + 1.1038642376267897e-9*coen(9)**2 + 0.00014899044762998738*coen(11)**2&
            - 0.000027460049387358062*coen(11)*coen(12) + 1.2740529163840806e-6*coen(12)**2&
            + 2.0231367393944403e-10*coen(5)*coep(4) + 2.833988808063157e-11*coen(6)*coep(4)&
            - 1.7288105846131693e-10*coep(4)**2 + 4.043987875665763e-11*coep(4)*coep(5)&
            + coen(4)*(7.880300331788053e-9*coen(5) + 1.1038642376267889e-9*coen(6)&
            - 1.3212378193964972e-8*coep(4) + 1.5751698032973473e-9*coep(5)&
            + 2.833988808063157e-11*coep(6)) + 7.275797412817936e-13*coep(4)*coep(6)&
            - 1.5751698032973473e-9*coen(9)*coep(8) - 5.056951207060763e-10*coep(8)**2&
            + coen(8)*(-7.880300331788053e-9*coen(9) + 2.5779320031971947e-9*coep(8)&
            - 2.0231367393944403e-10*coep(9)) + 5.667977616126365e-11*coen(9)*coep(9)&
            - 4.043987875665763e-11*coep(8)*coep(9) + 7.275797412818061e-13*coep(9)**2&
            + 0.00020898690331778533*coen(11)*coep(11) - 0.000019258921554049882*coen(12)*coep(11)&
            + 0.00007328578182881898*coep(11)**2 - 0.000019808714465606256*coen(11)*coep(12)&
            + 1.822732703115931e-6*coen(12)*coep(12) - 0.000013892709098888758*coep(11)*coep(12)&
            + 6.586163371889187e-7*coep(12)**2) + v**2*(1.6557963564401847e-9*coen(3)**2&
            - 4.6122975194059436e-7*coen(7)**2 + 8.501966424189548e-11*coen(3)*coep(3)&
            + 1.0913696119227093e-12*coep(3)**2 - 2.158509993849602e-8*coen(7)*coep(7)&
            - 2.501556654027428e-10*coep(7)**2 + jx*(0.00014899044762998738*coen(5)**2&
            - 0.018849944015569487*coen(8)**2 - 9.860036350321833e-8*coen(12)**2&
            + 5.519321188133945e-10*coen(13)**2 + 5.519321188133948e-10*coen(14)**2&
            - 1.1038642376267897e-9*coen(12)*coen(15) + 0.00020898690331778533*coen(5)*coep(5)&
            + 0.00007328578182881898*coep(5)**2 - 0.02440858458679133*coen(8)*coep(8)&
            - 0.007846841106718629*coep(8)**2 - 6.017344881132966e-9*coen(12)*coep(12)&
            - 2.833988808063182e-11*coen(15)*coep(12) - 8.949583666040266e-11*coep(12)**2&
            + 2.8339888080631575e-11*coen(13)*coep(13) + 3.6378987064089685e-13*coep(13)**2&
            + 2.8339888080631824e-11*coen(14)*coep(14) + 3.6378987064090306e-13*coep(14)**2&
            - 2.833988808063182e-11*coen(12)*coep(15) - 7.27579741281806e-13*coep(12)*coep(15))&
            + jx**2*(0.00014899044762998738*coen(5)**2 - 0.018849944015569487*coen(8)**2&
            - 9.860036350321833e-8*coen(12)**2 + 5.519321188133945e-10*coen(13)**2&
            + 5.519321188133948e-10*coen(14)**2 - 1.1038642376267897e-9*coen(12)*coen(15)&
            + 0.00020898690331778533*coen(5)*coep(5) + 0.00007328578182881898*coep(5)**2&
            - 0.02440858458679133*coen(8)*coep(8) - 0.007846841106718629*coep(8)**2&
            - 6.017344881132966e-9*coen(12)*coep(12) - 2.833988808063182e-11*coen(15)*coep(12)&
            - 8.949583666040266e-11*coep(12)**2 + 2.8339888080631575e-11*coen(13)*coep(13)&
            + 3.6378987064089685e-13*coep(13)**2 + 2.8339888080631824e-11*coen(14)*coep(14)&
            + 3.6378987064090306e-13*coep(14)**2 - 2.833988808063182e-11*coen(12)*coep(15)&
            - 7.27579741281806e-13*coep(12)*coep(15))))))/(EXP(16455.72762807003*Q)*mx**2)
        elseif (A==132) then 
            ans=(jx*mx*Q*(-5.475448362859454e-7*coen(8)**2 - 5.475448362859454e-7*jx*coen(8)**2&
            + Q**11*((-2.3899188793689975e28 - 2.3899188793689975e28*jx)*coen(5)**2&
            + (-1.8215492071078235e20 - 1.8215492071078235e20*jx)*coen(5)*coep(5)&
            + (-3.4708725289362054e11 - 3.4708725289362054e11*jx)*coep(5)**2)&
            - 7.581658216981977e-7*coen(8)*coep(8) - 7.581658216981977e-7*jx*coen(8)*coep(8)&
            - 2.6245129855042426e-7*coep(8)**2 - 2.6245129855042426e-7*jx*coep(8)**2&
            + Q**8*((7.195811117253994e21 + 7.195811117253994e21*jx)*coen(5)**2&
            + (-1.0759247066135678e22 - 1.0759247066135678e22*jx)*coen(8)**2&
            + (3.7521087994782475e21 + 3.7521087994782475e21*jx)*coen(5)*coep(5)&
            + 4.3680042510878854e20*coep(5)**2 + 4.3680042510878854e20*jx*coep(5)**2&
            + (-4.0052408633123034e21 - 4.0052408633123034e21*jx)*coen(8)*coep(8)&
            - 3.0516432550917394e20*coep(8)**2 - 3.0516432550917394e20*jx*coep(8)**2)&
            + Q**6*((5.860605741073919e15 + 5.860605741073919e15*jx)*coen(5)**2&
            + (-3.094634386781189e16 - 3.094634386781189e16*jx)*coen(8)**2&
            + (4.643362199378304e15 + 4.643362199378304e15*jx)*coen(5)*coep(5)&
            + 8.719109066869018e14*coep(5)**2 + 8.719109066869018e14*jx*coep(5)**2&
            + (-2.0419855066702916e16 - 2.0419855066702916e16*jx)*coen(8)*coep(8)&
            - 3.119832980498797e15*coep(8)**2 - 3.119832980498797e15*jx*coep(8)**2)&
            + Q**4*((5.616483007182814e8 + 5.616483007182814e8*jx)*coen(5)**2&
            + (-8.434733385564345e9 - 8.434733385564345e9*jx)*coen(8)**2&
            + (5.86348389993277e8 + 5.86348389993277e8*jx)*coen(5)*coep(5)&
            + 1.4935483603812963e8*coep(5)**2 + 1.4935483603812963e8*jx*coep(5)**2&
            + (-7.762149794439805e9 - 7.762149794439805e9*jx)*coen(8)*coep(8)&
            - 1.7214012281765425e9*coep(8)**2 - 1.7214012281765425e9*jx*coep(8)**2)&
            + Q**10*((4.696346594948489e26 + 4.696346594948489e26*jx)*coen(5)**2&
            + (-8.561939645548493e25 - 8.561939645548493e25*jx)*coen(8)**2&
            + (9.023893519157382e25 + 9.023893519157382e25*jx)*coen(5)*coep(5)&
            + 3.438917090603945e17*coep(5)**2 + 3.438917090603945e17*jx*coep(5)**2&
            + (-6.525742152709242e17 - 6.525742152709242e17*jx)*coen(8)*coep(8)&
            - 1.2434481089161592e9*coep(8)**2 - 1.2434481089161592e9*jx*coep(8)**2)&
            + Q**2*((5.427060915370515 + 5.427060915370515*jx)*coen(5)**2&
            + (-274.69334461627386 - 274.69334461627386*jx)*coen(8)**2&
            + (6.92687979789992 + 6.92687979789992*jx)*coen(5)*coep(5)&
            + 2.1943820719569356*coep(5)**2 + 2.1943820719569356*jx*coep(5)**2&
            + (-319.46295826152345 - 319.46295826152345*jx)*coen(8)*coep(8)&
            - 91.53194929358237*coep(8)**2 - 91.53194929358237*jx*coep(8)**2)&
            + Q*((-0.00015283776757538182 - 0.00015283776757538182*jx)*coen(5)**2&
            + (0.01944257121496351 + 0.01944257121496351*jx)*coen(8)**2&
            + (-0.00021162901001187267 - 0.00021162901001187267*jx)*coen(5)*coep(5)&
            - 0.00007325878706078944*coep(5)**2 - 0.00007325878706078944*jx*coep(5)**2&
            + (0.024815707040754052 + 0.024815707040754052*jx)*coen(8)*coep(8)&
            + 0.007861424511751426*coep(8)**2 + 0.007861424511751426*jx*coep(8)**2)&
            + Q**3*((-76675.94464728216 - 76675.94464728216*jx)*coen(5)**2&
            + (2.0121180238738682e6 + 2.0121180238738682e6*jx)*coen(8)**2 + (-89172.61588093972&
            - 89172.61588093972*jx)*coen(5)*coep(5) - 25549.576700872025*coep(5)**2&
            - 25549.576700872025*jx*coep(5)**2 + (2.1006066648222203e6&
            + 2.1006066648222203e6*jx)*coen(8)*coep(8) + 535067.1535206608*coep(8)**2&
            + 535067.1535206608*jx*coep(8)**2) + Q**5*((-2.354411429550883e12&
            - 2.354411429550883e12*jx)*coen(5)**2 + (2.0995755577560977e13&
            + 2.0995755577560977e13*jx)*coen(8)**2 + (-2.1666712341130344e12&
            - 2.1666712341130344e12*jx)*coen(5)*coep(5) - 4.804996840087568e11*coep(5)**2&
            - 4.804996840087568e11*jx*coep(5)**2 + (1.6634952444074191e13&
            + 1.6634952444074191e13*jx)*coen(8)*coep(8) + 3.1236409837139507e12*coep(8)**2&
            + 3.1236409837139507e12*jx*coep(8)**2) + Q**9*((-3.0032596299043265e24&
            - 3.0032596299043265e24*jx)*coen(5)**2 + (1.6824770266320765e24&
            + 1.6824770266320765e24*jx)*coen(8)**2 + (-1.1179944208818399e24&
            - 1.1179944208818399e24*jx)*coen(5)*coep(5) - 8.518139732782005e22*coep(5)**2&
            - 8.518139732782005e22*jx*coep(5)**2 + (3.232830718475304e23&
            + 3.232830718475304e23*jx)*coen(8)*coep(8) + 1.2319999992455875e15*coep(8)**2&
            + 1.2319999992455875e15*jx*coep(8)**2) + Q**7*((-8.638142117199035e18&
            - 8.638142117199035e18*jx)*coen(5)**2 + (2.5779159710626787e19&
            + 2.5779159710626787e19*jx)*coen(8)**2 + (-5.699852972365312e18&
            - 5.699852972365312e18*jx)*coen(5)*coep(5) - 8.708479677789727e17*coep(5)**2&
            - 8.708479677789727e17*jx*coep(5)**2 + (1.34420165311829e19&
            + 1.34420165311829e19*jx)*coen(8)*coep(8) + 1.5648476227439132e18*coep(8)**2&
            + 1.5648476227439132e18*jx*coep(8)**2)) + jx*Q*(-0.000033907235249691315*coen(8)**2&
            - 0.000033907235249691315*jx*coen(8)**2 + Q**11*((-1.4799800180769832e30&
            - 1.4799800180769832e30*jx)*coen(5)**2 + (-1.1280116876499715e22&
            - 1.1280116876499715e22*jx)*coen(5)*coep(5)&
            + (-2.1493708562502207e13 - 2.1493708562502207e13*jx)*coep(5)**2)&
            - 0.000046950140282523156*coen(8)*coep(8) - 0.000046950140282523156*jx*coen(8)*coep(8)&
            - 0.000016252546516371244*coep(8)**2 - 0.000016252546516371244*jx*coep(8)**2&
            + Q**8*((4.456074538481372e23 + 4.456074538481372e23*jx)*coen(5)**2&
            + (-6.662766173736588e23 - 6.662766173736588e23*jx)*coen(8)**2&
            + (2.3235290941526758e23 + 2.3235290941526758e23*jx)*coen(5)*coep(5)&
            + 2.7049282158866438e22*coep(5)**2 + 2.7049282158866438e22*jx*coep(5)**2&
            + (-2.4802835344991625e23 - 2.4802835344991625e23*jx)*coen(8)*coep(8)&
            - 1.8897591373593483e22*coep(8)**2 - 1.8897591373593483e22*jx*coep(8)**2)&
            + Q**6*((3.6292358981266797e17 + 3.6292358981266797e17*jx)*coen(5)**2&
            + (-1.9163818049336133e18 - 1.9163818049336133e18*jx)*coen(8)**2&
            + (2.8754462467731533e17 + 2.8754462467731533e17*jx)*coen(5)*coep(5)&
            + 5.399391295576956e16*coep(5)**2 + 5.399391295576956e16*jx*coep(5)**2&
            + (-1.2645189647076017e18 - 1.2645189647076017e18*jx)*coen(8)*coep(8)&
            - 1.9319862739838544e17*coep(8)**2 - 1.9319862739838544e17*jx*coep(8)**2)&
            + Q**4*((3.478060571116186e10 + 3.478060571116186e10*jx)*coen(5)**2&
            + (-5.223288947672552e11 - 5.223288947672552e11*jx)*coen(8)**2&
            + (3.631018225400095e10 + 3.631018225400095e10*jx)*coen(5)*coep(5)&
            + 9.248940407465086e9*coep(5)**2 + 9.248940407465086e9*jx*coep(5)**2&
            + (-4.8067851558728925e11 - 4.8067851558728925e11*jx)*coen(8)*coep(8)&
            - 1.0659940982880162e11*coep(8)**2 - 1.0659940982880162e11*jx*coep(8)**2)&
            + Q**10*((2.908257338141436e28 + 2.908257338141436e28*jx)*coen(5)**2&
            + (-5.30206263517133e27 - 5.30206263517133e27*jx)*coen(8)**2 + (5.588131969204511e27&
            + 5.588131969204511e27*jx)*coen(5)*coep(5) + 2.1295821468471955e19*coep(5)**2&
            + 2.1295821468471955e19*jx*coep(5)**2 + (-4.041127953130492e19&
            - 4.041127953130492e19*jx)*coen(8)*coep(8) - 7.700170790723283e10*coep(8)**2&
            - 7.700170790723283e10*jx*coep(8)**2) + Q**2*((336.0759137463106&
            + 336.0759137463106*jx)*coen(5)**2 + (-17010.646873426835&
            - 17010.646873426835*jx)*coen(8)**2 + (428.9536258745202&
            + 428.9536258745202*jx)*coen(5)*coep(5) + 135.88919885766575*coep(5)**2&
            + 135.88919885766575*jx*coep(5)**2 + (-19783.047819081105&
            - 19783.047819081105*jx)*coen(8)*coep(8) - 5668.203098420817*coep(8)**2&
            - 5668.203098420817*jx*coep(8)**2) + Q*((-0.00946462425866025&
            - 0.00946462425866025*jx)*coen(5)**2 + (1.2039997318144122&
            + 1.2039997318144122*jx)*coen(8)**2 + (-0.013105327915802746&
            - 0.013105327915802746*jx)*coen(5)*coep(5) - 0.004536620131104668*coep(5)**2&
            - 0.004536620131104668*jx*coep(5)**2 + (1.5367362830517977&
            + 1.5367362830517977*jx)*coen(8)*coep(8) + 0.4868261969663423*coep(8)**2&
            + 0.4868261969663423*jx*coep(8)**2) + Q**3*((-4.748230867782253e6&
            - 4.748230867782253e6*jx)*coen(5)**2 + (1.2460232416474801e8&
            + 1.2460232416474801e8*jx)*coen(8)**2 + (-5.522099130757511e6&
            - 5.522099130757511e6*jx)*coen(5)*coep(5) - 1.5821818603985196e6*coep(5)**2&
            - 1.5821818603985196e6*jx*coep(5)**2 + (1.3008206749666092e8&
            + 1.3008206749666092e8*jx)*coen(8)*coep(8) + 3.3134542865697075e7*coep(8)**2&
            + 3.3134542865697075e7*jx*coep(8)**2) + Q**5*((-1.4579916917461934e14&
            - 1.4579916917461934e14*jx)*coen(5)**2 + (1.3001821520997732e15&
            + 1.3001821520997732e15*jx)*coen(8)**2 + (-1.3417317884346453e14&
            - 1.3417317884346453e14*jx)*coen(5)*coep(5) - 2.975540036794144e13*coep(5)**2&
            - 2.975540036794144e13*jx*coep(5)**2 + (1.030135266574021e15&
            + 1.030135266574021e15*jx)*coen(8)*coep(8) + 1.934344416227029e14*coep(8)**2&
            + 1.934344416227029e14*jx*coep(8)**2) + Q**9*((-1.8597971168499313e26&
            - 1.8597971168499313e26*jx)*coen(5)**2 + (1.0418899159232068e26&
            + 1.0418899159232068e26*jx)*coen(8)**2 + (-6.923286884379661e25&
            - 6.923286884379661e25*jx)*coen(5)*coep(5) - 5.274939122215512e24*coep(5)**2&
            - 5.274939122215512e24*jx*coep(5)**2 + (2.0019611989642724e25&
            + 2.0019611989642724e25*jx)*coen(8)*coep(8) + 7.629277281728229e16*coep(8)**2&
            + 7.629277281728229e16*jx*coep(8)**2) + Q**7*((-5.349251741188459e20&
            - 5.349251741188459e20*jx)*coen(5)**2 + (1.5963990068406085e21&
            + 1.5963990068406085e21*jx)*coen(8)**2 + (-3.5296882157375154e20&
            - 3.5296882157375154e20*jx)*coen(5)*coep(5) - 5.39280894519782e19*coep(5)**2&
            - 5.39280894519782e19*jx*coep(5)**2 + (8.324096704932389e20&
            + 8.324096704932389e20*jx)*coen(8)*coep(8) + 9.690467877335368e19*coep(8)**2&
            + 9.690467877335368e19*jx*coep(8)**2))&
            + mx**2*((1.6557894649545754e-6 - 0.05879482821485974*Q + 830.6796374776401*Q**2&
            - 6.084695910520246e6*Q**3 + 2.550684757480667e10*Q**4 - 6.34916970999094e13*Q**5&
            + 9.35825283327575e16*Q**6 - 7.795683245553589e19*Q**7 + 3.253623587027546e22*Q**8&
            - 5.087853178603701e24*Q**9 + 2.5891522529622662e26*Q**10)*coen(1)**2&
            + 5.519298216515251e-7*jx*v**2*coen(8)**2 + 5.519298216515251e-7*jx**2*v**2*coen(8)**2&
            + 7.936593798439135e-7*coep(1)**2&
            + coen(1)*(2.2927126639925324e-6*coep(1) + Q**9*(-2.7122383411374768e23*coen(3)&
            - 9.776162043536268e23*coep(1) - 1.580718914296596e22*coep(3))&
            + Q**7*(-1.4732996878033738e18*coen(3) - 4.064899873962924e19*coep(1)&
            - 4.549200863080867e17*coep(3)) + Q**5*(-6.332086564155945e11*coen(3)&
            - 5.03045178797633e13*coep(1) - 3.1861056599444275e11*coep(3))&
            + Q**3*(-29755.877337191956*coen(3) - 6.352287803897017e6*coep(1)&
            - 20174.103150546955*coep(3)) + Q*(-0.00007901126761011023*coen(3)&
            - 0.07504332715873034*coep(1) - 0.00006813530398120589*coep(3))&
            + Q**2*(2.4531844847518527*coen(3) + 966.0640840313071*coep(1)&
            + 1.8890690146983724*coep(3)) + Q**4*(1.8333267111912528e8*coen(3)&
            + 2.347293774554158e10*coep(1) + 1.082569761578709e8*coep(3))&
            + Q**6*(1.2713512041435458e15*coen(3) + 6.175015935624462e16*coep(1)&
            + 5.2266276698085844e14*coep(3)) + Q**10*(2.1744712929222326e25*coen(3)&
            + 1.9734009694546926e18*coep(1) + 8.286696331137802e16*coep(3))&
            + Q**8*(9.291404625658473e20*coen(3) + 1.2111949901787994e22*coep(1)&
            + 1.773171426038139e20*coep(3))) + jx*Q**12*((-9.648292848889095e25&
            - 9.648292848889095e25*jx)*coen(5)**2 + (-7.353739217072609e17&
            - 7.353739217072609e17*jx)*coen(5)*coep(5) + (-1.4012188819221566e9&
            - 1.4012188819221566e9*jx)*coep(5)**2) + 7.642375546641774e-7*jx*v**2*coen(8)*coep(8)&
            + 7.642375546641774e-7*jx**2*v**2*coen(8)*coep(8)&
            + 2.645531266146378e-7*jx*v**2*coep(8)**2 + 2.645531266146378e-7*jx**2*v**2*coep(8)**2&
            + Q*(coep(1)*(-0.000054702043249433385*coen(3) - 0.023773147007263264*coep(1)&
            - 0.000047172263626818355*coep(3)) + jx*(-2.2104821104968457e-9*coen(8)**2&
            + 0.00015406176117318816*coen(11)**2 - 0.00002633708920337008*coen(11)*coen(12)&
            + 1.125591227868861e-6*coen(12)**2 - 3.0607758024382002e-9*coen(8)*coep(8)&
            - 1.0595367938405868e-9*coep(8)**2 + v**2*(0.00015406176117318816*coen(5)**2&
            - 0.01959827607161991*coen(8)**2 + 0.0002133238303267328*coen(5)*coep(5)&
            + 0.00007384547638351354*coep(5)**2 - 0.025014442386243445*coen(8)*coep(8)&
            - 0.007924382335754421*coep(8)**2) + 0.0002133238303267328*coen(11)*coep(11)&
            - 0.000018234014416477796*coen(12)*coep(11) + 0.00007384547638351354*coep(11)**2&
            - 0.000022711767993735294*coen(11)*coep(12) + 1.9413054059041598e-6*coen(12)*coep(12)&
            - 0.000015724087875606118*coep(11)*coep(12) + 8.37041588829748e-7*coep(12)**2)&
            + jx**2*(-2.2104821104968457e-9*coen(8)**2 + 0.00015406176117318816*coen(11)**2&
            - 0.00002633708920337008*coen(11)*coen(12) + 1.125591227868861e-6*coen(12)**2&
            - 3.0607758024382002e-9*coen(8)*coep(8) - 1.0595367938405868e-9*coep(8)**2&
            + v**2*(0.00015406176117318816*coen(5)**2 - 0.01959827607161991*coen(8)**2&
            + 0.0002133238303267328*coen(5)*coep(5) + 0.00007384547638351354*coep(5)**2&
            - 0.025014442386243445*coen(8)*coep(8) - 0.007924382335754421*coep(8)**2)&
            + 0.0002133238303267328*coen(11)*coep(11) - 0.000018234014416477796*coen(12)*coep(11)&
            + 0.00007384547638351354*coep(11)**2 - 0.000022711767993735294*coen(11)*coep(12)&
            + 1.9413054059041598e-6*coen(12)*coep(12) - 0.000015724087875606118*coep(11)*coep(12)&
            + 8.37041588829748e-7*coep(12)**2)) + jx*Q**11*((1.8959525220332866e24&
            + 2.4090584248755186e28*v**2 + jx*(1.8959525220332866e24&
            + 2.4090584248755186e28*v**2))*coen(5)**2 + (-3.4565232221010084e23&
            - 3.4565232221010084e23*jx)*coen(8)**2 + 2.4090584248755186e28*coen(11)**2&
            + 2.4090584248755186e28*jx*coen(11)**2 - 2.023221454771903e27*coen(11)*coen(15)&
            - 2.023221454771903e27*jx*coen(11)*coen(15) + 4.247951204484435e25*coen(15)**2&
            + 4.247951204484435e25*jx*coen(15)**2 + (3.64301767987248e23&
            + 1.836136992594088e20*v**2 + jx*(3.64301767987248e23&
            + 1.836136992594088e20*v**2))*coen(5)*coep(5) + 1.3883182169746642e15*coep(5)**2&
            + 1.3883182169746642e15*jx*coep(5)**2 + 3.498668837542483e11*v**2*coep(5)**2&
            + 3.498668837542483e11*jx*v**2*coep(5)**2 + (-2.6344940779873855e15&
            - 2.6344940779873855e15*jx)*coen(8)*coep(8) - 5.019899043765041e6*coep(8)**2&
            - 5.019899043765041e6*jx*coep(8)**2 + 1.836136992594088e20*coen(11)*coep(11)&
            + 1.836136992594088e20*jx*coen(11)*coep(11) - 7.710298179066943e18*coen(15)*coep(11)&
            - 7.710298179066943e18*jx*coen(15)*coep(11) + 3.498668837542483e11*coep(11)**2&
            + 3.498668837542483e11*jx*coep(11)**2 - 7.710298066895302e18*coen(11)*coep(15)&
            - 7.710298066895302e18*jx*coen(11)*coep(15) + 3.237704887218533e17*coen(15)*coep(15)&
            + 3.237704887218533e17*jx*coen(15)*coep(15) - 2.938318838258287e10*coep(11)*coep(15)&
            - 2.938318838258287e10*jx*coep(11)*coep(15) + 6.169287517740592e8*coep(15)**2&
            + 6.169287517740592e8*jx*coep(15)**2) + Q**2*(0.0009425685664583746*coen(3)**2&
            + 276.79493495930234*coep(1)**2 + coen(3)*(1.5464892054461874*coep(1)&
            + 0.001625646511980602*coep(3)) + 1.1768471084254728*coep(1)*coep(3)&
            + 0.0007009374903739175*coep(3)**2 + jx*((-6.170182396848007e-7&
            - 5.470523260579413*v**2)*coen(5)**2&
            + (0.00007849120839903887 + 276.89321249254664*v**2)*coen(8)**2&
            - 5.470523260579413*coen(11)**2 + 0.8177281615839509*coen(11)*coen(12)&
            - 0.029927716902627712*coen(12)**2 + 0.007351547583903489*coen(11)*coen(15)&
            - 0.0006283790443055827*coen(12)*coen(15) + (-8.543631675289897e-7&
            - 6.982353367423382*v**2)*coen(5)*coep(5) - 2.957515576861436e-7*coep(5)**2&
            - 2.211955670746198*v**2*coep(5)**2 + (0.00010018298564370096&
            + 322.021361343769*v**2)*coen(8)*coep(8) + 0.00003173719683692318*coep(8)**2&
            + 92.26497831976742*v**2*coep(8)**2 - 6.982353367423382*coen(11)*coep(11)&
            + 0.5154964018153958*coen(12)*coep(11) + 0.005089712974475782*coen(15)*coep(11)&
            - 2.211955670746198*coep(11)**2 + 0.6296896715661241*coen(11)*coep(12)&
            - 0.04516478480632723*coen(12)*coep(12) - 0.0005418821706602005*coen(15)*coep(12)&
            + 0.39228236947515754*coep(11)*coep(12) - 0.016692184763967576*coep(12)**2&
            + 0.006339601230463856*coen(11)*coep(15) - 0.0005418821706602005*coen(12)*coep(15)&
            + 0.004389109948270467*coep(11)*coep(15) - 0.00046729166024927815*coep(12)*coep(15))&
            + jx**2*((-6.170182396848007e-7 - 5.470523260579413*v**2)*coen(5)**2&
            + (0.00007849120839903887 + 276.89321249254664*v**2)*coen(8)**2&
            - 5.470523260579413*coen(11)**2 + 0.8177281615839509*coen(11)*coen(12)&
            - 0.029927716902627712*coen(12)**2 + 0.007351547583903489*coen(11)*coen(15)&
            - 0.0006283790443055827*coen(12)*coen(15) + (-8.543631675289897e-7&
            - 6.982353367423382*v**2)*coen(5)*coep(5) - 2.957515576861436e-7*coep(5)**2&
            - 2.211955670746198*v**2*coep(5)**2 + (0.00010018298564370096&
            + 322.021361343769*v**2)*coen(8)*coep(8) + 0.00003173719683692318*coep(8)**2&
            + 92.26497831976742*v**2*coep(8)**2 - 6.982353367423382*coen(11)*coep(11)&
            + 0.5154964018153958*coen(12)*coep(11) + 0.005089712974475782*coen(15)*coep(11)&
            - 2.211955670746198*coep(11)**2 + 0.6296896715661241*coen(11)*coep(12)&
            - 0.04516478480632723*coen(12)*coep(12) - 0.0005418821706602005*coen(15)*coep(12)&
            + 0.39228236947515754*coep(11)*coep(12) - 0.016692184763967576*coep(12)**2&
            + 0.006339601230463856*coen(11)*coep(15) - 0.0005418821706602005*coen(12)*coep(15)&
            + 0.004389109948270467*coep(11)*coep(15) - 0.00046729166024927815*coep(12)*coep(15)))&
            + Q**8*(5.164817344728757e18*coen(3)**2 + 9.228246561240175e20*coep(1)**2&
            + 2.984251579342816e19*coep(1)*coep(3) + 2.4126353155029754e17*coep(3)**2&
            + coen(3)*(2.1395117929565648e20*coep(1) + 2.6816108021257943e18*coep(3))&
            + jx*((-3.4872867667819772e16 - 7.253438409763345e21*v**2)*coen(5)**2&
            + (1.0407252080124174e17 + 1.0845411956758485e22*v**2)*coen(8)**2&
            - 7.253438409763345e21*coen(11)**2 + 3.097134875219491e20*coen(11)*coen(12)&
            - 2.887136259577154e18*coen(12)**2 + 1.3708212876274277e20*coen(11)*coen(15)&
            - 3.4432115631525033e18*coen(12)*coen(15) - 5.3519100455476915e17*coen(15)**2&
            + (-2.3010760385102e16 - 3.782157374654989e21*v**2)*coen(5)*coep(5)&
            - 3.5156826001599975e15*coep(5)**2 - 4.402985194105687e20*v**2*coep(5)**2&
            + (5.426649125710212e16 + 4.037316633929331e21*v**2)*coen(8)*coep(8)&
            + 6.317414477309584e15*coep(8)**2 + 3.076082187080058e20*v**2*coep(8)**2&
            - 3.782157374654989e21*coen(11)*coep(11) + 7.131705976521883e19*coen(12)*coep(11)&
            + 4.1116871005114065e19*coen(15)*coep(11) - 4.402985194105687e20*coep(11)**2&
            + 5.910571420127131e19*coen(11)*coep(12) - 7.926627950228677e17*coen(12)*coep(12)&
            - 8.938702673752645e17*coen(15)*coep(12) + 9.947505264476056e18*coep(11)*coep(12)&
            - 3.020759945440774e9*coep(12)**2 + 4.232771809041877e19*coen(11)*coep(15)&
            - 8.938702673752645e17*coen(12)*coep(15) - 3.9369776582653024e17*coen(15)*coep(15)&
            + 1.0984853220753721e19*coep(11)*coep(15) - 1.6084235436686496e17*coep(12)*coep(15)&
            - 6.250358728954863e16*coep(15)**2) + jx**2*((-3.4872867667819772e16&
            - 7.253438409763345e21*v**2)*coen(5)**2 + (1.0407252080124174e17&
            + 1.0845411956758485e22*v**2)*coen(8)**2 - 7.253438409763345e21*coen(11)**2&
            + 3.097134875219491e20*coen(11)*coen(12) - 2.887136259577154e18*coen(12)**2&
            + 1.3708212876274277e20*coen(11)*coen(15) - 3.4432115631525033e18*coen(12)*coen(15)&
            - 5.3519100455476915e17*coen(15)**2 + (-2.3010760385102e16&
            - 3.782157374654989e21*v**2)*coen(5)*coep(5) - 3.5156826001599975e15*coep(5)**2&
            - 4.402985194105687e20*v**2*coep(5)**2 + (5.426649125710212e16&
            + 4.037316633929331e21*v**2)*coen(8)*coep(8) + 6.317414477309584e15*coep(8)**2&
            + 3.076082187080058e20*v**2*coep(8)**2 - 3.782157374654989e21*coen(11)*coep(11)&
            + 7.131705976521883e19*coen(12)*coep(11) + 4.1116871005114065e19*coen(15)*coep(11)&
            - 4.402985194105687e20*coep(11)**2 + 5.910571420127131e19*coen(11)*coep(12)&
            - 7.926627950228677e17*coen(12)*coep(12) - 8.938702673752645e17*coen(15)*coep(12)&
            + 9.947505264476056e18*coep(11)*coep(12) - 3.020759945440774e9*coep(12)**2&
            + 4.232771809041877e19*coen(11)*coep(15) - 8.938702673752645e17*coen(12)*coep(15)&
            - 3.9369776582653024e17*coen(15)*coep(15)&
            + 1.0984853220753721e19*coep(11)*coep(15) - 1.6084235436686496e17*coep(12)*coep(15)&
            - 6.250358728954863e16*coep(15)**2)) + Q**10*(4.5655150236271136e23*coen(3)**2&
            + 3.7602186022367096e9*coep(1)**2 + 3.1579785535467136e8*coep(1)*coep(3)&
            + 6.63048455396237e6*coep(3)**2 + coen(3)*(8.286696451695054e16*coep(1)&
            + 3.4797457864069735e15*coep(3)) + jx*((-1.2124398305190072e22&
            - 4.7339570511628636e26*v**2)*coen(5)**2 + (6.792293748798874e21&
            + 8.630507509874219e25*v**2)*coen(8)**2 - 4.7339570511628636e26*coen(11)**2&
            + 7.248237643074109e24*coen(11)*coen(12) + 2.5235830061797695e25*coen(11)*coen(15)&
            - 3.043676682418074e23*coen(12)*coen(15) - 2.2495169862136396e23*coen(15)**2&
            + (-4.51343251405259e21 - 9.096160917915918e25*v**2)*coen(5)*coep(5)&
            - 3.438840848495153e20*coep(5)**2 - 3.466457485684696e17*v**2*coep(5)**2&
            + (1.305119507276733e21 + 6.578003231515642e17*v**2)*coen(8)*coep(8)&
            + 4.973682113298749e12*coep(8)**2 + 1.2534062007455697e9*v**2*coep(8)**2&
            - 9.096160917915918e25*coen(11)*coep(11) + 2.762232150565018e16*coen(12)*coep(11)&
            + 3.819655735309973e24*coen(15)*coep(11) - 3.466457485684696e17*coep(11)**2&
            + 2.7622321103792676e16*coen(11)*coep(12) - 1.1599152621356575e15*coen(15)*coep(12)&
            + 1.0526595178489044e8*coep(11)*coep(12) + 1.470768748145069e24*coen(11)*coep(15)&
            - 1.1599152621356575e15*coen(12)*coep(15) - 6.176045262251228e22*coen(15)*coep(15)&
            + 2.01612873346616e16*coep(11)*coep(15) - 4.420323035974911e6*coep(12)*coep(15)&
            - 2.3536301018013028e14*coep(15)**2) + jx**2*((-1.2124398305190072e22&
            - 4.7339570511628636e26*v**2)*coen(5)**2 + (6.792293748798874e21&
            + 8.630507509874219e25*v**2)*coen(8)**2 - 4.7339570511628636e26*coen(11)**2&
            + 7.248237643074109e24*coen(11)*coen(12) + 2.5235830061797695e25*coen(11)*coen(15)&
            - 3.043676682418074e23*coen(12)*coen(15) - 2.2495169862136396e23*coen(15)**2&
            + (-4.51343251405259e21 - 9.096160917915918e25*v**2)*coen(5)*coep(5)&
            - 3.438840848495153e20*coep(5)**2 - 3.466457485684696e17*v**2*coep(5)**2&
            + (1.305119507276733e21 + 6.578003231515642e17*v**2)*coen(8)*coep(8)&
            + 4.973682113298749e12*coep(8)**2 + 1.2534062007455697e9*v**2*coep(8)**2&
            - 9.096160917915918e25*coen(11)*coep(11) + 2.762232150565018e16*coen(12)*coep(11)&
            + 3.819655735309973e24*coen(15)*coep(11) - 3.466457485684696e17*coep(11)**2&
            + 2.7622321103792676e16*coen(11)*coep(12) - 1.1599152621356575e15*coen(15)*coep(12)&
            + 1.0526595178489044e8*coep(11)*coep(12) + 1.470768748145069e24*coen(11)*coep(15)&
            - 1.1599152621356575e15*coen(12)*coep(15) - 6.176045262251228e22*coen(15)*coep(15)&
            + 2.01612873346616e16*coep(11)*coep(15) - 4.420323035974911e6*coep(12)*coep(15)&
            - 2.3536301018013028e14*coep(15)**2)) + Q**6*(3.6142710266205596e12*coen(3)**2&
            + 9.4344540194513e15*coep(1)**2 + 1.7701743687855503e14*coep(1)*coep(3)&
            + 7.269338799688982e11*coep(3)**2 + coen(3)*(4.6758349946674956e14*coep(1)&
            + 3.3690127907665547e12*coep(3)) + jx*((-9.504946446163973e9&
            - 5.907540108280006e15*v**2)*coen(5)**2 + (8.476153736627649e10&
            + 3.1194176110919164e16*v**2)*coen(8)**2 - 5.907540108280006e15*coen(11)**2&
            + 4.237837347145152e14*coen(11)*coen(12) - 6.868893920425141e12*coen(12)**2&
            + 5.891645215907381e13*coen(11)*coen(15) - 2.4095140177470386e12*coen(12)*coen(15)&
            - 1.2060987053401004e11*coen(15)**2 + (-8.747024325572882e9&
            - 4.680548332717577e15*v**2)*coen(5)*coep(5) - 1.9398154912852888e9*coep(5)**2&
            - 8.788935614624362e14*v**2*coep(5)**2 + (6.715662782250886e10&
            + 2.0583386452081536e16*v**2)*coen(8)*coep(8) + 1.261038741767729e10*coep(8)**2&
            + 3.144818006483766e15*v**2*coep(8)**2 - 4.680548332717577e15*coen(11)*coep(11)&
            + 1.558611664889165e14*coen(12)*coep(11) + 2.56362555395316e13*coen(15)*coep(11)&
            - 8.788935614624362e14*coep(11)**2 + 1.742209223269528e14*coen(11)*coep(12)&
            - 5.052902920931049e12*coen(12)*coep(12) - 1.1230042635888513e12*coen(15)*coep(12)&
            + 5.900581229285168e13*coep(11)*coep(12) - 8.022005360405996e11*coep(12)**2&
            + 2.964489505725702e13*coen(11)*coep(15) - 1.1230042635888513e12*coen(12)*coep(15)&
            - 1.349548126088909e11*coen(15)*coep(15) + 1.2249467026617582e13*coep(11)*coep(15)&
            - 4.84622586645932e11*coep(12)*coep(15) - 3.601186918006424e10*coep(15)**2)&
            + jx**2*((-9.504946446163973e9 - 5.907540108280006e15*v**2)*coen(5)**2&
            + (8.476153736627649e10 + 3.1194176110919164e16*v**2)*coen(8)**2&
            - 5.907540108280006e15*coen(11)**2 + 4.237837347145152e14*coen(11)*coen(12)&
            - 6.868893920425141e12*coen(12)**2 + 5.891645215907381e13*coen(11)*coen(15)&
            - 2.4095140177470386e12*coen(12)*coen(15) - 1.2060987053401004e11*coen(15)**2&
            + (-8.747024325572882e9 - 4.680548332717577e15*v**2)*coen(5)*coep(5)&
            - 1.9398154912852888e9*coep(5)**2 - 8.788935614624362e14*v**2*coep(5)**2&
            + (6.715662782250886e10 + 2.0583386452081536e16*v**2)*coen(8)*coep(8)&
            + 1.261038741767729e10*coep(8)**2 + 3.144818006483766e15*v**2*coep(8)**2&
            - 4.680548332717577e15*coen(11)*coep(11)&
            + 1.558611664889165e14*coen(12)*coep(11) + 2.56362555395316e13*coen(15)*coep(11)&
            - 8.788935614624362e14*coep(11)**2 + 1.742209223269528e14*coen(11)*coep(12)&
            - 5.052902920931049e12*coen(12)*coep(12) - 1.1230042635888513e12*coen(15)*coep(12)&
            + 5.900581229285168e13*coep(11)*coep(12) - 8.022005360405996e11*coep(12)**2&
            + 2.964489505725702e13*coen(11)*coep(15) - 1.1230042635888513e12*coen(12)*coep(15)&
            - 1.349548126088909e11*coen(15)*coep(15) + 1.2249467026617582e13*coep(11)*coep(15)&
            - 4.84622586645932e11*coep(12)*coep(15) - 3.601186918006424e10*coep(15)**2))&
            + Q**4*(255828.63854581988*coen(3)**2 + 5.20556095078592e9*coep(1)**2&
            + 5.251465811255071e7*coep(1)*coep(3) + 105895.34421556526*coep(3)**2&
            + coen(3)*(9.200559251419438e7*coep(1) + 334435.3300066818*coep(3))&
            + jx*((-309.54689500486876 - 5.661462329715704e8*v**2)*coen(5)**2&
            + (8123.080707236719 + 8.502282524935555e9*v**2)*coen(8)**2&
            - 5.661462329715704e8*coen(11)**2 + 6.111089037304177e7*coen(11)*coen(12)&
            - 1.5479639967856454e6*coen(12)**2 + 2.768614588296654e6*coen(11)*coen(15)&
            - 170552.42569721318*coen(12)*coen(15) - 2331.8230065426187*coen(15)**2&
            + (-359.9969520086656 - 5.910441316729757e8*v**2)*coen(5)*coep(5)&
            - 103.14567590689614*coep(5)**2 - 1.5055093675336626e8*v**2*coep(5)**2&
            + (8480.31639797084 + 7.824312581847192e9*v**2)*coen(8)*coep(8)&
            + 2160.1087114521106*coep(8)**2 + 1.7351869835953064e9*v**2*coep(8)**2&
            - 5.910441316729757e8*coen(11)*coep(11) + 3.0668530838064793e7*coen(12)*coep(11)&
            + 1.5710489379931148e6*coen(15)*coep(11) - 1.5055093675336626e8*coep(11)**2&
            + 3.60856587192903e7*coen(11)*coep(12) - 1.7320737530566263e6*coen(12)*coep(12)&
            - 111478.44333556056*coen(15)*coep(12) + 1.7504886037516903e7*coep(11)*coep(12)&
            - 462193.3238206639*coep(12)**2 + 1.8770851773405222e6*coen(11)*coep(15)&
            - 111478.44333556056*coen(12)*coep(15) - 3519.0216694309*coen(15)*coep(15)&
            + 1.0419192776269214e6*coep(11)*coep(15) - 70596.89614371014*coep(12)*coep(15)&
            - 1300.5743334421286*coep(15)**2) + jx**2*((-309.54689500486876&
            - 5.661462329715704e8*v**2)*coen(5)**2 + (8123.080707236719&
            + 8.502282524935555e9*v**2)*coen(8)**2 - 5.661462329715704e8*coen(11)**2&
            + 6.111089037304177e7*coen(11)*coen(12) - 1.5479639967856454e6*coen(12)**2&
            + 2.768614588296654e6*coen(11)*coen(15) - 170552.42569721318*coen(12)*coen(15)&
            - 2331.8230065426187*coen(15)**2 + (-359.9969520086656&
            - 5.910441316729757e8*v**2)*coen(5)*coep(5) - 103.14567590689614*coep(5)**2&
            - 1.5055093675336626e8*v**2*coep(5)**2 + (8480.31639797084&
            + 7.824312581847192e9*v**2)*coen(8)*coep(8) + 2160.1087114521106*coep(8)**2&
            + 1.7351869835953064e9*v**2*coep(8)**2 - 5.910441316729757e8*coen(11)*coep(11)&
            + 3.0668530838064793e7*coen(12)*coep(11) + 1.5710489379931148e6*coen(15)*coep(11)&
            - 1.5055093675336626e8*coep(11)**2 + 3.60856587192903e7*coen(11)*coep(12)&
            - 1.7320737530566263e6*coen(12)*coep(12) - 111478.44333556056*coen(15)*coep(12)&
            + 1.7504886037516903e7*coep(11)*coep(12) - 462193.3238206639*coep(12)**2&
            + 1.8770851773405222e6*coen(11)*coep(15) - 111478.44333556056*coen(12)*coep(15)&
            - 3519.0216694309*coen(15)*coep(15) + 1.0419192776269214e6*coep(11)*coep(15)&
            - 70596.89614371014*coep(12)*coep(15) - 1300.5743334421286*coep(15)**2))&
            + Q**3*(-25.061429513528868*coen(3)**2 - 1.6180566359684686e6*coep(1)**2&
            + coen(3)*(-16884.957439457015*coep(1) - 37.8209294948953*coep(3))&
            - 11198.099710728087*coep(1)*coep(3) - 13.978012865132508*coep(3)**2&
            + jx*((0.021909477125884622 + 77290.00010518944*v**2)*coen(5)**2&
            + (-1.1089589087636227 - 2.028231970173415e6*v**2)*coen(8)**2&
            + 77290.00010518944*coen(11)**2 - 9918.625779063987*coen(11)*coen(12)&
            + 305.5040043047364*coen(12)**2 - 228.2548175374331*coen(11)*coen(15)&
            + 16.7076196756859*coen(12)*coen(15) + 0.08770062646765793*coen(15)**2&
            + (0.027964365400066523 + 89886.75030379485*v**2)*coen(5)*coep(5)&
            + 0.008858895184836785*coep(5)**2 + 25754.189204735674*v**2*coep(5)**2&
            + (-1.2896974044965974 - 2.1174292679656716e6*v**2)*coen(8)*coep(8)&
            - 0.36952176889255733*coep(8)**2 - 539352.2119894895*v**2*coep(8)**2&
            + 89886.75030379485*coen(11)*coep(11) - 5628.319146485673*coen(12)*coep(11)&
            - 143.89199573322583*coen(15)*coep(11) + 25754.189204735674*coep(11)**2&
            - 6724.701050182318*coen(11)*coep(12) + 399.37410087775595*coen(12)*coep(12)&
            + 12.606976498298433*coen(15)*coep(12) - 3732.699903576029*coep(11)*coep(12)&
            + 126.4575063955919*coep(12)**2 - 175.7670920983391*coen(11)*coep(15)&
            + 12.606976498298433*coen(12)*coep(15) + 0.15125713140568423*coen(15)*coep(15)&
            - 109.49890791857173*coep(11)*coep(15) + 9.318675243421668*coep(12)*coep(15)&
            + 0.06521823367337516*coep(15)**2) + jx**2*((0.021909477125884622&
            + 77290.00010518944*v**2)*coen(5)**2 + (-1.1089589087636227&
            - 2.028231970173415e6*v**2)*coen(8)**2 + 77290.00010518944*coen(11)**2&
            - 9918.625779063987*coen(11)*coen(12)&
            + 305.5040043047364*coen(12)**2 - 228.2548175374331*coen(11)*coen(15)&
            + 16.7076196756859*coen(12)*coen(15) + 0.08770062646765793*coen(15)**2&
            + (0.027964365400066523 + 89886.75030379485*v**2)*coen(5)*coep(5)&
            + 0.008858895184836785*coep(5)**2 + 25754.189204735674*v**2*coep(5)**2&
            + (-1.2896974044965974 - 2.1174292679656716e6*v**2)*coen(8)*coep(8)&
            - 0.36952176889255733*coep(8)**2 - 539352.2119894895*v**2*coep(8)**2&
            + 89886.75030379485*coen(11)*coep(11) - 5628.319146485673*coen(12)*coep(11)&
            - 143.89199573322583*coen(15)*coep(11) + 25754.189204735674*coep(11)**2&
            - 6724.701050182318*coen(11)*coep(12) + 399.37410087775595*coen(12)*coep(12)&
            + 12.606976498298433*coen(15)*coep(12) - 3732.699903576029*coep(11)*coep(12)&
            + 126.4575063955919*coep(12)**2 - 175.7670920983391*coen(11)*coep(15)&
            + 12.606976498298433*coen(12)*coep(15) + 0.15125713140568423*coen(15)*coep(15)&
            - 109.49890791857173*coep(11)*coep(15) + 9.318675243421668*coep(12)*coep(15)&
            + 0.06521823367337516*coep(15)**2)) + Q**5*(-1.2962629498649683e9*coen(3)**2&
            - 9.445969517705166e12*coep(1)**2 + coen(3)*(-2.755274347118602e11*coep(1)&
            - 1.450436209875091e9*coep(3)) - 1.3165199657286324e11*coep(1)*coep(3)&
            - 3.870400620348744e8*coep(3)**2 + jx*((2.267418919608541e6&
            + 2.373266615426004e12*v**2)*coen(5)**2 + (-3.405169041876153e7&
            - 2.1163899033303133e13*v**2)*coen(8)**2 + 2.373266615426004e12*coen(11)**2&
            - 2.1106955213853152e11*coen(11)*coen(12) + 4.316069840935379e9*coen(12)**2&
            - 1.7058058884299149e10*coen(11)*coen(15) + 8.641752999099785e8*coen(12)*coen(15)&
            + 2.3803394964823764e7*coen(15)**2 + (2.367135147124089e6&
            + 2.1840229120469424e12*v**2)*coen(5)*coep(5) + 602957.3676886294*coep(5)**2&
            + 4.8434774163420337e11*v**2*coep(5)**2 + (-3.1336416896908607e7&
            - 1.6768172626587766e13*v**2)*coen(8)*coep(8) - 6.94943385035301e6*coep(8)**2&
            - 3.1486565059017217e12*v**2*coep(8)**2 + 2.1840229120469424e12*coen(11)*coep(11)&
            - 9.184247823728674e10*coen(12)*coep(11) - 8.56059536585368e9*coen(15)*coep(11)&
            + 4.8434774163420337e11*coep(11)**2 - 1.0620352199814758e11*coen(11)*coep(12)&
            + 4.023188740648813e9*coen(12)*coep(12) + 4.834787366250302e8*coen(15)*coep(12)&
            - 4.388399885762108e10*coep(11)*coep(12) + 8.680858111024244e8*coep(12)**2&
            - 1.0072693877553436e10*coen(11)*coep(15) + 4.834787366250302e8*coen(12)*coep(15)&
            + 3.1117299046699304e7*coen(15)*coep(15) - 4.886189269509193e9*coep(11)*coep(15)&
            + 2.5802670802324948e8*coep(12)*coep(15) + 9.852957501658298e6*coep(15)**2)&
            + jx**2*((2.267418919608541e6 + 2.373266615426004e12*v**2)*coen(5)**2&
            + (-3.405169041876153e7 - 2.1163899033303133e13*v**2)*coen(8)**2&
            + 2.373266615426004e12*coen(11)**2 - 2.1106955213853152e11*coen(11)*coen(12)&
            + 4.316069840935379e9*coen(12)**2 - 1.7058058884299149e10*coen(11)*coen(15)&
            + 8.641752999099785e8*coen(12)*coen(15) + 2.3803394964823764e7*coen(15)**2&
            + (2.367135147124089e6 + 2.1840229120469424e12*v**2)*coen(5)*coep(5)&
            + 602957.3676886294*coep(5)**2 + 4.8434774163420337e11*v**2*coep(5)**2&
            + (-3.1336416896908607e7 - 1.6768172626587766e13*v**2)*coen(8)*coep(8)&
            - 6.94943385035301e6*coep(8)**2 - 3.1486565059017217e12*v**2*coep(8)**2&
            + 2.1840229120469424e12*coen(11)*coep(11) - 9.184247823728674e10*coen(12)*coep(11)&
            - 8.56059536585368e9*coen(15)*coep(11) + 4.8434774163420337e11*coep(11)**2&
            - 1.0620352199814758e11*coen(11)*coep(12) + 4.023188740648813e9*coen(12)*coep(12)&
            + 4.834787366250302e8*coen(15)*coep(12) - 4.388399885762108e10*coep(11)*coep(12)&
            + 8.680858111024244e8*coep(12)**2 - 1.0072693877553436e10*coen(11)*coep(15)&
            + 4.834787366250302e8*coen(12)*coep(15) + 3.1117299046699304e7*coen(15)*coep(15)&
            - 4.886189269509193e9*coep(11)*coep(15) + 2.5802670802324948e8*coep(12)*coep(15)&
            + 9.852957501658298e6*coep(15)**2)) + Q**7*(-5.752002445850689e15*coen(3)**2&
            - 4.732138879391148e18*coep(1)**2 + coen(3)*(-4.419064232663883e17*coep(1)&
            - 4.231294047709298e15*coep(3)) - 1.180604719723389e17*coep(1)*coep(3)&
            - 6.717616400578609e14*coep(3)**2 + jx*((2.3659732114711453e13&
            + 8.707320245197916e18*v**2)*coen(5)**2 + (-1.2493285475777544e14&
            - 2.598561081851196e19*v**2)*coen(8)**2 + 8.707320245197916e18*coen(11)**2&
            - 4.91099895934458e17*coen(11)*coen(12) + 6.167692519829395e15*coen(12)**2&
            - 1.182919747501724e17*coen(11)*coen(15) + 3.8346682972337915e15*coen(12)*coen(15)&
            + 3.3628729467346006e14*coen(15)**2 + (1.8745622995744598e13&
            + 5.745499959083902e18*v**2)*coen(5)*coep(5) + 3.5199737691836445e12*coep(5)**2&
            + 8.778221100615626e17*v**2*coep(5)**2 + (-8.243658113928923e13&
            - 1.3549666246543077e19*v**2)*coen(8)*coep(8) - 1.2595014205428758e13*coep(8)**2&
            - 1.5773796264637158e18*v**2*coep(8)**2 + 5.745499959083902e18*coen(11)*coep(11)&
            - 1.473021410887961e17*coen(12)*coep(11) - 4.3505976422761064e16*coen(15)*coep(11)&
            + 8.778221100615626e17*coep(11)**2 - 1.5164002876936227e17*coen(11)*coep(12)&
            + 3.2023109011289865e15*coen(12)*coep(12) + 1.4104313492364322e15*coen(15)*coep(12)&
            - 3.93534906574463e16*coep(11)*coep(12) + 2.881107267750901e14*coep(12)**2&
            - 4.863078796248516e16*coen(11)*coep(15) + 1.4104313492364322e15*coen(12)*coep(15)&
            + 3.134674153605224e14*coen(15)*coep(15) - 1.6470462375252532e16*coep(11)*coep(15)&
            + 4.478410933719071e14*coep(12)*coep(15) + 6.763704938027239e13*coep(15)**2)&
            + jx**2*((2.3659732114711453e13 + 8.707320245197916e18*v**2)*coen(5)**2&
            + (-1.2493285475777544e14 - 2.598561081851196e19*v**2)*coen(8)**2&
            + 8.707320245197916e18*coen(11)**2 - 4.91099895934458e17*coen(11)*coen(12)&
            + 6.167692519829395e15*coen(12)**2 - 1.182919747501724e17*coen(11)*coen(15)&
            + 3.8346682972337915e15*coen(12)*coen(15) + 3.3628729467346006e14*coen(15)**2&
            + (1.8745622995744598e13 + 5.745499959083902e18*v**2)*coen(5)*coep(5)&
            + 3.5199737691836445e12*coep(5)**2 + 8.778221100615626e17*v**2*coep(5)**2&
            + (-8.243658113928923e13 - 1.3549666246543077e19*v**2)*coen(8)*coep(8)&
            - 1.2595014205428758e13*coep(8)**2 - 1.5773796264637158e18*v**2*coep(8)**2&
            + 5.745499959083902e18*coen(11)*coep(11) - 1.473021410887961e17*coen(12)*coep(11)&
            - 4.3505976422761064e16*coen(15)*coep(11) + 8.778221100615626e17*coep(11)**2&
            - 1.5164002876936227e17*coen(11)*coep(12) + 3.2023109011289865e15*coen(12)*coep(12)&
            + 1.4104313492364322e15*coen(15)*coep(12) - 3.93534906574463e16*coep(11)*coep(12)&
            + 2.881107267750901e14*coep(12)**2 - 4.863078796248516e16*coen(11)*coep(15)&
            + 1.4104313492364322e15*coen(12)*coep(15) + 3.134674153605224e14*coen(15)*coep(15)&
            - 1.6470462375252532e16*coep(11)*coep(15) + 4.478410933719071e14*coep(12)*coep(15)&
            + 6.763704938027239e13*coep(15)**2)) + Q**9*(-2.417683985075161e21*coen(3)**2&
            - 3.7255992283882495e15*coep(1)**2 + coen(3)*(-4.105201497190372e22*coep(1)&
            - 6.637747486751376e20*coep(3)) - 2.1668483414990653e14*coep(1)*coep(3)&
            - 2.5295802782506353e12*coep(3)**2 + jx*((2.9050062553959588e19&
            + 3.0273110840564566e24*v**2)*coen(5)**2 + (-4.343593727124163e19&
            - 1.695951059534567e24*v**2)*coen(8)**2 + 3.0273110840564566e24*coen(11)**2&
            - 9.040794470458255e22*coen(11)*coen(12) + 5.452021045648903e20*coen(12)**2&
            - 8.645121802613483e22*coen(11)*coen(15) + 1.61178932338344e21*coen(12)*coen(15)&
            + 4.805567815885159e20*coen(15)**2 + (1.5147562041025417e19&
            + 1.1269478231413152e24*v**2)*coen(5)*coep(5) + 1.763398102901949e18*coep(5)**2&
            + 8.58635682770262e22*v**2*coep(5)**2 + (-1.6169476342133197e19&
            - 3.258720681178756e23*v**2)*coen(8)*coep(8) - 1.2319726853338243e18*coep(8)**2&
            - 1.2418664094627498e15*v**2*coep(8)**2 + 1.1269478231413152e24*coen(11)*coep(11)&
            - 1.3684004990634575e22*coen(12)*coep(11) - 1.9906936349708958e22*coen(15)*coep(11)&
            + 8.58635682770262e22*coep(11)**2 - 5.26906304765532e21*coen(11)*coep(12)&
            + 4.1554232464066504e12*coen(12)*coep(12) + 2.2125824955837917e20*coen(15)*coep(12)&
            - 7.222827804996884e13*coep(11)*coep(12) + 7917.954742011451*coep(12)**2&
            - 1.6498348282757287e22*coen(11)*coep(15) + 2.2125824955837917e20*coen(12)*coep(15)&
            + 2.4950858288488953e20*coen(15)*coep(15) - 2.776675802258686e21*coep(11)*coep(15)&
            + 1.6863868521670896e12*coep(12)*coep(15) + 2.244819487272289e19*coep(15)**2)&
            + jx**2*((2.9050062553959588e19 + 3.0273110840564566e24*v**2)*coen(5)**2&
            + (-4.343593727124163e19 - 1.695951059534567e24*v**2)*coen(8)**2&
            + 3.0273110840564566e24*coen(11)**2 - 9.040794470458255e22*coen(11)*coen(12)&
            + 5.452021045648903e20*coen(12)**2 - 8.645121802613483e22*coen(11)*coen(15)&
            + 1.61178932338344e21*coen(12)*coen(15) + 4.805567815885159e20*coen(15)**2&
            + (1.5147562041025417e19 + 1.1269478231413152e24*v**2)*coen(5)*coep(5)&
            + 1.763398102901949e18*coep(5)**2 + 8.58635682770262e22*v**2*coep(5)**2&
            + (-1.6169476342133197e19 - 3.258720681178756e23*v**2)*coen(8)*coep(8)&
            - 1.2319726853338243e18*coep(8)**2 - 1.2418664094627498e15*v**2*coep(8)**2&
            + 1.1269478231413152e24*coen(11)*coep(11) - 1.3684004990634575e22*coen(12)*coep(11)&
            - 1.9906936349708958e22*coen(15)*coep(11) + 8.58635682770262e22*coep(11)**2&
            - 5.26906304765532e21*coen(11)*coep(12) + 4.1554232464066504e12*coen(12)*coep(12)&
            + 2.2125824955837917e20*coen(15)*coep(12) - 7.222827804996884e13*coep(11)*coep(12)&
            + 7917.954742011451*coep(12)**2 - 1.6498348282757287e22*coen(11)*coep(15)&
            + 2.2125824955837917e20*coen(12)*coep(15) + 2.4950858288488953e20*coen(15)*coep(15)&
            - 2.776675802258686e21*coep(11)*coep(15) + 1.6863868521670896e12*coep(12)*coep(15)&
            + 2.244819487272289e19*coep(15)**2))))/(EXP(16618.23565606628*Q)*mx**2)
        elseif (A==134) then 
            ans=(jx*mx*Q*(-5.758439296286147e-7*coen(8)**2 - 5.758439296286147e-7*jx*coen(8)**2&
            + Q**11*((-5.870038435469319e28 - 5.870038435469319e28*jx)*coen(5)**2&
            + (-1.5475623930060513e27 - 1.5475623930060513e27*jx)*coen(5)*coep(5)&
            + (-1.0199887899265247e25 - 1.0199887899265247e25*jx)*coep(5)**2)&
            - 7.775148207454363e-7*coen(8)*coep(8) - 7.775148207454363e-7*jx*coen(8)*coep(8)&
            - 2.6245362040574675e-7*coep(8)**2 - 2.6245362040574675e-7*jx*coep(8)**2&
            + Q**10*((8.678948667314522e26 + 8.678948667314522e26*jx)*coen(5)**2&
            + (-2.0715256345622585e26 - 2.0715256345622585e26*jx)*coen(8)**2&
            + (1.3158168397162197e26 + 1.3158168397162197e26*jx)*coen(5)*coep(5)&
            + 1.583686436373453e24*coep(5)**2 + 1.583686436373453e24*jx*coep(5)**2&
            + (-5.461318871143368e24 - 5.461318871143368e24*jx)*coen(8)*coep(8)&
            - 3.5995214486700405e22*coep(8)**2 - 3.5995214486700405e22*jx*coep(8)**2)&
            + Q**8*((9.71346616080501e21 + 9.71346616080501e21*jx)*coen(5)**2&
            + (-1.5664139260347671e22 - 1.5664139260347671e22*jx)*coen(8)**2&
            + (4.132032125768703e21 + 4.132032125768703e21*jx)*coen(5)*coep(5)&
            + 3.635535154982413e20*coep(5)**2 + 3.635535154982413e20*jx*coep(5)**2&
            + (-4.4332718646281345e21 - 4.4332718646281345e21*jx)*coen(8)*coep(8)&
            - 2.3330486574928878e20*coep(8)**2 - 2.3330486574928878e20*jx*coep(8)**2)&
            + Q**6*((7.30005905444438e15 + 7.30005905444438e15*jx)*coen(5)**2&
            + (-3.9218890690703384e16 - 3.9218890690703384e16*jx)*coen(8)**2&
            + (5.196981712462061e15 + 5.196981712462061e15*jx)*coen(5)*coep(5)&
            + 8.544146108786734e14*coep(5)**2 + 8.544146108786734e14*jx*coep(5)**2&
            + (-2.2337527539537908e16 - 2.2337527539537908e16*jx)*coen(8)*coep(8)&
            - 2.814497153584054e15*coep(8)**2 - 2.814497153584054e15*jx*coep(8)**2)&
            + Q**4*((6.632866698850391e8 + 6.632866698850391e8*jx)*coen(5)**2&
            + (-1.007496046408569e10 - 1.007496046408569e10*jx)*coen(8)**2&
            + (6.53163875335027e8 + 6.53163875335027e8*jx)*coen(5)*coep(5)&
            + 1.5576148512665606e8*coep(5)**2 + 1.5576148512665606e8*jx*coep(5)**2&
            + (-8.570780733217058e9 - 8.570780733217058e9*jx)*coen(8)*coep(8)&
            - 1.7322680124696338e9*coep(8)**2 - 1.7322680124696338e9*jx*coep(8)**2)&
            + Q**2*((6.020250379320832 + 6.020250379320832*jx)*coen(5)**2&
            + (-310.43300082340977 - 310.43300082340977*jx)*coen(8)**2&
            + (7.432338598276443 + 7.432338598276443*jx)*coen(5)*coep(5)&
            + 2.273776159317634*coep(5)**2 + 2.273776159317634*jx*coep(5)**2&
            + (-345.5191780416412 - 345.5191780416412*jx)*coen(8)*coep(8)&
            - 94.3978360903592*coep(8)**2 - 94.3978360903592*jx*coep(8)**2)&
            + Q*((-0.0001631756780294899 - 0.0001631756780294899*jx)*coen(5)**2&
            + (0.021245351498706468 + 0.021245351498706468*jx)*coen(8)**2&
            + (-0.00022032273247186691 - 0.00022032273247186691*jx)*coen(5)*coep(5)&
            - 0.00007437092805445107*coep(5)**2 - 0.00007437092805445107*jx*coep(5)**2&
            + (0.026228584531994156 + 0.026228584531994156*jx)*coen(8)*coep(8)&
            + 0.00802411373121854*coep(8)**2 + 0.00802411373121854*jx*coep(8)**2)&
            + Q**3*((-87966.74374037186 - 87966.74374037186*jx)*coen(5)**2&
            + (2.340726308413751e6 + 2.340726308413751e6*jx)*coen(8)**2&
            + (-97909.03966895824 - 97909.03966895824*jx)*coen(5)*coep(5)&
            - 26749.31542387764*coep(5)**2 - 26749.31542387764*jx*coep(5)**2&
            + (2.305003154921178e6 + 2.305003154921178e6*jx)*coen(8)*coep(8)&
            + 549679.3809179863*coep(8)**2 + 549679.3809179863*jx*coep(8)**2)&
            + Q**5*((-2.854920266169623e12 - 2.854920266169623e12*jx)*coen(5)**2&
            + (2.5761772484698914e13 + 2.5761772484698914e13*jx)*coen(8)**2&
            + (-2.4286840329926885e12 - 2.4286840329926885e12*jx)*coen(5)*coep(5)&
            - 4.908691277614594e11*coep(5)**2 - 4.908691277614594e11*jx*coep(5)**2&
            + (1.8340051701647316e13 + 1.8340051701647316e13*jx)*coen(8)*coep(8)&
            + 3.015213253604871e12*coep(8)**2 + 3.015213253604871e12*jx*coep(8)**2)&
            + Q**7*((-1.1113374216078508e19 - 1.1113374216078508e19*jx)*coen(5)**2&
            + (3.427864122826968e19 + 3.427864122826968e19*jx)*coen(8)**2&
            + (-6.329737997094582e18 - 6.329737997094582e18*jx)*coen(5)*coep(5)&
            - 7.975381135724419e17*coep(5)**2 - 7.975381135724419e17*jx*coep(5)**2&
            + (1.4581864438303795e19 + 1.4581864438303795e19*jx)*coen(8)*coep(8)&
            + 1.282973587258329e18*coep(8)**2 + 1.282973587258329e18*jx*coep(8)**2)&
            + Q**9*((-4.438714056088177e24 - 4.438714056088177e24*jx)*coen(5)**2&
            + (3.062784825522962e24 + 3.062784825522962e24*jx)*coen(8)**2&
            + (-1.2562468842318237e24 - 1.2562468842318237e24*jx)*coen(5)*coep(5)&
            - 6.6111107016048605e22*coep(5)**2 - 6.6111107016048605e22*jx*coep(5)**2&
            + (4.6434931284107e23 + 4.6434931284107e23*jx)*coen(8)*coep(8)&
            + 5.588799947600114e21*coep(8)**2 + 5.588799947600114e21*jx*coep(8)**2))&
            + jx*Q*(-0.00003619998178121347*coen(8)**2 - 0.00003619998178121347*jx*coen(8)**2&
            + Q**11*((-3.6901541109595284e30 - 3.6901541109595284e30*jx)*coen(5)**2&
            + (-9.728630892790844e28 - 9.728630892790844e28*jx)*coen(5)*coep(5)&
            + (-6.412080376743e26 - 6.412080376743e26*jx)*coep(5)**2)&
            - 0.00004887786585466095*coen(8)*coep(8) - 0.00004887786585466095*jx*coen(8)*coep(8)&
            - 0.000016498943182797838*coep(8)**2 - 0.000016498943182797838*jx*coep(8)**2&
            + Q**10*((5.455953731065614e28 + 5.455953731065614e28*jx)*coen(5)**2&
            + (-1.3022485151286395e28 - 1.3022485151286395e28*jx)*coen(8)**2&
            + (8.271780455489253e27 + 8.271780455489253e27*jx)*coen(5)*coep(5)&
            + 9.95572188819425e25*coep(5)**2 + 9.95572188819425e25*jx*coep(5)**2&
            + (-3.4332157285098375e26 - 3.4332157285098375e26*jx)*coen(8)*coep(8)&
            - 2.2628112264199794e24*coep(8)**2 - 2.2628112264199794e24*jx*coep(8)**2)&
            + Q**8*((6.106295125492662e23 + 6.106295125492662e23*jx)*coen(5)**2&
            + (-9.847139592296903e23 - 9.847139592296903e23*jx)*coen(8)**2&
            + (2.5975699312951995e23 + 2.5975699312951995e23*jx)*coen(5)*coep(5)&
            + 2.2854509634268917e22*coep(5)**2 + 2.2854509634268917e22*jx*coep(5)**2&
            + (-2.786941955508807e23 - 2.786941955508807e23*jx)*coen(8)*coep(8)&
            - 1.4666529340753218e22*coep(8)**2 - 1.4666529340753218e22*jx*coep(8)**2)&
            + Q**6*((4.589125476118197e17 + 4.589125476118197e17*jx)*coen(5)**2&
            + (-2.465465129411892e18 - 2.465465129411892e18*jx)*coen(8)**2 + (3.267042224961186e17&
            + 3.267042224961186e17*jx)*coen(5)*coep(5) + 5.371211148714976e16*coep(5)**2&
            + 5.371211148714976e16*jx*coep(5)**2 + (-1.40423133485168e18&
            - 1.40423133485168e18*jx)*coen(8)*coep(8) - 1.769311795102704e17*coep(8)**2&
            - 1.769311795102704e17*jx*coep(8)**2) + Q**4*((4.169700179186716e10&
            + 4.169700179186716e10*jx)*coen(5)**2 + (-6.333545714054268e11&
            - 6.333545714054268e11*jx)*coen(8)**2 + (4.106064016776921e10&
            + 4.106064016776921e10*jx)*coen(5)*coep(5) + 9.791824891574776e9*coep(5)**2&
            + 9.791824891574776e9*jx*coep(5)**2 + (-5.3879547986784143e11&
            - 5.3879547986784143e11*jx)*coen(8)*coep(8) - 1.0889768436392586e11*coep(8)**2&
            - 1.0889768436392586e11*jx*coep(8)**2) + Q**2*((378.45836838170976&
            + 378.45836838170976*jx)*coen(5)**2 + (-19515.129700755017&
            - 19515.129700755017*jx)*coen(8)**2 + (467.22819848589637&
            + 467.22819848589637*jx)*coen(5)*coep(5) + 142.9391738052034*coep(5)**2&
            + 142.9391738052034*jx*coep(5)**2 + (-21720.79500470562&
            - 21720.79500470562*jx)*coen(8)*coep(8) - 5934.246713099626*coep(8)**2&
            - 5934.246713099626*jx*coep(8)**2) + Q*((-0.01025791237499773&
            - 0.01025791237499773*jx)*coen(5)**2 + (1.335572535573419&
            + 1.335572535573419*jx)*coen(8)**2 + (-0.013850417606403516&
            - 0.013850417606403516*jx)*coen(5)*coep(5) - 0.004675270680302897*coep(5)**2&
            - 0.004675270680302897*jx*coep(5)**2 + (1.6488396132222156&
            + 1.6488396132222156*jx)*coen(8)*coep(8) + 0.5044296830007982*coep(8)**2&
            + 0.5044296830007982*jx*coep(8)**2) + Q**3*((-5.529961083045335e6&
            - 5.529961083045335e6*jx)*coen(5)**2 + (1.4714794297481513e8&
            + 1.4714794297481513e8*jx)*coen(8)**2 + (-6.154975801374277e6&
            - 6.154975801374277e6*jx)*coen(5)*coep(5) - 1.681574956653299e6*coep(5)**2&
            - 1.681574956653299e6*jx*coep(5)**2 + (1.4490223465167165e8&
            + 1.4490223465167165e8*jx)*coen(8)*coep(8) + 3.455516773020961e7*coep(8)**2&
            + 3.455516773020961e7*jx*coep(8)**2) + Q**5*((-1.794723471146268e14&
            - 1.794723471146268e14*jx)*coen(5)**2 + (1.619493836115149e15&
            + 1.619493836115149e15*jx)*coen(8)**2 + (-1.5267733707527578e14&
            - 1.5267733707527578e14*jx)*coen(5)*coep(5) - 3.0858106802281004e13*coep(5)**2&
            - 3.0858106802281004e13*jx*coep(5)**2 + (1.1529331183439382e15&
            + 1.1529331183439382e15*jx)*coen(8)*coep(8) + 1.895490413823854e14*coep(8)**2&
            + 1.895490413823854e14*jx*coep(8)**2) + Q**7*((-6.986336461153837e20&
            - 6.986336461153837e20*jx)*coen(5)**2 + (2.1549001805895804e21&
            + 2.1549001805895804e21*jx)*coen(8)**2 + (-3.9791406731066514e20&
            - 3.9791406731066514e20*jx)*coen(5)*coep(5) - 5.013661462015543e19*coep(5)**2&
            - 5.013661462015543e19*jx*coep(5)**2 + (9.16677592387164e20&
            + 9.16677592387164e20*jx)*coen(8)*coep(8) + 8.065313897549115e19*coep(8)**2&
            + 8.065313897549115e19*jx*coep(8)**2) + Q**9*((-2.7903631469387574e26&
            - 2.7903631469387574e26*jx)*coen(5)**2 + (1.925395913354764e26&
            + 1.925395913354764e26*jx)*coen(8)**2 + (-7.897298552965142e25&
            - 7.897298552965142e25*jx)*coen(5)*coep(5) - 4.1560234403448513e24*coep(5)**2&
            - 4.1560234403448513e24*jx*coep(5)**2 + (2.91909592166871e25&
            + 2.91909592166871e25*jx)*coen(8)*coep(8) + 3.5133557179712184e23*coep(8)**2&
            + 3.5133557179712184e23*jx*coep(8)**2)) + mx**2*((1.7413314848227498e-6&
            - 0.06424518444552065*Q + 938.7383116297564*Q**2 - 7.078272789682019e6*Q**3&
            + 3.0466320754256416e10*Q**4 - 7.790268026508247e13*Q**5 + 1.1859652528348896e17*Q**6&
            - 1.0365738728239438e20*Q**7 + 4.736779789322992e22*Q**8 - 9.261751966995928e24*Q**9&
            + 6.264219562767858e26*Q**10)*coen(1)**2 + 5.804438282742499e-7*jx*v**2*coen(8)**2&
            + 5.804438282742499e-7*jx**2*v**2*coen(8)**2 + 7.936503781727725e-7*coep(1)**2&
            + coen(1)*(2.35117705964798e-6*coep(1) + Q**9*(-5.287671068239069e23*coen(3)&
            - 1.4041757441594533e24*coep(1) - 2.486132757531498e22*coep(3))&
            + Q**7*(-2.6234197968685507e18*coen(3) - 4.409503746998228e19*coep(1)&
            - 4.263298102335794e17*coep(3)) + Q**5*(-1.0715880209312236e12*coen(3)&
            - 5.545966158217979e13*coep(1) - 2.850643329593702e11*coep(3))&
            + Q**3*(-47928.13289028706*coen(3) - 6.970247248883331e6*coep(1)&
            - 17429.27996205089*coep(3)) + Q*(-0.00012100628407511348*coen(3)&
            - 0.07931430323030852*coep(1) - 0.00005531604578064018*coep(3))&
            + Q**2*(3.8558300899063807*coen(3) + 1044.8376589157144*coep(1)&
            + 1.5879986715102719*coep(3)) + Q**4*(3.0265606304932386e8*coen(3)&
            + 2.5917734949274437e10*coep(1) + 9.540431211245963e7*coep(3))&
            + Q**6*(2.2056567867432485e15*coen(3) + 6.7547885800895576e16*coep(1)&
            + 4.753028691399132e14*coep(3)) + Q**8*(1.7083903175046893e21*coen(3)&
            + 1.3406055845087475e22*coep(1) + 1.8100196129759193e20*coep(3))&
            + Q**10*(5.237836986286768e25*coen(3) + 1.6514833290179773e25*coep(1)&
            + 6.904451844552144e23*coep(3))) + jx*Q**12*((-2.334411395688794e26&
            - 2.334411395688794e26*jx)*coen(5)**2 + (-6.154384380080996e24&
            - 6.154384380080996e24*jx)*coen(5)*coep(5) + (-4.0563166337920785e22&
            - 4.0563166337920785e22*jx)*coep(5)**2) + 7.837256865493266e-7*jx*v**2*coen(8)*coep(8)&
            + 7.837256865493266e-7*jx**2*v**2*coen(8)*coep(8)&
            + 2.645501260575908e-7*jx*v**2*coep(8)**2 + 2.645501260575908e-7*jx**2*v**2*coep(8)**2&
            + Q*(coep(1)*(-0.00008169242952027985*coen(3) - 0.024264633451950988*coep(1)&
            - 0.000037344359475334835*coep(3)) + jx*(-2.2900303741465683e-9*coen(8)**2&
            + 0.00016447914159270315*coen(11)**2 - 0.00004033542802503783*coen(11)*coen(12)&
            + 2.4728770137793317e-6*coen(12)**2 - 3.092040159222503e-9*coen(8)*coep(8)&
            - 1.0437320454546087e-9*coep(8)**2 + v**2*(0.00016447914159270315*coen(5)**2&
            - 0.021415061481840213*coen(8)**2 + 0.00022208269239599708*coen(5)*coep(5)&
            + 0.00007496501043273182*coep(5)**2 - 0.02643810107676951*coen(8)*coep(8)&
            - 0.008088211150650329*coep(8)**2) + 0.00022208269239599708*coen(11)*coep(11)&
            - 0.00002723080984009329*coen(12)*coep(11) + 0.00007496501043273182*coep(11)**2&
            - 0.000018438681926880065*coen(11)*coep(12) + 2.260870650638265e-6*coen(12)*coep(12)&
            - 0.000012448119825111613*coep(11)*coep(12) + 5.167600400702364e-7*coep(12)**2)&
            + jx**2*(-2.2900303741465683e-9*coen(8)**2 + 0.00016447914159270315*coen(11)**2&
            - 0.00004033542802503783*coen(11)*coen(12) + 2.4728770137793317e-6*coen(12)**2&
            - 3.092040159222503e-9*coen(8)*coep(8) - 1.0437320454546087e-9*coep(8)**2&
            + v**2*(0.00016447914159270315*coen(5)**2 - 0.021415061481840213*coen(8)**2&
            + 0.00022208269239599708*coen(5)*coep(5) + 0.00007496501043273182*coep(5)**2&
            - 0.02643810107676951*coen(8)*coep(8) - 0.008088211150650329*coep(8)**2)&
            + 0.00022208269239599708*coen(11)*coep(11) - 0.00002723080984009329*coen(12)*coep(11)&
            + 0.00007496501043273182*coep(11)**2 - 0.000018438681926880065*coen(11)*coep(12)&
            + 2.260870650638265e-6*coen(12)*coep(12) - 0.000012448119825111613*coep(11)*coep(12)&
            + 5.167600400702364e-7*coep(12)**2)) + jx*Q**11*((3.451465760284626e24&
            + 5.916928886961195e28*v**2 + jx*(3.451465760284626e24&
            + 5.916928886961195e28*v**2))*coen(5)**2 + (-8.238094351416231e23&
            - 8.238094351416231e23*jx)*coen(8)**2 + 5.916928886961195e28*coen(11)**2&
            + 5.916928886961195e28*jx*coen(11)**2 - 4.947449344457541e27*coen(11)*coen(15)&
            - 4.947449344457541e27*jx*coen(11)*coen(15) + 1.0342043771183595e26*coen(15)**2&
            + 1.0342043771183595e26*jx*coen(15)**2 + (5.232772935033048e23&
            + 1.5599244754894348e27*v**2 + jx*(5.232772935033048e23&
            + 1.5599244754894348e27*v**2))*coen(5)*coep(5) + 6.298043368726913e21*coep(5)**2&
            + 6.298043368726913e21*jx*coep(5)**2 + 1.0281365619389379e25*v**2*coep(5)**2&
            + 1.0281365619389379e25*jx*v**2*coep(5)**2 + (-2.171870788997319e22&
            - 2.171870788997319e22*jx)*coen(8)*coep(8) - 1.4314665876850881e20*coep(8)**2&
            - 1.4314665876850881e20*jx*coep(8)**2 + 1.5599244754894348e27*coen(11)*coep(11)&
            + 1.5599244754894348e27*jx*coen(11)*coep(11) - 6.521666451552617e25*coen(15)*coep(11)&
            - 6.521666451552617e25*jx*coen(15)*coep(11) + 1.0281365619389379e25*coep(11)**2&
            + 1.0281365619389379e25*jx*coep(11)**2 - 6.521666451552672e25*coen(11)*coep(15)&
            - 6.521666451552672e25*jx*coen(11)*coep(15) + 2.726550802529241e24*coen(15)*coep(15)&
            + 2.726550802529241e24*jx*coen(15)*coep(15) - 8.596779945398445e23*coep(11)*coep(15)&
            - 8.596779945398445e23*jx*coep(11)*coep(15) + 1.797052749739534e22*coep(15)**2&
            + 1.797052749739534e22*jx*coep(15)**2) + Q**2*(0.0021022018083991525*coen(3)**2&
            + 285.4556862122239*coep(1)**2 + coen(3)*(2.3449234185758216*coep(1)&
            + 0.001921974422441879*coep(3)) + 0.9540484335486903*coep(1)*coep(3)&
            + 0.0004392996031306103*coep(3)**2 + jx*((-6.489210700727409e-7&
            - 6.0683407387765325*v**2)*coen(5)**2&
            + (0.00008448903902284099 + 312.91277054325207*v**2)*coen(8)**2&
            - 6.0683407387765325*coen(11)**2 + 1.2852766966354605*coen(11)*coen(12)&
            - 0.06635997983881767*coen(12)**2 + 0.011429765042135777*coen(11)*coen(15)&
            - 0.0014014678722661011*coen(12)*coen(15) + (-8.761848888481744e-7&
            - 7.491708859024287*v**2)*coen(5)*coep(5) - 2.957601451282175e-7*coep(5)**2&
            - 2.2919393096741345*v**2*coep(5)**2 + (0.00010430648333459965&
            + 348.2792196385714*v**2)*coen(8)*coep(8) + 0.000031910493841531156*coep(8)**2&
            + 95.15189540407461*v**2*coep(8)**2 - 7.491708859024287*coen(11)*coep(11)&
            + 0.781641139525274*coen(12)*coep(11) + 0.007716337066911621*coen(15)*coep(11)&
            - 2.2919393096741345*coep(11)**2 + 0.529332890503424*coen(11)*coep(12)&
            - 0.053533261981453945*coen(12)*coep(12) - 0.0006406581408139596*coen(15)*coep(12)&
            + 0.31801614451623017*coep(11)*coep(12) - 0.010604532468338862*coep(12)**2&
            + 0.005224930351057477*coen(11)*coep(15) - 0.0006406581408139596*coen(12)*coep(15)&
            + 0.0035273974216676134*coep(11)*coep(15) - 0.0002928664020870735*coep(12)*coep(15))&
            + jx**2*((-6.489210700727409e-7 - 6.0683407387765325*v**2)*coen(5)**2&
            + (0.00008448903902284099 + 312.91277054325207*v**2)*coen(8)**2&
            - 6.0683407387765325*coen(11)**2 + 1.2852766966354605*coen(11)*coen(12)&
            - 0.06635997983881767*coen(12)**2 + 0.011429765042135777*coen(11)*coen(15)&
            - 0.0014014678722661011*coen(12)*coen(15) + (-8.761848888481744e-7&
            - 7.491708859024287*v**2)*coen(5)*coep(5) - 2.957601451282175e-7*coep(5)**2&
            - 2.2919393096741345*v**2*coep(5)**2 + (0.00010430648333459965&
            + 348.2792196385714*v**2)*coen(8)*coep(8) + 0.000031910493841531156*coep(8)**2&
            + 95.15189540407461*v**2*coep(8)**2 - 7.491708859024287*coen(11)*coep(11)&
            + 0.781641139525274*coen(12)*coep(11) + 0.007716337066911621*coen(15)*coep(11)&
            - 2.2919393096741345*coep(11)**2 + 0.529332890503424*coen(11)*coep(12)&
            - 0.053533261981453945*coen(12)*coep(12) - 0.0006406581408139596*coen(15)*coep(12)&
            + 0.31801614451623017*coep(11)*coep(12) - 0.010604532468338862*coep(12)**2&
            + 0.005224930351057477*coen(11)*coep(15) - 0.0006406581408139596*coen(12)*coep(15)&
            + 0.0035273974216676134*coep(11)*coep(15) - 0.0002928664020870735*coep(12)*coep(15)))&
            + Q**10*(1.0949063973577998e24*coen(3)**2 + 1.088482435320001e23*coep(1)**2&
            + 9.101362909641559e21*coep(1)*coep(3) + 1.902529708452451e20*coep(3)**2&
            + coen(3)*(6.904451844552086e23*coep(1) + 2.8865841050957514e22*coep(3))&
            + jx*((-1.7651987782781578e22 - 8.748276973416602e26*v**2)*coen(5)**2&
            + (1.2180158405848462e22 + 2.088073187589286e26*v**2)*coen(8)**2&
            - 8.748276973416602e26*coen(11)**2 + 1.7459456620955892e25*coen(11)*coen(12)&
            + 4.994520606265049e25*coen(11)*coen(15) - 7.29937598238533e23*coen(12)*coen(15)&
            - 5.589998755621899e23*coen(15)**2 + (-4.995873663499861e21&
            - 1.3263277156454288e26*v**2)*coen(5)*coep(5) - 2.6291228464081817e20*coep(5)**2&
            - 1.5963370813120622e24*v**2*coep(5)**2 + (1.8466358259710246e21&
            + 5.504944430059922e24*v**2)*coen(8)*coep(8) + 2.222567778614477e19*coep(8)**2&
            + 3.6282747844000024e22*v**2*coep(8)**2 - 1.3263277156454288e26*coen(11)*coep(11)&
            + 2.3014839481840288e23*coen(12)*coep(11) + 5.239187347464056e24*coen(15)*coep(11)&
            - 1.5963370813120622e24*coep(11)**2 + 2.3014839481840486e23*coen(11)*coep(12)&
            - 9.621947016985837e21*coen(15)*coep(12) + 3.0337876365471866e21*coep(11)*coep(12)&
            + 2.3483006274701602e24*coen(11)*coep(15) - 9.621947016985837e21*coen(12)*coep(15)&
            - 8.538917756521648e22*coen(15)*coep(15) + 9.133875492799121e22*coep(11)*coep(15)&
            - 1.2683531389683003e20*coep(12)*coep(15) - 1.0284567057278346e21*coep(15)**2)&
            + jx**2*((-1.7651987782781578e22 - 8.748276973416602e26*v**2)*coen(5)**2&
            + (1.2180158405848462e22 + 2.088073187589286e26*v**2)*coen(8)**2&
            - 8.748276973416602e26*coen(11)**2 + 1.7459456620955892e25*coen(11)*coen(12)&
            + 4.994520606265049e25*coen(11)*coen(15) - 7.29937598238533e23*coen(12)*coen(15)&
            - 5.589998755621899e23*coen(15)**2 + (-4.995873663499861e21&
            - 1.3263277156454288e26*v**2)*coen(5)*coep(5) - 2.6291228464081817e20*coep(5)**2&
            - 1.5963370813120622e24*v**2*coep(5)**2 + (1.8466358259710246e21&
            + 5.504944430059922e24*v**2)*coen(8)*coep(8) + 2.222567778614477e19*coep(8)**2&
            + 3.6282747844000024e22*v**2*coep(8)**2 - 1.3263277156454288e26*coen(11)*coep(11)&
            + 2.3014839481840288e23*coen(12)*coep(11) + 5.239187347464056e24*coen(15)*coep(11)&
            - 1.5963370813120622e24*coep(11)**2 + 2.3014839481840486e23*coen(11)*coep(12)&
            - 9.621947016985837e21*coen(15)*coep(12) + 3.0337876365471866e21*coep(11)*coep(12)&
            + 2.3483006274701602e24*coen(11)*coep(15) - 9.621947016985837e21*coen(12)*coep(15)&
            - 8.538917756521648e22*coen(15)*coep(15) + 9.133875492799121e22*coep(11)*coep(15)&
            - 1.2683531389683003e20*coep(12)*coep(15) - 1.0284567057278346e21*coep(15)**2))&
            + Q**8*(1.2714640168003437e19*coen(3)**2 + 7.055055847399412e20*coep(1)**2&
            + 2.265295540249568e19*coep(1)*coep(3) + 1.7105334272377453e17*coep(3)**2&
            + coen(3)*(3.0333423502917933e20*coep(1) + 3.4528850037127654e18*coep(3))&
            + jx*((-4.419594130390645e16 - 9.79105829565017e21*v**2)*coen(5)**2&
            + (1.363201478008767e17 + 1.5789265964409974e22*v**2)*coen(8)**2&
            - 9.79105829565017e21*coen(11)**2 + 5.694634391682299e20*coen(11)*coen(12)&
            - 6.961622110406536e18*coen(12)**2 + 2.4779764219916186e20*coen(11)*coen(15)&
            - 8.476426778668957e18*coen(12)*coen(15) - 1.3258900281605115e18*coen(15)**2&
            + (-2.51722585372826e16 - 4.1650392098084755e21*v**2)*coen(5)*coep(5)&
            - 3.1716692851105665e15*coep(5)**2 - 3.6645761717842195e20*v**2*coep(5)**2&
            + (5.798951896035413e16 + 4.4686852816958243e21*v**2)*coen(8)*coep(8)&
            + 5.10216107677687e15*coep(8)**2 + 2.3516852824664703e20*v**2*coep(8)**2&
            - 4.1650392098084755e21*coen(11)*coep(11) + 1.0111141167639313e20*coen(12)*coep(11)&
            + 6.211370066039314e19*coen(15)*coep(11) - 3.6645761717842195e20*coep(11)**2&
            + 6.0333987099197325e19*coen(11)*coep(12) - 1.063412019420579e18*coen(12)*coep(12)&
            - 1.1509616679042548e18*coen(15)*coep(12) + 7.550985134165228e18*coep(11)*coep(12)&
            - 1.2808101137751016e16*coep(12)**2 + 4.026939260777044e19*coen(11)*coep(15)&
            - 1.1509616679042548e18*coen(12)*coep(15) - 5.0790236723455046e17*coen(15)*coep(15)&
            + 8.431133090131251e18*coep(11)*coep(15) - 1.1403556181584966e17*coep(12)*coep(15)&
            - 4.183235316098412e16*coep(15)**2) + jx**2*((-4.419594130390645e16&
            - 9.79105829565017e21*v**2)*coen(5)**2 + (1.363201478008767e17&
            + 1.5789265964409974e22*v**2)*coen(8)**2 - 9.79105829565017e21*coen(11)**2&
            + 5.694634391682299e20*coen(11)*coen(12) - 6.961622110406536e18*coen(12)**2&
            + 2.4779764219916186e20*coen(11)*coen(15) - 8.476426778668957e18*coen(12)*coen(15)&
            - 1.3258900281605115e18*coen(15)**2 + (-2.51722585372826e16&
            - 4.1650392098084755e21*v**2)*coen(5)*coep(5) - 3.1716692851105665e15*coep(5)**2&
            - 3.6645761717842195e20*v**2*coep(5)**2 + (5.798951896035413e16&
            + 4.4686852816958243e21*v**2)*coen(8)*coep(8) + 5.10216107677687e15*coep(8)**2&
            + 2.3516852824664703e20*v**2*coep(8)**2 - 4.1650392098084755e21*coen(11)*coep(11)&
            + 1.0111141167639313e20*coen(12)*coep(11) + 6.211370066039314e19*coen(15)*coep(11)&
            - 3.6645761717842195e20*coep(11)**2 + 6.0333987099197325e19*coen(11)*coep(12)&
            - 1.063412019420579e18*coen(12)*coep(12) - 1.1509616679042548e18*coen(15)*coep(12)&
            + 7.550985134165228e18*coep(11)*coep(12) - 1.2808101137751016e16*coep(12)**2&
            + 4.026939260777044e19*coen(11)*coep(15) - 1.1509616679042548e18*coen(12)*coep(15)&
            - 5.0790236723455046e17*coen(15)*coep(15) + 8.431133090131251e18*coep(11)*coep(15)&
            - 1.1403556181584966e17*coep(12)*coep(15) - 4.183235316098412e16*coep(15)**2))&
            + Q**6*(8.6570747805964795e12*coen(3)**2 + 8.510938911244166e15*coep(1)**2&
            + 1.3875105273227439e14*coep(1)*coep(3) + 4.7672831145043677e11*coep(3)**2&
            + coen(3)*(7.15655225291812e14*coep(1) + 4.242644277817359e12*coep(3))&
            + jx*((-1.1353517487822737e10 - 7.358372653025253e15*v**2)*coen(5)**2&
            + (1.0245005364537274e11 + 3.953217509449632e16*v**2)*coen(8)**2&
            - 7.358372653025253e15*coen(11)**2 + 7.352189289144162e14*coen(11)*coen(12)&
            - 1.6512249357348678e13*coen(12)**2 + 1.0121787802035417e14*coen(11)*coen(15)&
            - 5.771383187064318e12*coen(12)*coen(15) - 2.864785633171519e11*coen(15)**2&
            + (-9.658450699211243e9 - 5.238495719835488e15*v**2)*coen(5)*coep(5)&
            - 1.9521004815133781e9*coep(5)**2 - 8.612397598629242e14*v**2*coep(5)**2&
            + (7.293517097120024e10 + 2.2515961933631856e16*v**2)*coen(8)*coep(8)&
            + 1.1990974602680496e10*coep(8)**2 + 2.8369796370813885e15*v**2*coep(8)**2&
            - 5.238495719835488e15*coen(11)*coep(11) + 2.3855174176393738e14*coen(12)*coep(11)&
            + 4.029060761803484e13*coen(15)*coep(11) - 8.612397598629242e14*coep(11)**2&
            + 1.584342897133044e14*coen(11)*coep(12) - 6.325268580984681e12*coen(12)*coep(12)&
            - 1.4142147592724526e12*coen(15)*coep(12) + 4.625035091075813e13*coep(11)*coep(12)&
            - 5.2096797768149554e11*coep(12)**2 + 2.6926025970653355e13*coen(11)*coep(15)&
            - 1.4142147592724526e12*coen(12)*coep(15) - 1.7052810282960995e11*coen(15)*coep(15)&
            + 1.0035109532829475e13*coep(11)*coep(15) - 3.1781887430029114e11*coep(12)*coep(15)&
            - 2.4050611987653973e10*coep(15)**2) + jx**2*((-1.1353517487822737e10&
            - 7.358372653025253e15*v**2)*coen(5)**2 + (1.0245005364537274e11&
            + 3.953217509449632e16*v**2)*coen(8)**2 - 7.358372653025253e15*coen(11)**2&
            + 7.352189289144162e14*coen(11)*coen(12) - 1.6512249357348678e13*coen(12)**2&
            + 1.0121787802035417e14*coen(11)*coen(15) - 5.771383187064318e12*coen(12)*coen(15)&
            - 2.864785633171519e11*coen(15)**2 + (-9.658450699211243e9&
            - 5.238495719835488e15*v**2)*coen(5)*coep(5) - 1.9521004815133781e9*coep(5)**2&
            - 8.612397598629242e14*v**2*coep(5)**2 + (7.293517097120024e10&
            + 2.2515961933631856e16*v**2)*coen(8)*coep(8)&
            + 1.1990974602680496e10*coep(8)**2 + 2.8369796370813885e15*v**2*coep(8)**2&
            - 5.238495719835488e15*coen(11)*coep(11) + 2.3855174176393738e14*coen(12)*coep(11)&
            + 4.029060761803484e13*coen(15)*coep(11) - 8.612397598629242e14*coep(11)**2&
            + 1.584342897133044e14*coen(11)*coep(12) - 6.325268580984681e12*coen(12)*coep(12)&
            - 1.4142147592724526e12*coen(15)*coep(12) + 4.625035091075813e13*coep(11)*coep(12)&
            - 5.2096797768149554e11*coep(12)**2 + 2.6926025970653355e13*coen(11)*coep(15)&
            - 1.4142147592724526e12*coen(12)*coep(15) - 1.7052810282960995e11*coen(15)*coep(15)&
            + 1.0035109532829475e13*coep(11)*coep(15) - 3.1781887430029114e11*coep(12)*coep(15)&
            - 2.4050611987653973e10*coep(15)**2)) + Q**4*(585176.8966735813*coen(3)**2&
            + 5.238316625496312e9*coep(1)**2 + 4.306224335406756e7*coep(1)*coep(3)&
            + 69452.45928403157*coep(3)**2 + coen(3)*(1.423666689035687e8*coep(1)&
            + 409856.7324736557*coep(3)) + jx*((-349.82832103507656&
            - 6.685850698463616e8*v**2)*coen(5)**2 + (9308.65824580031&
            + 1.0155440251418804e10*v**2)*coen(8)**2 - 6.685850698463616e8*coen(11)**2&
            + 1.0088535434977463e8*coen(11)*coen(12) - 3.5677208309580516e6*coen(12)**2&
            + 4.527097927444784e6*coen(11)*coen(15) - 390117.9311157208*coen(12)*coen(15)&
            - 5328.531179070601*coen(15)**2 + (-389.3670892563561&
            - 6.583814134055753e8*v**2)*coen(5)*coep(5) - 106.37733881785307*coep(5)**2&
            - 1.570057233787434e8*v**2*coep(5)**2 + (9166.593525918566&
            + 8.639244983091478e9*v**2)*coen(8)*coep(8) + 2185.9785500493344*coep(8)**2&
            + 1.746105541832104e9*v**2*coep(8)**2 - 6.583814134055753e8*coen(11)*coep(11)&
            + 4.745555630118959e7*coen(12)*coep(11) + 2.4481792178284805e6*coen(15)*coep(11)&
            - 1.570057233787434e8*coep(11)**2 + 3.1801437370819878e7*coen(11)*coep(12)&
            - 2.1237074693627884e6*coen(12)*coep(12) - 136618.91082455186*coen(15)*coep(12)&
            + 1.4354081118022518e7*coep(11)*coep(12) - 299519.3371262805*coep(12)**2&
            + 1.6462994161211294e6*coen(11)*coep(15) - 136618.91082455186*coen(12)*coep(15)&
            - 4298.5795999092725*coen(15)*coep(15) + 868640.177377522*coep(11)*coep(15)&
            - 46301.63952268771*coep(12)*coep(15) - 851.5159593818366*coep(15)**2)&
            + jx**2*((-349.82832103507656 - 6.685850698463616e8*v**2)*coen(5)**2&
            + (9308.65824580031 + 1.0155440251418804e10*v**2)*coen(8)**2&
            - 6.685850698463616e8*coen(11)**2 + 1.0088535434977463e8*coen(11)*coen(12)&
            - 3.5677208309580516e6*coen(12)**2 + 4.527097927444784e6*coen(11)*coen(15)&
            - 390117.9311157208*coen(12)*coen(15) - 5328.531179070601*coen(15)**2&
            + (-389.3670892563561 - 6.583814134055753e8*v**2)*coen(5)*coep(5)&
            - 106.37733881785307*coep(5)**2 - 1.570057233787434e8*v**2*coep(5)**2&
            + (9166.593525918566 + 8.639244983091478e9*v**2)*coen(8)*coep(8)&
            + 2185.9785500493344*coep(8)**2 + 1.746105541832104e9*v**2*coep(8)**2&
            - 6.583814134055753e8*coen(11)*coep(11) + 4.745555630118959e7*coen(12)*coep(11)&
            + 2.4481792178284805e6*coen(15)*coep(11) - 1.570057233787434e8*coep(11)**2&
            + 3.1801437370819878e7*coen(11)*coep(12) - 2.1237074693627884e6*coen(12)*coep(12)&
            - 136618.91082455186*coen(15)*coep(12) + 1.4354081118022518e7*coep(11)*coep(12)&
            - 299519.3371262805*coep(12)**2 + 1.6462994161211294e6*coen(11)*coep(15)&
            - 136618.91082455186*coen(12)*coep(15) - 4298.5795999092725*coen(15)*coep(15)&
            + 868640.177377522*coep(11)*coep(15) - 46301.63952268771*coep(12)*coep(15)&
            - 851.5159593818366*coep(15)**2)) + Q**3*(-56.41286196004179*coen(3)**2&
            - 1.6622108236300894e6*coep(1)**2 + coen(3)*(-25918.736632576096*coep(1)&
            - 45.50882212088872*coep(3)) - 9196.24503874843*coep(1)*coep(3)&
            - 9.014951899325956*coep(3)**2 + jx*((0.023941480528419604&
            + 88669.43084806307*v**2)*coen(5)**2 + (-1.2345376315446517&
            - 2.3594242632273394e6*v**2)*coen(8)**2 + 88669.43084806307*coen(11)**2&
            - 15976.044296762353*coen(11)*coen(12) + 688.3594576872623*coen(12)**2&
            - 364.206142737764*coen(11)*coen(15) + 37.608574640027854*coen(12)*coen(15)&
            + 0.19856549537741602*coen(15)**2 + (0.029557107864229916&
            + 98691.14682646384*v**2)*coen(5)*coep(5) + 0.009042409237874445*coep(5)**2&
            + 26962.99161886561*v**2*coep(5)**2 + (-1.3740692084326105&
            - 2.3234157496277764e6*v**2)*coen(8)*coep(8) - 0.3754036480684117*coep(8)**2&
            - 554070.274543363*v**2*coep(8)**2 + 98691.14682646384*coen(11)*coep(11)&
            - 8639.578877525366*coen(12)*coep(11) - 221.4919987088143*coen(15)*coep(11)&
            + 26962.99161886561*coep(11)**2 - 5809.759987350296*coen(11)*coep(12)&
            + 482.12559261787413*coen(12)*coep(12) + 15.169607373629571*coen(15)*coep(12)&
            - 3065.4150129161435*coep(11)*coep(12) + 81.69881189699575*coep(12)**2&
            - 149.99594311415632*coen(11)*coep(15) + 15.169607373629571*coen(12)*coep(15)&
            + 0.18154194415117345*coen(15)*coep(15) - 90.11556315133458*coep(11)*coep(15)&
            + 6.009967932883969*coep(12)*coep(15) + 0.04149446688049337*coep(15)**2)&
            + jx**2*((0.023941480528419604 + 88669.43084806307*v**2)*coen(5)**2&
            + (-1.2345376315446517 - 2.3594242632273394e6*v**2)*coen(8)**2&
            + 88669.43084806307*coen(11)**2 - 15976.044296762353*coen(11)*coen(12)&
            + 688.3594576872623*coen(12)**2 - 364.206142737764*coen(11)*coen(15)&
            + 37.608574640027854*coen(12)*coen(15) + 0.19856549537741602*coen(15)**2&
            + (0.029557107864229916 + 98691.14682646384*v**2)*coen(5)*coep(5)&
            + 0.009042409237874445*coep(5)**2 + 26962.99161886561*v**2*coep(5)**2&
            + (-1.3740692084326105 - 2.3234157496277764e6*v**2)*coen(8)*coep(8)&
            - 0.3754036480684117*coep(8)**2 - 554070.274543363*v**2*coep(8)**2&
            + 98691.14682646384*coen(11)*coep(11) - 8639.578877525366*coen(12)*coep(11)&
            - 221.4919987088143*coen(15)*coep(11) + 26962.99161886561*coep(11)**2&
            - 5809.759987350296*coen(11)*coep(12) + 482.12559261787413*coen(12)*coep(12)&
            + 15.169607373629571*coen(15)*coep(12) - 3065.4150129161435*coep(11)*coep(12)&
            + 81.69881189699575*coep(12)**2 - 149.99594311415632*coen(11)*coep(15)&
            + 15.169607373629571*coen(12)*coep(15) + 0.18154194415117345*coen(15)*coep(15)&
            - 90.11556315133458*coep(11)*coep(15) + 6.009967932883969*coep(12)*coep(15)&
            + 0.04149446688049337*coep(15)**2)) + Q**5*(-3.0329325481661882e9*coen(3)**2&
            - 9.117897231882355e12*coep(1)**2 + coen(3)*(-4.2655441236225354e11*coep(1)&
            - 1.805371499564446e9*coep(3)) - 1.0624114409857852e11*coep(1)*coep(3)&
            - 2.5462248573174262e8*coep(3)**2 + jx*((2.637774825173374e6&
            + 2.8777256535151484e12*v**2)*coen(5)**2 + (-4.006635182550607e7&
            - 2.5967560088360816e13*v**2)*coen(8)**2 + 2.8777256535151484e12*coen(11)**2&
            - 3.571960069770746e11*coen(11)*coen(12) + 1.0183551905422487e10*coen(12)**2&
            - 2.8587669769978924e10*coen(11)*coen(15) + 2.0219550321107914e9*coen(12)*coen(15)&
            + 5.527344706257913e7*coen(15)**2 + (2.597518245572502e6&
            + 2.448084602868009e12*v**2)*coen(5)*coep(5) + 619436.1244587067*coep(5)**2&
            + 4.9479023922898895e11*v**2*coep(5)**2 + (-3.408449268862787e7&
            - 1.8486553860726594e13*v**2)*coen(8)*coep(8) - 6.888926253466653e6*coep(8)**2&
            - 3.039299077294118e12*v**2*coep(8)**2 + 2.448084602868009e12*coen(11)*coep(11)&
            - 1.421848041207512e11*coen(12)*coep(11) - 1.3447380752467773e10*coen(15)*coep(11)&
            + 4.9479023922898895e11*coep(11)**2 - 9.502144431979005e10*coen(11)*coep(12)&
            + 4.990737554472171e9*coen(12)*coep(12) + 6.017904998548152e8*coen(15)*coep(12)&
            - 3.5413714699526184e10*coep(11)*coep(12) + 5.607884449977503e8*coep(12)**2&
            - 9.011506136120308e9*coen(11)*coep(15) + 6.017904998548152e8*coen(12)*coep(15)&
            + 3.871341219108492e7*coen(15)*coep(15) - 4.067485647429203e9*coep(11)*coep(15)&
            + 1.697483238211617e8*coep(12)*coep(15) + 6.560198896135203e6*coep(15)**2)&
            + jx**2*((2.637774825173374e6 + 2.8777256535151484e12*v**2)*coen(5)**2&
            + (-4.006635182550607e7 - 2.5967560088360816e13*v**2)*coen(8)**2&
            + 2.8777256535151484e12*coen(11)**2 - 3.571960069770746e11*coen(11)*coen(12)&
            + 1.0183551905422487e10*coen(12)**2 - 2.8587669769978924e10*coen(11)*coen(15)&
            + 2.0219550321107914e9*coen(12)*coen(15) + 5.527344706257913e7*coen(15)**2&
            + (2.597518245572502e6 + 2.448084602868009e12*v**2)*coen(5)*coep(5)&
            + 619436.1244587067*coep(5)**2 + 4.9479023922898895e11*v**2*coep(5)**2&
            + (-3.408449268862787e7 - 1.8486553860726594e13*v**2)*coen(8)*coep(8)&
            - 6.888926253466653e6*coep(8)**2 - 3.039299077294118e12*v**2*coep(8)**2&
            + 2.448084602868009e12*coen(11)*coep(11) - 1.421848041207512e11*coen(12)*coep(11)&
            - 1.3447380752467773e10*coen(15)*coep(11) + 4.9479023922898895e11*coep(11)**2&
            - 9.502144431979005e10*coen(11)*coep(12) + 4.990737554472171e9*coen(12)*coep(12)&
            + 6.017904998548152e8*coen(15)*coep(12) - 3.5413714699526184e10*coep(11)*coep(12)&
            + 5.607884449977503e8*coep(12)**2 - 9.011506136120308e9*coen(11)*coep(15)&
            + 6.017904998548152e8*coen(12)*coep(15) + 3.871341219108492e7*coen(15)*coep(15)&
            - 4.067485647429203e9*coep(11)*coep(15) + 1.697483238211617e8*coep(12)*coep(15)&
            + 6.560198896135203e6*coep(15)**2)) + Q**7*(-1.4037123668639382e16*coen(3)**2&
            - 3.879666324050281e18*coep(1)**2 + coen(3)*(-6.575942794414276e17*coep(1)&
            - 5.37713399229441e15*coep(3)) - 8.925993509214534e16*coep(1)*coep(3)&
            - 4.4287678630904125e14*coep(3)**2 + jx*((2.9031055304382586e13&
            + 1.120214895585556e19*v**2)*coen(5)**2 + (-1.5596665398551403e14&
            - 3.455246242746479e19*v**2)*coen(8)**2 + 1.120214895585556e19*coen(11)**2&
            - 8.744732656228504e17*coen(11)*coen(12) + 1.4956576140458314e16*coen(12)**2&
            - 2.083374349419619e17*coen(11)*coen(15) + 9.358082445759586e15*coen(12)*coen(15)&
            + 8.177123316421785e14*coen(15)**2 + (2.0667485342943477e13&
            + 6.380300574456193e18*v**2)*coen(5)*coep(5) + 3.397857145578833e12*coep(5)**2&
            + 8.039089274331182e17*v**2*coep(5)**2 + (-8.883243170049277e13&
            - 1.4698345823327422e19*v**2)*coen(8)*coep(8) - 1.1192761854437486e13*coep(8)**2&
            - 1.29322210801676e18*v**2*coep(8)**2&
            + 6.380300574456193e18*coen(11)*coep(11) - 2.1919809314714256e17*coen(12)*coep(11)&
            - 6.759790318976034e16*coen(15)*coep(11) + 8.039089274331182e17*coep(11)**2&
            - 1.4210993674452646e17*coen(11)*coep(12) + 4.0617222964939115e15*coen(12)*coep(12)&
            + 1.7923779974314695e15*coen(15)*coep(12) - 2.9753311697381784e16*coep(11)*coep(12)&
            + 2.012146872206595e14*coep(12)**2 - 4.489519002789957e16*coen(11)*coep(15)&
            + 1.7923779974314695e15*coen(12)*coep(15) + 4.0074304920158544e14*coen(15)*coep(15)&
            - 1.3105864246640784e16*coep(11)*coep(15) + 2.952511908726941e14*coep(12)*coep(15)&
            + 4.502983155346106e13*coep(15)**2) + jx**2*((2.9031055304382586e13&
            + 1.120214895585556e19*v**2)*coen(5)**2 + (-1.5596665398551403e14&
            - 3.455246242746479e19*v**2)*coen(8)**2 + 1.120214895585556e19*coen(11)**2&
            - 8.744732656228504e17*coen(11)*coen(12) + 1.4956576140458314e16*coen(12)**2&
            - 2.083374349419619e17*coen(11)*coen(15) + 9.358082445759586e15*coen(12)*coen(15)&
            + 8.177123316421785e14*coen(15)**2 + (2.0667485342943477e13&
            + 6.380300574456193e18*v**2)*coen(5)*coep(5) + 3.397857145578833e12*coep(5)**2&
            + 8.039089274331182e17*v**2*coep(5)**2 + (-8.883243170049277e13&
            - 1.4698345823327422e19*v**2)*coen(8)*coep(8) - 1.1192761854437486e13*coep(8)**2&
            - 1.29322210801676e18*v**2*coep(8)**2 + 6.380300574456193e18*coen(11)*coep(11)&
            - 2.1919809314714256e17*coen(12)*coep(11) - 6.759790318976034e16*coen(15)*coep(11)&
            + 8.039089274331182e17*coep(11)**2 - 1.4210993674452646e17*coen(11)*coep(12)&
            + 4.0617222964939115e15*coen(12)*coep(12) + 1.7923779974314695e15*coen(15)*coep(12)&
            - 2.9753311697381784e16*coep(11)*coep(12) + 2.012146872206595e14*coep(12)**2&
            - 4.489519002789957e16*coen(11)*coep(15) + 1.7923779974314695e15*coen(12)*coep(15)&
            + 4.0074304920158544e14*coen(15)*coep(15) - 1.3105864246640784e16*coep(11)*coep(15)&
            + 2.952511908726941e14*coep(12)*coep(15) + 4.502983155346106e13*coep(15)**2))&
            + Q**9*(-5.918100458834253e21*coen(3)**2 - 1.6900331514145419e22*coep(1)**2&
            + coen(3)*(-5.546698380525158e22*coep(1) - 9.040104533475285e20*coep(3))&
            - 9.669982965650167e20*coep(1)*coep(3) - 1.0888213697610967e19*coep(3)**2&
            + jx*((3.862875234411949e19 + 4.474170945923119e24*v**2)*coen(5)**2&
            + (-6.229353622637444e19 - 3.087250655665309e24*v**2)*coen(8)**2&
            + 4.474170945923119e24*coen(11)**2 - 1.7625570227463566e23*coen(11)*coen(12)&
            + 1.287968096806026e21*coen(12)**2 - 1.6136765192473357e23*coen(11)*coen(15)&
            + 3.945400305889502e21*coen(12)*coen(15) + 1.2009735760943763e21*coen(15)**2&
            + (1.6432367501143034e19 + 1.2662819094253482e24*v**2)*coen(5)*coep(5)&
            + 1.445788607436884e18*coep(5)**2 + 6.6639209121458824e22*v**2*coep(5)**2&
            + (-1.7630345141252897e19 - 4.680585813864844e23*v**2)*coen(8)*coep(8)&
            - 9.278125573827524e17*coep(8)**2 - 5.633443838048473e21*v**2*coep(8)**2&
            + 1.2662819094253482e24*coen(11)*coep(11) - 1.8488994601750527e22*coen(12)*coep(11)&
            - 2.865172715713988e22*coen(15)*coep(11) + 6.6639209121458824e22*coep(11)**2&
            - 8.287109191771661e21*coen(11)*coep(12) + 3.395567187370965e19*coen(12)*coep(12)&
            + 3.013368177825094e20*coen(15)*coep(12) - 3.223327655216723e20*coep(11)*coep(12)&
            + 2.2379973060945334e17*coep(12)**2 - 1.7096714485613292e22*coen(11)*coep(15)&
            + 3.013368177825094e20*coen(12)*coep(15) + 3.261455767491636e20*coen(15)*coep(15)&
            - 2.1397067081225724e21*coep(11)*coep(15) + 7.258809131740643e18*coep(12)*coep(15)&
            + 1.6157008141751206e19*coep(15)**2) + jx**2*((3.862875234411949e19&
            + 4.474170945923119e24*v**2)*coen(5)**2 + (-6.229353622637444e19&
            - 3.087250655665309e24*v**2)*coen(8)**2 + 4.474170945923119e24*coen(11)**2&
            - 1.7625570227463566e23*coen(11)*coen(12) + 1.287968096806026e21*coen(12)**2&
            - 1.6136765192473357e23*coen(11)*coen(15) + 3.945400305889502e21*coen(12)*coen(15)&
            + 1.2009735760943763e21*coen(15)**2 + (1.6432367501143034e19&
            + 1.2662819094253482e24*v**2)*coen(5)*coep(5) + 1.445788607436884e18*coep(5)**2&
            + 6.6639209121458824e22*v**2*coep(5)**2 + (-1.7630345141252897e19&
            - 4.680585813864844e23*v**2)*coen(8)*coep(8) - 9.278125573827524e17*coep(8)**2&
            - 5.633443838048473e21*v**2*coep(8)**2 + 1.2662819094253482e24*coen(11)*coep(11)&
            - 1.8488994601750527e22*coen(12)*coep(11) - 2.865172715713988e22*coen(15)*coep(11)&
            + 6.6639209121458824e22*coep(11)**2 - 8.287109191771661e21*coen(11)*coep(12)&
            + 3.395567187370965e19*coen(12)*coep(12) + 3.013368177825094e20*coen(15)*coep(12)&
            - 3.223327655216723e20*coep(11)*coep(12) + 2.2379973060945334e17*coep(12)**2&
            - 1.7096714485613292e22*coen(11)*coep(15) + 3.013368177825094e20*coen(12)*coep(15)&
            + 3.261455767491636e20*coen(15)*coep(15) - 2.1397067081225724e21*coep(11)*coep(15)&
            + 7.258809131740643e18*coep(12)*coep(15)&
            + 1.6157008141751206e19*coep(15)**2))))/(EXP(16944.76814318306*Q)*mx**2)
        elseif (A==136) then 
            ans=(jx*mx*Q*(-6.052130632379088e-7*coen(8)**2 - 6.052130632379088e-7*jx*coen(8)**2&
            + Q**11*((-8.155134474860593e28 - 8.155134474860593e28*jx)*coen(5)**2&
            + (-2.0340486035063333e27 - 2.0340486035063333e27*jx)*coen(5)*coep(5)&
            + (-1.2683278657697395e25 - 1.2683278657697395e25*jx)*coep(5)**2)&
            - 7.971104152873692e-7*coen(8)*coep(8) - 7.971104152873692e-7*jx*coen(8)*coep(8)&
            - 2.6246335908558907e-7*coep(8)**2 - 2.6246335908558907e-7*jx*coep(8)**2&
            + Q**10*((1.2009953567008999e27 + 1.2009953567008999e27*jx)*coen(5)**2&
            + (-2.8355418333495068e26 - 2.8355418333495068e26*jx)*coen(8)**2&
            + (1.6885986135073446e26 + 1.6885986135073446e26*jx)*coen(5)*coep(5)&
            + 1.9190612281277475e24*coep(5)**2 + 1.9190612281277475e24*jx*coep(5)**2&
            + (-7.072390926339624e24 - 7.072390926339624e24*jx)*coen(8)*coep(8)&
            - 4.409978441041551e22*coep(8)**2 - 4.409978441041551e22*jx*coep(8)**2)&
            + Q**8*((1.291667188065136e22 + 1.291667188065136e22*jx)*coen(5)**2&
            + (-2.1098139673629047e22 - 2.1098139673629047e22*jx)*coen(8)**2&
            + (5.146535684776478e21 + 5.146535684776478e21*jx)*coen(5)*coep(5)&
            + 4.215817502424771e20*coep(5)**2 + 4.215817502424771e20*jx*coep(5)**2&
            + (-5.549920788542419e21 - 5.549920788542419e21*jx)*coen(8)*coep(8)&
            - 2.7159061737634156e20*coep(8)**2 - 2.7159061737634156e20*jx*coep(8)**2)&
            + Q**6*((9.084113342047626e15 + 9.084113342047626e15*jx)*coen(5)**2&
            + (-4.973784868042673e16 - 4.973784868042673e16*jx)*coen(8)**2&
            + (6.14486073293116e15 + 6.14486073293116e15*jx)*coen(5)*coep(5)&
            + 9.543424554838854e14*coep(5)**2 + 9.543424554838854e14*jx*coep(5)**2&
            + (-2.6729869911604064e16 - 2.6729869911604064e16*jx)*coen(8)*coep(8)&
            - 3.156437902169201e15*coep(8)**2 - 3.156437902169201e15*jx*coep(8)**2)&
            + Q**4*((7.724207549458926e8 + 7.724207549458926e8*jx)*coen(5)**2&
            + (-1.194413997236935e10 - 1.194413997236935e10*jx)*coen(8)**2&
            + (7.319589132267709e8 + 7.319589132267709e8*jx)*coen(5)*coep(5)&
            + 1.6745672322795743e8*coep(5)**2 + 1.6745672322795743e8*jx*coep(5)**2&
            + (-9.71849929983629e9 - 9.71849929983629e9*jx)*coen(8)*coep(8)&
            - 1.8703564300458004e9*coep(8)**2 - 1.8703564300458004e9*jx*coep(8)**2)&
            + Q**2*((6.59973053314041 + 6.59973053314041*jx)*coen(5)**2&
            + (-345.2350114396449 - 345.2350114396449*jx)*coen(8)**2 + (7.918571156130923&
            + 7.918571156130923*jx)*coen(5)*coep(5) + 2.3525604668344706*coep(5)**2&
            + 2.3525604668344706*jx*coep(5)**2 + (-371.7659583398842&
            - 371.7659583398842*jx)*coen(8)*coep(8) - 98.09313404426567*coep(8)**2&
            - 98.09313404426567*jx*coep(8)**2) + Q*((-0.0001740617563316724&
            - 0.0001740617563316724*jx)*coen(5)**2 + (0.02294727582143711&
            + 0.02294727582143711*jx)*coen(8)**2 + (-0.00022925222091686273&
            - 0.00022925222091686273*jx)*coen(5)*coep(5) - 0.00007548553729282174*coep(5)**2&
            - 0.00007548553729282174*jx*coep(5)**2 + (0.027532887216979735&
            + 0.027532887216979735*jx)*coen(8)*coep(8) + 0.008179857290835684*coep(8)**2&
            + 0.008179857290835684*jx*coep(8)**2) + Q**3*((-99291.00359611274&
            - 99291.00359611274*jx)*coen(5)**2 + (2.6857084580863724e6&
            + 2.6857084580863724e6*jx)*coen(8)**2 + (-106921.41261255303&
            - 106921.41261255303*jx)*coen(5)*coep(5) - 28211.987204101653*coep(5)**2&
            - 28211.987204101653*jx*coep(5)**2 + (2.5450225562136173e6&
            + 2.5450225562136173e6*jx)*coen(8)*coep(8) + 582247.3503683337*coep(8)**2&
            + 582247.3503683337*jx*coep(8)**2) + Q**5*((-3.435183586981965e12&
            - 3.435183586981965e12*jx)*coen(5)**2 + (3.1585479650480883e13&
            + 3.1585479650480883e13*jx)*coen(8)**2 + (-2.7950802118966475e12&
            - 2.7950802118966475e12*jx)*coen(5)*coep(5) - 5.3792217147175555e11*coep(5)**2&
            - 5.3792217147175555e11*jx*coep(5)**2 + (2.136569264681667e13&
            + 2.136569264681667e13*jx)*coen(8)*coep(8) + 3.3182505625234395e12*coep(8)**2&
            + 3.3182505625234395e12*jx*coep(8)**2) + Q**7*((-1.430480903891328e19&
            - 1.430480903891328e19*jx)*coen(5)**2 + (4.491129309779073e19&
            + 4.491129309779073e19*jx)*coen(8)**2 + (-7.687620089426219e18&
            - 7.687620089426219e18*jx)*coen(5)*coep(5) - 9.078044714766112e17*coep(5)**2&
            - 9.078044714766112e17*jx*coep(5)**2 + (1.7894514524555661e19&
            + 1.7894514524555661e19*jx)*coen(8)*coep(8) + 1.4658405605380063e18*coep(8)**2&
            + 1.4658405605380063e18*jx*coep(8)**2) + Q**9*((-6.067911401772239e24&
            - 6.067911401772239e24*jx)*coen(5)**2 + (4.1758631768510826e24&
            + 4.1758631768510826e24*jx)*coen(8)**2 + (-1.5961799548527066e24&
            - 1.5961799548527066e24*jx)*coen(5)*coep(5) - 7.81105741684e22*coep(5)**2&
            - 7.81105741684e22*jx*coep(5)**2 + (5.8712606433358666e23&
            + 5.8712606433358666e23*jx)*coen(8)*coep(8) + 6.672579599870215e21*coep(8)**2&
            + 6.672579599870215e21*jx*coep(8)**2)) + jx*Q*(-0.00003861410404638442*coen(8)**2&
            - 0.00003861410404638442*jx*coen(8)**2 + Q**11*((-5.203179347117551e30&
            - 5.203179347117551e30*jx)*coen(5)**2 + (-1.297773778890184e29&
            - 1.297773778890184e29*jx)*coen(5)*coep(5)&
            + (-8.092248358246233e26 - 8.092248358246233e26*jx)*coep(5)**2)&
            - 0.000050857634082930725*coen(8)*coep(8) - 0.000050857634082930725*jx*coen(8)*coep(8)&
            - 0.000016745817418204864*coep(8)**2 - 0.000016745817418204864*jx*coep(8)**2&
            + Q**10*((7.662650144192773e28 + 7.662650144192773e28*jx)*coen(5)**2&
            + (-1.809146464801146e28 - 1.809146464801146e28*jx)*coen(8)**2&
            + (1.077368062839079e28 + 1.077368062839079e28*jx)*coen(5)*coep(5)&
            + 1.2244089633137568e26*coep(5)**2 + 1.2244089633137568e26*jx*coep(5)**2& 
            + (-4.512361937882202e26 - 4.512361937882202e26*jx)*coen(8)*coep(8)&
            - 2.8136763184463987e24*coep(8)**2 - 2.8136763184463987e24*jx*coep(8)**2)&
            + Q**8*((8.241159059985708e23 + 8.241159059985708e23*jx)*coen(5)**2&
            + (-1.3461139721341588e24 - 1.3461139721341588e24*jx)*coen(8)**2&
            + (3.283618224418084e23 + 3.283618224418084e23*jx)*coen(5)*coep(5)&
            + 2.6897967933518642e22*coep(5)**2 + 2.6897967933518642e22*jx*coep(5)**2&
            + (-3.540987989112945e23 - 3.540987989112945e23*jx)*coen(8)*coep(8)&
            - 1.7328159278791563e22*coep(8)**2 - 1.7328159278791563e22*jx*coep(8)**2)&
            + Q**6*((5.795891051695402e17 + 5.795891051695402e17*jx)*coen(5)**2&
            + (-3.1733988914815324e18 - 3.1733988914815324e18*jx)*coen(8)**2&
            + (3.920574523333974e17 + 3.920574523333974e17*jx)*coen(5)*coep(5)&
            + 6.088943069864077e16*coep(5)**2 + 6.088943069864077e16*jx*coep(5)**2&
            + (-1.7054324181156388e18 - 1.7054324181156388e18*jx)*coen(8)*coep(8)&
            - 2.0138861662739885e17*coep(8)**2 - 2.0138861662739885e17*jx*coep(8)**2)&
            + Q**4*((4.92823721277523e10 + 4.92823721277523e10*jx)*coen(5)**2&
            + (-7.620659428105356e11 - 7.620659428105356e11*jx)*coen(8)**2&
            + (4.6700805633315384e10 + 4.6700805633315384e10*jx)*coen(5)*coep(5)&
            + 1.0684156913924862e10*coep(5)**2 + 1.0684156913924862e10*jx*coep(5)**2&
            + (-6.200645127038077e11 - 6.200645127038077e11*jx)*coen(8)*coep(8)&
            - 1.1933340864657147e11*coep(8)**2 - 1.1933340864657147e11*jx*coep(8)**2)&
            + Q**2*((421.07928094176884 + 421.07928094176884*jx)*coen(5)**2&
            + (-22026.855436437905 - 22026.855436437905*jx)*coen(8)**2&
            + (505.2246045147586 + 505.2246045147586*jx)*coen(5)*coep(5)&
            + 150.09922977496444*coep(5)**2 + 150.09922977496444*jx*coep(5)**2&
            + (-23719.59606991663 - 23719.59606991663*jx)*coen(8)*coep(8)&
            - 6258.586792486723*coep(8)**2 - 6258.586792486723*jx*coep(8)**2)&
            + Q*((-0.011105574512104503 - 0.011105574512104503*jx)*coen(5)**2&
            + (1.464093473808138 + 1.464093473808138*jx)*coen(8)**2 + (-0.014626863908039248&
            - 0.014626863908039248*jx)*coen(5)*coep(5) - 0.00481616569118311*coep(5)**2&
            - 0.00481616569118311*jx*coep(5)**2 + (1.756666926529801&
            + 1.756666926529801*jx)*coen(8)*coep(8) + 0.5218953120791147*coep(8)**2&
            + 0.5218953120791147*jx*coep(8)**2) + Q**3*((-6.335013859776969e6&
            - 6.335013859776969e6*jx)*coen(5)**2 + (1.7135490315422195e8&
            + 1.7135490315422195e8*jx)*coen(8)**2 + (-6.821853000526764e6&
            - 6.821853000526764e6*jx)*coen(5)*coep(5) - 1.799995200741747e6*coep(5)**2&
            - 1.799995200741747e6*jx*coep(5)**2 + (1.623787914627291e8&
            + 1.623787914627291e8*jx)*coen(8)*coep(8) + 3.7148834242886215e7*coep(8)**2&
            + 3.7148834242886215e7*jx*coep(8)**2) + Q**5*((-2.1917328706768247e14&
            - 2.1917328706768247e14*jx)*coen(5)**2 + (2.0152324390578882e15&
            + 2.0152324390578882e15*jx)*coen(8)**2 + (-1.78333094039215e14&
            - 1.78333094039215e14*jx)*coen(5)*coep(5) - 3.4320777193637996e13*coep(5)**2&
            - 3.4320777193637996e13*jx*coep(5)**2 + (1.3631845196357505e15&
            + 1.3631845196357505e15*jx)*coen(8)*coep(8) + 2.117126682423995e14*coep(8)**2&
            + 2.117126682423995e14*jx*coep(8)**2) + Q**7*((-9.126825214860284e20&
            - 9.126825214860284e20*jx)*coen(5)**2 + (2.865452598226621e21&
            + 2.865452598226621e21*jx)*coen(8)**2 + (-4.9048935000513597e20&
            - 4.9048935000513597e20*jx)*coen(5)*coep(5) - 5.7920191160168595e19*coep(5)**2&
            - 5.7920191160168595e19*jx*coep(5)**2 + (1.1417146913749046e21&
            + 1.1417146913749046e21*jx)*coen(8)*coep(8) + 9.352428650036391e19*coep(8)**2&
            + 9.352428650036391e19*jx*coep(8)**2) + Q**9*((-3.871478929399277e26&
            - 3.871478929399277e26*jx)*coen(5)**2 + (2.664304936375885e26&
            + 2.664304936375885e26*jx)*coen(8)**2 + (-1.0184026518476998e26&
            - 1.0184026518476998e26*jx)*coen(5)*coep(5) - 4.983649595937045e24*coep(5)**2&
            - 4.983649595937045e24*jx*coep(5)**2 + (3.7460108371139407e25&
            + 3.7460108371139407e25*jx)*coen(8)*coep(8) + 4.2572723323040095e23*coep(8)**2&
            + 4.2572723323040095e23*jx*coep(8)**2)) + mx**2*((1.8300991871101505e-6&
            - 0.06939009313930813*Q + 1043.9535300468094*Q**2 - 8.121293416342865e6*Q**3&
            + 3.6117794181799194e10*Q**4 - 9.551109211617652e13*Q**5 + 1.5040190301192278e17*Q**6&
            - 1.3580691822909789e20*Q**7 + 6.379850438070719e22*Q**8 - 1.2627360957068796e25*Q**9&
            + 8.574373422256815e26*Q**10)*coen(1)**2 + 6.100330623700501e-7*jx*v**2*coen(8)**2&
            + 6.100330623700501e-7*jx**2*v**2*coen(8)**2 + 7.936609589008777e-7*coep(1)**2&
            + coen(1)*(2.4103761330759636e-6*coep(1) + Q**9*(-7.07474861720527e23*coen(3)&
            - 1.7754060484409838e24*coep(1) - 3.188496383264471e22*coep(3))&
            + Q**7*(-3.2681937649210255e18*coen(3) - 5.41110866145442e19*coep(1)&
            - 5.2754224522082784e17*coep(3)) + Q**5*(-1.2633725330436362e12*coen(3)&
            - 6.4607555785050555e13*coep(1) - 3.3214397444494604e11*coep(3))&
            + Q**3*(-53742.012265062345*coen(3) - 7.695874385765897e6*coep(1)&
            - 19179.16200812759*coep(3)) + Q*(-0.00012837658228611275*coen(3)&
            - 0.0832564886240445*coep(1) - 0.0000578756262489392*coep(3))&
            + Q**2*(4.207709427806255*coen(3) + 1124.1802589538552*coep(1)&
            + 1.7025443117906847*coep(3)) + Q**4*(3.481010912953381e8*coen(3)&
            + 2.9387696249327938e10*coep(1) + 1.0795137480810429e8*coep(3))&
            + Q**6*(2.668891196100939e15*coen(3) + 8.082825069087598e16*coep(1)&
            + 5.707563639508688e14*coep(3)) + Q**8*(2.2015746098073035e21*coen(3)&
            + 1.6782363337131957e22*coep(1) + 2.3005484290102863e20*coep(3))&
            + Q**10*(7.138437691821156e25*coen(3) + 2.1386149228129596e25*coep(1)&
            + 8.902323599338277e23*coep(3))) + jx*Q**12*((-3.1954605956405665e26&
            - 3.1954605956405665e26*jx)*coen(5)**2 + (-7.970098080121871e24&
            - 7.970098080121871e24*jx)*coen(5)*coep(5) + (-4.969742350556863e22&
            - 4.969742350556863e22*jx)*coep(5)**2) + 8.03458711025321e-7*jx*v**2*coen(8)*coep(8)&
            + 8.03458711025321e-7*jx**2*v**2*coen(8)*coep(8)&
            + 2.645536529669592e-7*jx*v**2*coep(8)**2 + 2.645536529669592e-7*jx**2*v**2*coep(8)**2&
            + Q*(coep(1)*(-0.0000845407320454933*coen(3) - 0.024735008359776184*coep(1)&
            - 0.00003811324248975566*coep(3)) + jx*(-2.3714317667051334e-9*coen(8)**2&
            + 0.00017544800782791259*coen(11)**2 - 0.00004279219409537092*coen(11)*coen(12)&
            + 2.6092799487526725e-6*coen(12)**2 - 3.1233512215861015e-9*coen(8)*coep(8)&
            - 1.0284212042644426e-9*coep(8)**2 + v**2*(0.00017544800782791259*coen(5)**2&
            - 0.02313003104643604*coen(8)**2 + 0.00023107801677782613*coen(5)*coep(5)&
            + 0.00007608671437630052*coep(5)**2 - 0.027752162874681498*coen(8)*coep(8)&
            - 0.00824500278659206*coep(8)**2) + 0.00023107801677782613*coen(11)*coep(11)&
            - 0.000028180244015164433*coen(12)*coep(11) + 0.00007608671437630052*coep(11)**2&
            - 0.00001929187541631307*coen(11)*coep(12) + 2.352667572288173e-6*coen(12)*coep(12)&
            - 0.000012704414163251887*coep(11)*coep(12) + 5.303230023614629e-7*coep(12)**2)&
            + jx**2*(-2.3714317667051334e-9*coen(8)**2 + 0.00017544800782791259*coen(11)**2&
            - 0.00004279219409537092*coen(11)*coen(12) + 2.6092799487526725e-6*coen(12)**2&
            - 3.1233512215861015e-9*coen(8)*coep(8) - 1.0284212042644426e-9*coep(8)**2&
            + v**2*(0.00017544800782791259*coen(5)**2 - 0.02313003104643604*coen(8)**2&
            + 0.00023107801677782613*coen(5)*coep(5) + 0.00007608671437630052*coep(5)**2&
            - 0.027752162874681498*coen(8)*coep(8) - 0.00824500278659206*coep(8)**2)&
            + 0.00023107801677782613*coen(11)*coep(11) - 0.000028180244015164433*coen(12)*coep(11)&
            + 0.00007608671437630052*coep(11)**2 - 0.00001929187541631307*coen(11)*coep(12)&
            + 2.352667572288173e-6*coen(12)*coep(12) - 0.000012704414163251887*coep(11)*coep(12)&
            + 5.303230023614629e-7*coep(12)**2)) + jx*Q**11*((4.705910552076599e24&
            + 8.22008307474875e28*v**2 + jx*(4.705910552076599e24&
            + 8.22008307474875e28*v**2))*coen(5)**2 + (-1.1110622668075191e24&
            - 1.1110622668075191e24*jx)*coen(8)**2 + 8.22008307474875e28*coen(11)**2&
            + 8.22008307474875e28*jx*coen(11)**2 - 6.84347974609708e27*coen(11)*coen(15)&
            - 6.84347974609708e27*jx*coen(11)*coen(15) + 1.4243534587596734e26*coen(15)**2&
            + 1.4243534587596734e26*jx*coen(15)**2 + (6.61650687422693e23&
            + 2.0502480431733847e27*v**2 + jx*(6.61650687422693e23&
            + 2.0502480431733847e27*v**2))*coen(5)*coep(5) + 7.519538217312641e21*coep(5)**2&
            + 7.519538217312641e21*jx*coep(5)**2 + 1.278429001359203e25*v**2*coep(5)**2&
            + 1.278429001359203e25*jx*v**2*coep(5)**2 + (-2.771204643130116e22&
            - 2.771204643130116e22*jx)*coen(8)*coep(8) - 1.7279803759721064e20*coep(8)**2&
            - 1.7279803759721064e20*jx*coep(8)**2 + 2.0502480431733847e27*coen(11)*coep(11)&
            + 2.0502480431733847e27*jx*coen(11)*coep(11) - 8.534482456164889e25*coen(15)*coep(11)&
            - 8.534482456164889e25*jx*coen(15)*coep(11) + 1.278429001359203e25*coep(11)**2&
            + 1.278429001359203e25*jx*coep(11)**2 - 8.534482456164849e25*coen(11)*coep(15)&
            - 8.534482456164849e25*jx*coen(11)*coep(15) + 3.55261359897936e24*coen(15)*coep(15)&
            + 3.55261359897936e24*jx*coen(15)*coep(15) - 1.0643326713449541e24*coep(11)*coep(15)&
            - 1.0643326713449541e24*jx*coep(11)*coep(15) + 2.215226723752175e22*coep(15)**2&
            + 2.215226723752175e22*jx*coep(15)**2) + Q**2*(0.002251318807682627*coen(3)**2&
            + 296.6230833073218*coep(1)**2 + coen(3)*(2.4855940500586247*coep(1)&
            + 0.0020299104955178743*coep(3)) + 0.9925502197438392*coep(1)*coep(3)&
            + 0.000457569204076327*coep(3)**2 + jx*((-6.82033491023895e-7&
            - 6.652291684533797*v**2)*coen(5)**2&
            + (0.00008991527471526002 + 347.9845100156031*v**2)*coen(8)**2&
            - 6.652291684533797*coen(11)**2 + 1.4025698092687515*coen(11)*coen(12)&
            - 0.07211155376565571*coen(12)**2 + 0.012307210326354259*coen(11)*coen(15)&
            - 0.0015008792051217513*coen(12)*coen(15) + (-8.982886065964516e-7&
            - 7.981635733580976*v**2)*coen(5)*coep(5) - 2.957781514253837e-7*coep(5)**2&
            - 2.3712965782921227*v**2*coep(5)**2 + (0.00010788326845778786&
            + 374.7267529846184*v**2)*coen(8)*coep(8) + 0.00003205147840468375*coep(8)**2&
            + 98.87436110244059*v**2*coep(8)**2 - 7.981635733580976*coen(11)*coep(11)&
            + 0.8285313500195416*coen(12)*coep(11) + 0.00810475362328131*coen(15)*coep(11)&
            - 2.3712965782921227*coep(11)**2 + 0.5675147705968949*coen(11)*coep(12)&
            - 0.05711704177033269*coen(12)*coep(12) - 0.0006766368318392913*coen(15)*coep(12)&
            + 0.3308500732479464*coep(11)*coep(12) - 0.011093593201806548*coep(12)**2&
            + 0.005548422401740604*coen(11)*coep(15) - 0.0006766368318392913*coen(12)*coep(15)&
            + 0.003653841558854979*coep(11)*coep(15) - 0.00030504613605088465*coep(12)*coep(15))&
            + jx**2*((-6.82033491023895e-7 - 6.652291684533797*v**2)*coen(5)**2&
            + (0.00008991527471526002 + 347.9845100156031*v**2)*coen(8)**2&
            - 6.652291684533797*coen(11)**2 + 1.4025698092687515*coen(11)*coen(12)&
            - 0.07211155376565571*coen(12)**2 + 0.012307210326354259*coen(11)*coen(15)&
            - 0.0015008792051217513*coen(12)*coen(15) + (-8.982886065964516e-7&
            - 7.981635733580976*v**2)*coen(5)*coep(5) - 2.957781514253837e-7*coep(5)**2&
            - 2.3712965782921227*v**2*coep(5)**2 + (0.00010788326845778786&
            + 374.7267529846184*v**2)*coen(8)*coep(8) + 0.00003205147840468375*coep(8)**2&
            + 98.87436110244059*v**2*coep(8)**2 - 7.981635733580976*coen(11)*coep(11)&
            + 0.8285313500195416*coen(12)*coep(11) + 0.00810475362328131*coen(15)*coep(11)&
            - 2.3712965782921227*coep(11)**2 + 0.5675147705968949*coen(11)*coep(12)&
            - 0.05711704177033269*coen(12)*coep(12) - 0.0006766368318392913*coen(15)*coep(12)&
            + 0.3308500732479464*coep(11)*coep(12) - 0.011093593201806548*coep(12)**2&
            + 0.005548422401740604*coen(11)*coep(15) - 0.0006766368318392913*coen(12)*coep(15)&
            + 0.003653841558854979*coep(11)*coep(15) - 0.00030504613605088465*coep(12)*coep(15)))&
            + Q**10*(1.4857439188425484e24*coen(3)**2 + 1.333530032703681e23*coep(1)**2&
            + 1.110206026707183e22*coep(1)*coep(3) + 2.3107042801990484e20*coep(3)**2&
            + coen(3)*(8.902323599338317e23*coep(1) + 3.705733305325241e22*coep(3))&
            + jx*((-2.3776152118610843e22 - 1.2105602470323046e27*v**2)*coen(5)**2&
            + (1.6362460086401178e22 + 2.858124474085605e26*v**2)*coen(8)**2&
            - 1.2105602470323046e27*coen(11)**2 + 2.379479230607052e25*coen(11)*coen(12)&
            + 6.782422283526404e25*coen(11)*coen(15) - 9.904959458950322e23*coen(12)*coen(15)& 
            - 7.256651739621578e23*coen(15)**2 + (-6.254378962120489e21&
            - 1.7020468424799585e26*v**2)*coen(5)*coep(5) - 3.060639436754951e20*coep(5)**2&
            - 1.9343448927013684e24*v**2*coep(5)**2 + (2.3005607191824703e21&
            + 7.128716409376532e24*v**2)*coen(8)*coep(8) + 2.6145448917351977e19*coep(8)**2&
            + 4.445100109012269e22*v**2*coep(8)**2 - 1.7020468424799585e26*coen(11)*coep(11)&
            + 2.9674411997794392e23*coen(12)*coep(11) + 6.674012146250384e24*coen(15)*coep(11)&
            - 1.9343448927013684e24*coep(11)**2 + 2.9674411997794258e23*coen(11)*coep(12)&
            - 1.2352444351084134e22*coen(15)*coep(12) + 3.70068675569061e21*coep(11)*coep(12)&
            + 3.0567487399063343e24*coen(11)*coep(15) - 1.2352444351084134e22*coen(12)*coep(15)&
            - 1.1013233523758746e23*coen(15)*coep(15) + 1.1080365143780885e23*coep(11)*coep(15)&
            - 1.5404695201326986e20*coep(12)*coep(15) - 1.2605978239698055e21*coep(15)**2)&
            + jx**2*((-2.3776152118610843e22 - 1.2105602470323046e27*v**2)*coen(5)**2&
            + (1.6362460086401178e22 + 2.858124474085605e26*v**2)*coen(8)**2&
            - 1.2105602470323046e27*coen(11)**2 + 2.379479230607052e25*coen(11)*coen(12)&
            + 6.782422283526404e25*coen(11)*coen(15) - 9.904959458950322e23*coen(12)*coen(15)&
            - 7.256651739621578e23*coen(15)**2 + (-6.254378962120489e21&
            - 1.7020468424799585e26*v**2)*coen(5)*coep(5) - 3.060639436754951e20*coep(5)**2&
            - 1.9343448927013684e24*v**2*coep(5)**2 + (2.3005607191824703e21&
            + 7.128716409376532e24*v**2)*coen(8)*coep(8) + 2.6145448917351977e19*coep(8)**2&
            + 4.445100109012269e22*v**2*coep(8)**2 - 1.7020468424799585e26*coen(11)*coep(11)&
            + 2.9674411997794392e23*coen(12)*coep(11) + 6.674012146250384e24*coen(15)*coep(11)&
            - 1.9343448927013684e24*coep(11)**2 + 2.9674411997794258e23*coen(11)*coep(12)&
            - 1.2352444351084134e22*coen(15)*coep(12) + 3.70068675569061e21*coep(11)*coep(12)&
            + 3.0567487399063343e24*coen(11)*coep(15) - 1.2352444351084134e22*coen(12)*coep(15)&
            - 1.1013233523758746e23*coen(15)*coep(15) + 1.1080365143780885e23*coep(11)*coep(15)&
            - 1.5404695201326986e20*coep(12)*coep(15) - 1.2605978239698055e21*coep(15)**2))&
            + Q**8*(1.5557028544975434e19*coen(3)**2&
            + 8.212608059515753e20*coep(1)**2 + 2.6744294664094118e19*coep(1)*coep(3)&
            + 2.0500730273920147e17*coep(3)**2 + coen(3)*(3.6702188649644143e20*coep(1)&
            + 4.1981156019162593e18*coep(3)) + jx*((-5.605113410811264e16&
            - 1.3019542011910277e22*v**2)*coen(5)**2 + (1.7597780617309523e17&
            + 2.126616812690239e22*v**2)*coen(8)**2 - 1.3019542011910277e22*coen(11)**2&
            + 7.338582032691011e20*coen(11)*coen(12) - 8.772959919686893e18*coen(12)**2&
            + 3.1331530514279576e20*coen(11)*coen(15) - 1.0371352363316955e19*coen(12)*coen(15)&
            - 1.5979872161633247e18*coen(15)**2 + (-3.0122724702753868e16&
            - 5.187523394792918e21*v**2)*coen(5)*coep(5) - 3.557088391481625e15*coep(5)**2&
            - 4.2493928462784535e20*v**2*coep(5)**2 + (7.0116827892421576e16&
            + 5.59412111237732e21*v**2)*coen(8)*coep(8) + 5.743664638676543e15*coep(8)**2&
            + 2.7375360198385838e20*v**2*coep(8)**2 - 5.187523394792918e21*coen(11)*coep(11)&
            + 1.2234062883214714e20*coen(12)*coep(11) + 7.382974605028806e19*coen(15)*coep(11)&
            - 4.2493928462784535e20*coep(11)**2 + 7.66849476336762e19*coen(11)*coep(12)&
            - 1.3314495411505864e18*coen(12)*coep(12) - 1.3993718673054195e18*coen(15)*coep(12)&
            + 8.914764888031372e18*coep(11)*coep(12) - 1.5240050895853432e16*coep(12)**2&
            + 5.0574436960004825e19*coen(11)*coep(15) - 1.3993718673054195e18*coen(12)*coep(15)&
            - 6.067266450157341e17*coen(15)*coep(15) + 9.899913859003509e18*coep(11)*coep(15)&
            - 1.3667153515946763e17*coep(12)*coep(15) - 4.9583853741702184e16*coep(15)**2)&
            + jx**2*((-5.605113410811264e16 - 1.3019542011910277e22*v**2)*coen(5)**2&
            + (1.7597780617309523e17 + 2.126616812690239e22*v**2)*coen(8)**2&
            - 1.3019542011910277e22*coen(11)**2 + 7.338582032691011e20*coen(11)*coen(12)&
            - 8.772959919686893e18*coen(12)**2 + 3.1331530514279576e20*coen(11)*coen(15)&
            - 1.0371352363316955e19*coen(12)*coen(15) - 1.5979872161633247e18*coen(15)**2&
            + (-3.0122724702753868e16 - 5.187523394792918e21*v**2)*coen(5)*coep(5)&
            - 3.557088391481625e15*coep(5)**2 - 4.2493928462784535e20*v**2*coep(5)**2&
            + (7.0116827892421576e16 + 5.59412111237732e21*v**2)*coen(8)*coep(8)&
            + 5.743664638676543e15*coep(8)**2 + 2.7375360198385838e20*v**2*coep(8)**2&
            - 5.187523394792918e21*coen(11)*coep(11) + 1.2234062883214714e20*coen(12)*coep(11)&
            + 7.382974605028806e19*coen(15)*coep(11) - 4.2493928462784535e20*coep(11)**2&
            + 7.66849476336762e19*coen(11)*coep(12) - 1.3314495411505864e18*coen(12)*coep(12)&
            - 1.3993718673054195e18*coen(15)*coep(12) + 8.914764888031372e18*coep(11)*coep(12)&
            - 1.5240050895853432e16*coep(12)**2 + 5.0574436960004825e19*coen(11)*coep(15)&
            - 1.3993718673054195e18*coen(12)*coep(15) - 6.067266450157341e17*coen(15)*coep(15)&
            + 9.899913859003509e18*coep(11)*coep(15) - 1.3667153515946763e17*coep(12)*coep(15)&
            - 4.9583853741702184e16*coep(15)**2)) + Q**6*(1.0076503974000293e13*coen(3)**2&
            + 9.544728608497912e15*coep(1)**2 + 1.5700344963438847e14*coep(1)*coep(3)&
            + 5.423411867275352e11*coep(3)**2 + coen(3)*(8.178581579292468e14*coep(1)&
            + 4.873951094212695e12*coep(3)) + jx*((-1.3460224138339216e10&
            - 9.156460455956056e15*v**2)*coen(5)**2 + (1.2376271161272784e11&
            + 5.013396767064092e16*v**2)*coen(8)**2 - 9.156460455956056e15*coen(11)**2&
            + 8.896303987003129e14*coen(11)*coen(12) - 1.9318934272438926e13*coen(12)**2&
            + 1.2111703869833397e14*coen(11)*coen(15) - 6.717669316000194e12*coen(12)*coen(15)&
            - 3.3332264322096136e11*coen(15)**2 + (-1.095208019720987e10&
            - 6.193799239382754e15*v**2)*coen(5)*coep(5) - 2.1077630390500536e9*coep(5)**2&
            - 9.61942968570282e14*v**2*coep(5)**2 + (8.371809093023834e10&
            + 2.694275023029199e16*v**2)*coen(8)*coep(8) + 1.300204055701614e10*coep(8)**2&
            + 3.1815762028326365e15*v**2*coep(8)**2 - 6.193799239382754e15*coen(11)*coep(11)&
            + 2.7261938597641562e14*coen(12)*coep(11) + 4.5846048469649734e13*coen(15)*coep(11)&
            - 9.61942968570282e14*coep(11)**2 + 1.9025212131695625e14*coen(11)*coep(12)&
            - 7.335047525936125e12*coen(12)*coep(12) - 1.624650364737565e12*coen(15)*coep(12)&
            + 5.233448321146283e13*coep(11)*coep(12) - 5.994461042748823e11*coep(12)**2&
            + 3.1841989242358746e13*coen(11)*coep(15) - 1.624650364737565e12*coen(12)*coep(15)&
            - 1.9494874421796133e11*coen(15)*coep(15) + 1.1272743456472484e13*coep(11)*coep(15)&
            - 3.615607911516901e11*coep(12)*coep(15) - 2.708556725092799e10*coep(15)**2)&
            + jx**2*((-1.3460224138339216e10 - 9.156460455956056e15*v**2)*coen(5)**2&
            + (1.2376271161272784e11 + 5.013396767064092e16*v**2)*coen(8)**2&
            - 9.156460455956056e15*coen(11)**2 + 8.896303987003129e14*coen(11)*coen(12)&
            - 1.9318934272438926e13*coen(12)**2 + 1.2111703869833397e14*coen(11)*coen(15)&
            - 6.717669316000194e12*coen(12)*coen(15) - 3.3332264322096136e11*coen(15)**2&
            + (-1.095208019720987e10 - 6.193799239382754e15*v**2)*coen(5)*coep(5)&
            - 2.1077630390500536e9*coep(5)**2 - 9.61942968570282e14*v**2*coep(5)**2&
            + (8.371809093023834e10 + 2.694275023029199e16*v**2)*coen(8)*coep(8)&
            + 1.300204055701614e10*coep(8)**2 + 3.1815762028326365e15*v**2*coep(8)**2&
            - 6.193799239382754e15*coen(11)*coep(11) + 2.7261938597641562e14*coen(12)*coep(11)&
            + 4.5846048469649734e13*coen(15)*coep(11) - 9.61942968570282e14*coep(11)**2&
            + 1.9025212131695625e14*coen(11)*coep(12) - 7.335047525936125e12*coen(12)*coep(12)&
            - 1.624650364737565e12*coen(15)*coep(12) + 5.233448321146283e13*coep(11)*coep(12)&
            - 5.994461042748823e11*coep(12)**2 + 3.1841989242358746e13*coen(11)*coep(15)&
            - 1.624650364737565e12*coen(12)*coep(15) - 1.9494874421796133e11*coen(15)*coep(15)&
            + 1.1272743456472484e13*coep(11)*coep(15) - 3.615607911516901e11*coep(12)*coep(15)&
            - 2.708556725092799e10*coep(15)**2)) + Q**4*(660169.9917674549*coen(3)**2&
            + 5.655756608953942e9*coep(1)**2 + 4.6646001175373174e7*coep(1)*coep(3)&
            + 75299.71290104554*coep(3)**2 + coen(3)*(1.5691888750223106e8*coep(1)&
            + 453196.52560241125*coep(3)) + jx*((-389.05610995262924&
            - 7.785724188715996e8*v**2)*coen(5)**2 + (10523.524260267077&
            + 1.2039264727266396e10*v**2)*coen(8)**2 - 7.785724188715996e8*coen(11)**2&
            + 1.1603369709844604e8*coen(11)*coen(12) - 4.029718242278611e6*coen(12)**2&
            + 5.152140963166782e6*coen(11)*coen(15) - 440113.32784496987*coen(12)*coen(15)&
            - 5964.78767566325*coen(15)**2 + (-418.95466210504145&
            - 7.377883335430492e8*v**2)*coen(5)*coep(5) - 110.54421446185137*coep(5)**2&
            - 1.6879037134241116e8*v**2*coep(5)**2 + (9972.268781669674&
            + 9.795898749775978e9*v**2)*coen(8)*coep(8) + 2281.4442493297356*coep(8)**2&
            + 1.8852522029846475e9*v**2*coep(8)**2 - 7.377883335430492e8*coen(11)*coep(11)&
            + 5.230629583407702e7*coen(12)*coep(11) + 2.6902254608291592e6*coen(15)*coep(11)&
            - 1.6879037134241116e8*coep(11)**2 + 3.598379160270143e7*coen(11)*coep(12)&
            - 2.3568411173424385e6*coen(12)*coep(12) - 151065.50853413704*coen(15)*coep(12)&
            + 1.5548667058457725e7*coep(11)*coep(12) - 327452.1148602988*coep(12)**2&
            + 1.8386685212664616e6*coen(11)*coep(15) - 151065.50853413704*coen(12)*coep(15)&
            - 4724.499875972484*coen(15)*coep(15) + 935481.9290718851*coep(11)*coep(15)&
            - 50199.808600697026*coep(12)*coep(15) - 917.6189466669387*coep(15)**2)&
            + jx**2*((-389.05610995262924 - 7.785724188715996e8*v**2)*coen(5)**2&
            + (10523.524260267077 + 1.2039264727266396e10*v**2)*coen(8)**2&
            - 7.785724188715996e8*coen(11)**2 + 1.1603369709844604e8*coen(11)*coen(12)&
            - 4.029718242278611e6*coen(12)**2 + 5.152140963166782e6*coen(11)*coen(15)&
            - 440113.32784496987*coen(12)*coen(15) - 5964.78767566325*coen(15)**2&
            + (-418.95466210504145 - 7.377883335430492e8*v**2)*coen(5)*coep(5)&
            - 110.54421446185137*coep(5)**2 - 1.6879037134241116e8*v**2*coep(5)**2&
            + (9972.268781669674 + 9.795898749775978e9*v**2)*coen(8)*coep(8)&
            + 2281.4442493297356*coep(8)**2 + 1.8852522029846475e9*v**2*coep(8)**2&
            - 7.377883335430492e8*coen(11)*coep(11) + 5.230629583407702e7*coen(12)*coep(11)&
            + 2.6902254608291592e6*coen(15)*coep(11) - 1.6879037134241116e8*coep(11)**2&
            + 3.598379160270143e7*coen(11)*coep(12) - 2.3568411173424385e6*coen(12)*coep(12)&
            - 151065.50853413704*coen(15)*coep(12) + 1.5548667058457725e7*coep(11)*coep(12)&
            - 327452.1148602988*coep(12)**2 + 1.8386685212664616e6*coen(11)*coep(15)&
            - 151065.50853413704*coen(12)*coep(15) - 4724.499875972484*coen(15)*coep(15)&
            + 935481.9290718851*coep(11)*coep(15) - 50199.808600697026*coep(12)*coep(15)&
            - 917.6189466669387*coep(15)**2)) + Q**3*(-62.218734835809705*coen(3)**2&
            - 1.7606533423209535e6*coep(1)**2 + coen(3)*(-28061.757383050764*coep(1)&
            - 49.28128560456462*coep(3)) - 9758.017427190876*coep(1)*coep(3)&
            - 9.571688554133965*coep(3)**2 + jx*((0.025860001359274448&
            + 100081.77065028419*v**2)*coen(5)**2 + (-1.3527488463760295&
            - 2.7070978054476213e6*v**2)*coen(8)**2 + 100081.77065028419*coen(11)**2&
            - 17914.00408835412*coen(11)*coen(12) + 765.1374458423087*coen(12)**2&
            - 403.3848229794882*coen(11)*coen(15) + 41.47915655720647*coen(12)*coen(15)&
            + 0.21582950398286507*coen(15)**2 + (0.031027669968158155&
            + 107772.94928170991*v**2)*coen(5)*coep(5) + 0.009218136492788898*coep(5)**2&
            + 28436.671306444467*v**2*coep(5)**2 + (-1.4567061700058108&
            - 2.5652914619219652e6*v**2)*coen(8)*coep(8) - 0.38436244737300584*coep(8)**2&
            - 586884.4474403177*v**2*coep(8)**2 + 107772.94928170991*coen(11)*coep(11)&
            - 9353.919127683588*coen(12)*coep(11) - 238.28901046632228*coen(15)*coep(11)&
            + 28436.671306444467*coep(11)**2 - 6393.054002709197*coen(11)*coep(12)&
            + 525.2550651926373*coen(12)*coep(12) + 16.42709520152154*coen(15)*coep(12)&
            - 3252.672475730292*coep(11)*coep(12) + 87.27241577205754*coep(12)**2&
            - 163.21957293150876*coen(11)*coep(15) + 16.42709520152154*coen(12)*coep(15)&
            + 0.19460352477941747*coen(15)*coep(15) - 95.15383644216233*coep(11)*coep(15)&
            + 6.381125702755976*coep(12)*coep(15) + 0.043866259197329095*coep(15)**2)&
            + jx**2*((0.025860001359274448 + 100081.77065028419*v**2)*coen(5)**2&
            + (-1.3527488463760295 - 2.7070978054476213e6*v**2)*coen(8)**2&
            + 100081.77065028419*coen(11)**2 - 17914.00408835412*coen(11)*coen(12)&
            + 765.1374458423087*coen(12)**2 - 403.3848229794882*coen(11)*coen(15)&
            + 41.47915655720647*coen(12)*coen(15) + 0.21582950398286507*coen(15)**2&
            + (0.031027669968158155 + 107772.94928170991*v**2)*coen(5)*coep(5)&
            + 0.009218136492788898*coep(5)**2 + 28436.671306444467*v**2*coep(5)**2&
            + (-1.4567061700058108 - 2.5652914619219652e6*v**2)*coen(8)*coep(8)&
            - 0.38436244737300584*coep(8)**2 - 586884.4474403177*v**2*coep(8)**2&
            + 107772.94928170991*coen(11)*coep(11) - 9353.919127683588*coen(12)*coep(11)&
            - 238.28901046632228*coen(15)*coep(11) + 28436.671306444467*coep(11)**2&
            - 6393.054002709197*coen(11)*coep(12) + 525.2550651926373*coen(12)*coep(12)&
            + 16.42709520152154*coen(15)*coep(12) - 3252.672475730292*coep(11)*coep(12)&
            + 87.27241577205754*coep(12)**2 - 163.21957293150876*coen(11)*coep(15)&
            + 16.42709520152154*coen(12)*coep(15) + 0.19460352477941747*coen(15)*coep(15)&
            - 95.15383644216233*coep(11)*coep(15) + 6.381125702755976*coep(12)*coep(15)&
            + 0.043866259197329095*coep(15)**2)) + Q**5*(-3.476890424440825e9*coen(3)**2&
            - 1.003403268365138e13*coep(1)**2 + coen(3)*(-4.782203974570874e11*coep(1)&
            - 2.0335114814833202e9*coep(3)) - 1.1758605236730098e11*coep(1)*coep(3)&
            - 2.825297090689137e8*coep(3)**2 + jx*((3.026608688419857e6&
            + 3.4625418561829233e12*v**2)*coen(5)**2 + (-4.680122534849323e7&
            - 3.183703070539217e13*v**2)*coen(8)**2 + 3.4625418561829233e12*coen(11)**2&
            - 4.211241776812121e11*coen(11)*coen(12) + 1.1678674598711958e10*coen(12)**2&
            - 3.337176663464278e10*coen(11)*coen(15) + 2.3179269496272163e9*coen(12)*coen(15)&
            + 6.328919803864056e7*coen(15)**2 + (2.8680653544759867e6&
            + 2.8173406107775474e12*v**2)*coen(5)*coep(5) + 656152.7123796121*coep(5)**2&
            + 5.422062567916956e11*v**2*coep(5)**2 + (-3.80804040167813e7&
            - 2.153585192835018e13*v**2)*coen(8)*coep(8) - 7.328696161219933e6*coep(8)**2&
            - 3.3446775612171265e12*v**2*coep(8)**2 + 2.8173406107775474e12*coen(11)*coep(11)&
            - 1.5940679915236246e11*coen(12)*coep(11) - 1.504350496232515e10*coen(15)*coep(11)&
            + 5.422062567916956e11*coep(11)**2 - 1.1071465814831535e11*coen(11)*coep(12)&
            + 5.648912458747223e9*coen(12)*coep(12) + 6.7783716049444e8*coen(15)*coep(12)&
            - 3.9195350789100334e10*coep(11)*coep(12) + 6.285737848774633e8*coep(12)**2&
            - 1.0349085877838955e10*coen(11)*coep(15) + 6.7783716049444e8*coen(12)*coep(15)&
            + 4.344705911652258e7*coen(15)*coep(15) - 4.471860343417067e9*coep(11)*coep(15)&
            + 1.8835313937927577e8*coep(12)*coep(15) + 7.218835302234934e6*coep(15)**2)&
            + jx**2*((3.026608688419857e6 + 3.4625418561829233e12*v**2)*coen(5)**2&
            + (-4.680122534849323e7 - 3.183703070539217e13*v**2)*coen(8)**2&
            + 3.4625418561829233e12*coen(11)**2 - 4.211241776812121e11*coen(11)*coen(12)&
            + 1.1678674598711958e10*coen(12)**2 - 3.337176663464278e10*coen(11)*coen(15)&
            + 2.3179269496272163e9*coen(12)*coen(15) + 6.328919803864056e7*coen(15)**2&
            + (2.8680653544759867e6 + 2.8173406107775474e12*v**2)*coen(5)*coep(5)&
            + 656152.7123796121*coep(5)**2 + 5.422062567916956e11*v**2*coep(5)**2&
            + (-3.80804040167813e7 - 2.153585192835018e13*v**2)*coen(8)*coep(8)&
            - 7.328696161219933e6*coep(8)**2 - 3.3446775612171265e12*v**2*coep(8)**2&
            + 2.8173406107775474e12*coen(11)*coep(11) - 1.5940679915236246e11*coen(12)*coep(11)&
            - 1.504350496232515e10*coen(15)*coep(11) + 5.422062567916956e11*coep(11)**2&
            - 1.1071465814831535e11*coen(11)*coep(12) + 5.648912458747223e9*coen(12)*coep(12)&
            + 6.7783716049444e8*coen(15)*coep(12) - 3.9195350789100334e10*coep(11)*coep(12)&
            + 6.285737848774633e8*coep(12)**2 - 1.0349085877838955e10*coen(11)*coep(15)&
            + 6.7783716049444e8*coen(12)*coep(15) + 4.344705911652258e7*coen(15)*coep(15)&
            - 4.471860343417067e9*coep(11)*coep(15) + 1.8835313937927577e8*coep(12)*coep(15)&
            + 7.218835302234934e6*coep(15)**2)) + Q**7*(-1.6668613918838974e16*coen(3)**2&
            - 4.432544142258793e18*coep(1)**2 + coen(3)*(-7.701185092036424e17*coep(1)&
            - 6.328769152685194e15*coep(3)) - 1.0326605887479565e17*coep(1)*coep(3)&
            - 5.172094659261278e14*coep(3)**2 + jx*((3.559466287199652e13&
            + 1.4418734483258665e19*v**2)*coen(5)**2 + (-1.9488990164439153e14&
            - 4.526897274303262e19*v**2)*coen(8)**2 + 1.4418734483258665e19*coen(11)**2&
            - 1.089397921640342e18*coen(11)*coen(12) + 1.8030606107875504e16*coen(12)**2&
            - 2.5586134716816378e17*coen(11)*coen(15) + 1.1112409279225982e16*coen(12)*coen(15)&
            + 9.660146076016965e14*coen(15)**2 + (2.4077665915026117e13&
            + 7.748845341176446e18*v**2)*coen(5)*coep(5) + 3.7394401289719346e12*coep(5)**2&
            + 9.150343497301664e17*v**2*coep(5)**2 + (-1.0473677202065914e14&
            - 1.8037028871514733e19*v**2)*coen(8)*coep(8) - 1.2368003213264576e13*coep(8)**2&
            - 1.4775147140862643e18*v**2*coep(8)**2 + 7.748845341176446e18*coen(11)*coep(11)&
            - 2.5670616973454746e17*coen(12)*coep(11) - 7.84064522322834e16*coen(15)*coep(11)&
            + 9.150343497301664e17*coep(11)**2 - 1.7584741507360928e17*coen(11)*coep(12)&
            + 4.865618687697681e15*coen(12)*coep(12) + 2.1095897175617312e15*coen(15)*coep(12)&
            - 3.442201962493189e16*coep(11)*coep(12) + 2.37603595972213e14*coep(12)**2&
            - 5.471728948659208e16*coen(11)*coep(15) + 2.1095897175617312e15*coen(12)*coep(15)&
            + 4.6725610051802275e14*coen(15)*coep(15) - 1.505161176753505e16*coep(11)*coep(15)&
            + 3.448063106174185e14*coep(12)*coep(15) + 5.199318236112899e13*coep(15)**2)&
            + jx**2*((3.559466287199652e13 + 1.4418734483258665e19*v**2)*coen(5)**2&
            + (-1.9488990164439153e14 - 4.526897274303262e19*v**2)*coen(8)**2&
            + 1.4418734483258665e19*coen(11)**2 - 1.089397921640342e18*coen(11)*coen(12)&
            + 1.8030606107875504e16*coen(12)**2 - 2.5586134716816378e17*coen(11)*coen(15)&
            + 1.1112409279225982e16*coen(12)*coen(15) + 9.660146076016965e14*coen(15)**2&
            + (2.4077665915026117e13 + 7.748845341176446e18*v**2)*coen(5)*coep(5)&
            + 3.7394401289719346e12*coep(5)**2 + 9.150343497301664e17*v**2*coep(5)**2&
            + (-1.0473677202065914e14 - 1.8037028871514733e19*v**2)*coen(8)*coep(8)&
            - 1.2368003213264576e13*coep(8)**2 - 1.4775147140862643e18*v**2*coep(8)**2&
            + 7.748845341176446e18*coen(11)*coep(11) - 2.5670616973454746e17*coen(12)*coep(11)&
            - 7.84064522322834e16*coen(15)*coep(11) + 9.150343497301664e17*coep(11)**2&
            - 1.7584741507360928e17*coen(11)*coep(12) + 4.865618687697681e15*coen(12)*coep(12)&
            + 2.1095897175617312e15*coen(15)*coep(12) - 3.442201962493189e16*coep(11)*coep(12)&
            + 2.37603595972213e14*coep(12)**2 - 5.471728948659208e16*coen(11)*coep(15)&
            + 2.1095897175617312e15*coen(12)*coep(15) + 4.6725610051802275e14*coen(15)*coep(15)&
            - 1.505161176753505e16*coep(11)*coep(15) + 3.448063106174185e14*coep(12)*coep(15)&
            + 5.199318236112899e13*coep(15)**2)) + Q**9*(-7.56941763787305e21*coen(3)**2&
            - 2.0177162793411037e22*coep(1)**2 + coen(3)*(-6.961665940143504e22*coep(1)&
            - 1.1487910275421528e21*coep(3)) - 1.1557935307197557e21*coep(1)*coep(3)&
            - 1.3149303212281528e19*coep(3)**2 + jx*((5.061193797438333e19&
            + 6.116237073286959e24*v**2)*coen(5)**2 + (-8.26697268772567e19&
            - 4.209120319022931e24*v**2)*coen(8)**2 + 6.116237073286959e24*coen(11)**2&
            - 2.358249539068424e23*coen(11)*coen(12) + 1.7219781592850217e21*coen(12)**2&
            - 2.110606256184064e23*coen(11)*coen(15) + 5.0462784252487e21*coen(12)*coen(15)&
            + 1.491421713731207e21*coen(15)**2 + (2.016588694577277e19&
            + 1.6088921490607603e24*v**2)*coen(5)*coep(5) + 1.6519014798514977e18*coep(5)**2&
            + 7.873265740250685e22*v**2*coep(5)**2 + (-2.1746487741413822e19&
            - 5.918020161469946e23*v**2)*coen(8)*coep(8) - 1.0641849238011848e18*coep(8)**2&
            - 6.725720931137011e21*v**2*coep(8)**2 + 1.6088921490607603e24*coen(11)*coep(11)&
            - 2.3205553133811678e22*coen(12)*coep(11) - 3.5185666038529954e22*coen(15)*coep(11)&
            + 7.873265740250685e22*coep(11)**2 - 1.0628321277548236e22*coen(11)*coep(12)&
            + 4.294947288679701e19*coen(12)*coep(12) + 3.829303425140509e20*coen(15)*coep(12)&
            - 3.852645102399185e20*coep(11)*coep(12) + 2.6781077496645306e17*coep(12)**2&
            - 2.2054905090627483e22*coen(11)*coep(15) + 3.829303425140509e20*coen(12)*coep(15)&
            + 4.0246508176999376e20*coen(15)*coep(15) - 2.56392290244516e21*coep(11)*coep(15)&
            + 8.766202141521018e18*coep(12)*coep(15) + 1.9653646703467885e19*coep(15)**2)&
            + jx**2*((5.061193797438333e19 + 6.116237073286959e24*v**2)*coen(5)**2&
            + (-8.26697268772567e19 - 4.209120319022931e24*v**2)*coen(8)**2&
            + 6.116237073286959e24*coen(11)**2 - 2.358249539068424e23*coen(11)*coen(12)&
            + 1.7219781592850217e21*coen(12)**2 - 2.110606256184064e23*coen(11)*coen(15)&
            + 5.0462784252487e21*coen(12)*coen(15) + 1.491421713731207e21*coen(15)**2&
            + (2.016588694577277e19 + 1.6088921490607603e24*v**2)*coen(5)*coep(5)&
            + 1.6519014798514977e18*coep(5)**2 + 7.873265740250685e22*v**2*coep(5)**2&
            + (-2.1746487741413822e19 - 5.918020161469946e23*v**2)*coen(8)*coep(8)&
            - 1.0641849238011848e18*coep(8)**2 - 6.725720931137011e21*v**2*coep(8)**2&
            + 1.6088921490607603e24*coen(11)*coep(11) - 2.3205553133811678e22*coen(12)*coep(11)&
            - 3.5185666038529954e22*coen(15)*coep(11) + 7.873265740250685e22*coep(11)**2&
            - 1.0628321277548236e22*coen(11)*coep(12) + 4.294947288679701e19*coen(12)*coep(12)&
            + 3.829303425140509e20*coen(15)*coep(12) - 3.852645102399185e20*coep(11)*coep(12)&
            + 2.6781077496645306e17*coep(12)**2 - 2.2054905090627483e22*coen(11)*coep(15)&
            + 3.829303425140509e20*coen(12)*coep(15) + 4.0246508176999376e20*coen(15)*coep(15)&
            - 2.56392290244516e21*coep(11)*coep(15) + 8.766202141521018e18*coep(12)*coep(15)&
            + 1.9653646703467885e19*coep(15)**2))))/(EXP(17272.86157621062*Q)*mx**2)
        else
            write(*,*)'do not prepare this target isotope in the code yet.'
            stop
        endif
    else
        write(*,*)'do not prepare this target isotope in the code yet.'
        stop
    endif
  
    return
    end subroutine 
    
    
!=======================================================================
! Transition probability considering interference between operators
!=======================================================================
    subroutine Calc_total_transition_prob_rel(jx,mx,Z,A,v,Q,ans)
    implicit none
    ! we follow the notation of 1308.6288
    real*8, intent(in) :: jx ! the DM spin
    real*8, intent(in) :: mx
    integer, intent(in) :: Z
    integer, intent(in) :: A
    real*8, intent(in) :: v ! relative velocity 
    real*8, intent(in) :: Q ! recoil energy
    real*8, intent(out) :: ans
    
    real*8 mT ! mass of target nuclear [GeV]
    real*8, parameter :: mM=246.2d0 ! the fiducial scale [GeV]
    real*8, parameter :: atomicmassunit=0.931494028d0 ! [GeV] 
    real*8, parameter :: mN=0.938272d0 ! [GeV]
    
    if (Z==54) then
            if (A==128) then
                mT=127.9035312*atomicmassunit
            elseif (A==129) then 
                mT=128.9047801*atomicmassunit
            elseif (A==130) then 
                mT=129.9035094*atomicmassunit
            elseif (A==131) then 
                mT=130.905072*atomicmassunit
            elseif (A==132) then 
                mT=131.904144*atomicmassunit
            elseif (A==134) then 
                mT=133.905395*atomicmassunit
            elseif (A==136) then 
                mT=135.907214*atomicmassunit
            else
                write(*,*)'do not prepare this target isotope in the code yet.'
                stop             
            endif
    else
        write(*,*)'do not prepare this target isotope in the code yet.'
        stop  
    endif  
    
    DD__coep(1) = DD__dp(1)*(1/(2*mN*mM))*((2*mT*Q)**0.5)**2 - DD__dp(3)*(1/(2*mx*mM))*((2*mT*Q)**0.5)**2 
    DD__coen(1) = DD__dn(1)*(1/(2*mN*mM))*((2*mT*Q)**0.5)**2 - DD__dn(3)*(1/(2*mx*mM))*((2*mT*Q)**0.5)**2   
    DD__coep(3) = -DD__dp(1)*2*(mN/mM)
    DD__coen(3) = -DD__dn(1)*2*(mN/mM)
    DD__coep(7) = -DD__dp(2)*2
    DD__coen(7) = -DD__dn(2)*2
    DD__coep(9) = DD__dp(2)*2*mN/mx + DD__dp(6)*2
    DD__coen(9) = DD__dn(2)*2*mN/mx + DD__dn(6)*2
    DD__coep(5) = DD__dp(3)*2*(mN/mM)
    DD__coen(5) = DD__dn(3)*2*(mN/mM) 
    DD__coep(4) = DD__dp(1)*2*mN**2/(mM*mx)*(((2*mT*Q)**0.5)/mN)**2 - DD__dp(3)*2*mN/mM*(((2*mT*Q)**0.5)/mN)**2&
    + DD__dp(4)*4*(((2*mT*Q)**0.5)/mM)**2
    DD__coen(4) = DD__dn(1)*2*mN**2/(mM*mx)*(((2*mT*Q)**0.5)/mN)**2 - DD__dn(3)*2*mN/mM*(((2*mT*Q)**0.5)/mN)**2&
    + DD__dn(4)*4*(((2*mT*Q)**0.5)/mM)**2  
    DD__coep(6) = -DD__dp(1)*2*mN**2/(mM*mx) + DD__dp(3)*2*mN/mM - DD__dp(4)*4*(mN/mM)**2
    DD__coen(6) = -DD__dn(1)*2*mN**2/(mM*mx) + DD__dn(3)*2*mN/mM - DD__dn(4)*4*(mN/mM)**2 
    DD__coep(10) = -DD__dp(5)*(mN/mx)*(((2*mT*Q)**0.5)/mM)**2
    DD__coen(10) = -DD__dn(5)*(mN/mx)*(((2*mT*Q)**0.5)/mM)**2
    DD__coep(12) = -DD__dp(5)*4*(((2*mT*Q)**0.5)/mM)**2
    DD__coen(12) = -DD__dn(5)*4*(((2*mT*Q)**0.5)/mM)**2    
    DD__coep(15) = -DD__dp(5)*4*(mN/mM)**2 + DD__dp(7)*4*(mN/mM)**2
    DD__coen(15) = -DD__dn(5)*4*(mN/mM)**2 + DD__dn(7)*4*(mN/mM)**2
    DD__coep(8) = DD__dp(6)*2
    DD__coen(8) = DD__dn(6)*2
    DD__coep(11) = DD__dp(7)*(((2*mT*Q)**0.5)/mM)**2
    DD__coen(11) = DD__dn(7)*(((2*mT*Q)**0.5)/mM)**2    
    !set value for coefficients  
    
    call Calc_total_transition_prob(jx,mx,Z,A,v,Q,ans)
    
    return
    end subroutine     
    
    
!=======================================================================
! Transition probability calculated by DirectDM Effective Theory
!=======================================================================
    subroutine Calc_total_transition_prob_DirectDM(jx,mx,Z,A,v,Q,ans)
    implicit none
    ! we follow the notation of 1611.00368
    real*8, intent(in) :: jx ! the DM spin
    real*8, intent(in) :: mx
    integer, intent(in) :: Z
    integer, intent(in) :: A
    real*8, intent(in) :: v ! relative velocity 
    real*8, intent(in) :: Q ! recoil energy
    real*8, intent(out) :: ans
    
    real*8 mT ! mass of target nuclear [GeV]
    real*8 qsq ! q square
    real*8, parameter :: mM=246.2d0 ! the fiducial scale [GeV]
    real*8, parameter :: atomicmassunit=0.931494028d0 ! [GeV] 
    real*8, parameter :: mN=0.938272d0 ! [GeV]
    
    if (Z==54) then
            if (A==128) then
                mT=127.9035312*atomicmassunit
            elseif (A==129) then 
                mT=128.9047801*atomicmassunit
            elseif (A==130) then 
                mT=129.9035094*atomicmassunit
            elseif (A==131) then 
                mT=130.905072*atomicmassunit
            elseif (A==132) then 
                mT=131.904144*atomicmassunit
            elseif (A==134) then 
                mT=133.905395*atomicmassunit
            elseif (A==136) then 
                mT=135.907214*atomicmassunit
            else
                write(*,*)'do not prepare this target isotope in the code yet.'
                stop             
            endif
    else
        write(*,*)'do not prepare this target isotope in the code yet.'
        stop  
    endif  
    qsq=2*mT*Q
    DD__coep(1) = (-0.00116171*DD__Q5(1)/mx + 2.99383*DD__Q6(1))*mM**2
    DD__coep(3) = 0.0 
    DD__coep(4) = (-0.0138082*DD__Q5(1) - 1.96*DD__Q6(4))*mM**2
    DD__coep(5) = 0.00436302*DD__Q5(1)/qsq*mM**2
    DD__coep(6) = 0.0121728*DD__Q5(1)/qsq*mM**2
    DD__coep(7) = -0.98*DD__Q6(3)*mM**2
    DD__coep(8) = 5.98765*DD__Q6(2)*mM**2
    DD__coep(9) = (5.70437*DD__Q6(2) + 0.92014*DD__Q6(3)/mx)*mM**2
    DD__coep(10) = 0.0
    DD__coep(11) = 0.00436302*DD__Q5(2)/qsq*mM**2
    DD__coep(12) = 0.0
    DD__coep(13) = 0.0
    DD__coep(14) = 0.0
    DD__coep(15) = 0.0
    DD__coen(1) = 3.0*DD__Q6(1)*mM**2
    DD__coen(3) = 0.0 
    DD__coen(4) = (0.0094529*DD__Q5(1) - 1.96*DD__Q6(4))*mM**2
    DD__coen(5) = 0.0
    DD__coen(6) = -0.00833337*DD__Q5(1)/qsq*mM**2
    DD__coen(7) = -0.98*DD__Q6(3)*mM**2
    DD__coen(8) = 6.0*DD__Q6(2)*mM**2
    DD__coen(9) = (5.76253*DD__Q6(2) + 0.92014*DD__Q6(3)/mx)*mM**2
    DD__coen(10) = 0.0
    DD__coen(11) = 0.0
    DD__coen(12) = 0.0
    DD__coen(13) = 0.0
    DD__coen(14) = 0.0
    DD__coen(15) = 0.0
    !set value for coefficients  
    
    call Calc_total_transition_prob(jx,mx,Z,A,v,Q,ans)
    
    return
    end subroutine     
        

!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!   Compute the dRdQ per nuclei mass per time. unit:[GeV-1]   
!   Detector_type=PandaX is now availible.
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  subroutine Calc_dRdQ_EFT(Detector_type,jx,mx,Q_,ans)
    implicit none 
    real*8, intent(in) :: jx,mx,Q_
    real*8, intent(out) :: ans
    real*8 Q
    CHARACTER(LEN=*), intent(in) :: Detector_type

    real*8, parameter :: c = 299792458d-3  ! Speed of light in km/s
    real*8, parameter :: Mp=0.9383d0 ! [GeV]
    real*8, parameter :: Mn=0.9396d0 ! [GeV]
    real*8, parameter :: NA=6.022d23
    real*8, parameter :: Gev2kg=1.782662e-27
    real*8, parameter :: atomicmassunit=0.931494028d0 ! [GeV]
    real*8, parameter :: cmGeV = 1d0/(197.327053d-16)  ! 1 cm-1=197.327053d-16 GeV
    real*8, parameter :: pi=dacos(-1.0d0)
    integer,parameter :: ngrid=200

    real*8 f(ngrid),v(ngrid),ProbDen(ngrid),avgP(ngrid)

    real*8 eta(7),Armbar,ans_iso(7),xi(7),Arm(1:7)
    integer A(7),Z
    real*8 mr  ! reduced mass
    real*8 mT  ! target mass
    real*8 dsigmadE ! d\sigma/dE_R
    real*8 vmin,vmax
    integer i,j,niso



    if (trim(Detector_type).eq. 'Xenon') then 
      if (Q_<=0d0) then 
        Q=2d-7 !set a nuclear- recoil energy threshold
      else
        Q=Q_
      endif

      Z=54
      ! eta is the natural abundance of the ith isotope
      eta(1:7)=(/1.92d-2,26.44d-2,4.08d-2,21.18d-2,26.89d-2,10.44d-2,8.87d-2/)  
      A(1:7)=(/128,129,130,131,132,134,136/)
      Arm(1:7)=(/127.9035312d0,128.9047801d0,129.9035094d0,130.905072d0,131.904144d0,133.905395d0,135.907214d0/)
      niso=7
      Armbar=sum(Arm(:)*eta(:))
    endif

    do i=1,niso,1  ! run over all the targets 
      mT=Arm(i)*atomicmassunit
      mr=mT*mx/(mT+mx)

      vmin=dsqrt(Q*mT/2d0/mr/mr)
      !vmax=dmin1(DD__vesc/c,dsqrt(DD__detQmax*mT/2d0/mr/mr))
      vmax=(DD__vesc+DD__vobs)/c

      if (vmax>vmin) then 

        call Gen_linspace(vmin,vmax,ngrid,.false.,v)



        do j=1,ngrid,1

          call Maxwell_Boltzmann_f(v(j)*c,f(j)) ! all the theta and phi were integrated out. 

          f(j)=f(j)*(c**3) ! to nature unit

          call Calc_dsigmadQ_EFT(jx,mx,Z,A(i),v(j),Q,dsigmadE)

          !avgP(j)=4d0*pi*v(j)**3*f(j)*dsigmadE  ! v*f*dsigmadQ *dv^3  
          avgP(j)=v(j)**3*f(j)*dsigmadE  ! v*f(v)*dsigmadQ *v^2*dv where the sin(theta) is inside the subroutine: Maxwell_Boltzmann_f   
    
        enddo 
        call TableArea(v,avgP,ngrid,ans_iso(i))
      else
        ans_iso(i)=0d0
      endif

      !xi(i)=1d3*NA*mT*Gev2kg*eta(i)*Abar ! The expression of 1307.5955, not sure what is the definition.
      ans_iso(i)=eta(i)*ans_iso(i)
      ans_iso(i)=ans_iso(i)/(Armbar*atomicmassunit) ! per target mass


    enddo
    ans=sum(ans_iso(:))*DD__rho0/(cmGeV)**3/mx
    return
  end subroutine 









!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
!    The total evints  per day per kg 
!@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
  subroutine Predict_R_EFT(What_experiment,jx,mx,ans)
    implicit none
    real*8, intent(in) :: jx,mx
    real*8, intent(out) :: ans
    CHARACTER(LEN=*), intent(in) :: What_experiment
    integer :: i
    integer, parameter :: nQ=100
    real*8 dRdQ(nQ),Q(nQ),eff
    !real*8 :: exposure  ! kg-days
    real*8, parameter :: Gev2kg=1.782662d-27
    real*8, parameter :: cmGeV = 1d0/(197.327053d-16)  ! 1 cm-1=197.327053d-16 GeV
    real*8, parameter :: secGeV = 1d25/(6.582119d0)  ! 1 sec-1=6.582119d-25 GeV


    if (.not. DD__loaded) call loadDirectDet('formula',220d0,544d0)
    if (.not. DirectDetLike__loaded) call Read_DDLikeTable()


    if (trim(What_experiment).eq. 'PandaX-II') then 

      !exposure=33000d0
      do i=1,nQ,1
        Q(i)=PandaX16__Q(i)
        eff=PandaX16__eff(i)
        call Calc_dRdQ_EFT('Xenon',jx,mx,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff  

      enddo 

      
    elseif (trim(What_experiment).eq. 'PandaXRun9') then 

      do i=1,nQ,1
        Q(i)=PandaXRun9__Q(i)
        eff=PandaXRun9__eff(i)
        call Calc_dRdQ_EFT('Xenon',jx,mx,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff       
      enddo

      
    elseif (trim(What_experiment).eq. 'PandaXRun10') then 

      do i=1,nQ,1
        Q(i)=PandaXRun10__Q(i)
        eff=PandaXRun10__eff(i)
        call Calc_dRdQ_EFT('Xenon',jx,mx,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff       
      enddo 
      
      
    elseif (trim(What_experiment).eq. 'LUX-2013') then 

      do i=1,nQ,1
        Q(i)=LUX13__Q(i)
        eff=LUX13__eff(i)
        call Calc_dRdQ_EFT('Xenon',jx,mx,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff  

      enddo 


    elseif (trim(What_experiment).eq. 'LUX-2014-2016') then 

      do i=1,nQ,1
        Q(i)=LUX16__Q(i)
        eff=LUX16__eff(i)
        call Calc_dRdQ_EFT('Xenon',jx,mx,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff       
      enddo 
      
    elseif (trim(What_experiment).eq. 'XENON1T') then 

      do i=1,nQ,1
        Q(i)=XENON1T__Q(i)
        eff=XENON1T__eff(i)
        call Calc_dRdQ_EFT('Xenon',jx,mx,Q(i),dRdQ(i))
        dRdQ(i)=dRdQ(i)*eff       
      enddo 


    else
      write(*,*)'Not prepared yet!'
      STOP
    endif




    call TableArea(Q,dRdQ,nQ,ans)
    ! from unit less to 1/kg/s
    ans=ans/Gev2kg*secGeV 
    ans=ans*60d0*60d0*24d0

    return
  end subroutine









  
!=======================================================================
end module DMFormFactor
  
    
